package it.prato.comune.sit;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Map;

public class AdditionalFilterServerParams {
	public enum ServerParameterFormats {
		GEOSERVER,
		MAPSERVER,
		ALL
	};
	
	private Map<String, String> serverParams = null;
	private ServerParameterFormats defaultFormat = ServerParameterFormats.ALL; 
	
	public AdditionalFilterServerParams(Map<String, String> serverParams) {
		this.serverParams = serverParams;
	}
	
	public Map<String, String> getMap() {
		return serverParams;
	}
	
	public String getGeoserverParamsString() throws UnsupportedEncodingException {

		StringBuffer s = new StringBuffer();
		
		if (serverParams.size()>0) {
			s.append("viewparams=");
			for (String k: serverParams.keySet()) {
				String v = serverParams.get(k).replaceAll(",", "\\,").replaceAll(";", "\\;");
				s.append(k + ":" + URLEncoder.encode(v,"UTF-8") +  ";");
			}
		}

		return s.toString();		
	}

	public String getMapserverParamsString() throws UnsupportedEncodingException {
		
		StringBuffer s = new StringBuffer();
		
		if (serverParams.size()>0) {
			for (String k: serverParams.keySet()) {
				String v = serverParams.get(k);
				s.append(k + "=" + URLEncoder.encode(v,"UTF-8") + "&");
			}
			s.deleteCharAt(s.length()-1);
		}	

		return s.toString();
	}
	
	public String getDefaultParamsString() throws UnsupportedEncodingException {
		StringBuffer retVal = new StringBuffer();
		
		if (defaultFormat==ServerParameterFormats.ALL || defaultFormat==ServerParameterFormats.GEOSERVER){
			retVal.append(getGeoserverParamsString());
		}
		
		if (defaultFormat==ServerParameterFormats.ALL || defaultFormat==ServerParameterFormats.MAPSERVER){
			if (retVal.length()>0) {
				retVal.append("&");
			}
			retVal.append(getMapserverParamsString());
		}
		
		return retVal.toString();
	}

	/**
	 * @return the defaultFormat
	 */
	public ServerParameterFormats getDefaultFormat() {
		return defaultFormat;
	}

	/**
	 * @param defaultFormat the defaultFormat to set
	 */
	public void setDefaultFormat(ServerParameterFormats defaultFormat) {
		this.defaultFormat = defaultFormat;
	}

		
	/*
	 * customQueryForServer: function(customQuery, serverID){
			var retVal = {};
			
			for (var p in customQuery) {
				if (customQuery[p] instanceof Object) {
					if (p == serverID) {
						Ext.apply(retVal, customQuery[p]);
						}
				} else {
					retVal[p] = customQuery[p];
				}
			}
			return retVal;
		},
		
	 */
	
}
