/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
/*
 * Copyright  Comune di Prato - Sistema Informativo
 * Project:    Package SIT per elaborazione/interrogazione/gestione dati cartografici 
 * File:       ConfigBean.java
 * Author:     Alessandro Radaelli
 * Version:    1.0.0
 * CreationDate:  
 * ModifyDate:     
 *             
 */

package it.prato.comune.sit;

import it.prato.comune.utilita.core.type.IdType;

/**
 *  Classe ConfigBean. I parametri richiesti da org.geotools.data.DataStore per i vari tipi di Datastore 
 *  devono essere inseriti nella forma <i>nomeparametro1=valore1!!nomeparametro2=valore2!!</i><br><br> 
 *  Esempi validi sono:<br>
 *  per ShapeFile - url;%BASEPATH%/circoscrizioni/circoscrizioni.shp<br>
 *  per Postgres/postgis - dbtype;postgis!!host;localhost!!port;5432!!database;aaa!!user;postgres!!passwd;poposa  
 *  
 *  typeName nome della feature all'interno del datastore. Per ShapeFile e' il nome dell'unico layer contenuto, 
 *  per altri datastore (come per esempio Postgres, che puo' contenere piu' di un layer all'interno dello stesso datastore), 
 *  e' il nome del layer (o tabella nel caso di postgres). 
 * 
 */
public class ConfigBean {
	
	// ///////////////////////////////////////////////////////////////
	// TODOH Riferimento circolare con layer e territorio? 
	// Problemi di garbage collection? Quale altra soluzione? 
	// Singleton statico che ha il riferimento?
	// ///////////////////////////////////////////////////////////////

    private SITDataStorePool dsPool = null;
    private SITLayersManager  layersManager;
    
    private ConfigLayerBean layerConfig = new ConfigLayerBean();

	/**
	 * @return
	 * @see it.prato.comune.sit.ConfigLayerBean#getEnte()
	 */
	public String getEnte() {
		return layerConfig.getEnte();
	}

	/**
	 * @param ente
	 * @see it.prato.comune.sit.ConfigLayerBean#setEnte(java.lang.String)
	 */
	public void setEnte(String ente) {
		layerConfig.setEnte(ente);
	}

	/**
	 * @return
	 * @see it.prato.comune.sit.ConfigLayerBean#isDataStoreConHints()
	 */
	public boolean isDataStoreConHints() {
		return layerConfig.isDataStoreConHints();
	}

	/**
	 * @param bdataStoreConHints
	 * @see it.prato.comune.sit.ConfigLayerBean#setDataStoreConHints(boolean)
	 */
	public void setDataStoreConHints(boolean bdataStoreConHints) {
		layerConfig.setDataStoreConHints(bdataStoreConHints);
	}

	/**
	 * @return
	 * @see it.prato.comune.sit.ConfigLayerBean#getDataStoreParams()
	 */
	public String getDataStoreParams() {
		return layerConfig.getDataStoreParams();
	}

	/**
	 * @param dataStoreParams
	 * @see it.prato.comune.sit.ConfigLayerBean#setDataStoreParams(java.lang.String)
	 */
	public void setDataStoreParams(String dataStoreParams) {
		layerConfig.setDataStoreParams(dataStoreParams);
	}

	/**
	 * @return
	 * @see it.prato.comune.sit.ConfigLayerBean#getDataStoreFactory()
	 */
	public String getDataStoreFactory() {
		return layerConfig.getDataStoreFactory();
	}

	/**
	 * @param dataStoreConnectionPoolDriver
	 * @see it.prato.comune.sit.ConfigLayerBean#setDataStoreFactory(java.lang.String)
	 */
	public void setDataStoreFactory(String dataStoreConnectionPoolDriver) {
		layerConfig.setDataStoreFactory(dataStoreConnectionPoolDriver);
	}

	/**
	 * @return
	 * @see it.prato.comune.sit.ConfigLayerBean#getTypeName()
	 */
	public String getTypeName() {
		return layerConfig.getTypeName();
	}

	/**
	 * @param typeName
	 * @see it.prato.comune.sit.ConfigLayerBean#setTypeName(java.lang.String)
	 */
	public void setTypeName(String typeName) {
		layerConfig.setTypeName(typeName);
	}

	
	/**
	 * @return
	 * @see it.prato.comune.sit.ConfigLayerBean#getTypeNameScrittura()
	 */
	public String getTypeNameScrittura() {
		return layerConfig.getTypeNameScrittura();
	}

	/**
	 * @param typeNameScrittura
	 * @see it.prato.comune.sit.ConfigLayerBean#setTypeNameScrittura(java.lang.String)
	 */
	public void setTypeNameScrittura(String typeNameScrittura) {
		layerConfig.setTypeNameScrittura(typeNameScrittura);
	}

	/**
	 * @return
	 * @see it.prato.comune.sit.ConfigLayerBean#getStoricoCodiceEvento()
	 */
	public IdType getStoricoCodiceEvento() {
		return layerConfig.getStoricoCodiceEvento();
	}

	/**
	 * @param storicoCodiceEvento
	 * @see it.prato.comune.sit.ConfigLayerBean#setStoricoCodiceEvento(it.prato.comune.utilita.core.type.IdType)
	 */
	public void setStoricoCodiceEvento(IdType storicoCodiceEvento) {
		layerConfig.setStoricoCodiceEvento(storicoCodiceEvento);
	}

	/**
	 * @param storicoCodiceEvento
	 * @see it.prato.comune.sit.ConfigLayerBean#setStoricoCodiceEvento(java.lang.String)
	 */
	public void setStoricoCodiceEvento(String storicoCodiceEvento) {
		layerConfig.setStoricoCodiceEvento(storicoCodiceEvento);
	}

	/**
	 * @return
	 * @see it.prato.comune.sit.ConfigLayerBean#getStoricoCodiceNumeratore()
	 */
	public IdType getStoricoCodiceNumeratore() {
		return layerConfig.getStoricoCodiceNumeratore();
	}

	/**
	 * @param storicoCodiceNumeratore
	 * @see it.prato.comune.sit.ConfigLayerBean#setStoricoCodiceNumeratore(it.prato.comune.utilita.core.type.IdType)
	 */
	public void setStoricoCodiceNumeratore(IdType storicoCodiceNumeratore) {
		layerConfig.setStoricoCodiceNumeratore(storicoCodiceNumeratore);
	}

	/**
	 * @param storicoCodiceNumeratore
	 * @see it.prato.comune.sit.ConfigLayerBean#setStoricoCodiceNumeratore(java.lang.String)
	 */
	public void setStoricoCodiceNumeratore(String storicoCodiceNumeratore) {
		layerConfig.setStoricoCodiceNumeratore(storicoCodiceNumeratore);
	}

	/**
	 * @return
	 * @see it.prato.comune.sit.ConfigLayerBean#getStoricoCodiceVariazione()
	 */
	public IdType getStoricoCodiceVariazione() {
		return layerConfig.getStoricoCodiceVariazione();
	}

	/**
	 * @param storicoCodiceVariazione
	 * @see it.prato.comune.sit.ConfigLayerBean#setStoricoCodiceVariazione(it.prato.comune.utilita.core.type.IdType)
	 */
	public void setStoricoCodiceVariazione(IdType storicoCodiceVariazione) {
		layerConfig.setStoricoCodiceVariazione(storicoCodiceVariazione);
	}

	/**
	 * @param storicoCodiceVariazione
	 * @see it.prato.comune.sit.ConfigLayerBean#setStoricoCodiceVariazione(java.lang.String)
	 */
	public void setStoricoCodiceVariazione(String storicoCodiceVariazione) {
		layerConfig.setStoricoCodiceVariazione(storicoCodiceVariazione);
	}

	/**
	 * @return
	 * @see it.prato.comune.sit.ConfigLayerBean#getQueryDirettaJdbcDriver()
	 */
	public String getQueryDirettaJdbcDriver() {
		return layerConfig.getQueryDirettaJdbcDriver();
	}

	/**
	 * @param queryDirettaJdbcDriver
	 * @see it.prato.comune.sit.ConfigLayerBean#setQueryDirettaJdbcDriver(java.lang.String)
	 */
	public void setQueryDirettaJdbcDriver(String queryDirettaJdbcDriver) {
		layerConfig.setQueryDirettaJdbcDriver(queryDirettaJdbcDriver);
	}

	/**
	 * @return
	 * @see it.prato.comune.sit.ConfigLayerBean#getQueryDirettaJdbcUrl()
	 */
	public String getQueryDirettaJdbcUrl() {
		return layerConfig.getQueryDirettaJdbcUrl();
	}

	/**
	 * @param queryDirettaJdbcUrl
	 * @see it.prato.comune.sit.ConfigLayerBean#setQueryDirettaJdbcUrl(java.lang.String)
	 */
	public void setQueryDirettaJdbcUrl(String queryDirettaJdbcUrl) {
		layerConfig.setQueryDirettaJdbcUrl(queryDirettaJdbcUrl);
	}

	/**
	 * @return
	 * @see it.prato.comune.sit.ConfigLayerBean#getQueryDirettaPasswd()
	 */
	public String getQueryDirettaPasswd() {
		return layerConfig.getQueryDirettaPasswd();
	}

	/**
	 * @param queryDirettaPasswd
	 * @see it.prato.comune.sit.ConfigLayerBean#setQueryDirettaPasswd(java.lang.String)
	 */
	public void setQueryDirettaPasswd(String queryDirettaPasswd) {
		layerConfig.setQueryDirettaPasswd(queryDirettaPasswd);
	}

	/**
	 * @return
	 * @see it.prato.comune.sit.ConfigLayerBean#getQueryDirettaUser()
	 */
	public String getQueryDirettaUser() {
		return layerConfig.getQueryDirettaUser();
	}

	/**
	 * @param queryDirettaUser
	 * @see it.prato.comune.sit.ConfigLayerBean#setQueryDirettaUser(java.lang.String)
	 */
	public void setQueryDirettaUser(String queryDirettaUser) {
		layerConfig.setQueryDirettaUser(queryDirettaUser);
	}

	/**
	 * @return
	 * @see it.prato.comune.sit.ConfigLayerBean#getDsPoolID()
	 */
	public String getDsPoolID() {
		return layerConfig.getDsPoolID();
	}

	/**
	 * @param dsPoolID
	 * @see it.prato.comune.sit.ConfigLayerBean#setDsPoolID(java.lang.String)
	 */
	public void setDsPoolID(String dsPoolID) {
		layerConfig.setDsPoolID(dsPoolID);
	}

	/**
	 * @return
	 * @see it.prato.comune.sit.ConfigLayerBean#getSRID()
	 */
	public String getSRID() {
		return layerConfig.getSRID();
	}

	/**
	 * @param sRID
	 * @see it.prato.comune.sit.ConfigLayerBean#setSRID(java.lang.String)
	 */
	public void setSRID(String sRID) {
		layerConfig.setSRID(sRID);
	}

	/**
	 * @return the dsPool
	 */
	public SITDataStorePool getDsPool() {
		return dsPool;
	}

	/**
	 * @param dsPool the dsPool to set
	 */
	public void setDsPool(SITDataStorePool dsPool) {
		this.dsPool = dsPool;
	}

	/**
	 * @return the layersManager
	 */
	public SITLayersManager getLayersManager() {
		return layersManager;
	}

	/**
	 * @param layersManager the layersManager to set
	 */
	public void setLayersManager(SITLayersManager layersManager) {
		this.layersManager = layersManager;
	}

	/**
	 * @param layerConfig the layerConfig to set
	 */
	public void setLayerConfig(ConfigLayerBean layerConfig) {
		this.layerConfig = layerConfig;
	}
    

}
