/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 */
package it.prato.comune.sit;

import java.util.EnumSet;
import java.util.Set;

/**
 * Bean che detiene i parametri di configurazione della feaure
 *  
 * @author bf44
 *
 */
public class FeatureAttributeConfig implements IFeatureAttributeConfig {    
   
    private String realName;
    private String defaultValue;
    private String humanReadableName;
    private String validatingRegExp;
    private final EnumSet<Permission> permissionSet = EnumSet.noneOf(Permission.class);
    private ISuggestConfig suggestConfig;
    
    @Override
    public final String getRealName() {
        return realName;
    }

    @Override
    public final String getHumanReadableName() {
        return humanReadableName;
    }

    @Override
    public final String getValidatingRegExp() {
        return validatingRegExp;
    }            

    @Override
    public final ISuggestConfig getSuggestConfig() {
        return suggestConfig;
    }

    public void setRealName(final String realName) {
        this.realName = realName;
    }

    public void setHumanReadableName(final String humanReadableName) {
        this.humanReadableName = humanReadableName;
    }

    public void setValidatingRegExp(final String validatingRegExp) {
        this.validatingRegExp = validatingRegExp;
    }

    public void setSuggestConfig(final ISuggestConfig suggestConfig) {
        this.suggestConfig = suggestConfig;
    }        
    
    public void addPermission(Permission permission){
        this.permissionSet.add(permission);
    }
    
    public void removePermission(Permission permission){
        this.permissionSet.remove(permission);
    }
    
    public void removePermissions(Set<Permission> permissions){
        this.permissionSet.remove(permissions);
    }
    
    @Override
    public boolean hasPermission(Permission permission){
        return this.permissionSet.contains(permission);
    }
    
    public void addPermissions(Set<Permission> permissions){
        this.permissionSet.addAll(permissions);
    }
    
    public void setPermissions(Set<Permission> permissions){
        this.permissionSet.clear();
        this.permissionSet.addAll(permissions);
    }
    
    public void addAllPermission(){
        this.permissionSet.addAll(EnumSet.allOf(Permission.class));
    }
    
    public void removeAllPermission(){
        this.permissionSet.clear();
    }

    @Override
    public String getDefaultValue() {
        return this.defaultValue;
    }

    public void setDefaultValue(String defaultValue) {
        this.defaultValue = defaultValue;
    }

    @Override
    public boolean hasSuggestProvider() {       
        return (this.suggestConfig != null) && (this.suggestConfig.getSuggestProvider() != null);
    }
    
    /*
    public static void main(String[] args){
        FeatureAttributeConfig f = new FeatureAttributeConfig();
        f.addAllPermission();
        System.out.println(f.permissionSet);
        f.removePermission(Permission.QUERY);
        System.out.println(f.permissionSet);
        System.out.println(f.hasPermission(Permission.QUERY));
    }
    */
    
        
}
