/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/


package it.prato.comune.sit;

import it.prato.comune.utilita.logging.interfaces.LogInterface;

import java.util.HashMap;

import org.opengis.feature.simple.SimpleFeatureType;
import org.opengis.filter.Filter;
import org.opengis.geometry.BoundingBox;

import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.io.ParseException;
import com.vividsolutions.jts.io.WKTReader;

/**
 * La classe Filtro fornisce un controllo ad alto livello sulle operazioni di filtraggio dei dati, sia <br>
 * che queste riguardino filtri spaziali che filtri di altro genere. 
 * 
 * @author Ing. Alessandro Radaelli
 */
public class Filtro {

	private Filter filtroTotale = null;
	private SimpleFeatureType featureType;
	private HashMap nomiCampi;
	protected LogInterface logger;
	
	public static final short OpAddFiltroAND = 1;
    public static final short OpAddFiltroOR  = 2;
    
    public static final short GEOMETRY_BBOX = 4;
    public static final short GEOMETRY_CONTAINS = 11;
    public static final short GEOMETRY_CROSSES = 9;
    public static final short GEOMETRY_DISJOINT = 6;
    public static final short GEOMETRY_EQUALS = 5;
    public static final short GEOMETRY_INTERSECTS = 7;
    public static final short GEOMETRY_OVERLAPS = 12;
    public static final short GEOMETRY_TOUCHES = 8;
    public static final short GEOMETRY_WITHIN = 10;

    /**
     * Costruttore della classe Filtro.
     * 
     * @param filtro
     */
    protected Filtro(Filtro filtro) {		
		this(filtro.featureType,filtro.nomiCampi,filtro.filtroTotale,filtro.logger);
	}
    
    /**
     * Costruttore della classe Filtro.
     * 
     * @param featureType
     * @param nomiCampi
     * @param logger
     */
	protected Filtro(SimpleFeatureType featureType, HashMap nomiCampi, LogInterface logger) {		
		this(featureType,nomiCampi,null,logger);
	}
	
	/**
     * Costruttore della classe Filtro.
     * 
     * @param featureType
     * @param nomiCampi
     * @param logger
     */
    protected Filtro(SimpleFeatureType featureType, HashMap nomiCampi, Filter filtroTotale, LogInterface logger) {
        this.featureType = featureType;
        this.nomiCampi = nomiCampi;
        this.logger = logger;
        this.filtroTotale = filtroTotale != null ? filtroTotale : Filter.INCLUDE;
    }

	/**
	 * Consente di aggiungere al filtro una specifica funzione.
	 * 
	 * @param funzione
	 * @param Valore
	 * @param tipoFiltro
	 * @param operazione
	 * 
	 * @see  Filtro#AddFiltroFunzione(Filter, short)
	 */
	public void AddFiltroFunzione(Funzione funzione, Object Valore, String tipoFiltro, short operazione ) {
        
		String funz = funzione.componiFunzione();
		
		if (tipoFiltro.equals("=")) {
			AddFiltroFunzione(SITFilterFactory.equals("NOAPICI" + funz, Valore), operazione);
		} else if (tipoFiltro.equals("<")) {
			AddFiltroFunzione(SITFilterFactory.less("NOAPICI" + funz, Valore), operazione);
		} else if (tipoFiltro.equals(">")) {
			AddFiltroFunzione(SITFilterFactory.greater("NOAPICI" + funz, Valore), operazione);
		} else if (tipoFiltro.equals("<=")) {
			AddFiltroFunzione(SITFilterFactory.lessOrEqual("NOAPICI" + funz, Valore), operazione);
		} else if (tipoFiltro.equals(">=")) {
			AddFiltroFunzione(SITFilterFactory.greaterOrEqual("NOAPICI" + funz, Valore), operazione);
		} else if (tipoFiltro.equals("<>")) {
			AddFiltroFunzione(SITFilterFactory.notEqual("NOAPICI" + funz, Valore), operazione);
		} else if (tipoFiltro.toUpperCase().equals("IS NULL")) {
			AddFiltroFunzione(SITFilterFactory.isNull("NOAPICI" + funz), operazione);
		}        
    }

	/**
	 * Consente di aggiungere al filtro una specifica funzione.
	 * 
	 * @param filter
	 * @param operazione
	 */
	protected void AddFiltroFunzione(Filter filter, short operazione) {
		
		switch (operazione) {
    		case OpAddFiltroAND: 
    			filtroTotale  = SITFilterFactory.and(filtroTotale, filter);
    			break;
    		case OpAddFiltroOR:
    			filtroTotale  = SITFilterFactory.or(filtroTotale, filter);
    			break;
		}
		
	}

	/** 
	 * Consente di aggiungere la clausola AND al filtro sulla base del tipo di filtro passato come argomento.
	 * 
	 * @param NLCampo
	 * @param Valore
	 * @param tipoFiltro
	 * @throws SITException
	 * 
	 * @see Filtro#AndFiltro(Filter)
	 */
	public void AddFiltroAnd(String NLCampo, Object Valore, String tipoFiltro) throws SITException {
		
	      if (tipoFiltro.equals("=")) {
	    	  AndFiltro(SITFilterFactory.equals((String)nomiCampi.get(NLCampo), Valore));
	      } else if (tipoFiltro.equals("<")) {
	    	  AndFiltro(SITFilterFactory.less((String)nomiCampi.get(NLCampo), Valore));
	      } else if (tipoFiltro.equals(">")) {
	    	  AndFiltro(SITFilterFactory.greater((String)nomiCampi.get(NLCampo), Valore));
	      } else if (tipoFiltro.equals("<=")) {
	    	  AndFiltro(SITFilterFactory.lessOrEqual((String)nomiCampi.get(NLCampo), Valore));
	      } else if (tipoFiltro.equals(">=")) {
	    	  AndFiltro(SITFilterFactory.greaterOrEqual((String)nomiCampi.get(NLCampo), Valore));
	      } else if (tipoFiltro.equals("<>")) {
	    	  AndFiltro(SITFilterFactory.notEqual((String)nomiCampi.get(NLCampo), Valore));
	      } else if (tipoFiltro.toUpperCase().equals("IS NULL")) {
	    	  AndFiltro(SITFilterFactory.isNull((String)nomiCampi.get(NLCampo)));
	      } else if (tipoFiltro.toUpperCase().equals("IS NOT NULL")) {
	    	  AndFiltro(SITFilterFactory.isNotNull((String)nomiCampi.get(NLCampo)));
	      } else {
	    	  String msg = "Il tipo filtro " + tipoFiltro + " non e' presente!";
	    	  logger.error(msg);
	    	  throw new SITException(msg);
	      }
	}
	
	/** 
	 * Consente di aggiungere la clausola OR al filtro sulla base del tipo di filtro passato come argomento.
	 * 
	 * @param NLCampo
	 * @param Valore
	 * @param tipoFiltro
	 * @throws SITException
	 * 
	 * @see Filtro#OrFiltro(Filter)
	 */
	public void AddFiltroOr(String NLCampo, Object Valore, String tipoFiltro) throws SITException {
		
	      if (tipoFiltro.equals("=")) {
	    	  OrFiltro(SITFilterFactory.equals((String)nomiCampi.get(NLCampo), Valore));
	      } else if (tipoFiltro.equals("<")) {
	    	  OrFiltro(SITFilterFactory.less((String)nomiCampi.get(NLCampo), Valore));
	      } else if (tipoFiltro.equals(">")) {
	    	  OrFiltro(SITFilterFactory.greater((String)nomiCampi.get(NLCampo), Valore));
	      } else if (tipoFiltro.equals("<=")) {
	    	  OrFiltro(SITFilterFactory.lessOrEqual((String)nomiCampi.get(NLCampo), Valore));
	      } else if (tipoFiltro.equals(">=")) {
	    	  OrFiltro(SITFilterFactory.greaterOrEqual((String)nomiCampi.get(NLCampo), Valore));
	      } else if (tipoFiltro.equals("<>")) {
	    	  OrFiltro(SITFilterFactory.notEqual((String)nomiCampi.get(NLCampo), Valore));
	      } else if (tipoFiltro.toUpperCase().equals("IS NULL")) {
	    	  OrFiltro(SITFilterFactory.isNull((String)nomiCampi.get(NLCampo)));
	      }
	}
	
	/**
	 * Consente di impostare un filtro o di aggiungere ad un filtro esistente una ulteriore condizione AND/OR di filtraggio.
	 * 
	 * @param filter
	 * @param operazione
	 * @throws SITException
	 */
	protected void AddFiltro (Filter filter, short operazione) throws SITException {
	                
         switch (operazione) {
         	case OpAddFiltroAND: 
         		filtroTotale  = SITFilterFactory.and(filtroTotale, filter);
				break;
            case OpAddFiltroOR:
            	filtroTotale  = SITFilterFactory.or(filtroTotale, filter);
                break;
         }
		
	}

	/**
	 * Aggiunge in AND al filtro una funzione passata come argomento.
	 * 
	 * @param funzione
	 * @param Valore
	 * @param tipoFiltro
	 * 
	 * @see Filtro#AddFiltroFunzione(Funzione, Object, String, short)
	 */
	public void andFiltroFunzione(Funzione funzione, Object Valore, String tipoFiltro){
		AddFiltroFunzione(funzione, Valore, tipoFiltro, OpAddFiltroAND);
	}

	/**
	 * Aggiunge in AND al filtro corrente un filtro su una funzione specifica.
	 * 
	 * @param filter
	 * 
	 * @see Filtro#AddFiltroFunzione(Filter, short)
	 */
	protected void andFiltroFunzione(Filter filter){
		AddFiltroFunzione(filter, OpAddFiltroAND);
	}
	
	/**
	 * @param NLCampo
	 * @param Valore
	 * @param tipoFiltro
	 * @throws SITException
	 * 
	 * @see Filtro#AddFiltroAnd(String, Object, String)
	 */
	public void AndFiltro(String NLCampo, Object Valore, String tipoFiltro) throws SITException {
		AddFiltroAnd(NLCampo, Valore, tipoFiltro);
    }

    /**
     * @param filter
     * @throws SITException
     * 
     * @see Filtro#AddFiltro(Filter, short)
     */
	protected void AndFiltro (Filter filter) throws SITException {
        AddFiltro(filter, OpAddFiltroAND);
    }
    
    /**
     * @param NLCampo
     * @param Valore
     * @param tipoFiltro
     * @throws SITException
     * 
     * @see Filtro#AddFiltroOr(String, Object, String)
     */
    public void OrFiltro (String NLCampo, Object Valore, String tipoFiltro) throws SITException {
    	AddFiltroOr(NLCampo, Valore, tipoFiltro);
    }
    
    /**
     * @param filter
     * @throws SITException
     * 
     * @see Filtro#AddFiltro(Filter, short)
     */
    protected void OrFiltro (Filter filter) throws SITException {
    	AddFiltro(filter, OpAddFiltroOR);
    }  
     
	/**
	 * Agginge una condizione NOT al filtro corrente.
	 * 
	 */
	public void NotFiltro(){
		if (filtroTotale!=Filter.INCLUDE) {
			filtroTotale = SITFilterFactory.not(filtroTotale);
		} else {
			filtroTotale = Filter.EXCLUDE;
		}
    }
	
	/**
	 * Cancella il Filtro esistente.
	 * 
	 */
	public void ResetFiltro() {
	    filtroTotale = Filter.INCLUDE;        
    }
	
	/**
	 * Imposta un nuovo filtro.
	 * 
	 * @param nuovoFiltro
	 */
	protected void setFiltro(Filter nuovoFiltro){
		filtroTotale = nuovoFiltro != null ? nuovoFiltro : Filter.INCLUDE;
	}
	
	/**
	 * Restituisce il filtro corrente.
	 * 
	 * @return Filter
	 */
	protected Filter getFiltro(){
		return filtroTotale;
	}
	
	/**
	 * Aggiunge in AND un altro filtro al filtro corrente.
	 * 
	 * @param altroFiltro
	 * 
	 * @see Filtro#AndFiltroPriv(Filter)
	 */
	protected void AndFiltro(Filtro altroFiltro){
		AndFiltroPriv(altroFiltro.getFiltro());
	}
	
	/**
	 * Aggiunge in OR un altro filtro al filtro corrente.
	 * 
	 * @param altroFiltro
	 * 
	 * @see Filtro#OrFiltroPriv(Filter)
	 */
	protected void OrFiltro(Filtro altroFiltro){
		OrFiltroPriv(altroFiltro.getFiltro());
	}	
	
	/**
	 * Se il filtro esiste gia aggiunge una nuova condizione di AND. <br>
	 * In caso contrario imposta come filtro quello passato come argomento.
	 * 
	 * @param altroFiltro
	 */
	protected void AndFiltroPriv(Filter altroFiltro){
		filtroTotale  = SITFilterFactory.and(filtroTotale, altroFiltro);
	}
	
	/**
	 * Se il filtro esiste gia aggiunge una nuova condizione di OR. <br>
	 * In caso contrario imposta come filtro quello passato come argomento.
	 * 
	 * @param altroFiltro
	 */
	protected void OrFiltroPriv(Filter altroFiltro){
        filtroTotale = SITFilterFactory.or(filtroTotale, altroFiltro); 
	}
	
	/**
	 * Restituisce il filtro in formato stringa.
	 * 
	 * @return String
	 */
	public String getFilterString(){
		return filtroTotale.toString();
	}

	/**
	 * Restituisce la funzione vuota.
	 * 
	 * @return Funzione
	 */
	public Funzione getFunzioneVuota(){
		return new Funzione(nomiCampi, logger);
	}
	
	/**
	 * Crea un filtro basato sull'ID passato come argomento e lo aggiunge al filtro totale. 
	 * 
	 * @param chiave
	 */
	public void addFiltroByFID (String chiave) { 
		Filter idFilter = SITFilterFactory.id(chiave); 
        filtroTotale = SITFilterFactory.and(filtroTotale, idFilter); 
		
//TOB		
//        filtroTotale = SITGeometryFilter.id(chiave);              
	}

	/**
	 * Restituisce il nome dell'attributo corrispondente alla stringa passata come argomento.
	 * 
	 * @param nlCampo
	 * @return String
	 */
	public String getNomeCampo(String nlCampo){
		return (String)nomiCampi.get(nlCampo);
	}	
	
	/**
	 * Aggiunge in AND un filtro geografico sulla base dell'ogetto territorio passato come argomento.
	 * 
	 * @param ogTer
	 * @param tipoFiltro
	 */
	public void AndFiltroGeog (OggettoTerritorio ogTer, short tipoFiltro) {
		AddFiltroGeog(ogTer, tipoFiltro, OpAddFiltroAND);
	}
	
	/**
     * Aggiunge in AND un filtro geografico sulla base della geometria WKT passata come argomento.
     * 
     * @param ogTer
     * @param tipoFiltro
	 * @throws SITParseException 
     */
    public void AndFiltroGeog (String wktString, short tipoFiltro) throws SITParseException {
        AddFiltroGeog(wktString, tipoFiltro, OpAddFiltroAND);
    }
	
	/**
	 * Aggiunge in OR un filtro geografico sulla base dell'ogetto territorio passato come argomento.
	 * 
	 * @param ogTer
	 * @param tipoFiltro
	 */
	public void OrFiltroGeog (OggettoTerritorio ogTer, short tipoFiltro) {
		AddFiltroGeog(ogTer, tipoFiltro, OpAddFiltroOR);
	}
	
	/**
     * Aggiunge in OR un filtro geografico sulla base della geometria WKT passata come argomento.
     * 
     * @param ogTer
     * @param tipoFiltro
     * @throws SITParseException 
     */
    public void OrFiltroGeog (String wktString, short tipoFiltro) throws SITParseException {
        AddFiltroGeog(wktString, tipoFiltro, OpAddFiltroOR);
    }
	
	/**
	 * Aggiunge un filtro geografico sulla base dell'ogetto territorio passato come argomento e all'operazione 
	 * scelta.
	 * 
	 * @param ogTer
	 * @param tipoFiltro
	 * @param operazione
	 */
	public void AddFiltroGeog (OggettoTerritorio ogTer, short tipoFiltro, short operazione) {
		AddFiltroGeogPriv((Geometry) ogTer.getFeat().getDefaultGeometry(), tipoFiltro, operazione);        
	}
	
	/**
     * Aggiunge un filtro geografico sulla base della geometria wkt passata come argomento e all'operazione 
     * scelta.
     * 
     * @param wktString
     * @param tipoFiltro
     * @param operazione
	 * @throws SITParseException 
     */
    public void AddFiltroGeog(String wktString, short tipoFiltro, short operazione) throws SITParseException {
        try {
            WKTReader wktReader = new WKTReader();
            Geometry geometry = (Geometry) wktReader.read(wktString);
            AddFiltroGeogPriv(geometry, tipoFiltro, operazione);  
        } catch (ParseException e) {
            String errMsg = "ParseException durante setting dell'attributo WKTString";
            logger.error(errMsg, e);
            throw new SITParseException(errMsg, e);
        }              
    }
	
	/**
	 * Aggiunge un filtro sulla base della geometria pasata come argomento e dell'operazione scelta.
	 * 
	 * @param geom
	 * @param tipoFiltro
	 * @param operazione
	 */
	protected void AddFiltroGeogPriv (Geometry geom, short tipoFiltro, short operazione ) {
	     
		try{
			Filter filter = null;
			String geomName = featureType.getGeometryDescriptor().getName().getLocalPart();

			switch(tipoFiltro){
    			case GEOMETRY_BBOX:
    				Geometry featureGeom = (Geometry)featureType.getGeometryDescriptor().getDefaultValue();
    				filter = SITFilterFactory.bbox(geom, (BoundingBox)featureGeom.getEnvelope());
    				break;
    			case GEOMETRY_CONTAINS:
    				filter = SITFilterFactory.contains(geomName, geom);
    				break;
    			case GEOMETRY_CROSSES:
    				filter = SITFilterFactory.crosses(geomName, geom);
    				break;
    			case GEOMETRY_DISJOINT:
    				filter = SITFilterFactory.disjoint(geomName, geom);
    				break;
    			case GEOMETRY_EQUALS:
    				filter = SITFilterFactory.geometryEquals(geomName, geom);
    				break;
    			case GEOMETRY_INTERSECTS:
    				filter = SITFilterFactory.intersects(geomName, geom);
    				break;
    			case GEOMETRY_OVERLAPS:
    				filter = SITFilterFactory.overlaps(geomName, geom);
    				break;
    			case GEOMETRY_TOUCHES:
    				filter = SITFilterFactory.touches(geomName, geom);
    				break;
    			case GEOMETRY_WITHIN:
    				filter = SITFilterFactory.within(geomName, geom);
    				break;
			}
		
			switch (operazione) {
    			case OpAddFiltroAND: 
    				this.AndFiltro(filter);
    				break;
    			case OpAddFiltroOR:
    				this.OrFiltro(filter);
    				break;				
			}
			
		}catch(SITException e){
			logger.error("Errore aggiungendo il filtro geografico",e);
		}
	}	
}
