/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
package it.prato.comune.sit;

import it.prato.comune.utilita.core.type.DateType;
import it.prato.comune.utilita.core.type.TsType;
import it.prato.comune.utilita.logging.interfaces.LogInterface;

import java.util.HashMap;

import org.opengis.feature.simple.SimpleFeatureType;

/**
 * Classe FiltroTemporale.
 * 
 * @author Mattia Gennari
 *
 */
public class FiltroTemporale extends Filtro {
	
	private TsType dtInizioFiltro;
    private TsType dtFineFiltro;

	/**
	 * Costruttore della classe FiltroTemporale.
	 * 
	 * @param featureType
	 * @param nomiCampi
	 * @param dtInizioFiltro
	 * @param dtFineFiltro
	 * @param logger
	 */
	protected FiltroTemporale(SimpleFeatureType featureType, HashMap nomiCampi, TsType dtInizioFiltro, TsType dtFineFiltro, LogInterface logger) {
		super(featureType, nomiCampi, logger);
		setIntervalloTemporale(dtInizioFiltro, dtFineFiltro);
	}
	
	/**
	 * Costruttore della classe FiltroTemporale.
	 * 
	 * @param filtro
	 * @param dtInizioFiltro
	 * @param dtFineFiltro
	 */
	protected FiltroTemporale(Filtro filtro, TsType dtInizioFiltro, TsType dtFineFiltro) {			
		super(filtro);
		setIntervalloTemporale(dtInizioFiltro, dtFineFiltro);
	}
	
	/**
	 * Imposta l-intervallo temporale per il filtro.
	 * 
	 * @param dtInizioFiltro
	 * @param dtFineFiltro
	 */
	private void setIntervalloTemporale(TsType dtInizioFiltro, TsType dtFineFiltro){
		this.dtInizioFiltro = (dtInizioFiltro == null) ? new TsType(DateType.INIZIO_DEI_TEMPI) : dtInizioFiltro;
		this.dtFineFiltro = (dtFineFiltro == null) ? new TsType(DateType.FINE_DEI_TEMPI) : dtFineFiltro;  		
	}
	
	/* 
	 * Viene lanciata una SITRuntimeException non potendo altrimenti sovrascrivee il metodo padre
	 * che non lancia eccezioni.
	 * 
	 * @see it.prato.comune.sit.Filtro#resetFiltro()
	 */
	@Override
	public void ResetFiltro() {
		super.ResetFiltro();
		try {			
//TOB			
//			if (dtInizioFiltro!=null && dtFineFiltro.compareTo(dtInizioFiltro)>=0) {
//				this.AndFiltro(Costanti.NL_DTINIZIOFILTRO, "to_date('" + dtFineFiltro.getFormattedTimeStamp() + "', 'DD/MM/YYYY HH24:MI:SS')", "<=");
//				this.AndFiltro(Costanti.NL_DTFINEFILTRO, "to_date('" + dtInizioFiltro.getFormattedTimeStamp() + "', 'DD/MM/YYYY HH24:MI:SS')", ">=");
//			}
			
			logger.info("Imposto il filtro temporale tra le date: " + dtInizioFiltro.getFormatted() + " e " + dtFineFiltro.getFormatted());
			
			if (dtInizioFiltro!=null && dtFineFiltro.compareTo(dtInizioFiltro)>=0) {
				this.AndFiltro(Layers.NL_DTINIZIOFILTRO, dtFineFiltro.toSqlTimeStamp(), "<=");
				this.AndFiltro(Layers.NL_DTFINEFILTRO, dtInizioFiltro.toSqlTimeStamp(), ">=");
			}
		} catch (SITException e) {
			throw new SITRuntimeException("Eccezione durante l'impostazione del filtro", e);
		}
	}
}
