/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
/*
 * Copyright  Comune di Prato - Sistema Informativo
 * Project:    Package SIT per elaborazione/interrogazione/gestione dati cartografici 
 * File:       Funzione.java
 * Author:     Alessandro Radaelli
 * Version:    1.0.0
 * CreationDate:  
 * ModifyDate:     
 *             
 */
package it.prato.comune.sit;

import it.prato.comune.utilita.logging.interfaces.LogInterface;

import java.util.Collection;
import java.util.HashMap;
import java.util.Vector;

/**
 * Classe Funzione.
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public class Funzione {
	
	private HashMap nomiCampi;
	private String tipoFunzione;
	private Vector parametri;
	private LogInterface logger;

	/**
	 * Costruttore della classe Funzione.
	 * 
	 * @param nomiCampi
	 * @param logger
	 */
	protected Funzione(HashMap nomiCampi, LogInterface logger){
		this.nomiCampi = nomiCampi;
		parametri = new Vector();
		this.logger = logger;
	}
	
	/**
	 * Imposta la tipologia di Funzione.
	 * 
	 * @param tipo
	 * @throws SITFunzioneException
	 */
	public void setTipo(String tipo) throws SITFunzioneException{
		if(tipo == null || tipo.equals("")){
			throw new SITFunzioneException("Funzione da applicare mancante.");
		}
		tipoFunzione = tipo;
	}
	
	/**
	 * Aggiunge un parametro alla funzione.
	 * 
	 * @param parametro
	 */
	public void addParametro(String parametro){
		parametri.add("'" + parametro + "'");
	}
	
	/**
	 * Aggiunge un parametro alla funzione.
	 * 
	 * @param parametro
	 */
	public void addParametro(Number parametro){
		parametri.add(parametro);
	}
	
	/**
	 * Consente di aggiungere un nuovo campo.
	 * 
	 * @param NLCampo
	 */
	public void addCampo(String NLCampo){
		parametri.add((String)nomiCampi.get(NLCampo));
	}	
	
	/**
	 * Consente di aggiungere un insieme di parametri.
	 * 
	 * @param c
	 */
	public void addParametri(Collection c){
		parametri = new Vector(c);
	}
	
	/**
	 * Costruisce la funzione sulla base dei parametri impostati e restituisce la stringa corrispondente.
	 * 
	 * @return String
	 */
	public String componiFunzione(){
		
		String funz = tipoFunzione + "(";
		Object param;
		String lista = "";
		for(int i=0; i<parametri.size(); i++){
			param = parametri.get(i);
			if(param != null){
				if(param instanceof Number){
					lista += param.toString() + ",";
				}else if(param instanceof String && !param.equals("")){
					lista += param.toString().replace("'","''") + ",";
				}
			}
		}
		if(lista.equals("")){
			funz += ")";
		}else{
			lista = lista.substring(0,lista.length()-1);//Levo l'ultima virgola
			funz = funz + lista + ")";
		}		
		logger.debug("Funzione = " + funz);
		return funz;
	}
}
