/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
package it.prato.comune.sit;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.fileupload.MultipartStream;
import org.apache.commons.fileupload.MultipartStream.MalformedStreamException;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.geotools.GML;
import org.geotools.GML.Version;
import org.geotools.data.simple.SimpleFeatureCollection;
import org.geotools.data.simple.SimpleFeatureIterator;
import org.geotools.ows.ServiceException;
import org.opengis.feature.simple.SimpleFeature;
import org.opengis.feature.simple.SimpleFeatureType;
import org.opengis.feature.type.AttributeDescriptor;
import org.xml.sax.SAXException;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.io.WKTWriter;

import it.prato.comune.utilita.core.type.TsType;
import it.prato.comune.utilita.logging.interfaces.LogInterface;

/**
 * Implementazione dell'interfaccia {@link IGetFeatureInfoLayer} per la gestione dei layer di tipo WMS utilizzando la chiamata getFeatureInfo
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public class GetFeatureInfoLayer implements IGetFeatureInfoLayer {
        
    private String  serverURL   = null;
    private String  nome        = null;
    private String  WMSLayerName = null;
    private int     codTPN;
    private String extraParams  = null;
    private LogInterface logger = null;
    private String  defaultInfoFormat = InfoFormat.GML_2.getDescription();
    private String espressioneDescrizione = null;
    private String espressioneNomeOggetto = null;
    private String espressioneIDTPN = null;
    private Version WFSVersion = Version.WFS1_0;
    
    private String textToCheckIfDataFound;
    private String regExpToCheckIfDataFound;
    
    private AdditionalFilterServerParams additionalFilterParams = null;
    
    public static enum InfoFormat {
        
        HTML    ("text/html"),
        TEXT    ("text/plain"),
        GML     ("text/gml"),
        GML_2   ("application/vnd.ogc.gml"),
        GML_3   ("application/vnd.ogc.gml/3.1.1"),       
        JSON    ("application/json"),
        JSONP   ("text/javascript");       

        InfoFormat(String description){                  
            this.description = description;                       
        }

        private final String description;             

        public String getDescription() {
            return description;
        }

        public static InfoFormat decode(String description){
            if(description == null) return null;
            for(InfoFormat og: InfoFormat.values()){
                if(og.getDescription().equals(description))
                    return og;
            }
            return null;
        }
    }	

	/**
	 * Costruttore
	 * 
	 * @param szURL Urll del server WMS
	 * @param nome Nome da assegnare al layer
	 * @param WMSLayerName Nome del layer WMS
	 * @param codTPN CodTPN da assegnare al layer
	 * @param defaultInfoFormat formato di default con cui fare le chiamate WMS per ottenere il gml
	 * @param logger
	 * @throws SITException
	 */
	public GetFeatureInfoLayer(String  szURL, String nome, String WMSLayerName, 
			int codTPN, String  defaultInfoFormat, LogInterface logger) throws SITException {
		super();

		this.serverURL = szURL;
		this.nome = nome;
		this.WMSLayerName = WMSLayerName;
		this.codTPN = codTPN;
		if (defaultInfoFormat!=null) this.defaultInfoFormat = defaultInfoFormat;
		this.logger = logger;

	}
	
	/** @see IGetFeatureInfoLayer#getFeatureInfo(GetFeatureInfoParams) */
	public List<IGetFeatureInfoObject> getFeatureInfo(GetFeatureInfoParams params) throws SITException {

		if (params.getInfoFormat()==null) params.setInfoFormat(defaultInfoFormat);
		InfoFormat infoFormat = InfoFormat.decode(params.getInfoFormat());
		
		if(infoFormat.equals(InfoFormat.TEXT) || infoFormat.equals(InfoFormat.HTML)){
		    
		    return getTextFeatureInfo(params);
		        		    
		} else {
		    
			List<IGetFeatureInfoObject> retVal = null;
			try {
				retVal = getGmlFeatureInfo(params);
			} catch (Exception e) {
				String errMsg = "Errore nel recupero di GMLFeatureInfo in metodo getFeatureInfo della classe getFeatureInfo.\n";
				errMsg += "Layer codTPN: " + getCodTPN();
				errMsg += "Layer nome: " + getNome();
				errMsg += "Provo con text/html";
				// Se non possibile con GML per un qualsiasi motivo prova con text/html
				logger.warn(errMsg, e);
				retVal = getTextFeatureInfo(params);
			}
			
		    return retVal;
		    
		}
    		    	
	}
	
	public List<IGetFeatureInfoObject> getTextFeatureInfo(GetFeatureInfoParams params) throws SITException {
	    
	    
	    List<IGetFeatureInfoObject> retVal = new ArrayList<IGetFeatureInfoObject>();
	    Map<String, Object> attributes = new HashMap<String, Object>();
	    	    
	    String getFeatureInfoLink;
	    
        try {
            
            getFeatureInfoLink = params.getFeatureInfoReqStringHtml(this.serverURL, this.WMSLayerName, this.extraParams, additionalFilterParams);
            
            logger.debug("getFeatureInfoLink = " + getFeatureInfoLink);
            
        } catch (UnsupportedEncodingException e) {
            String msg = "UnsupportedEncodingException nel metodo getFeatureInfo di GetFeatureInfoLayer";
            this.logger.warn(msg + e.getMessage(),e);
            throw new SITException(msg + e.getMessage());
        }
	    
	    
        int nRecord = -1;
        boolean checkData = false;
        
        // In questo caso non viene chiamato il server WMS per controllare la presenza di features
        if(getRegExpToCheckIfDataFound() != null || getTextToCheckIfDataFound() != null){ 
            
            checkData = true;
            nRecord = 0;            
            URL url;
            InputStream in = null;
            
            try {
                
                url = new URL(getFeatureInfoLink);
            
                HttpURLConnection connection = (HttpURLConnection) url.openConnection();
                connection.setConnectTimeout(params.getConnectTimeout()*1000);
                connection.setReadTimeout(params.getReadTimeout()*1000);
                String encoding = connection.getContentEncoding();
                in = connection.getInputStream();
                
                String html = IOUtils.toString(in, encoding);                                         
                        
                if(getTextToCheckIfDataFound() != null){
                    
                    nRecord = StringUtils.countMatches(html, getTextToCheckIfDataFound());
                    
                } else if(getRegExpToCheckIfDataFound() != null){
                    Pattern p = Pattern.compile(getRegExpToCheckIfDataFound());
                    Matcher m = p.matcher(html);                    
                    while(m.find()) {
                        nRecord++;
                    }                    
                }
                
                attributes.put("$V_MATCH_COUNT", nRecord);
                
            } catch (IOException e) {
                String msg = "IOException nel metodo getFeatureInfo di GetFeatureInfoLayer";
                this.logger.warn(msg + e.getMessage(),e);
                throw new SITException(msg + e.getMessage());
            } finally {
                if(in != null){
                    try {
                        in.close();
                    } catch (IOException e) {
                        logger.error("Impossibile chiudere l'imput stream della connessione http",e);
                    }
                }
            }
                                
        }
        
        if(!checkData || nRecord > 0){            
            Coordinate xy = new Coordinate(params.getCoordX(), params.getCoordY());
            Point p = new GeometryFactory().createPoint(xy);
            String WKTGeom = WKTWriter.toPoint(xy);
            WKTWriter writer = new WKTWriter();            
            retVal.add(createGetFeatureInfoObject(WKTGeom, writer.write(p.getEnvelope()) , attributes, params, getFeatureInfoLink));            
        }       
        
        return retVal;
        
	}
	
	public List<IGetFeatureInfoObject> getGmlFeatureInfo(GetFeatureInfoParams params) throws SITException {
	    
	    List<IGetFeatureInfoObject> retVal = new ArrayList<IGetFeatureInfoObject>();
        InputStream is = null;
        InputStream responseStream = null;
        File xsdFile = null;
        File gmlFile = null;
        File gmlFileTmp = null;
        
        try {
            //String url = params.getFeatureInfoReqString(this.serverURL, this.WMSLayerName, this.extraParams, additionalFilterParams);
            //String getFeatureInfoLink = params.getFeatureInfoReqStringHtml(this.serverURL, this.WMSLayerName, this.extraParams, additionalFilterParams);
            //String url = (serverURL.contains("?")?"&":"?") + params.getFeatureInfoReqStringParams(WMSLayerName, extraParams, additionalFilterParams, params.getInfoFormat());
            String getFeatureInfoLink = params.getFeatureInfoReqStringHtml(serverURL, WMSLayerName, extraParams, additionalFilterParams);
            String urlParams = params.getFeatureInfoReqStringParams(WMSLayerName, extraParams, 
					additionalFilterParams, params.getInfoFormat());
            logger.debug("getFeatureInfo request server: " + serverURL);
            logger.debug("getFeatureInfo request parametri: " + urlParams);
            
            // Scelta metodo in base alla lunghezza della url complessiva
            int reqLength = serverURL.length() + urlParams.length();
            String httpMethod = (reqLength>2000) ? "POST" : "GET";
            logger.debug("getFeatureInfo request: scelto metodo "+httpMethod + " perch richiesta lunga " + reqLength);
            
            String actualUrl = serverURL;
            if (httpMethod.equals("GET")) {
            	actualUrl += (serverURL.contains("?")?"&":"?") + urlParams; 
            }
            URL url1 = new URL(actualUrl);
            HttpURLConnection connection = (HttpURLConnection) url1.openConnection();
            connection.setConnectTimeout(params.getConnectTimeout()*1000);
            connection.setReadTimeout(params.getReadTimeout()*1000);
            connection.setRequestMethod(httpMethod);
            if (httpMethod.equals("GET")){
            	connection.connect();
			} else {
				connection.setDoOutput(true);
				connection.connect();
				connection.getOutputStream().write(urlParams.getBytes("UTF-8"));
			}
            
            responseStream = connection.getInputStream();
            
            String contentType = connection.getHeaderField("Content-type");
            
            if (contentType.indexOf("multipart")==-1) {
                // Non  multipart
                is= responseStream;
            } else {
                // E' multipart (come per esempio mapserver regione toscana
                int boundaryIndex = contentType.indexOf("boundary=");
                byte[] boundary = (contentType.substring(boundaryIndex + 9)).getBytes();
            
                MultipartStream multipartStream =  new MultipartStream(responseStream, boundary);
                
                try {
                    xsdFile = File.createTempFile("tolotmp", ".tolo");
                    gmlFile = File.createTempFile("tolotmp", ".tolo");
                    gmlFileTmp = File.createTempFile("tolotmp", ".tolo");
                } catch (IOException e1) {
                    String msg = "IOException nel metodo getFeatureInfo di GetFeatureInfoLayer creando i file temporanei";
                    this.logger.warn(msg + e1.getMessage(), e1);
                    throw new SITException(msg + e1.getMessage());
                }
                    
                boolean nextPart = multipartStream.skipPreamble();
                while(nextPart) {
                  String headersString = multipartStream.readHeaders();
                  String[] headers = headersString.split("\r\n");
                  String filename = null;
                  String replaceSrc = null;
                  String replaceDst = null;
                  for (String h: headers) {
                      // Cerca Content-Disposition:
                      if (h.indexOf("Content-Disposition")!=-1) {
                          int start = h.indexOf("filename");
                          filename = h.substring(start+9);
                          break;
                      }
                  }
                  File f = null;
                  if (filename.indexOf(".xsd")!=-1) {
                      f = xsdFile;
                  } else {
                      f= gmlFileTmp;
                  }
                  
                  if (f!=null) {
                      FileOutputStream fos = null;
                      try {
                          fos = new FileOutputStream(f);
                          multipartStream.readBodyData(fos);
                      } catch (FileNotFoundException e) {
                          String msg = "FileNotFoundException nel metodo getFeatureInfo di GetFeatureInfoLayer scrivendo sul file" + f.getAbsolutePath();
                          this.logger.warn(msg + e.getMessage(), e);
                          throw new SITException(msg + e.getMessage());
                      } catch (MalformedStreamException e) {
                          String msg = "MalformedStreamException nel metodo getFeatureInfo di GetFeatureInfoLayer leggendo lo stream multipart ricevuto dalla url " + actualUrl;
                          this.logger.warn(msg + e.getMessage(), e);
                          throw new SITException(msg + e.getMessage());
                      } catch (IOException e) {
                          String msg = "IOException nel metodo getFeatureInfo di GetFeatureInfoLayer scrivendo sul file" + f.getAbsolutePath();
                          this.logger.warn(msg + e.getMessage(), e);
                          throw new SITException(msg + e.getMessage());
                      } finally {
                          if (fos!=null)
                            try {
                                fos.close();
                            } catch (IOException e) {
                                String msg = "IOException nel metodo getFeatureInfo di GetFeatureInfoLayer chiudendo lo stream di scrittura del il file" + f.getAbsolutePath();
                                this.logger.warn(msg + e.getMessage(), e);
                                throw new SITException(msg + e.getMessage());
                            }
                      }
                  
                      if (filename.indexOf(".dat")!=-1) {
                          // Se si tratta del file che contiene le geometrie (il file gml) sostituisci schemalocation con il  file salvato 
                          replaceDst = "file://" + xsdFile.getAbsolutePath();
                          replaceSrc = "result.xsd";
                      
                          FileWriter f1 = null;
                          BufferedReader br = null;
                          try {
                      
                              f1 = new  FileWriter(gmlFile);
                              br = new BufferedReader(new FileReader(gmlFileTmp));
                              String strLine = null;
                              boolean bFatto = false;
                              while ((strLine = br.readLine()) != null)   {
                                  if ((!bFatto) && strLine.indexOf(replaceSrc)!=-1) {
                                      strLine = strLine.replace(replaceSrc,replaceDst);
                                      bFatto = true;
                                  }
                                  f1.write(strLine+"\r\n");
                              }
                          } catch (IOException e) {
                                String msg = "IOException nel metodo getFeatureInfo di GetFeatureInfoLayer chiudendo lo stream di scrittura del il file" + gmlFile.getAbsolutePath();
                                this.logger.warn(msg + e.getMessage(), e);
                                throw new SITException(msg + e.getMessage());
                          } finally {
                        	  if (br!=null){
                        		  br.close();
                        	  }
                              if (f1!=null)
                                  try {
                                      f1.close();
                                  } catch (IOException e) {
                                      String msg = "IOException nel metodo getFeatureInfo di GetFeatureInfoLayer chiudendo lo stream di scrittura del il file" + gmlFile.getAbsolutePath();
                                      this.logger.warn(msg + e.getMessage(), e);
                                      throw new SITException(msg + e.getMessage());
                                  }
                          }
                      }
                  }
                  nextPart = multipartStream.readBoundary();
                }
                is = new FileInputStream(gmlFile);
            }
            
            GML gml = new GML(getWFSVersion());
            SimpleFeatureCollection featureCollection = gml.decodeFeatureCollection(is);
            if (featureCollection!=null) {
                SimpleFeatureIterator fi = featureCollection.features();
                try {
                    
                    WKTWriter wktWriter = new WKTWriter();
                    while (fi.hasNext()) {
                        SimpleFeature f = fi.next();
                        
                        SimpleFeatureType ft = f.getFeatureType();
                        Geometry geom = (Geometry) f.getDefaultGeometry();
                        String WKTGeom = wktWriter.write(geom);
                        String boundingboxWKT = wktWriter.write(geom.getEnvelope());
                        Map<String, Object> attributes = new HashMap<String, Object>();
                        for (AttributeDescriptor ad: ft.getAttributeDescriptors()){
                            if (!ad.getType().getName().getLocalPart().equals(f.getDefaultGeometryProperty().getName().getLocalPart())) {
                                attributes.put(ad.getType().getName().getLocalPart(), f.getAttribute(ad.getName()));
                            }
                        }
                        attributes.put("##FID##",f.getID());
                        retVal.add(createGetFeatureInfoObject(WKTGeom, boundingboxWKT, attributes, params, getFeatureInfoLink));
                    }
                } finally {
                    if (fi!=null) fi.close();
                }
            }           
        } catch (ServiceException e) {
            String msg = "ServiceException nel metodo getFeatureInfo di GetFeatureInfoLayer";
            this.logger.warn(msg + e.getMessage(),e);
            throw new SITException(msg + e.getMessage(), e);
        } catch (SAXException e) {
            String msg = "SAXException nel metodo getFeatureInfo di GetFeatureInfoLayer";
            this.logger.warn(msg + e.getMessage(),e);
            throw new SITException(msg + e.getMessage(), e);
        } catch (ParserConfigurationException e) {
            String msg = "ParserConfigurationException nel metodo getFeatureInfo di GetFeatureInfoLayer";
            this.logger.warn(msg + e.getMessage(),e);
            throw new SITException(msg + e.getMessage(), e);
        } catch (Error e) {
            String msg = "Error nel metodo getFeatureInfo di GetFeatureInfoLayer";
            this.logger.warn(msg + e.getMessage(),e);
            throw new SITException(msg + e.getMessage());
        } catch (UnsupportedEncodingException e) {
            String msg = "UnsupportedEncodingException nel metodo getFeatureInfo di GetFeatureInfoLayer";
            this.logger.warn(msg + e.getMessage(),e);
            throw new SITException(msg + e.getMessage());
        }  catch (IOException e) {
            String msg = "IOException nel metodo getFeatureInfo di GetFeatureInfoLayer";
            this.logger.warn(msg + e.getMessage(),e);
            throw new SITException(msg + e.getMessage());
        } finally {
            try {
                if (responseStream!=null) responseStream.close();
            } catch (IOException e) {
                String msg = "IOException nel metodo getFeatureInfo di GetFeatureInfoLayer chiudendo il responseStream";
                this.logger.warn(msg,e);
                throw new SITException(msg, e);
            } finally {
                if (xsdFile!=null && xsdFile.exists()) {
                    xsdFile.delete();
                }
                if (gmlFile!=null && gmlFile.exists()) {
                    gmlFile.delete();
                }
                if (gmlFileTmp!=null && gmlFileTmp.exists()) {
                    gmlFileTmp.delete();
                }

                if (is!=null)
                    try {
                        is.close();
                    } catch (IOException e) {
                        String msg = "IOException nel metodo getFeatureInfo di GetFeatureInfoLayer";
                        this.logger.warn(msg,e);
                        throw new SITException(msg, e);
                    }
            }
        }
        
        return retVal;
	    
	}

	/**
	 * Metodo per la creazione di un nuovo oggetto di questo layer
	 * 
	 * @param wkt	Geometria in formato WKT
	 * @param boundingBoxWKT Boundingbox in formato WKT
	 * @param featureInfoAttributes	attributi 
	 * @param srid sistema di riferimento
	 * 
	 * @return
	 */
	private IGetFeatureInfoObject createGetFeatureInfoObject(String wkt, String boundingBoxWKT,
															Map<String, Object> featureInfoAttributes,
															GetFeatureInfoParams params, String getFeatureInfoLink) {
		
		String srid = params.getSourceSRID();
		GetFeatureInfoObject retVal = new GetFeatureInfoObject(wkt,	boundingBoxWKT, featureInfoAttributes, this, srid);
		
		retVal.setSourceHTTP(serverURL);
		retVal.setSourceLayer(WMSLayerName);
		String bbox = params.getBboxLeft()+":"+params.getBboxBottom()+":"+params.getBboxRight()+":"+params.getBboxTop();
		retVal.setSourceBBOX(bbox);
		retVal.setSourceStyle(params.getStyle());
		retVal.setSourceWidth(params.getWidth());
		retVal.setSourceHeight(params.getHeight());
		retVal.setSourceX(params.getX());
		retVal.setSourceY(params.getY());
		retVal.setGetFeatureInfoLink(getFeatureInfoLink);
		return retVal ;
	}

	/** @see IGetFeatureInfoLayer#setFiltroTemporale(TsType, TsType) */
	public void setFiltroTemporale(TsType dtInizioFiltro, TsType dtFineFiltro) {
		
	}

	/** @see IGetFeatureInfoLayer#getNome() */
	public String getNome() {
		return this.nome;
	}

	/** @see IGetFeatureInfoLayer#getCodTPN() */
	public int getCodTPN() {
		return this.codTPN;
	}

	/**
	 * Metodo per recuperare parametri extra da inserire nella chiamata WMS
	 * 
	 * @return 
	 */
	public String getExtraParams() {
		return extraParams;
	}

	/**
	 * Metodo per impostare i parametri extra da inserire nella chiamata WMS
	 * 
	 * @param extraParams
	 */
	public void setExtraParams(String extraParams) {
		this.extraParams = extraParams;
	}

	/**
	 * @see IGetFeatureInfoLayer#setEspressioneDescrizione(String) 
	 */
	public void setEspressioneDescrizione(String espressioneDescrizione) {
		this.espressioneDescrizione = espressioneDescrizione;
		
	}
	
	public void setEspressioneIDTPN(String espressioneIDTPN) {
		this.espressioneIDTPN = espressioneIDTPN;
		
	}

	/** @see IGetFeatureInfoLayer#getEspressioneDescrizione */
	public String getEspressioneDescrizione() {
		return espressioneDescrizione;
	}

	/**
	 * @return the wFSVersion
	 */
	public Version getWFSVersion() {
		return WFSVersion;
	}

	/**
	 * @param wFSVersion the wFSVersion to set
	 */
	public void setWFSVersion(Version wFSVersion) {
		WFSVersion = wFSVersion;
	}

	public String getEspressioneIDTPN() {
		return this.espressioneIDTPN;
	}

	public void setEspressioneNomeOggetto(String espressioneNomeOggetto) {
		this.espressioneNomeOggetto = espressioneNomeOggetto;
		
	}

	public String getEspressioneNomeOggetto() {
		return espressioneNomeOggetto;
	}
		
	
	public String getTextToCheckIfDataFound() {
        return textToCheckIfDataFound;
    }
	
	public String getRegExpToCheckIfDataFound() {
        return regExpToCheckIfDataFound;
    }	
	
	/**
	 * Metodo per impostare il testo da cercare all'interno della risposta di testo 
     * del WMS per poter capire se un o pi dati sono stati trovati.
     * 
	 * @param textToCheckIfDataFound stringa di testo da cercare 
	 */	
	public void setTextToCheckIfDataFound(String textToCheckIfDataFound) {
	    this.textToCheckIfDataFound = textToCheckIfDataFound;
    }	
	
	
	/**
	 * Metodo per impostare la regularExpression da cercare all'interno della risposta di testo 
     * del WMS per poter capire se un o pi dati sono stati trovati.
	 * 
	 * @param regExpToCheckIfDataFound stringa passata al costruttore della regular expression	
	 */
    public void setRegExpToCheckIfDataFound(String regExpToCheckIfDataFound) {
        this.regExpToCheckIfDataFound = regExpToCheckIfDataFound;        
    }

	@Override
	public void setAdditionalFilterServerParams(AdditionalFilterServerParams additionalFilterParams) {
		this.additionalFilterParams = additionalFilterParams;
		
	}

	@Override
	public AdditionalFilterServerParams getAdditionalFilterServerParams() {
		return additionalFilterParams;
	}

    
}
