/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
package it.prato.comune.sit;

import java.util.HashMap;
import java.util.Map;

import bsh.EvalError;
import bsh.Interpreter;
import it.prato.comune.sit.IFeatureAttributeConfig.Permission;

/**
 * Implementazione dell'interfaccia {@link IGetFeatureInfoObject} per la gestione per esempio dei layer di {@link GetFeatureInfoLayer}
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public class GetFeatureInfoObject implements IGetFeatureInfoObject {

	private String wkt			= null;
	private Map<String, Object> featureInfoAttributes = null;
	private IGetFeatureInfoLayer featureInfoLayer = null;
	private String srid			= null;
	private String boundingBoxWKT = null;
	private String getFeatureInfoLink = null;
	
	private String sourceHTTP 	= null;
	private String sourceLayer	= null;
	private String sourceBBOX	= null;
	private String sourceStyle	= null;
	
	private Integer sourceWidth 	= null;
	private Integer sourceHeight	= null;
	private Integer sourceX			= null;
	private Integer sourceY			= null;
	
	
	/** 
	 * Costruttore
	 * 
	 * @param wkt geometria in formato WKT
	 * @param boundingBoxWKT bounding box in formato WKT
	 * @param featureInfoAttributes attributi della feature
	 * @param featureInfoLayer layer del quale questo oggetto fa parte
	 * @param srid sistema di riferimento
	 * 
	 */
	public GetFeatureInfoObject(String wkt, String boundingBoxWKT,
			Map<String, Object> featureInfoAttributes,
			IGetFeatureInfoLayer featureInfoLayer, String srid) {
		super();
	
		this.wkt = wkt;
		this.boundingBoxWKT = boundingBoxWKT;
		this.featureInfoAttributes = featureInfoAttributes;
		this.featureInfoLayer = featureInfoLayer;
		this.srid = srid;
	}

	/**
	 * Ritorna una descrizione dell'oggetto, comprensibile per l'utente ed in
	 * gradi di fargli discriminare tra pi oggetti.<br>
	 * 
	 * La descrizione viene calcolato valutando di {@link GetFeatureInfoLayer#getEspressioneDescrizione()} tramite beanshell.
	 *<br>
	 * La shell di BeanShell per utilizzato per il calcolo ha a disposizione un oggetto attributes 
	 * che contiene il risultato del metodo {@link OggettoTerritorio#getFeatureInfoAttributes()} 
	 *
	 * @return Stringa per la descrizione dell'oggetto
	 */
	public String getDescrizione() {
		
		return evalEspressione(featureInfoLayer.getEspressioneDescrizione());
		
	}

	/**
	 * Ritorna il nome dell'oggetto, comprensibile per l'utente ed in
	 * gradi di fargli discriminare tra pi oggetti.<br>
	 * 
	 * La descrizione viene calcolato valutando di {@link GetFeatureInfoLayer#getEspressioneNomeOggetto()} tramite beanshell.
	 *<br>
	 * La shell di BeanShell per utilizzato per il calcolo ha a disposizione un oggetto attributes 
	 * che contiene il risultato del metodo {@link OggettoTerritorio#getFeatureInfoAttributes()} 
	 *
	 * @return Stringa per la descrizione dell'oggetto
	 */
	public String getNomeOggetto() {
		
		return evalEspressione(featureInfoLayer.getEspressioneNomeOggetto());
		
	}
	
	private String evalEspressione(String espr) {
		
		String retVal = null;
		
		if (espr!=null) {
			Interpreter i = new Interpreter();  // Construct an interpreter
			
			try {
				i.set("attributes", getFeatureInfoAttributes());
				
				return (String) i.eval(espr);
			} catch (EvalError e) {
				//TODO logger.warn("Errore di valutazione dell'espressione descrizione nel metodo getDescrizione di OggettoTerritorio, proseguo con descrizione standard. Espressione: " + getLayer().getEspressioneDescrizione(), e );
				retVal = e.getMessage();
			}             
		} else {
			retVal = "";	
		}
		
		return retVal;
	}
	
	/**
	 * @see IGetFeatureInfoObject#getIDTPN() 
	 */
	public String getIDTPN() {
		return evalEspressione(featureInfoLayer.getEspressioneIDTPN());
	}

	/**
	 * @see IGetFeatureInfoObject#getJSGeomAdditionalFields() 
	 */
	public Map<String, String> getJSGeomAdditionalFields() {
		return null;
	}

	/**
	 * @see IGetFeatureInfoObject#getGeometryAttributeWKT() 
	 */
	public String getGeometryAttributeWKT() {
		return wkt;
	}

	/**
	 * @see IGetFeatureInfoObject#getFeatureInfoAttributes() 
	 */
	public Map<String, Object> getFeatureInfoAttributes() {
		return featureInfoAttributes;
	}

	/**
	 * Ritorna tutti gli attributi non essendo supportate le permission su questo tipo di fonte.
	 * @see IGetFeatureInfoObject#getFeatureInfoAttributes(Permission) 
	 */
	public Map<String, Object> getFeatureInfoAttributes(Permission perm) {
		return new HashMap<String, Object>(getFeatureInfoAttributes());
	}
	
	/**
	 * @see IGetFeatureInfoObject#getFeatureInfoLayer() 
	 */
	public IGetFeatureInfoLayer getFeatureInfoLayer() {
		return featureInfoLayer;
	}

	/**
	 * @see IGetFeatureInfoObject#getSRID() 
	 */
	public String getSRID() {
		return srid;
	}

	/**
	 * @see IGetFeatureInfoObject#getBoundingBoxWKT() 
	 */
	public String getBoundingBoxWKT() {
		return this.boundingBoxWKT;
	}

	/**
	 * @see IGetFeatureInfoObject#getGetFeatureInfoLink() 
	 */
	public String getGetFeatureInfoLink() {
		return getFeatureInfoLink;
	}

	/**
	 * @param getFeatureInfoLink the getFeatureInfoLink to set
	 */
	public void setGetFeatureInfoLink(String getFeatureInfoLink) {
		this.getFeatureInfoLink = getFeatureInfoLink;
	}

	/**
	 * @return the sourceHTTP
	 */
	public String getSourceHTTP() {
		return sourceHTTP;
	}

	/**
	 * @param sourceHTTP the sourceHTTP to set
	 */
	public void setSourceHTTP(String sourceHTTP) {
		this.sourceHTTP = sourceHTTP;
	}

	/**
	 * @return the sourceLayer
	 */
	public String getSourceLayer() {
		return sourceLayer;
	}

	/**
	 * @param sourceLayer the sourceLayer to set
	 */
	public void setSourceLayer(String sourceLayer) {
		this.sourceLayer = sourceLayer;
	}

	/**
	 * @return the sourceBBOX
	 */
	public String getSourceBBOX() {
		return sourceBBOX;
	}

	/**
	 * @param sourceBBOX the sourceBBOX to set
	 */
	public void setSourceBBOX(String sourceBBOX) {
		this.sourceBBOX = sourceBBOX;
	}

	/**
	 * @return the sourceStyle
	 */
	public String getSourceStyle() {
		return sourceStyle;
	}

	/**
	 * @param sourceStyle the sourceStyle to set
	 */
	public void setSourceStyle(String sourceStyle) {
		this.sourceStyle = sourceStyle;
	}

	/**
	 * @return the sourceWidth
	 */
	public Integer getSourceWidth() {
		return sourceWidth;
	}

	/**
	 * @param sourceWidth the sourceWidth to set
	 */
	public void setSourceWidth(Integer sourceWidth) {
		this.sourceWidth = sourceWidth;
	}

	/**
	 * @return the sourceHeight
	 */
	public Integer getSourceHeight() {
		return sourceHeight;
	}

	/**
	 * @param sourceHeight the sourceHeight to set
	 */
	public void setSourceHeight(Integer sourceHeight) {
		this.sourceHeight = sourceHeight;
	}

	/**
	 * @return the sourceX
	 */
	public Integer getSourceX() {
		return sourceX;
	}

	/**
	 * @param sourceX the sourceX to set
	 */
	public void setSourceX(Integer sourceX) {
		this.sourceX = sourceX;
	}

	/**
	 * @return the sourceY
	 */
	public Integer getSourceY() {
		return sourceY;
	}

	/**
	 * @param sourceY the sourceY to set
	 */
	public void setSourceY(Integer sourceY) {
		this.sourceY = sourceY;
	}
	
}
