/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
package it.prato.comune.sit;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;


/**
 * Classe utilizzata per passare i parametri al metodo GetFeatureInfo dell'interfaccia IGetFeatureInfoLayer
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public class GetFeatureInfoParams {

	//private String layerName= null;//application/vnd.ogc.gml
	private String infoFormat	= null;
	private String infoFormatHtml	= "text/html";
	private int featureCount	= 0;
	private double bboxLeft		= 0.0;
	private double bboxRight 	= 0.0;
	private double bboxBottom 	= 0;
	private double bboxTop 		= 0;
	private int width 			= 0;
	private int height 			= 0;
	private int x 				= 0;
	private int y 				= 0;
	private double coordX		= 0.0;
	private double coordY		= 0.0;
	private double range 		= 0.0;
	private String sourceSRID 	= null;
	private String style		= "";
	private String sld_body		= null;
	
	private int connectTimeout = 20;
	private int readTimeout = 20;
	
	/**
	 * @return the infoFormat
	 */
	public String getInfoFormat() {
		return infoFormat;
	}
	/**
	 * @param infoFormat the infoFormat to set
	 */
	public void setInfoFormat(String infoFormat) {
		this.infoFormat = infoFormat;
	}
	/**
	 * @return the featureCount
	 */
	public int getFeatureCount() {
		return featureCount;
	}
	/**
	 * @param featureCount the featureCount to set
	 */
	public void setFeatureCount(int featureCount) {
		this.featureCount = featureCount;
	}
	
	public void setBBox(String bbox) {
		String[] arr = bbox.split(",");
		
		setBboxLeft(Double.parseDouble(arr[0]));
		setBboxBottom(Double.parseDouble(arr[1]));
		setBboxRight(Double.parseDouble(arr[2]));
		setBboxTop(Double.parseDouble(arr[3]));
		
	}
	
	/**
	 * @return the bboxLeft
	 */
	public double getBboxLeft() {
		return bboxLeft;
	}
	/**
	 * @param bboxLeft the bboxLeft to set
	 */
	public void setBboxLeft(double bboxLeft) {
		this.bboxLeft = bboxLeft;
	}
	/**
	 * @return the bboxRight
	 */
	public double getBboxRight() {
		return bboxRight;
	}
	/**
	 * @param bboxRight the bboxRight to set
	 */
	public void setBboxRight(double bboxRight) {
		this.bboxRight = bboxRight;
	}
	/**
	 * @return the bboxBottom
	 */
	public double getBboxBottom() {
		return bboxBottom;
	}
	/**
	 * @param bboxBottom the bboxBottom to set
	 */
	public void setBboxBottom(double bboxBottom) {
		this.bboxBottom = bboxBottom;
	}
	/**
	 * @return the bboxTop
	 */
	public double getBboxTop() {
		return bboxTop;
	}
	/**
	 * @param bboxTop the bboxTop to set
	 */
	public void setBboxTop(double bboxTop) {
		this.bboxTop = bboxTop;
	}
	/**
	 * @return the width
	 */
	public int getWidth() {
		return width;
	}
	/**
	 * @param width the width to set
	 */
	public void setWidth(int width) {
		this.width = width;
	}
	/**
	 * @return the height
	 */
	public int getHeight() {
		return height;
	}
	/**
	 * @param height the height to set
	 */
	public void setHeight(int height) {
		this.height = height;
	}
	/**
	 * @return the x
	 */
	public int getX() {
		return x;
	}
	/**
	 * @param x the x to set
	 */
	public void setX(int x) {
		this.x = x;
	}
	/**
	 * @return the y
	 */
	public int getY() {
		return y;
	}
	/**
	 * @param y the y to set
	 */
	public void setY(int y) {
		this.y = y;
	}
	/**
	 * @return the range
	 */
	public double getRange() {
		return range;
	}
	/**
	 * @param range the range to set
	 */
	public void setRange(double range) {
		this.range = range;
	}
	/**
	 * @return the sourceSRID
	 */
	public String getSourceSRID() {
		return sourceSRID;
	}
	/**
	 * @param sourceSRID the sourceSRID to set
	 */
	public void setSourceSRID(String sourceSRID) {
		this.sourceSRID = sourceSRID;
	}
	/**
	 * @return the coordX
	 */
	public double getCoordX() {
		return coordX;
	}
	/**
	 * @param coordX the coordX to set
	 */
	public void setCoordX(double coordX) {
		this.coordX = coordX;
	}
	/**
	 * @return the coordY
	 */
	public double getCoordY() {
		return coordY;
	}
	/**
	 * @param coordY the coordY to set
	 */
	public void setCoordY(double coordY) {
		this.coordY = coordY;
	}
	/**
	 * @return the style
	 */
	public String getStyle() {
		return style;
	}
	/**
	 * @param style the style to set
	 */
	public void setStyle(String style) {
		this.style = style;
	}
	
	
	
	/**
	 * Metodo che compone la richiesta getFeatureInfo di tipo html compatibile con i parametri inseriti
	 * 
	 * @param server url del server
	 * @param WMSLayerName nome del layer WMS
	 * @param extraParams
	 * @return
	 * @throws UnsupportedEncodingException
	 */
	public String getFeatureInfoReqStringHtml(String server, String WMSLayerName, String extraParams, AdditionalFilterServerParams wmsFilterServerParams) throws UnsupportedEncodingException {
		
		String sep =  (server.contains("?")) ? "&" : "?";
		String retVal = server + sep + getFeatureInfoReqStringParams(WMSLayerName, extraParams, wmsFilterServerParams, getInfoFormatHtml());

		return retVal;
		
	}
	
	/**
	 * Metodo che compone la richiesta getFeatureInfo compatibile con i parametri inseriti
	 * 
	 * @param server url del server
	 * @param WMSLayerName nome del layer WMS
	 * @param extraParams
	 * @return
	 * @throws UnsupportedEncodingException
	 */
	public String getFeatureInfoReqString(String server, String WMSLayerName, String extraParams, AdditionalFilterServerParams wmsFilterServerParams) throws UnsupportedEncodingException {
		
		String sep =  (server.contains("?")) ? "&" : "?";
		String retVal = server + sep + getFeatureInfoReqStringParams(WMSLayerName, extraParams, wmsFilterServerParams, getInfoFormat());

		return retVal;
		
	}
	
	/**
	 * Metodo di base che compone la richiesta getFeatureInfo compatibile con i parametri inseriti, ma priva dell'indicazione INFO_FORMAT
	 * 
	 * 
	 * @param WMSLayerName nome del layer WMS
	 * @param extraParams
	 * @param wmsFilterServerParams
	 * @param infoFormat
	 * @return
	 * @throws UnsupportedEncodingException
	 */
	public String getFeatureInfoReqStringParams(String WMSLayerName, String extraParams, 
			AdditionalFilterServerParams wmsFilterServerParams, String infoFormat ) throws UnsupportedEncodingException {
		
		
		StringBuilder retVal = new StringBuilder("");
		//String sep =  (server.contains("?")) ? "&" : "?";
		//retVal.append(sep);
		retVal.append("SERVICE=WMS&FORMAT=image%2Fpng&REQUEST=GetFeatureInfo");
		//
		retVal.append("&LAYERS=");
		retVal.append(URLEncoder.encode(WMSLayerName.trim(), "UTF-8"));
		retVal.append("&QUERY_LAYERS=");
		retVal.append(URLEncoder.encode(WMSLayerName.trim(), "UTF-8")); 
		retVal.append("&STYLES=");
		
		retVal.append((getStyle()!=null ? getStyle() : ""));
		retVal.append("&FEATURE_COUNT=");
		retVal.append(getFeatureCount());
		retVal.append("&HEIGHT=");
		retVal.append(getHeight());
		retVal.append("&WIDTH=");
		retVal.append(getWidth());
		retVal.append("&SRS=");
		retVal.append(getSourceSRID());
		retVal.append("&VERSION=1.1.1");
		retVal.append("&TRANSPARENT=TRUE");
		retVal.append("&X=");
		retVal.append(String.valueOf(getX()));
		retVal.append("&Y=");
		retVal.append(String.valueOf(getY()));
		retVal.append("&BBOX=");
		retVal.append(URLEncoder.encode(String.valueOf(getBboxLeft())+","+
										String.valueOf(getBboxBottom())+","+
										String.valueOf(getBboxRight()) +","+
										String.valueOf(getBboxTop()), "UTF-8"));
		retVal.append("&");
		retVal.append(extraParams);
		if (wmsFilterServerParams!=null) {
			retVal.append("&");
			retVal.append(wmsFilterServerParams.getGeoserverParamsString());
			retVal.append("&");
			retVal.append(wmsFilterServerParams.getMapserverParamsString());
		}
		String sldBody = getSld_body();
		if (sldBody != null && !sldBody.equals("")){
			retVal.append("&SLD_BODY=");
			retVal.append(URLEncoder.encode(sldBody, "UTF-8"));
		}
		retVal.append("&INFO_FORMAT=");
		retVal.append(URLEncoder.encode(infoFormat.trim(), "UTF-8"));
		
		return retVal.toString();
		
	}
	/**
	 * @return the infoFormatHtml
	 */
	public String getInfoFormatHtml() {
		return infoFormatHtml;
	}
	/**
	 * @param infoFormatHtml the infoFormatHtml to set
	 */
	public void setInfoFormatHtml(String infoFormatHtml) {
		this.infoFormatHtml = infoFormatHtml;
	}
	/**
	 * @return the sld_body
	 */
	public String getSld_body() {
		return sld_body;
	}
	/**
	 * @param sld_body the sld_body to set
	 */
	public void setSld_body(String sld_body) {
		this.sld_body = sld_body;
	}
	/**
	 * @return the connectTimeout
	 */
	public int getConnectTimeout() {
		return connectTimeout;
	}
	/**
	 * @param connectTimeout the connectTimeout to set
	 */
	public void setConnectTimeout(int connectTimeout) {
		this.connectTimeout = connectTimeout;
	}
	/**
	 * @return the readTimeout
	 */
	public int getReadTimeout() {
		return readTimeout;
	}
	/**
	 * @param readTimeout the readTimeout to set
	 */
	public void setReadTimeout(int readTimeout) {
		this.readTimeout = readTimeout;
	}
	
}
