/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 */

package it.prato.comune.sit;

/**
 * Interfaccia che espone i metodi per il recupero dei parametri di configurazione della feature
 * 
 * @author bf44
 *
 */
public interface IFeatureAttributeConfig {
    
    public enum Permission {
        QUERY,
        READ,
        WRITE;
    }
    
    /**
     * Restituisce il nome reale del campo in tabella
     * @return
     */
    public String getRealName();
    
    /**
     * Restituisce il nome del campo per come lo si vuole esporre
     * @return
     */
    public String getHumanReadableName();
    
    /**
     * Stringa della regula expression necessaria alla validazione/filtraggio del campo (regexp javascript?)
     * @return
     */
    public String getValidatingRegExp();   
    
    /**
     * Restituisce il valore di default del campo o null altrimenti
     * @return
     */
    public String getDefaultValue();
    
    /**
     * Restituisce true o false in base al fatto che a questa feature attribute sia stata assegnata o meno quella specifica autorizzazione 
     * 
     * @param permission
     * @return
     */
    public boolean hasPermission(Permission permission);
    
    /**
     * Restituisce true o false in base al fatto che a questa feature attribute sia stato assegnato un seggest providere 
     * 
     * @param permission
     * @return
     */
    public boolean hasSuggestProvider();
    
    /**
     * Resistuisce il SuggestConfig necessario a definire un suggest per il campo. Null altrimenti 
     * @return
     */
    public ISuggestConfig getSuggestConfig();    
}
