/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
package it.prato.comune.sit;

import java.util.Map;

import it.prato.comune.sit.IFeatureAttributeConfig.Permission;


/**
 * Definisce l'interfaccia che deve essere implementata da oggetti utilizzati da {@link IGetFeatureInfoLayer}.<br>
 * Hanno la capacit di fornire informazioni relative alla loro geometria, gli attributi etc.
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public interface IGetFeatureInfoObject {

	/**
	 * Metodo per recuperare la descrizione dell'oggetto
	 * @return
	 */
    public String getDescrizione();
    
    /**
     * Metodo per recuperare la chiave primaria
     * @return
     */
    public String getIDTPN();
    
    /**
     * Metodo per recuperare informazioni aggiuntive sull'oggetto nella sua serializzazione javascript
     * @return
     */
    public Map<String, String> getJSGeomAdditionalFields();
    
    /**
     * Metodo per recuperare la geometria in formato WKT
     * @return
     */
    public String getGeometryAttributeWKT();
    
    /**
     * Metodo per recuperare il buonding box in formato WKT
     * @return
     */
    public String getBoundingBoxWKT();
    
    /**
     * Metodo per recuperare gli attributi
     * @return
     */
    public Map<String, Object> getFeatureInfoAttributes();
    
    /**
     * Metodo per recuperare gli attributi che hanno una certa permission
     * @param perm 
     * @return
     */
    public Map<String, Object> getFeatureInfoAttributes(Permission perm);
    
    /**
     * Metodo per recuperare il layer di cui questo oggetto fa parte
     * @return
     */
    public IGetFeatureInfoLayer getFeatureInfoLayer();
    
    /**
     * Metodo per recuperare il link per ottenere la getfeatureinfo in formato text/html 
     * @return
     */
    public String getGetFeatureInfoLink(); 
    
    
    /**
     * Metodo per recuperare il sistema di riferimento
     * @return
     * @throws SITException
     */
    public String getSRID() throws SITException;
    
    /**
     * Metodo che permette di recuperare la URL del server dal quale la feature  stata recuperata
     * @return
     */
    public String getSourceHTTP();
    
    /**
     * Metodo che permette di recuperare il nome del layer dal quale la feature  stata recuperata
     * @return
     */
    public String getSourceLayer();
    
    /**
     * Metodo che permette di recuperare il BoundingBox utilizzato nella richiesta dalla quale la feature  stata recuperata
     * @return
     */
    public String getSourceBBOX();
    
    /**
     * Metodo che permette di recuperare lo stile utilizzato nella richiesta dalla quale la feature  stata recuperata
     * @return
     */
    public String getSourceStyle();
    
    /**
     * Metodo che permette di recuperare la width utilizzata nella richiesta dalla quale la feature  stata recuperata
     * @return
     */
    public Integer getSourceWidth();
    
    /**
     * Metodo che permette di recuperare la height utilizzata nella richiesta dalla quale la feature  stata recuperata
     * @return
     */
    public Integer getSourceHeight();

    /**
     * Metodo che permette di recuperare la x utilizzata nella richiesta dalla quale la feature  stata recuperata
     * @return
     */
    public Integer getSourceX();
    
    /**
     * Metodo che permette di recuperare la Y utilizzata nella richiesta dalla quale la feature  stata recuperata
     * @return
     */
    public Integer getSourceY();
    
}
