/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
/*
 * Copyright  Comune di Prato - Sistema Informativo
 * Project:    Package SIT per elaborazione/interrogazione/gestione dati cartografici 
 * File:       ISITLayersManager.java
 * Author:     Nieri Federico
 * Version:    1.0.0
 * CreationDate:   Settembre 2010
 * ModifyDate:     
 *
 * Function:   Interfaccia della classe che gestisce un certo insieme di layer
 * 
 *             
 */

package it.prato.comune.sit;



import it.prato.comune.utilita.logging.interfaces.LogInterface;

import java.sql.Connection;
import java.util.Properties;

/**
 * Interfaccia ILayersManager. 
 * Espone i metodi di gestione dei Layers e le costanti relative alle operazioni
 * che con essi si possono fare.
 * Il metodo initLayers  il metodo mediante il quale si configurano i layers.
 * 
 * @author bf44
 *
 */
public interface ILayersManager {
    
    enum  EditOperationType {EdOpModify, EdOpModifyUnione, EdOpModifyDifferenza, EdOpModifyCopertura, EdOpModifyAndClipCopertura };

    // /////////////////////////////////////////////////////////////////////
    // Attenzione se modificate deve essere modificato anche in javascript
    // /////////////////////////////////////////////////////////////////////
    
    /** Operazione di inserimento */
    public static final String digitizeOperationInsert       = "N".intern();
    /** Operazione di subtract */
    public static final String digitizeOperationSubtract     = "D".intern();   
    /** Operazione di add */
    public static final String digitizeOperationAdd          = "O".intern();
    /** Operazione di AddSub */
    public static final String digitizeOperationAddSub       = "M".intern();  
    /** Operazione di Cancellazione */
    public static final String operationFeatureDelete        = "C".intern();     
    /** Operazione di editing vertici */
    public static final String operationFeatureVertexEditing = "V".intern();   
    /** Operazione di generica modifica geometria */
    public static final String operationGeometryModify       = "G".intern();   
    /** Operazione di dragdrop */
    public static final String operationFeatureDragDrop      = "R".intern();
    /** Operazione di clona */
    public static final String operationFeatureClone         = "CL".intern(); 
    /** Operazione di aggiornamento alfanumerico */
    public static final String operationUpdateAlfa           = "A".intern();
    /** Operazione di identify */
    public static final String operationIdentify             = "I".intern();
    /** Operazione di identify */
    public static final String operationInsFromLayer         = "F".intern();
                

    /**
     * Inizializza un layer
     * 
     * @param pr
     * @param layer
     * @throws SITException
     */
    public void initLayers(Properties pr, int codTPN, LayerTerritorio layer) throws SITException;

    /**
     * Recupera l'oggetto layer passando il codTPN definito nel file di config.
     * 
     * @param codTPN
     * @return LayerTerritorio
     */
    public LayerTerritorio getLayerByCodTPN(int codTPN);
    
    
    /**
    *  Recupera un IGetFeatureInfoLayer passando il codTPN definito.
    * 
    * @param codTPN
    * @return IGetFeatureInfoLayer
    */
	public IGetFeatureInfoLayer getGetFeatureInfoLayerByCodTPN(int codTPN) ;
    
	/**
	 * Aggiunge un IGetFeatureInfoLayer
	 * 
	 * @param layer
	 */
	public void addGetFeatureInfoLayer(IGetFeatureInfoLayer layer);
    
    
    /**
     * Recupera l'oggetto layer passando il nome del layer.
     * 
     * @param name
     * @return LayerTerritorio
     */
    /* **********************************************************************************************
     * Metodo che non pu pi funzionare, perch allo stesso nome possono corrispondere layer diversi
     * **********************************************************************************************
    public LayerTerritorio getLayerByName(String name);            
     */
    /**
     * 
     * @param codTPN
     * @param idTPN
     * @param szJSGeometry
     * @return OggettoTerritorio
     * @throws SITException
     * 
     * @see SITLayersManager#modify(int, String, String) 
     */   
    public OggettoTerritorio modify (int codTPN, String idTPN, String szJSGeometry) throws SITException;

    /**
     * Modifica LA GEOMETRIA dell'oggetto identificato da idTPN all'interno del layer codTPN con quella contenuta in szJSGeometry
     * 
     * N.B. Il valore degli attributi rimane invariato
     * 
     * @param codTPN
     * @param idTPN
     * @param szJSGeometry
     * @return OggettoTerritorio
     * @throws SITException
     */
    public OggettoTerritorio modify(int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica) throws SITException;

    /**
     * @param codTPN
     * @param idTPN
     * @param szJSGeometry
     * @return OggettoTerritorio
     * @throws SITException
     * 
     * @see SITLayersManager#modifyUnione(int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica)
     */
    public OggettoTerritorio modifyUnione(int codTPN, String idTPN, String szJSGeometry) throws SITException;
   
    /**
     * @param codTPN
     * @param idTPN
     * @param szJSGeometry
     * @param tr
     * @param bSoloImmagineStorica
     * @return OggettoTerritorio
     * @throws SITException
     * 
     * @see SITLayersManager#editOperation(int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica, EditOperationType op, Integer clippingCodTPN)
     */
    public OggettoTerritorio modifyUnione (int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica ) throws SITException;

    /**
     * @param codTPN
     * @param idTPN
     * @param szJSGeometry
     * @return OggettoTerritorio
     * @throws SITException
     * 
     * @see SITLayersManager#modifyDifferenza(int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica)
     */
    public OggettoTerritorio modifyDifferenza (int codTPN, String idTPN, String szJSGeometry) throws SITException;
   
    /**
     * @param codTPN
     * @param idTPN
     * @param szJSGeometry
     * @param tr
     * @param bSoloImmagineStorica
     * @return OggettoTerritorio
     * @throws SITException
     * 
     * @see SITLayersManager#editOperation(int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica, EditOperationType op, Integer clippingCodTPN)
     */
    public OggettoTerritorio modifyDifferenza (int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica ) throws SITException;

    /**
     * @param codTPN
     * @param idTPN
     * @param szJSGeometry
     * @return OggettoTerritorio
     * @throws SITException
     * 
     * @see SITLayersManager#appendFeature(int codTPN, String idTPN, String szJSGeometry, SITTransaction tr)
     */
    public OggettoTerritorio appendFeature (int codTPN, String idTPN, String szJSGeometry) throws SITException;
    
    /**
     * Consente di inserire una nuova feature.
     * 
     * @param codTPN
     * @param idTPN
     * @param szJSGeometry
     * @param tr
     * @return OggettoTerritorio
     * @throws SITException
     */
    public OggettoTerritorio appendFeature (int codTPN, String idTPN, String szJSGeometry, SITTransaction tr) throws SITException;

    /**
     * 
     * @param codTPN
     * @param keyValue
     * @throws SITException
     * 
     * @see SITLayersManager#removeFeature(int codTPN, String keyValue, SITTransaction tr)
     */
    public void removeFeature (int codTPN, String keyValue) throws SITException;

    /**
     * Consente la rimozione di una feature.
     * 
     * @param codTPN
     * @param keyValue
     * @param tr
     * @throws SITException
     */
    public void removeFeature (int codTPN, String keyValue, SITTransaction tr) throws SITException;

    /**
     * @param codTPN
     * @param idTPN
     * @param szJSGeometry
     * @param clippingCodTPN
     * @return OggettoTerritorio
     * @throws SITException
     * 
     * @see SITLayersManager#modifyAndClipCopertura(int codTPN, String idTPN, String szJSGeometry, int clippingCodTPN, SITTransaction tr, boolean bSoloImmagineStorica)
     */
    public OggettoTerritorio modifyAndClipCopertura (int codTPN, String idTPN, String szJSGeometry, int clippingCodTPN) throws SITException;
    
   /**
    * Il metodo permette di modificare la copertura di un poligono e di vincolare tale modifica ad un layer
    * che ne limita i confini.
    * 
    * @param codTPN codice tpn del layer da modificare
    * @param clippingCodTPN codice tpn del layer di vincolo
    * @param idTPN id del poligono interessato dalla modifica (o idstorico se bSoloImmagineStorica=true)
    * @param szJSGeometry stringa rrappresentante la geometry della modifica richiesta
    * @param tr Transazione - se null viene usata transazione di default con autocommit
    * @param bSoloImmagineStorica - indica se, in caso di layer storici, deve essere modificata l'immagine identificata da IDTPN (inteso come valore di IDStorico) oppure se deve essere fatto una modifica storica completa (con generazione di nuova immagine) 
    * @return OggettoTerritorio oggetto iniziale con la geometria modificata
    * @throws SITException
    * 
    * @see SITLayersManager#editOperation(int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica, EditOperationType op, Integer clippingCodTPN)
    */
   public OggettoTerritorio modifyAndClipCopertura (int codTPN, String idTPN, String szJSGeometry, int clippingCodTPN, SITTransaction tr, boolean bSoloImmagineStorica) throws SITException;

   /**
    * @param codTPN
    * @param idTPN
    * @param szJSGeometry
    * @return OggettoTerritorio
    * @throws SITException
    * 
    * @see SITLayersManager#modifyCopertura(int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica)
    */
   public OggettoTerritorio modifyCopertura (int codTPN, String idTPN, String szJSGeometry) throws SITException;
   
   /**
    * @param codTPN
    * @param idTPN
    * @param szJSGeometry
    * @param tr
    * @param bSoloImmagineStorica
    * @return OggettoTerritorio
    * @throws SITException
    * 
    * @see SITLayersManager#editOperation(int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica, EditOperationType op, Integer clippingCodTPN)
    */
   public OggettoTerritorio modifyCopertura (int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica) throws SITException;
   
   /**
    * Restituisce una connessione standard alla base di dati.
    * 
    * @return Connection
    * @throws SITException
    */
   public Connection getSitStandardConnection() throws SITException;

   /**
    * @param codTPN
    * @param idTPN
    * @param szClippingCodTPN
    * @param geoCoord
    * @param geoOp
    * @throws SITException
    * 
    * @see SITLayersManager#EditManager(Integer codTPN, String idTPN, String szClippingCodTPN, String geoCoord, String geoOp, SITTransaction tr, boolean bStoricoSoloImmagine)
    */
   public void EditManager (Integer codTPN, String idTPN, String szClippingCodTPN, String geoCoord, String geoOp ) throws SITException;
   
   /**
    * Esegue append, modify, remove e quanto necessari per la gestione della geometria a partire dai parametri nel formato gestito in Tolomeo.
    * 
    * @param codTPN Codice del layer interessato all'operazione
    * @param idTPN  ID dell'oggetto interessato dall'operazione (non utilizzato se si tratta di una operazione di inserimento). Se si tratta di un layer storico contiene IDStorico se BStoricoSoloImmagine=true, l'ID in caso contrario
    * @param szClippingCodTPN codice del layer su cui fare il clipping. null se clipping non necessario
    * @param geoCoord geometria nel formato utilizzato in tolomeo (ed utilizzato da jsGeometryToOggettoTerritorio)
    * @param geoOp (operazione richiesta. E' uno dei valori Costanti.digitizeOperation*)
    * @param tr
    * @param bStoricoSoloImmagine flag che indica se eventuali operazioni storiche devono generare lo storico o sono riferite ad una specifica immagine identificata da idTPN
    * @throws SITException
    */
   public void EditManager (Integer codTPN, String idTPN, String szClippingCodTPN, String geoCoord, 
		   String geoOp, SITTransaction tr, boolean bStoricoSoloImmagine) throws SITException;

   /**
    * @return String
    */
   public String getUser();

   /**
    * @param user
    */
   public void setUser(String user);

   /**
    * @return String
    */
   public String getIpAddress();

   /**
    * @param ipAddress
    */
   public void setIpAddress(String ipAddress);
   
   public LogInterface getLogger();
   

   public void dispose() throws SITException;
}