/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/

package it.prato.comune.sit;

import java.util.List;
import java.util.Map;

/**
 * Interfaccia che definisce i metodi che devono essere implementati da un suggestProvider
 * 
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public interface ISuggestProvider {

	/**
	 * Metodo che restituisce i valori suggeriti. Questa particolare implementazione non prevede argomenti di filtraggio
	 * @return Lista contanente un elenco di elementi definiti da una mappa di coppie nomecampo,valore campo
	 * @throws SITException
	 */
	public List<Map<String,Object>> 	getValori() throws SITException;
	//public void 						setValori(List<Map<Object,Object>> valori);
	
	/**
	 * Metodo che definisce se il provider  locale (filtraggio del suggerimento tra i valori locali) o no (filtraggio tramite richiesta remota al server)
	 * @return false se  locale, true se remoto
	 */
	public boolean						isRemote();
	
	/**
	 * Restituisce il nome del campo da visualizzare nella combo box di suggest
	 * @return nome del campo
	 */
	public String						getDisplayFieldName();
	
	/**
	 * Restituisce il nome del campo da utilizzare come value (e quindi inviare per la ricerca) nella combo box di suggest
	 * @return
	 */
	public String						getValueFieldName();
	
	/**
	 * Ritorna una stringa codificata che indica in che modo questo suggest deve essere filtrato in funzione dei valori contenuti in altri campi della ricerca. <br>
	 * la stringa  nella forma posizionecampofiltro1#nomecampofiltro1,posizionecampofiltro2#nomecampofiltro2.... Per esempio 0#codreg,1#codprov
	 * @return
	 */
	public String getFilterBind();
	
}
