/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
/*
 * Copyright  Comune di Prato - Sistema Informativo
 * Project:    Package SIT per elaborazione/interrogazione/gestione dati cartografici 
 * File:       JSGeometry.java
 * Author:     Alessandro Radaelli
 * Version:    1.0.0
 * CreationDate:  
 * ModifyDate:     
 *             
 */
package it.prato.comune.sit;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.geotools.referencing.CRS;
import org.opengis.referencing.FactoryException;
import org.opengis.referencing.NoSuchAuthorityCodeException;
import org.opengis.referencing.crs.CoordinateReferenceSystem;

import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.MultiLineString;
import com.vividsolutions.jts.geom.MultiPoint;
import com.vividsolutions.jts.geom.MultiPolygon;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.Polygon;
import com.vividsolutions.jts.io.ParseException;
import com.vividsolutions.jts.io.WKTReader;
import com.vividsolutions.jts.io.WKTWriter;

import it.prato.comune.sit.IFeatureAttributeConfig.Permission;
import it.prato.comune.utilita.logging.interfaces.LogInterface;
import net.sf.json.JSONArray;
import net.sf.json.JSONNull;
import net.sf.json.JSONObject;

/**
 * Classe JSGeometry.
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public class JSGeometry {

    private int codTPN;
    private String key=null;
    private String description=null;
    private List<String> descriptionSuggest = null;
    private Map<String, String> additionalFields = null;
    private String geometry;
    private String boundingbox;
    private String SRID;
    private LogInterface logger;
    private String getFeatureInfoLink=null; 
    
    private String sourceHTTP 	= null;
	private String sourceLayer	= null;
	private String sourceBBOX	= null;
	private String sourceStyle	= null;
	
	private Integer sourceWidth 	= null;
	private Integer sourceHeight	= null;
	private Integer sourceX		= null;
	private Integer sourceY		= null;
	
	private Map<String, Object> attributes = new HashMap<String, Object>();

	/**
	 * Tipo di geometria.
	 * 
	 *
	 */
	public enum GeometryType{
		POINTS(1),
		LINES(2),
		POLYGONS(3);
		
		private int presetValue;
		private GeometryType(int presetValue){
			this.presetValue = presetValue;
		}
		public int getPresetValue() {
			return presetValue;
		}
		
		/**
		 * Restituisce l'oggetto che corrisponde al parametro passato in ingresso.
		 * 
		 * @param prValue Valore utilizzato per individuare il tipo di geometria all'interno del preset file.
		 * 
		 * @return
		 */
		public static GeometryType fromPresetValue(int prValue){
			for (GeometryType gt: values()){
				if (prValue == gt.getPresetValue()) {
					return gt;
				}
			}
			return null;
		}

	}
    /**
     * Costruttore della classe JSGeometry.
     * 
     * @param logger
     */
    public JSGeometry(LogInterface logger) {
        this.logger = logger;
    }
    
    /**
     * Costruttore della classe JSGeometry.
     * 
     * @param JSGeometry
     * @param logger
     */
    @SuppressWarnings("unchecked")
	public JSGeometry(String JSGeometry, LogInterface logger) {
        
        this(logger);
        
        JSONObject json = JSONObject.fromObject(JSGeometry);
        
        codTPN = json.getInt("codTPN");
        key = json.optString("key");        
        description = json.optString("description");
        setDescriptionSuggest((List<String>) json.optJSONArray("descriptionSuggest"));
        geometry = json.getString("geometry");
        boundingbox = json.optString("boundingbox");
        SRID = json.optString("SRID");
        getFeatureInfoLink = json.optString("getFeatureInfoLink");
        sourceHTTP = json.optString("sourceHTTP");
        sourceLayer = json.optString("sourceLayer");
        sourceBBOX = json.optString("sourceBBOX");
        sourceStyle = json.optString("sourceStyle");
        sourceWidth = json.optInt("sourceWidth");
        sourceHeight = json.optInt("sourceHeight");
        sourceX = json.optInt("sourceX");
        sourceY = json.optInt("sourceY");
        if (json.containsKey("attributes") && json.opt("attributes")!=null && !(json.opt("attributes") instanceof JSONNull)) {
        	
        	for (Object k: json.keySet()) {
        		Object v = json.get(k);
        		attributes.put((String)k, v);
        	}
        	//attributes.putAll((Map<String, Object>)json.opt("attributes"));
        }
  
    }

    /**
     * Decodifica il tipo di geometria della geometria passata. Package-private per non esporre il tipo Geometry al di fuori del package.
     * 
     * @param g Geometria di cui decodificare il tipo
     * @return tipo di geometria decodificato
     */
    static GeometryType getGeometryType(final Geometry g){
		
		if (Point.class.isAssignableFrom(g.getClass()) ||
				MultiPoint.class.isAssignableFrom(g.getClass())) {
			return GeometryType.POINTS;
		}
		
		if (LineString.class.isAssignableFrom(g.getClass()) ||
				MultiLineString.class.isAssignableFrom(g.getClass())) {
			return GeometryType.LINES;
		}
		
		if (Polygon.class.isAssignableFrom(g.getClass()) ||
				MultiPolygon.class.isAssignableFrom(g.getClass())) {
			return GeometryType.POLYGONS;
		}
		
		return null;
		
	}
    /**
     * Converte una Geometria da WKT a com.vividsolutions.jts.geom.Geometry.
     * 
     * @param WKTGeom
     * @param logger
     * @return Geometry
     */
    //TODO gestione eccezioni
    protected static Geometry WKTToGeometry(String WKTGeom, LogInterface logger) {
        
        WKTReader wktReader = new WKTReader();
        Geometry geom = null;
        
        try {
            geom = wktReader.read(WKTGeom);
        } catch (ParseException e) {
            logger.error("Errore durante conversione geometria da WKT a com.vividsolutions.jts.geom.Geometry", e);
        }  
        
        return geom;
    }
    
    /**
     * Converte un oggetto JSGeometry nel corrispondente OggettoTerritorio.
     * 
     * @param layer
     * @param jsG
     * @param logger
     * @return OggettoTerritorio
     */
    //TODO gestione eccezioni
    public static OggettoTerritorio jsGeometryToOggettoTerritorio(LayerTerritorio layer, JSGeometry jsG, LogInterface logger) {
        
        if(layer.getCodTPN() != jsG.getCodTPN()) throw new IllegalStateException("Il layer passato e quello relativo alla geometria non corrispondono"); 
        OggettoTerritorio retVal=null;

        try {
        
            retVal = layer.creaNuovoOggettoTerritorio();
            
            Geometry geom =  JSGeometry.WKTToGeometry(jsG.getGeometry(), logger);
            
            Geometry geomTrasf = OggettoTerritorio.trasforma(geom, jsG.getSRID() , layer.getCRS(), logger);
                
            //Geometry newGeom = OggettoTerritorio.castToSameGeom(retVal.getGeometryAttribute(), geom);
            
            // TODO decidere se e come gestire l'eventuale presenza di key
            
            retVal.setGeometryAttributeGeom(geomTrasf);
            
        } catch (SITIllegalAttributeException e) {
            logger.error("Errore durante conversione da JSGeometry a OggettoTerritorio: SITIllegalAttributeException ", e);
        } catch (IOException e) {
            logger.error("Errore durante conversione da JSGeometry a OggettoTerritorio: IOException ", e);
        } catch (SITException e) {
            logger.error("Errore durante conversione da JSGeometry a OggettoTerritorio: SITException ", e);
        } 
        
        return retVal;
    }
    
    /**
     * Converte un oggetto JSGeometry nel corrispondente OggettoTerritorio.
     * 
     * @param terr
     * @param JSGeom
     * @param logger
     * @return OggettoTerritorio
     */
    //TODO gestione eccezioni
    public static OggettoTerritorio jsGeometryToOggettoTerritorio(SITLayersManager terr, String JSGeom, LogInterface logger) {
                
        JSGeometry jsG = new JSGeometry(JSGeom, logger);
        
        LayerTerritorio layer = terr.getLayerByCodTPN(jsG.getCodTPN());
        
        return jsGeometryToOggettoTerritorio(layer,jsG,logger);
        
    }
    
    /** 
     * @param ogg
     * @param logger
     * @return JSONObject
     * @throws SITException
     * 
     * @see JSGeometry#oggettoTerritorioToJSGeometry(OggettoTerritorio ogg, String toCRS, LogInterface logger)
     */
    public static JSONObject oggettoTerritorioToJSGeometry(OggettoTerritorio ogg, LogInterface logger) throws SITException {       
        return oggettoTerritorioToJSGeometry(ogg, null, logger);        
    }
    
    /**
     * Converte un OggettoTerritorio passato come argomento in un JSGeometry (JSONObject). <br />
     * Non supporta per retrocompatibilit la presenza di attributi nel valore ritornato
     * 
     * @param ogg
     * @param toCRS
     * @param logger
     * @return JSONObject
     * @throws SITException
     */
    // TODO controllo errori
    public static JSONObject oggettoTerritorioToJSGeometry(OggettoTerritorio ogg,  String toCRS, LogInterface logger) throws SITException {

        JSONObject json = new JSONObject();

        json.put("codTPN", new Integer(ogg.getLayer().getCodTPN()));
        json.put("key", ogg.getIDTPN());
        json.put("description", ogg.getDescrizione());
        json.put("getFeatureInfoLink", ogg.getGetFeatureInfoLink());
        json.put("sourceHTTP", ogg.getSourceHTTP());
        json.put("sourceLayer", ogg.getSourceLayer());
        json.put("sourceBBOX", ogg.getSourceBBOX());
        json.put("sourceStyle", ogg.getSourceStyle());
        
        json.put("sourceWidth", ogg.getSourceWidth());
        json.put("sourceHeight", ogg.getSourceHeight());
        json.put("sourceX", ogg.getSourceX());
        json.put("sourceY", ogg.getSourceY());
        
        int i=0;
        for (String descSugg: ogg.getDescrizioneSuggest()) {
            json.put("descriptionSuggest"+i, descSugg);
            i++;
        }
               
        Geometry geom = ogg.getGeometryAttribute();
        
        if (geom!=null) {
            Geometry boundingBox = geom.getEnvelope();
           
            if (toCRS!= null && !toCRS.equals("") ) {                
                geom = OggettoTerritorio.trasforma(geom, ogg.getLayer().getCRS(),  toCRS, logger);
                boundingBox = OggettoTerritorio.trasforma(boundingBox, ogg.getLayer().getCRS(),  toCRS, logger);                 
            } else {
            	// /////////////////////////////////////////////////////////////////////////////////
                // TODOH Se non viene indicato un sistema diverso dovrei recuperare da oggetto, 
                // ma non e' sotto forma di codice...quindi per il momento resta NULL
                // toCRS =  ogg.getLayer().getCRS();
            	// /////////////////////////////////////////////////////////////////////////////////
            }
            
            WKTWriter wktWriter = new WKTWriter();
    
            json.put("geometry", wktWriter.write(geom));
            json.put("boundingbox", wktWriter.write(boundingBox));
            json.put("SRID", toCRS);
            
        } else {
            json.put("geometry", "");
            json.put("boundingbox", "");
            json.put("SRID", "");
        }
                
        logger.debug("Ho convertito l'oggettoTerritorio " + ogg.getDescrizione() + " in JSONObject: " + json.toString());
        
        return json;
    }
    
    /**
     * Non supporta per retrocompatibilit la presenza di attributi nel valore ritornato
     * @param oggetti
     * @param logger
     * @return
     * @throws SITException
     * 
     * @see JSGeometry#oggettoTerritorioToJSGeometryArray(List, String, LogInterface)
     */
    public static JSONObject oggettoTerritorioToJSGeometryArray (List<? extends OggettoTerritorio> oggetti, LogInterface logger) throws SITException {    
        return oggettoTerritorioToJSGeometryArray (oggetti, null, logger);
    }
    
    /**
     * Converte una lista di OggettoTerritorio in un array di JSGeometry (JSONArray).
     * Non supporta per retrocompatibilit la presenza di attributi nel valore ritornato
     * @param oggetti
     * @param toCRS
     * @param logger
     * @return JSONObject
     * @throws SITException
     */
    public static JSONObject oggettoTerritorioToJSGeometryArray (List<? extends OggettoTerritorio> oggetti, 
    		String toCRS, LogInterface logger) throws SITException {
    	
        JSONObject[] buff = new JSONObject[oggetti.size()];
        JSONObject retVal = new JSONObject();
        CoordinateReferenceSystem layerCRS = null;
        
        Geometry bb = null;
        for (int i = 0; i < oggetti.size(); i++) {
            if (bb==null) {
                Geometry  geom = oggetti.get(0).getGeometryAttribute();
                if (geom!=null) {
                    bb = (Geometry) geom.getEnvelope().clone();
                    layerCRS = oggetti.get(0).getLayer().getCRS();
                }
            } else {
                Geometry geom = oggetti.get(i).getGeometryAttribute();
                if (geom != null) bb = bb.union(geom.getEnvelope());
            }
            
            buff[i] =  oggettoTerritorioToJSGeometry(oggetti.get(i),  toCRS, logger);
        }
        
        if (bb!=null && toCRS!= null && !toCRS.equals("")) {            
            bb = OggettoTerritorio.trasforma(bb, layerCRS,  toCRS, logger);            
        }
        
        JSONArray jsonGeometries = JSONArray.fromObject(buff);
        retVal.put("geometries", jsonGeometries);
        
        if (bb!=null) {
            WKTWriter wktWriter = new WKTWriter();
            retVal.put("boundingbox", wktWriter.write(bb));
        } else {
            retVal.put("boundingbox", (String) null);
        }
        
        return retVal;        
    }

    /**
     * 
     * @return String
     */
    public String getBoundingbox() {
        return boundingbox;
    }

    /**
     * 
     * @param oggTer
     * @param logger
     * @return JSGeometry
     * @throws SITException
     * 
     * @see JSGeometry#toJSGeometry(OggettoTerritorio oggTer, String toCRS, LogInterface logger)
     */
    public static JSGeometry toJSGeometry(OggettoTerritorio oggTer, LogInterface logger) throws SITException {
        return toJSGeometry(oggTer, null, logger);
    }
    
    /**
     * 
     * @param oggTer
     * @param toCRS
     * @param logger
     * @return JSGeometry
     * @throws SITException
     * 
     * @see JSGeometry#toJSGeometry(OggettoTerritorio oggTer, String toCRS, LogInterface logger, boolean withGeometry)
     */
    public static JSGeometry toJSGeometry(OggettoTerritorio oggTer, String toCRS, 
    		LogInterface logger) throws SITException {        
        return toJSGeometry(oggTer,  toCRS, logger, true);
    }
    
    /**
     * Converte IGetFeatureInfoObject passato come argomento in un oggetto JSGeometry. 
     * 
     * @param oggTer oggetto da convertire
     * @param toCRS sistema di riferimento nel quale convertire la geometria (se null non viene effettuata alcuna conversione)
     * @param logger
     * @param withGeometry flag che stabilisce se nel {@link JSGeometry} risultante deve essere valorizzata anche la geometria
     * @return JSGeometry 
     * @throws SITException
     */
    public static JSGeometry toJSGeometry(IGetFeatureInfoObject oggTer, String toCRS, LogInterface logger, 
    		boolean withGeometry) throws SITException {
    	return toJSGeometry(oggTer, toCRS, logger, withGeometry, false);
    }
    
    /**
     * Converte IGetFeatureInfoObject passato come argomento in un oggetto JSGeometry. 
     * 
     * @param oggTer oggetto da convertire
     * @param toCRS sistema di riferimento nel quale convertire la geometria (se null non viene effettuata alcuna conversione)
     * @param logger
     * @param withGeometry flag che stabilisce se nel {@link JSGeometry} risultante deve essere valorizzata anche la geometria
     * @param withAttributes flag che stabilisce se nel {@link JSGeometry} risultante devono essere valorizzati anche gli attributi. 
     *        Nell'elenco degli attributi ##FID## sar sostituito da __FID__ per problemi di compatibilit con javascript 

     * @return JSGeometry 
     * @throws SITException
     */
    public static JSGeometry toJSGeometry(IGetFeatureInfoObject oggTer, String toCRS, LogInterface logger, 
    		boolean withGeometry, boolean withAttributes) throws SITException {
        
        JSGeometry retVal =  new JSGeometry(logger);
        
        retVal.codTPN = oggTer.getFeatureInfoLayer().getCodTPN();
        retVal.description = oggTer.getDescrizione();
        retVal.key = oggTer.getIDTPN();
        retVal.additionalFields = oggTer.getJSGeomAdditionalFields();
        retVal.getFeatureInfoLink = oggTer.getGetFeatureInfoLink();
        
        retVal.sourceHTTP = oggTer.getSourceHTTP();
        retVal.sourceLayer = oggTer.getSourceLayer();
        retVal.sourceBBOX = oggTer.getSourceBBOX();
        retVal.sourceStyle = oggTer.getSourceStyle();
        
        retVal.sourceX = oggTer.getSourceX();
        retVal.sourceY = oggTer.getSourceY();
        retVal.sourceWidth = oggTer.getSourceWidth();
        retVal.sourceHeight = oggTer.getSourceHeight();
        
        
        if (withAttributes) {
        	Map<String, Object> fa = new HashMap<String, Object>(oggTer.getFeatureInfoAttributes(Permission.READ));
        	if (fa.containsKey("##FID##")) {
        		fa.put("__FID__", fa.get("##FID##"));
        		fa.remove("##FID##");
        	}
        	retVal.getAttributes().putAll(fa);
        }
        
        if (withGeometry) {
        	
        	String WKTGeom = oggTer.getGeometryAttributeWKT();
        	String WKTBound = oggTer.getBoundingBoxWKT();
        	String fromCRS = oggTer.getSRID();
        	logger.info("fromCRS = " + fromCRS + ", toCRS = " + toCRS);
        	
        	if (toCRS!=null && fromCRS!=null && !fromCRS.equals(toCRS)) {
        		CoordinateReferenceSystem sourceCRS = null;
        		CoordinateReferenceSystem targetCRS = null;
        		try {
	        		// Trasformazione geometria
        		    //System.setProperty("org.geotools.referencing.forceXY", "true");
	        		sourceCRS = CRS.decode(fromCRS,true);
	        		targetCRS = CRS.decode(toCRS,true);
        		} catch (NoSuchAuthorityCodeException e) {
                	String errMsg = "Errore durante trasformazione CRS: authority code non valido";
                	logger.error(errMsg,e);
                    throw new SITException(errMsg, e);
                } catch (FactoryException e) {
                	String errMsg = "Errore durante trasformazione CRS: FactoryException";
                	logger.error(errMsg,e);
                    throw new SITException(errMsg, e);
                }
        		Geometry geom = WKTToGeometry(oggTer.getGeometryAttributeWKT(), logger);
        		Geometry outGeom = OggettoTerritorio.trasforma(geom, sourceCRS, targetCRS, logger);

        		if (outGeom!=null) {
        			WKTWriter wktw = new WKTWriter();
            		WKTGeom = wktw.write(outGeom);
//            		logger.info("Geometria trasformata = " + WKTGeom.toString());
                    Geometry boundingBox = outGeom.getEnvelope();
                    WKTBound = wktw.write(boundingBox);              
                    
        		}

        	}
        	retVal.geometry = WKTGeom;
        	retVal.boundingbox = WKTBound;
        }
        
        return retVal;        
    }
    
    /**
     * Converte l'OggettoTerritorio passato come argomento il un oggeto JSGeometry.
     * 
     * @param oggTer oggetto da convertire
     * @param toCRS sistema di riferimento nel quale convertire la geometria (se null non viene effettuata alcuna conversione)
     * @param logger
     * @param withGeometry flag che stabilisce se nel {@link JSGeometry} risultante deve essere valorizzata anche la geometria
     *
     * @return JSGeometry 
     * @throws SITException
     */ 
    public static JSGeometry toJSGeometry(OggettoTerritorio oggTer, String toCRS, LogInterface logger, 
    		boolean withGeometry) throws SITException {
    	return toJSGeometry(oggTer, toCRS, logger, withGeometry, false);
    }
    
    /**
     * Converte l'OggettoTerritorio passato come argomento il un oggeto JSGeometry.
     * 
     * @param oggTer oggetto da convertire
     * @param toCRS sistema di riferimento nel quale convertire la geometria (se null non viene effettuata alcuna conversione)
     * @param logger
     * @param withGeometry   flag che stabilisce se nel {@link JSGeometry} risultante deve essere valorizzata anche la geometria
     * @param withAttributes flag che stabilisce se nel {@link JSGeometry} risultante devono essere valorizzati anche gli attributi. 
     *        Nell'elenco degli attributi ##FID## sar sostituito da __FID__ per problemi di compatibilit con javascript 
     *
     * @return JSGeometry 
     * @throws SITException
     */ 
    public static JSGeometry toJSGeometry(OggettoTerritorio oggTer, String toCRS, LogInterface logger, 
    		boolean withGeometry, boolean withAttributes) throws SITException {
        
        JSGeometry retVal =  new JSGeometry(logger);
        
        retVal.codTPN = oggTer.getLayer().getCodTPN();
        retVal.description = oggTer.getDescrizione();
        retVal.key = oggTer.getIDTPN();
        retVal.setDescriptionSuggest(oggTer.getDescrizioneSuggest());
        retVal.additionalFields = oggTer.getJSGeomAdditionalFields();
        retVal.getFeatureInfoLink = oggTer.getGetFeatureInfoLink();
        
        retVal.sourceHTTP = oggTer.getSourceHTTP();
        retVal.sourceLayer = oggTer.getSourceLayer();
        retVal.sourceBBOX = oggTer.getSourceBBOX();
        retVal.sourceStyle = oggTer.getSourceStyle();
        
        retVal.sourceX = oggTer.getSourceX();
        retVal.sourceY = oggTer.getSourceY();
        retVal.sourceWidth = oggTer.getSourceWidth();
        retVal.sourceHeight = oggTer.getSourceHeight();
        
        if (withAttributes) {
        	Map<String, Object> fa = new HashMap<String, Object>(oggTer.getFeatureInfoAttributes(Permission.READ));
        	if (fa.containsKey("##FID##")) {
        		fa.put("__FID__", fa.get("##FID##"));
        		fa.remove("##FID##");
        	}
        	retVal.getAttributes().putAll(fa);
        }
        
        if (withGeometry) {
        	retVal.setGeometry(oggTer, toCRS);
        	
        }
        
        return retVal;        
    }
    
    /**
     * Setta la geometria 
     * 
     * @param oggTer oggetto dal quale prendere la geometria
     * @param toCRS sistema di riferimento al quale convertire (se null nessuna conversione viene effettuata)
     * @throws SITException
     */

    private void setGeometry(OggettoTerritorio oggTer, String toCRS) throws SITException  {
        
        Geometry geom = oggTer.getGeometryAttribute();
        
        if (geom!=null) {
            Geometry boundingBox = geom.getEnvelope();
           
            if (toCRS!= null && !toCRS.equals("") ) {
                
                try {
                    geom = OggettoTerritorio.trasforma(geom, oggTer.getLayer().getCRS(),  toCRS, logger);
                    boundingBox = OggettoTerritorio.trasforma(boundingBox, oggTer.getLayer().getCRS(),  toCRS, logger);
                } catch (SITException e) {
                    logger.warn("Errore durante trasformazione toJSGeometry: " + e.getMessage(), e);
                    throw e;
                } 
                
            } else {
            	// /////////////////////////////////////////////////////////////////////////////////
                // TODOH Se non viene indicato un sistema diverso dovrei recuperare da oggetto, 
                // ma non e' sotto forma di codice...quindi per il momento resta NULL
                // toCRS =  ogg.getLayer().getCRS();
            	// /////////////////////////////////////////////////////////////////////////////////
            }
            
            WKTWriter wktWriter = new WKTWriter();
    
            this.geometry = wktWriter.write(geom);
            this.boundingbox = wktWriter.write(boundingBox);
            this.SRID = toCRS;
            
        } else {
        	this.geometry = "";
        	this.boundingbox = "";
        	this.SRID = "";
        }
    }
    
    /**
     * 
     * @param oggTerCollection
     * @param logger
     * @throws SITException
     * 
     * @see JSGeometry#toJSGeometryList(List, String, LogInterface, boolean)
     */
    public static List<JSGeometry> toJSGeometryArrayList(List<? extends OggettoTerritorio> oggTerCollection, LogInterface logger) throws SITException {
        return toJSGeometryList(oggTerCollection, null, logger, true);
    }
    
    /**
     * Converte una lista di OggettoTerritorio in una lista di JSGeometry.
     * 
     * @param oggTerCollection
     * @param toCRS
     * @param logger
     * @param withGeometry
     * @return
     * @throws SITException
     */
    public static List<JSGeometry> toJSGeometryList(List<? extends OggettoTerritorio> oggTerCollection, String toCRS, LogInterface logger, boolean withGeometry) throws SITException {
        List <JSGeometry> retVal = new ArrayList<JSGeometry>();
        
        for (OggettoTerritorio oggTer: oggTerCollection ) {
            retVal.add(toJSGeometry(oggTer, toCRS, logger, withGeometry));
        }
        
        return retVal;        
    }
    
    /**
     * Converte una lista di OggettoTerritorio in una lista di JSGeometry.
     * 
     * @param getFeatureInfoObjectCollection
     * @param toCRS
     * @param logger
     * @param withGeometry
     * @return
     * @throws SITException
     */
    public static List<JSGeometry> toJSGeometryList1(List<IGetFeatureInfoObject> getFeatureInfoObjectCollection, String toCRS, LogInterface logger, boolean withGeometry) throws SITException {
        return toJSGeometryList1(getFeatureInfoObjectCollection, toCRS, logger, withGeometry, false) ;        
    }
    
    /**
     * Converte una lista di OggettoTerritorio in una lista di JSGeometry.
     * 
     * @param getFeatureInfoObjectCollection
     * @param toCRS
     * @param logger
     * @param withGeometry
     * @param withAttributes
     * @return
     * @throws SITException
     */
    public static List<JSGeometry> toJSGeometryList1(List<IGetFeatureInfoObject> getFeatureInfoObjectCollection, String toCRS, LogInterface logger, 
    		boolean withGeometry, boolean withAttributes) throws SITException {
        List <JSGeometry> retVal = new ArrayList<JSGeometry>();
        
        for (IGetFeatureInfoObject oggTer: getFeatureInfoObjectCollection ) {
            retVal.add(toJSGeometry(oggTer, toCRS, logger, withGeometry, withAttributes));
        }
        
        return retVal;        
    }
    
    /**
     * Verifica se la geometria  dentro il boundingbox passato in ingresso.
     * 
     * @param x1 
     * @param x2
     * @param y1
     * @param y2
     * @return
     * @throws ParseException
     */
    public boolean isInside(double x1, double x2, double y1, double y2) throws ParseException{
    	WKTReader wkr = new WKTReader();
    	Geometry g = wkr.read(this.boundingbox);
    	return g.getEnvelopeInternal().getMinX()>=x1 &&
    			g.getEnvelopeInternal().getMaxX()<=x2 &&
    			g.getEnvelopeInternal().getMinY()>=y1 &&
    			g.getEnvelopeInternal().getMaxY()<=y2;
    }
    
    /**
     * 
     * @return int
     */
    public int getCodTPN() {
        return codTPN;
    }

    /**
     * 
     * @param codTPN
     */
    public void setCodTPN(int codTPN) {
        this.codTPN = codTPN;
    }

    /**
     * 
     * @return String
     */
    public String getKey() {
        return key;
    }

    /**
     * 
     * @param key
     */
    public void setKey(String key) {
        this.key = key;
    }

    /**
     * 
     * @return String
     */
    public String getGeometry() {
        return geometry;
    }
    
    /**
     * 
     * @param geometry
     */
    public void setGeometry(String geometry) {
        this.geometry = geometry;
        
        // Calcolo BoundingBox
        WKTReader wkr = new WKTReader();
        WKTWriter wkw = new WKTWriter();
        try {
			Geometry g = wkr.read(geometry);
			boundingbox = wkw.write(g.getEnvelope());
		} catch (ParseException e) {
			logger.error("Errore durante conversione geometria da WKT per calcolo boundingbox", e);
			// Non potuto aggiungere propagazione errore per non alterare API
		}
        
    }


    /**
     * 
     * @return String
     */
    public String getDescription() {
        return description;
    }

    /**
     * 
     * @param description
     */
    public void setDescription(String description) {
        this.description = description;
    }

    /**
     * 
     * @return String
     */
    public String getSRID() {
        return SRID;
    }

    /**
     * 
     * @param srid
     */
    public void setSRID(String srid) {
        SRID = srid;
    }

    /**
     * 
     * @param descriptionSuggest
     */
    public void setDescriptionSuggest(List<String> descriptionSuggest) {
        this.descriptionSuggest = descriptionSuggest;
    }

    /**
     * 
     * @return
     */
    public List<String> getDescriptionSuggest() {
        return descriptionSuggest;
    }
   
    /**
     * 
     * @return
     */
    public Map<String, String> getAdditionalFields() {
        return additionalFields;
    }

	/**
	 * @return
	 */
	public String getGetFeatureInfoLink() {
		return getFeatureInfoLink;
	}

	/**
	 * 
	 * @param getFeatureInfoLink
	 */
	public void setGetFeatureInfoLink(String getFeatureInfoLink) {
		this.getFeatureInfoLink = getFeatureInfoLink;
	}

	/**
	 * @return the sourceHTTP
	 */
	public String getSourceHTTP() {
		return sourceHTTP;
	}

	/**
	 * @param sourceHTTP the sourceHTTP to set
	 */
	public void setSourceHTTP(String sourceHTTP) {
		this.sourceHTTP = sourceHTTP;
	}

	/**
	 * @return the sourceLayer
	 */
	public String getSourceLayer() {
		return sourceLayer;
	}

	/**
	 * @param sourceLayer the sourceLayer to set
	 */
	public void setSourceLayer(String sourceLayer) {
		this.sourceLayer = sourceLayer;
	}

	/**
	 * @return the sourceBBOX
	 */
	public String getSourceBBOX() {
		return sourceBBOX;
	}

	/**
	 * @param sourceBBOX the sourceBBOX to set
	 */
	public void setSourceBBOX(String sourceBBOX) {
		this.sourceBBOX = sourceBBOX;
	}

	/**
	 * @return the sourceStyle
	 */
	public String getSourceStyle() {
		return sourceStyle;
	}

	/**
	 * @param sourceStyle the sourceStyle to set
	 */
	public void setSourceStyle(String sourceStyle) {
		this.sourceStyle = sourceStyle;
	}

	/**
	 * @return the attributes
	 */
	public Map<String, Object> getAttributes() {
		return attributes;
	}

	/**
	 * @return the sourceWidth
	 */
	public Integer getSourceWidth() {
		return sourceWidth;
	}

	/**
	 * @param sourceWidth the sourceWidth to set
	 */
	public void setSourceWidth(Integer sourceWidth) {
		this.sourceWidth = sourceWidth;
	}

	/**
	 * @return the sourceHeight
	 */
	public Integer getSourceHeight() {
		return sourceHeight;
	}

	/**
	 * @param sourceHeight the sourceHeight to set
	 */
	public void setSourceHeight(Integer sourceHeight) {
		this.sourceHeight = sourceHeight;
	}

	/**
	 * @return the sourceX
	 */
	public Integer getSourceX() {
		return sourceX;
	}

	/**
	 * @param sourceX the sourceX to set
	 */
	public void setSourceX(Integer sourceX) {
		this.sourceX = sourceX;
	}

	/**
	 * @return the sourceY
	 */
	public Integer getSourceY() {
		return sourceY;
	}

	/**
	 * @param sourceY the sourceY to set
	 */
	public void setSourceY(Integer sourceY) {
		this.sourceY = sourceY;
	}
		
}
