/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
/*
 * Copyright  Comune di Prato - Sistema Informativo
 * Project:    Package SIT per elaborazione/interrogazione/gestione dati cartografici 
 * File:       JSGeometryArrayList.java
 * Author:     Alessandro Radaelli
 * Version:    1.0.0
 * CreationDate:  
 * ModifyDate:     
 *             
 */
package it.prato.comune.sit;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.FilenameUtils;
import org.geotools.data.DataStore;
import org.geotools.data.DataStoreFinder;
import org.geotools.data.FeatureSource;
import org.geotools.feature.FeatureCollection;
import org.geotools.feature.FeatureIterator;
import org.opengis.feature.simple.SimpleFeature;
import org.opengis.feature.simple.SimpleFeatureType;
import org.opengis.filter.Filter;

import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryCollection;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.io.ParseException;
import com.vividsolutions.jts.io.WKTReader;
import com.vividsolutions.jts.io.WKTWriter;

import it.prato.comune.sit.JSGeometry.GeometryType;
import it.prato.comune.utilita.logging.interfaces.LogInterface;

/**
 * Classe JSGeometryArrayList.
 * 
 * @param <T>
 * 
 * @author Ing. Alessandro Radaelli
 * 
 */
public class JSGeometryArrayList<T extends JSGeometry>  {

    private LogInterface logger;
    private String boundingbox=null;
    private List<T> geometries = new ArrayList<T>();
    
    public enum ImportFileType {
    	SHPZIP
    }
    
    /**
     * Costruttore della classe JSGeometryArrayList.
     * 
     * @param logger
     */
    public JSGeometryArrayList(LogInterface logger) {
        super();
        this.logger = logger;
    }
    
    /**
     * 
     * @return String
     */
    public String getBoundingbox() {
        return boundingbox;
    }
    
    /**
     * 
     * @return
     */
    public List<T> getGeometries() {
        return geometries;
    }
    
    /**
     * Imposta le geometrie.
     * 
     * @param geometries
     * @throws SITException
     */
    public void setGeometries(List<T> geometries) throws SITException {
        this.geometries = geometries;
        
        // //////////////////////////////
        // Calcolo bounding box
        // //////////////////////////////
        
        WKTReader wktReader = new WKTReader();
        
        /*
        for (JSGeometry jsGeom: geometries) {
            Geometry bb;
            if ((jsGeom.getBoundingbox()!=null)&&(!jsGeom.getBoundingbox().equals(""))) {
                try {
                    bb = wktReader.read(jsGeom.getBoundingbox());
                } catch (ParseException e) {
                    logger.warn("Errore nel parsing della stringa WKT in JSGeometryArrayList.setGeometries", e);
                    throw new SITException("Errore nel parsing della stringa WKT in JSGeometryArrayList.setGeometries", e);
                }
                
                
                
                if (boundingBoxBuff==null) boundingBoxBuff = (Geometry) bb.clone();
                else boundingBoxBuff = boundingBoxBuff.union(bb);
            }
        }
        */
        List<Geometry> bblist = new ArrayList<Geometry>();
        for (JSGeometry jsGeom: geometries) {
            if ((jsGeom.getBoundingbox()!=null)&&(!jsGeom.getBoundingbox().equals(""))) {
                try {
                	Geometry bb = wktReader.read(jsGeom.getBoundingbox());
                	bblist.add(bb);
                } catch (ParseException e) {
                    logger.warn("Errore nel parsing della stringa WKT in JSGeometryArrayList.setGeometries", e);
                    throw new SITException("Errore nel parsing della stringa WKT in JSGeometryArrayList.setGeometries", e);
                }
            }
        }
        
        if (bblist.size()>0) {
	        GeometryFactory fact = new GeometryFactory();
	        GeometryCollection bbcoll = fact.createGeometryCollection(bblist.toArray(new Geometry[0]));
	        
	        WKTWriter wktWriter = new WKTWriter();
	        
	        this.boundingbox = wktWriter.write(bbcoll.getEnvelope());
	     
        }
    }
    
    /**
     * 
     * @param oggTerCollection
     * @param toCRS
     * @param logger
     * @return
     * @throws SITException
     * 
     * @see JSGeometryArrayList#toJSGeometryArrayList(List, String, LogInterface, boolean)
     */
    public static JSGeometryArrayList<JSGeometry> toJSGeometryArrayList(List<? extends OggettoTerritorio> oggTerCollection,  String toCRS, LogInterface logger) throws SITException {
        return toJSGeometryArrayList( oggTerCollection, toCRS, logger, true);
            
    }
    
    /**
     * Converte una lista di {@link OggettoTerritorio} in una lista di JSGeometryArrayList.
     * 
     * @param oggTerCollection
     * @param toCRS
     * @param logger
     * @param withGeometry
     * @return
     * @throws SITException
     */
    public static JSGeometryArrayList<JSGeometry> toJSGeometryArrayList(List<? extends OggettoTerritorio> oggTerCollection,  String toCRS, LogInterface logger, boolean withGeometry) throws SITException {
        JSGeometryArrayList<JSGeometry> retVal = new JSGeometryArrayList<JSGeometry>(logger);
        List<JSGeometry> jsgl = JSGeometry.toJSGeometryList(oggTerCollection, toCRS, logger, withGeometry);
        retVal.setGeometries(jsgl);
        
        return retVal;
    }
    
    /**
     * Converte una lista di {@link IGetFeatureInfoObject} in una lista di JSGeometryArrayList. 
     *      
     * @param oggTerCollection
     * @param toCRS
     * @param logger
     * @param withGeometry
     * @return
     * @throws SITException
     */
    public static JSGeometryArrayList<JSGeometry> toJSGeometryArrayList1(List<IGetFeatureInfoObject> oggTerCollection,  String toCRS, LogInterface logger, boolean withGeometry) throws SITException {
        return toJSGeometryArrayList1(oggTerCollection,  toCRS, logger, withGeometry, false);
    }
    
    /**
     * Converte una lista di {@link IGetFeatureInfoObject} in una lista di JSGeometryArrayList. 
     *      
     * @param oggTerCollection
     * @param toCRS
     * @param logger
     * @param withGeometry
     * @param withAttributes
     * @return
     * @throws SITException
     */
    public static JSGeometryArrayList<JSGeometry> toJSGeometryArrayList1(List<IGetFeatureInfoObject> oggTerCollection,  String toCRS, LogInterface logger, 
    		boolean withGeometry, boolean withAttributes) throws SITException {
        JSGeometryArrayList<JSGeometry> retVal = new JSGeometryArrayList<JSGeometry>(logger);
        List<JSGeometry> jsgl = JSGeometry.toJSGeometryList1(oggTerCollection, toCRS, logger, withGeometry, withAttributes);
        retVal.setGeometries(jsgl);
        
        return retVal;
    }
    
    /**
     * Restituisce un JSGeometryArrayList contenente le geometrie contenute nel file in ingresso.
     * 
     * @param file File contenente le geometrie 
     * @param fileType Tipo di file in ingresso
     * @param geomType Tipo di geometria attesa. Se presente anche una sola di tipo diverso viene lanciata l'eccezione SITImportWrongGeometryType
     * @param logger
     * @return
     * @throws SITException
     * @throws IOException
     */
    public static JSGeometryArrayList<JSGeometry> fromImport(final File file, 
    		final ImportFileType fileType, final JSGeometry.GeometryType geomType,
    		final LogInterface logger) throws SITException, IOException{
    	
    	switch (fileType) {
		case SHPZIP:
			return fromImportShpZip(file, geomType, logger);

		default:
			throw new SITException("Tipo di file non supportato");
		}
    	
    }
    
    /**
     * Importazione shapefile (@see {@link JSGeometryArrayList}{@link #fromImport(File, ImportFileType, GeometryType, LogInterface)}
     * 
     * @param file
     * @param geomType
     * @param logger
     * @return
     * @throws SITException
     * @throws IOException
     */
    private static JSGeometryArrayList<JSGeometry> fromImportShpZip(final File file,
    		final JSGeometry.GeometryType geomType,
    		final LogInterface logger) throws SITException, IOException{
    	
    	
		// Apertura zip
		Path outDir = Files.createTempDirectory(null);
		List<File> files = jZip.uncompress_file(file, outDir.toFile(), logger);
		
		File shpFile = null;
		List<JSGeometry> jsGeomList = new ArrayList<JSGeometry>();
	    WKTWriter wkr = new WKTWriter();
	    
		for (File f: files){
			if (FilenameUtils.getExtension(f.getAbsolutePath()).toLowerCase().equals("shp")) {
				shpFile = f;
				// Apertura e lettura con shapefiledatastore
			    Map<String, Object> map = new HashMap<String, Object>();
			    map.put("url", shpFile.toURI().toURL());

			    DataStore dataStore = DataStoreFinder.getDataStore(map);
			    String typeName = dataStore.getTypeNames()[0];

			    FeatureSource<SimpleFeatureType, SimpleFeature> source = dataStore.getFeatureSource(typeName);
			    Filter filter = Filter.INCLUDE;
			    FeatureCollection<SimpleFeatureType, SimpleFeature> collection = source.getFeatures(filter);

			    FeatureIterator<SimpleFeature> features = collection.features();
			    
				try {
			        while (features.hasNext()) {
			            SimpleFeature feature = features.next();
			            Geometry geom = (Geometry) feature.getDefaultGeometry();
			            GeometryType actualGeomType = JSGeometry.getGeometryType(geom);
			            
			            // Verifica tipo geometria
			            if (actualGeomType!=geomType){
			            	throw new SITImportWrongGeometryType();
			            }
			            
			            // Lettura e conversione geometria
			            JSGeometry s = new JSGeometry(logger);
			            s.setGeometry(wkr.write(geom));
			            jsGeomList.add(s);
			            
			        }
			    } finally {
			    	features.close();
			    }
   
			}
		}
		
		JSGeometryArrayList<JSGeometry> jsGeomArr = new JSGeometryArrayList<JSGeometry>(logger);
		jsGeomArr.setGeometries(jsGeomList);
    	return jsGeomArr;
    }
}
