/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
/*
 * Copyright  Comune di Prato - Sistema Informativo
 * Project:    Package SIT per elaborazione/interrogazione/gestione dati cartografici 
 * File:       LayerPoligoniUtenteGenerici.java
 * Author:     Federico Nieri
 * Version:    1.0.0
 * CreationDate:   Febbraio 2008
 * ModifyDate:     
 *
 * Function:   Layer per la gestione dei poligoni utente generici. 
 * 
 *             
 */
package it.prato.comune.sit;

import it.prato.comune.utilita.core.type.TsType;
import it.prato.comune.utilita.logging.interfaces.LogInterface;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Properties;

import org.opengis.feature.simple.SimpleFeature;

/**
 * <p>Per la gestione dei poligoni utente generici sono state implementate una serie di classi e 2 tabelle su DB.
 * Lo scopo era quello di rendere generica la gestione dei poligoni utente e di poter distinguere a quale applicazione tali poligoni 
 * facevano riferimento assegnando loro un identificativo di categoria.
 * In questo modo non si rende necessario, a meno di casi non pi cos generici, il dover creare una nuova tabella per ogni poligono 
 * ed il creare una classe PoligonoPoligonoUtenteXxxx per ognuno di essi.
 * </p>
 * <p>DATABASE</p>
 * <p>Le tabelle create nel DB sono 2:
 * <br>
 * 
 * CATEGORIE_POLIGONI_UTENTE<br>
 * ID      NUMBER(3)   PRIMARY KEY not null <br>
 * DESCRIZIONE DESCRIZIONE nullable <br>
 * <br>
 * POLIGONI_UTENTE<br>
 * ID1 NUMBER(8)   PRIMARY KEY not null
 * IDCATEGORIA   NUMBER(3)   FOREIGN KEY not null
 * ID  VARCHAR2(50)    not null
 * GEOM    SDO_GEOMETRY    
 * <br>
 * La tabella CATEGORIE_POLIGONI_UTENTE  soltanto una tabella di decodifica creata a scopo di promemoria, nel senso che dallapplicazione non viene mai 
 * utilizzata, ma deve essere inserita la riga relativa alla categoria di interesse se si vuole poter inserire dati nella tabella POLIGONI_UTENTE che 
 * rappresenta i poligoni veri e propri.<br>
 * ID_CATEGORIA  infatti la foreign key relativa alla tabella CATEGORIE_POLIGONI_UTENTE.<br>
 * Un esempio di dati inseriti in CATEGORIE_POLIGONI_UTENTE pu essere il seguente.<br>
 * ID  DESCRIZIONE<br>
 *  9  Poligoni Elettorale
 * 10  Poligoni Commercianti
 * 
 * La seconda riga  un esempio che non corrisponde ad un caso reale. La numerazione  stata fatta iniziare da 9 visto che esistevano gi 8 tabelle 
 * relative a poligoni utenti specifici (vedi ad es. POLIGONIUT1) che in seguito potrebbero essere convertiti a questa gestione e quindi potrebbero 
 * andare a riempire i primi 8 ID della tabella.<br>
 * <br>
 * La seconda tabella rispetta quella che era la struttura delle tabelle relative ai poligoni utente gi esistenti, ma con in pi il campo ID_CATEGORIA  
 * con il quale si lega il poligono alla categoria di interesse.<br>
 * Il campo ID1  lidentificativo numerico univoco, lID  invece lidentificativo descrittivo del poligono, univoco allinterno di una categoria. 
 * Un esempio di ID  la data di inserimento del poligono approssimata al millesimo di secondo (es. 24/10/2007 16:15:58:869). <br>
 * Questo  anche lID che viene creato di default dalla classe LayerPoligoniUtenteGenerici qualora lID non sia stato settato esplicitamente 
 * durante linserimento del poligono.<br>
 * <br>
 * Esempio dati in tabella<br>
 * ID1 IDCATEGORIA ID  GEOM<br>
 *  1   9   24/10/2007 16:15:58:869 <Unknown (2.002)
 *  <br>
 *  2   9   24/10/2007 16:23:22:100 <Unknown (2.002)
 *  <br>
 *  <br>
 *  CLASSI<br>
 *  A livello di classi Java, sono state implementate delle classi per la gestione dei LayerTerritorio e per la gestione degli OggettiTerritorio 
 *  dei poligoni utente generici nel package it.prato.comune.sit.<br>
 *  <strong>LayerTerritorio</strong><br>
 *  Esiste una classe astratta che si chiama LayerPoligoniUtenteGenerici, che estende LayerPoligoniUtente e che sovrascrive 4 metodi fondamentali:<br>
 *  <ul>
 *  <li>creaOggetto : quando crea il nuovo OggettoTerritorio relativo, imposta immediatamente lattributo relativo alla categoria di appartenenza del poligono</li>
 *  <li>getFiltroVuoto : questo metodo dovrebbe restituire il filtro vuoto per le interrogazioni su DB ed anche questo metodo  stato sovrascritto in modo 
 *  da impostare immediatamente sul filtro la condizione di uguaglianza per la categoria a cui appartiene il layer. Tale condizione andr in AND con le altre.</li>
 *  <li>initNomiCampi : imposta immediatamente, senza leggere dal file di configurazione, i nomi relativi ai campi comuni sul DB: identificativo 
 *  autoincrementale (ID1), identificativo per categoria (ID) ed identificativo di categoria (IDCATEGORIA). 
 *  Questo per non doverli ripetere ogni volta sul file di configurazione del SIT per ogni poligono utente.</li>
 *  <li>appendFeature : metodo utilizzato per linserimento in tabella dei poligoni. Si assicura di aver impostato lIDCATEGORIA del poligono utente specifico.</li>
 *  </ul>
 *  Esiste poi un metodo di ricerca dei poligoni, comune a tutti questi, che si chiama cercaPoligonoUtente (String ID) e che si preoccupa di impostare 
 *  lIDCATEGORIA corretto per permettere la ricerca per ID il quale  un identificativo univoco allinterno di una specifica categoria.<br>
 *  <strong>OggettoTerritorio</strong><br>
 *  Esiste una classe PoligonoPoligoniUtGenerici che estende PoligonoPoligoniUtente e dove sono stati messi i metodi per impostare e recuperare lIDCATEGORIA.
 *  Non esistono classi specifiche per i singoli poligoni utenti e PoligonoPoligoniUtGenerici  lOggettoTerritorio a cui fanno capo tutti i poligoni generici, 
 *  qualunque sia la categoria a cui appartengono.<br>
 *  <strong>FILE .MAP</strong><br>
 *  Sul file .map che referenzia il layer del poligono utente generico desiderato,  necessario aggiungere un filtro oltre a dare la giusta sorgente dati:<br>
 *  <br>
 *  Esempio<br>
 *  DATA "GEOM FROM POLIGONI_UTENTE USING UNIQUE ID1"<br>
 *  FILTER "IDCATEGORIA=9"<br>
 *  <br>
 *  Questo perch tutti i poligoni utente generici si trovano sulla stessa tabella e se non vengano filtrati il Map Server li visualizzer tutti.<br>
 *  <br>
 *  <strong>IMPLEMENTAZIONE DI UN NUOVO Poligono Utente Generico</strong><br>
 *  <br>
 *  Quando si ha la necessit di creare un nuovo PoligonoUtente i passi da seguire sono i seguenti:<br>
 *  <ul>
 *  <li>Inserire un record nella tabella CATEGORIE_POLIGONI_UTENTE con lid e la descrizione della propria categoria.</li>
 *  <li>Creare una classe LayerPoligoniUtXxxx che estenda LayerPoligoniUtenteGenerici e nella quale si implementa il metodo getIdCategoria 
 *  che deve restituire lidentificativo della categoria inserito in tabella (punto 1)<br>
 *  <br>
 *  Un esempio di classe  la seguente:<br>
 *  <br>
 *  package it.prato.comune.sit;<br>
 *  import it.prato.comune.utilita.logging.interfaces.LogInterface;<br>
 *  import java.util.HashMap;<br>
 *  import org.geotools.feature.Feature;<br>
 *  public class LayerPoligoniUtElettorale extends LayerPoligoniUtenteGenerici {<br>
 *  <br>
 *  public static final Integer ID_CATEGORIA = 9;<br>
 *  <br>
 *  public LayerPoligoniUtElettorale(    String configFilePath, <br>
 *  LogInterface logger,<br>
 *  ConfigBean configBean, <br>
 *  String nome, <br>
 *  int codTPN, <br>
 *  int layer, <br>
 *  HashMap (nomiCampi) {<br>
 *           super(configFilePath, logger, configBean, nome, codTPN, layer, nomiCampi);<br>
 *           <br>
 *           }<br>
 *  public LayerPoligoniUtElettorale() {<br>
 *        super();
 *  }<br>
 *  <br>
 *  protected Integer getIdCategoria() {<br>
 *   return ID_CATEGORIA;<br>
 *   }
 *   <br>
 *   }
 *   <br>
 *   <br>
 *   <br>
 *   </li
 *   <li>Inserire, come per tutti gli altri Layer, i valori relativi nel file di configurazione e nella classe Territorio del package it.prato.comune.sit, 
 *   dove ci sono tutte le inizializzazioni.<br>
 *   <strong>Esempio FILE Config.txt</strong><br>
 *   * Poligoni Utente Elettorale<br>
 *   001POLIGONIUTELECODTPN=-3700<br>
 *   001POLIGONIUTELENOMELAYER=PoligoniUtenteElettorale<br>
 *   001POLIGONIUTELEDSPOOLID=1<br>
 *   001POLIGONIUTELETYPENAME=POLIGONI_UTENTE<br>
 *   <br>
 *   <br>
 *   <strong>Esempio classe Territorio</strong><br>
 *   <br>
 *   public final static int idPoligonoUtElettorale=47;<br>
 *   <br>
 *   // PoligonoUtentiElettorale<br>
 *   initLayers(pr, idPoligonoUtElettorale, "POLIGONIUTELE", new LayerPoligoniUtElettorale());<br>
 *   <br>
 *   public LayerPoligoniUtElettorale getPoligoniUtElettorale() {<br>
 *   return    (LayerPoligoniUtElettorale)getTipologiaDivisioneTerritorio(idPoligonoUtElettorale);<br>
 *   }<br>
 *    </li>
 *    <li>Aggiungere il filtro corretto nel file .map che viene passato al MapServer per costruire limmagine della mappa<br>
 *    <br>
 *    <strong>Esempio .map Elettorale</strong><br>
 *    <br>
 *    DATA "GEOM FROM POLIGONI_UTENTE USING UNIQUE ID1"<br>
 *    FILTER "IDCATEGORIA=9 <br>
 */    

public abstract class LayerPoligoniUtenteGenerici extends LayerPoligoniUtente {
    
   /**
    * Costruttore della classe LayerPoligoniUtenteGenerici.
    * 
 	* @param configFilePath
 	* @param logger
 	* @param configBean
 	* @param nome
 	* @param codTPN
 	* @param layer
 	* @param nomiCampi
 	*/
   public LayerPoligoniUtenteGenerici(String configFilePath, LogInterface logger,
            ConfigBean configBean, String nome, int codTPN, int layer, HashMap<String, String> nomiCampi) {
         super(configFilePath, logger, configBean, nome, codTPN, layer, nomiCampi);        
   }
	
   /**
    * Costruttore di default della classe LayerPoligoniUtenteGenerici.
    */   
   public LayerPoligoniUtenteGenerici() {
       super();
   }
   
   /* 
    * @see it.prato.comune.sit.LayerTerritorio#creaOggetto(org.opengis.feature.simple.SimpleFeature)
 	*/
   protected OggettoTerritorio creaOggetto(SimpleFeature curFeat) {
       
       PoligonoPoligoniUtGenerici ppug = new PoligonoPoligoniUtGenerici(logger, this, curFeat);
       
       try {
           ppug.setIdCategoria(getIdCategoria());
       }catch (SITIllegalAttributeException e){
           logger.error("Impossibile impostare l'id categoria",e);
           throw new RuntimeException("Impossibile impostare l'id categoria");
       }
       
       return ppug;
   }
      
   /* 
    * @see it.prato.comune.sit.LayerTerritorio#getFiltroVuoto()
 	*/
   @Override
   public Filtro getFiltroVuoto() throws SITException {        

       Filtro filtro = new Filtro(super.getFiltroVuoto()){
           @Override
           public void ResetFiltro() {
               super.ResetFiltro();
               try {       
                   this.AndFiltro(ILayerUtente.NL_IDCAT,getIdCategoria(),"=");
               } catch (SITException e) {
                   throw new SITRuntimeException(e);
               }
           };
       };
       
       try {
           filtro.ResetFiltro();
       } catch(SITRuntimeException e){
           if(e.getCause() instanceof SITException){
               throw (SITException)(e.getCause());
           } else {
               throw new SITException(e.getMessage(),e);
           }           
       }
       
       return filtro;    
   }

   /*
 	* @see it.prato.comune.sit.LayerPoligoniUtente#initNomiCampi(java.util.Properties, java.lang.String, java.lang.String)
 	*/
   protected void initNomiCampi(Properties pr, String ente, String nomeLayer) {
       addnomicampi(Layers.NL_IDTPN,Layers.NL_FID);
       addnomicampi(getCampoChiave(),"ID");
       addnomicampi(getCampoCategoria(),"IDCATEGORIA");      
   }
   
   /*
 	* @see it.prato.comune.sit.LayerTerritorio#initRicerche()
 	*/
   protected void initRicerche() throws SITException  {
       addRicerca(1, "Per id poligono", "cercaPoligonoUtente", new Class[]{String.class},"ID");  
   };
   
   /*
    * @see it.prato.comune.sit.LayerPoligoniUtente#cercaPoligonoUtente(java.lang.String)
 	*/
   public PoligonoPoligoniUtente cercaPoligonoUtente (String ID) {
       
       ArrayList valore = new ArrayList();
       ArrayList<String> campi = new ArrayList<String>();        
       campi.add(getCampoChiave());        
       valore.add(ID);
       campi.add(getCampoCategoria());       
       valore.add(getIdCategoria());
       
       return (PoligonoPoligoniUtente) super.cercaPoligono(campi, valore);         
   }

   /*
    * @see it.prato.comune.sit.LayerPoligoniUtente#getCampoChiave()
    */
   protected String getCampoChiave(){       
       return ILayerUtente.NL_ID;
   }
   
   /**
    * Restituisce il campo categortia.
    * 
    * @return String
 	*/
   private String getCampoCategoria(){       
       return ILayerUtente.NL_IDCAT;
   }
   
   /**
    * @return Integer
 	*/
   protected abstract Integer getIdCategoria();

   /* 
 	* @see it.prato.comune.sit.LayerTerritorio#appendFeature(it.prato.comune.sit.OggettoTerritorio, it.prato.comune.sit.SITTransaction)
 	*/
   @Override
   public void appendFeature(OggettoTerritorio oggettoTer,
            SITTransaction transaction) throws IOException, SITException {
        oggettoTer.setAttribute(getNomiCampiScrittura(getCampoCategoria()), getIdCategoria());
        if(oggettoTer.getAttribute(getNomiCampiScrittura(getCampoChiave())) == null){
            TsType ts = new TsType();
            oggettoTer.setAttribute(getNomiCampiScrittura(getCampoChiave()), ts.getFormattedTimeStamp() + ":" + ts.getMillis());
        }

        super.appendFeature(oggettoTer, transaction);
   }   
}
