/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
/*
 * Copyright  Comune di Prato - Sistema Informativo
 * Project:    Package SIT per elaborazione/interrogazione/gestione dati cartografici 
 * File:       LayerPuntiTerritorio.java
 * Author:     Alessandro Radaelli
 * Version:    1.0.0
 * CreationDate:   Giugno 2005
 * ModifyDate:     
 *
 * Function:   Layer astratto contenente punti sul territorio 
 * 
 *             
 */
package it.prato.comune.sit;


import it.prato.comune.utilita.logging.interfaces.LogInterface;

import java.util.HashMap;

import org.geotools.geometry.DirectPosition2D;
import org.geotools.referencing.CRS;
import org.opengis.geometry.DirectPosition;
import org.opengis.geometry.MismatchedDimensionException;
import org.opengis.referencing.FactoryException;
import org.opengis.referencing.NoSuchAuthorityCodeException;
import org.opengis.referencing.crs.CoordinateReferenceSystem;
import org.opengis.referencing.operation.MathTransform;
import org.opengis.referencing.operation.TransformException;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.Point;





/**
 * Classe LayerPuntiTerritorioStorico.
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public abstract class LayerPuntiTerritorioStorico extends LayerTerritorioStorico implements ILayerPuntiTerritorio{

    /**
     * Costruttore della classe LayerPuntiTerritorioStorico.
     * 
     * @param configFilePath
     * @param logger
     * @param configBean
     * @param nome
     * @param codTPN
     * @param layer
     * @param nomicampi
     */
    public LayerPuntiTerritorioStorico(String configFilePath, 
            LogInterface logger,  
            ConfigBean configBean,
            String nome, int codTPN, int layer, HashMap<String, String> nomicampi) {
        
        super(configFilePath, 
                logger, 
                configBean, nome, codTPN, layer, nomicampi);        
    } 
    
    /**
     * Costruttore di default della classe LayerPuntiTerritorioStorico.
     */
    public LayerPuntiTerritorioStorico(){
        super();
    }
    
    // [regionstart] Funzioni static

   /**
    * Crea un punto nel sistema di coordinate identificato da targetSRID utilizzando le coordinate passate nel sistema +
    * di riferimento identificato da sourceSRID
    * 
    * @param x coordinata x, espressa nel sistema di riferimento sourceSRID
    * @param y coordinata y, espressa nel sistema di riferimento sourceSRID
    * @param sourceCRS Sistema di riferimento nel quale sono espresse le coordinate x e y
    * @param targetCRS Sistema di riferimento nel quale saranno espresse le coordinate del punto creato
    * @return Ritorna il punto creato
    * @throws SITException
    */
   static protected Point creaPoint (double x, double y, CoordinateReferenceSystem sourceCRS, CoordinateReferenceSystem targetCRS) throws SITException {
       DirectPosition pt1 = new DirectPosition2D(new Double(x),new Double(y) );         
       DirectPosition pt2 = null;
       
       try {
//TOB
//           MathTransform math = CRS.transform(sourceCRS, targetCRS);
//           pt2 = math.transform( pt1, null );
    	   
			MathTransform math = CRS.findMathTransform(sourceCRS, targetCRS, true);			
			pt2 = math.transform( pt1, null );

       } catch (MismatchedDimensionException e) {
           throw new SITException("creaPoint-MismatchedDimensionException ", e);
       } catch (FactoryException e) {
           throw new SITException("creaPoint-FactoryException ", e);
       } catch (TransformException e) {
           throw new SITException("creaPoint-TransformException ", e);
       }
       
       GeometryFactory geomFac = new GeometryFactory();
//TOB       
//       return geomFac.createPoint(new Coordinate(pt2.getCoordinates()[0], pt2.getCoordinates()[1]));
       
       double[] pt2_coordinates = pt2.getCoordinate();
       return geomFac.createPoint(new Coordinate(pt2_coordinates[0], pt2_coordinates[1]));
       
   }
    
    /**
     * Crea un punto nel sistema di coordinate identificato da targetSRID utilizzando le coordinate passate nel sistema +
     * di riferimento identificato da sourceSRID
     * 
     * @param x coordinata x, espressa nel sistema di riferimento sourceSRID
     * @param y coordinata y, espressa nel sistema di riferimento sourceSRID
     * @param sourceSRID Sistema di riferimento nel quale sono espresse le coordinate x e y (per es. epsg:4326)
     * @param targetSRID Sistema di riferimento nel quale saranno espresse le coordinate del punto creato (per es. epsg:4326)
     * @return Ritorna il punto creato
     * @throws SITException
     */
    static protected Point creaPoint (double x, double y, String sourceSRID, String targetSRID) throws SITException {      
        CoordinateReferenceSystem sourceCRS;
        CoordinateReferenceSystem targetCRS;
        
        try {
            sourceCRS = CRS.decode(sourceSRID,true);
            targetCRS = CRS.decode(targetSRID,true);
        } catch (NoSuchAuthorityCodeException e) {
            throw new SITException("creaPoint-NoSuchAuthorityCodeException ", e);
        } catch (FactoryException exc) {
        	throw new SITException("creaPoint-FactoryException ", exc);
		}

        return creaPoint(x,y,sourceCRS,targetCRS );
    }
    
    /**
     * Crea un punto nel sistema di coordinate identificato da targetSRID utilizzando le coordinate passate nel sistema +
     * di riferimento identificato da sourceSRID
     * 
     * @param x coordinata x, espressa nel sistema di riferimento sourceSRID
     * @param y coordinata y, espressa nel sistema di riferimento sourceSRID
     * @param sourceSRID Sistema di riferimento nel quale sono espresse le coordinate x e y (per es. epsg:4326)
     * @param targetCRS Sistema di riferimento nel quale saranno espresse le coordinate del punto creato
     * @return Ritorna il punto creato
     * @throws SITException
     */
    static protected Point creaPoint (double x, double y, String sourceSRID, CoordinateReferenceSystem targetCRS) throws SITException {      
        CoordinateReferenceSystem sourceCRS;
        
        try {
            sourceCRS = CRS.decode(sourceSRID,true);
        } catch (NoSuchAuthorityCodeException e) {
            throw new SITException("creaPoint-NoSuchAuthorityCodeException ", e);
        } catch (FactoryException exc) {
        	throw new SITException("creaPoint-FactoryException ", exc);
		}

        return creaPoint(x,y,sourceCRS,targetCRS );
    }
 
    /**
     * Crea un com.vividsolutions.jts.geom.Point.
     * @param x
     * @param y
     * @return Point
     */
    static protected Point creaPoint (double x, double y) {        
        GeometryFactory geomFac = new GeometryFactory();		
		return geomFac.createPoint(new Coordinate(x, y));        
    }
    
    /**
     * Calcola la distanza tra due punti
     * 
     * @param x1 - Coordinata x del primo punto
     * @param y1 - Coordinata y del primo punto
     * @param x2 - Coordinate x del secondo punto
     * @param y2 - Coordinate y del secondo punto
     * @return double
     */
    static double Distanza(double x1, double y1, double x2, double y2) {
        
    	// ////////////////////////////////////////
        // TODOH Fissare sistema di riferimento
    	// ////////////////////////////////////////
        
		Point punto1 = creaPoint(x1,y1);
		Point punto2 = creaPoint(x2,y2);
		
		return Distanza(punto1, punto2);        
    }
 
    /**
     * Calcola la distanza tra due punti.
     * 
     * @param p1
     * @param p2
     * @return double
     */
    static double Distanza(Point p1, Point p2) {
        return p1.distance(p2);        
    }
       
    /*
     * @see it.prato.comune.sit.LayerTerritorioStorico#getContentTypeLayer()
     */
    public int getContentTypeLayer(){
       return LAYER_OF_POINTS;
    }
}
