/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/

package it.prato.comune.sit;

import it.prato.comune.utilita.logging.interfaces.LogInterface;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.geotools.feature.simple.SimpleFeatureBuilder;
import org.geotools.feature.simple.SimpleFeatureTypeBuilder;
import org.geotools.referencing.CRS;
import org.geotools.referencing.crs.DefaultGeographicCRS;
import org.opengis.feature.simple.SimpleFeature;
import org.opengis.feature.simple.SimpleFeatureType;
import org.opengis.referencing.FactoryException;
import org.opengis.referencing.NoSuchAuthorityCodeException;
import org.opengis.referencing.crs.CoordinateReferenceSystem;

import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.io.ParseException;
import com.vividsolutions.jts.io.WKTReader;
import com.vividsolutions.jts.io.WKTWriter;

/**
 * Implementazione ricerca tramite servizio Nominatim (http://nominatim.openstreetmap.org/).
 * Esempio di configurazione Spring:
 * 
 * <pre>
 * {@code
 * <bean id="wktgeom3003" class="java.lang.String">
		<constructor-arg type="java.lang.String">
			<value>POLYGON ((1660445.6 4863959.77, 1675751.56 4862948.37, 1662266.14 4854637.97, 1660445.6 4863959.77))</value>
		</constructor-arg>
	</bean>

	<bean id="wktgeom4326" class="java.lang.String">
		<constructor-arg type="java.lang.String">
			<value>POLYGON ((10.8445989 43.9740513, 11.3347684 43.8887781, 10.8525905 43.7955161, 10.8445989 43.9740513))</value>
		</constructor-arg>
	</bean>

	<bean id="ricerchenominatim" class="it.prato.comune.sit.LayerTerritorioNominatim">
		<constructor-arg value="-15080" />  		   <!-- codTPN -->
		<constructor-arg value="Ricerche nominatim" />  <!-- nome -->
		<constructor-arg ref="logger1" />		   <!-- logger -->
		<property name="tipoLayer">           <!-- tipoLayer valori possibili (default poligono) it.prato.comune.sit.Layers.LAYER_OF_POINTS 
			it.prato.comune.sit.Layers.LAYER_OF_LINES it.prato.comune.sit.Layers.LAYER_OF_POLYGONS -->
			<util:constant static-field="it.prato.comune.sit.Layers.LAYER_OF_POLYGONS"/> 
		</property >
		<property name="countryCodes" value="it,de" />
		<property name="viewbox" value="10.8820852,43.9938559,11.7216801,43.5232622" />
		<property name="includeRE" >		
			<util:map map-class="java.util.HashMap" > 
				<entry key="display_name">
				     <array>
					<value>(.*)FI(.*)</value>
					<value>(.*)vecchio(.*)</value>
					<value>(.*)PO(.*)</value>
				     </array>
				</entry>
			</util:map>
		</property>
<!--
		<property name="filterGeom">
		    <bean class="it.prato.comune.sit.GeomIntersecabileDefault">
			<constructor-arg ref="ricerche_wfs_comuni2"/>
			<constructor-arg value="100005" />
			<constructor-arg ref="logger1" />
	   	     </bean>
		</property>
-->
		<property name="filterGeom">
		    <bean class="it.prato.comune.sit.GeomIntersecabileWKT">
			<constructor-arg ref="wktgeom4326"/>
			<constructor-arg value="EPSG:4326" />
			<constructor-arg ref="logger1" />
	   	     </bean>
		</property>


		<!--<property name="excludeRE" >		
			<util:map map-class="java.util.HashMap" > 
				<entry key="display_name">
				     <array>
					<value>(.*)FI(.*)</value>
				     </array>
				</entry>
			</util:map>
		</property>-->


	</bean>
	}
 * </pre>
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public class LayerTerritorioNominatim extends LayerTerritorioGenerico {

	/** Url del servizio. */
	private String urlServizio  = "http://nominatim.openstreetmap.org/search";
	/** Codici nazione nei quali effettuare la ricerca. */
	private String countryCodes = "it";
	/** Box geografico sul quale filtrare il risultato espresso in WGS84 (EPSG:4326) */
	private String viewbox = null;
	/** Regular expression per controllare quali risposte includere. */
	private Map<String, List<String>> includeRE = null;
	/** Regular expression per controllare quali risposte escludere. */
	private Map<String, List<String>> excludeRE = null;
	/** Geometria di filtro: nella risposta sar incluso solo ci che interseca questa geometria. */
	private IGeomIntersecabile filterGeom = null;
	
	/**
	 * Costruttore.
	 * 
	 * @param codTPN Identificativo del layer
	 * @param nome Nome del layer
	 * @param logger 
	 * @throws SITException
	 */
	public LayerTerritorioNominatim(int codTPN, String nome, LogInterface logger) throws SITException {
		super(codTPN, nome, logger);
		initRicerche();
		
	}

	@Override
    public final OggettoTerritorio creaNuovoOggettoTerritorio() 
    throws IOException, SITException {
		
		// Creare SimpleFeatureType come richiesto
		// senza utilizzare datastore
		
		SimpleFeatureBuilder featureBuilder = new SimpleFeatureBuilder( createFeatureType());
		 /* Longitude (= x coord) first ! */
         Point point = null; //geometryFactory.createPoint(new Coordinate(longitude, latitude));
         featureBuilder.add(point);
         featureBuilder.add(null);
         featureBuilder.add(null);
         SimpleFeature feature = featureBuilder.buildFeature(null);
		 
         OggettoTerritorio ogg = new OggettoTerritorioGenerico(logger,this,feature){
 
			private static final long serialVersionUID = 1L;

			@Override
        	public String getDescrizione() {
        		return (String) getAttributeByNL("NL_DESC");
        	} 
         };
		 
    	return ogg;
    }
	
	/**
	 Crea una featuretype contenente i campi necessari.
	 */
	private static SimpleFeatureType createFeatureType() {

	        SimpleFeatureTypeBuilder builder = new SimpleFeatureTypeBuilder();
	        builder.setName("layerosm");
	        builder.setCRS(DefaultGeographicCRS.WGS84); // <- Coordinate reference system

	        // add attributes in order
	        builder.add("geom", Geometry.class);
	        builder.length(100).add("osm_id", String.class);
	        builder.length(100).add("desc", String.class);
	        builder.add("rawjson", JSONObject.class);
	        
	        // build the type
	        final SimpleFeatureType LOCATION = builder.buildFeatureType();

	        return LOCATION;
	}
	
	@Override
	public final String getSRID() throws SITException {
		return "EPSG:4326";
	};
	
	@Override
	protected final CoordinateReferenceSystem getCRS() throws SITException {
		
		CoordinateReferenceSystem crs = null;
    	try {
	        crs = CRS.decode("EPSG:4326",true);
        } catch (NoSuchAuthorityCodeException e) {
            throw new SITException("NoSuchAuthorityCodeException durante decodifica CRS", e);
        } catch (FactoryException e) {
            throw new SITException("FactoryException durante decodifica CRS", e);
        }

        return crs;
	};
	
	@Override
	public final void setDSPool(final SITDataStorePool dsPool) {
		
		// Creo un ds pool apposito contenete un unico datastore fittizio con dsCaseSensitive=L 
		try {
			SITDataStorePool dsP = new SITDataStorePool("", logger);
			dsP.setParam("1", "", "it.prato.comune.sit.NullDatastoreFactory", null, "L");
			configBean.setDsPool(dsP);
			configBean.setDsPoolID("1");
		} catch (IOException e) {
			throw new SITRuntimeException(e);
		}
		
		setElencoCampi(new HashMap<String, String>());
		addnomicampi(Layers.NL_IDTPN, "osm_id");
		addnomicampi("NL_DESC", "desc");
		addnomicampi("NL_RAWJSON", "rawjson");
		
	}
 
	 /* 
     * @see it.prato.comune.sit.LayerTerritorio#initRicerche()
     */
    @Override
    protected final void initRicerche() throws SITException  {
    	logger.info("Inizio inizializzazione ricerche per LayerTerritorioNominatim");
    	addRicerca(1, "Per localit", "cercaLocalita",new Class[]{String.class}, "Localit&agrave;");
        addRicerca(2, "Per via", "cercaVia",new Class[]{String.class, String.class}, "Citt&agrave;", "Via");
        addRicerca(3, "Per via e civico", "cercaViaCivico",new Class[]{String.class, String.class, String.class}, "Citt&agrave;", "Via", "Civico");
        addRicerca(4, "Per via, civico e CAP", "cercaViaCivicoCAP",new Class[]{String.class, String.class, String.class, String.class}, "Citt&agrave;", "Via", "Civico", "CAP");
        addRicerca(5, "Per CAP", "cercaCAP",new Class[]{String.class}, "CAP");
        
        logger.info("Fine inizializzazione ricerche per LayerTerritorioNominatim");
    };
    
    /**
     * Metodo che implementa la ricerca per citt.
     * @param citta citt da ricercare
     * @return Lista di risultati
     * @throws SITException
     */
    public final List<OggettoTerritorio> cercaLocalita(final String citta) throws SITException {
    	return cercaViaCivicoCAP(citta, null, null, null);
    }
    
    /**
     * Metodo che implementa la ricerca per citt.
     * 
     * @param cap CAP da ricercare.
     * @return Lista di risultati
     * @throws SITException
     */
    public final List<OggettoTerritorio> cercaCAP(final String cap) throws SITException {
    	return cercaViaCivicoCAP(null, null, null, cap);
    }
    
    /**
     * Metodo che implementa la ricerca per citt e via.
     * @param citta citt da ricercare
     * @param via via da ricercare
     * @return Lista di risultati
     * @throws SITException
     */
    public final List<OggettoTerritorio> cercaVia(final String citta, final String via) throws SITException {
    	return cercaViaCivicoCAP(citta, via, null, null);
    }
    
    /**
     * 
     * Metodo che implementa la ricerca per citt e via.
     * 
     * @param citta citt da ricercare
     * @param via via da ricercare
     * @param civico civico da ricercare
     * @return Lista di risultati
     * @throws SITException
     */
    public final List<OggettoTerritorio> cercaViaCivico(final String citta, final String via, final String civico) throws SITException {
    	return cercaViaCivicoCAP(citta, via, civico, null);
    }
    
    /**
     * Metodo generale di ricerca. 
     * @param citta Citt da ricercare. Se non si vuole mettere una condizione su questo campo utilizzare null
     * @param via Via da ricercare. Se non si vuole mettere una condizione su questo campo utilizzare null
     * @param civico Civico da ricercare. Se non si vuole mettere una condizione su questo campo utilizzare null
     * @param cap CAP da ricercare. Se non si vuole mettere una condizione su questo campo utilizzare null
     * @return
     * @throws SITException
     */
    public final List<OggettoTerritorio> cercaViaCivicoCAP(final String citta, final String via, final String civico, final String cap) throws SITException {
    	
    	List<OggettoTerritorio> retVal = new ArrayList<OggettoTerritorio>();
    	
		try {
			String url = getUrlServizio() + "?";
			String  args = "polygon_text=1";
			args += "&addressdetails=1";
			args += "&format=json";			
			if (countryCodes!=null && !countryCodes.equals("")) {
				args += "&countrycodes=" + countryCodes;
			}
			if (viewbox!=null && !viewbox.equals("")) {
				args += "&viewbox=" + viewbox;
				args += "&bounded=1";
			}
			
			if (via!=null) {
				args += "&street=" +
						((civico!=null) ? URLEncoder.encode(civico, "UTF-8")+"%20" :"" ) +
							URLEncoder.encode(via, "UTF-8"); // <housenumber> <streetname>
			}
			if (citta!=null) {
				args += "&city=" + URLEncoder.encode(citta, "UTF-8");
			}
			if (cap!=null) {
				args += "&postalcode=" + URLEncoder.encode(cap, "UTF-8");
			}
			
			url += args;
			logger.debug("Ricerca su Nominatim: " + url);
			
			URL url1 = new URL(url);
			HttpURLConnection connection = (HttpURLConnection) url1.openConnection();
			// Setto Referer per policy Nominatim
			connection.setRequestProperty("Referer", "http://www.droneplanner.it");
			BufferedReader read = new BufferedReader(new InputStreamReader(connection.getInputStream()));
			String line = read.readLine();
			String html = "";
			while (line != null) {
				html += line;
				line = read.readLine();
			}
			
			logger.debug("Risultato ricerca su Nominatim: " + html);
			
			JSONArray resJson = JSONArray.fromObject(html);
			
			for(Object obj: resJson) {
				JSONObject jsonobj = (JSONObject) obj;
				String displayname = (String) jsonobj.get("display_name");
				String osm_id = (String) jsonobj.get("osm_id");
				String geomWKT = (String) jsonobj.get("geotext");
				String lat = (String) jsonobj.get("lat");
				String lon = (String) jsonobj.get("lon");
				
				
				OggettoTerritorio ogg = creaNuovoOggettoTerritorio();
				ogg.setIDTPN(osm_id);
				ogg.setAttributeByNL("NL_DESC", displayname);
				ogg.setAttributeByNL("NL_RAWJSON", jsonobj);
				
				if (geomWKT!=null && !geomWKT.equals("")) {
			        WKTReader wktReader = new WKTReader();   
	                String wkt1;
			        try {
	                    Geometry bb = wktReader.read(geomWKT);
	                    Geometry bb1 = bb.buffer(0.000005);
	                    WKTWriter wktw = new WKTWriter();
	                    wkt1 = wktw.write(bb1);
	                } catch (ParseException e) {
	                    logger.warn("Errore nel parsing della stringa WKT in JSGeometryArrayList.setGeometries", e);
	                    throw new SITException("Errore nel parsing della stringa WKT in JSGeometryArrayList.setGeometries", e);
	                }
			        
					ogg.setGeometryAttributeWKT(wkt1);
				} else {
					double l = 0.001;
					double dlon = Double.parseDouble(lon);
					double dlat = Double.parseDouble(lat);
					String bd = "" + (dlon+l) + " " + (dlat-l) ;
					String ad = "" + (dlon+l) + " " + (dlat+l);
					String bs = "" + (dlon-l) + " " + (dlat-l);
					String as = "" + (dlon-l) + " " + (dlat+l);
					String v = bd + "," + bs + "," + as + "," + ad + "," + bd;
					String a = "POLYGON((" + v + "))";
					//ogg.setGeometryAttributeWKT("POINT ("+lon + " "+lat+")");
					ogg.setGeometryAttributeWKT(a);
				}
				retVal.add(ogg);
			}
			
			localFilter(retVal);
			postFilter(retVal);
			
			return retVal;
			
		} catch (MalformedURLException e) {
			String messaggio = "Errore in cercaVia di LayerTerritorioNominatim";
			throw new SITException(messaggio, e);
		} catch (IOException e) {
			String messaggio = "Errore in cercaVia di LayerTerritorioNominatim";
			throw new SITException(messaggio, e);
		} 
    	
    }
    
    /**
     * Metodo di utilit per verificare una condizione sotto forma di regular expression.
     * @param jsonobj Oggetto json contenete le informazioni
     * @param mapRE Mappa contenete le regular expression da verificare. La chiave della mappa  il nome dell'attributo json da testare, mentre il valore  la regular expression.
     * 
     * @return True se la condizione  verificata
     */
    private static boolean verificaCondizione(final JSONObject jsonobj, final Map<String, List<String>> mapRE) {
    	
    	// Verifica condizione include
		boolean bInclude = false;

		// Loop sulla mappa (campi da testare)
		for (String incKey: mapRE.keySet()) {
			List<String> incValueList = mapRE.get(incKey);
			Object value;
			if (incKey.contains(".")) {
				String [] s = incKey.split("\\.");
				JSONObject l1    = (JSONObject) jsonobj.get(s[0]);
				value  = l1.get(s[1]);
			} else {
				value = (Object) jsonobj.get(incKey);
			}
			// Condizioni da testare
			for (String incValue: incValueList) {
				bInclude = Pattern.matches(incValue, value.toString());
				// Se la condizione  soddisfatta esce (sono in or)
				if (bInclude) return true;
			}
		}
		return false;    	

    }
    
    /**
     * Metodo che implementa il filtraggio locale del risultato nminatim
     * @param oggList Lista degli oggetti trovati
     */
    private void localFilter(final List<OggettoTerritorio> oggList) {
    	
    	for (Iterator<OggettoTerritorio> iterator = oggList.iterator(); iterator.hasNext();) {
    		OggettoTerritorio ogg = iterator.next();
    		
			JSONObject jsonobj = (JSONObject) ogg.getAttributeByNL("NL_RAWJSON");
			//JSONObject addr    = (JSONObject) jsonobj.get("address");
			
			if ((includeRE != null) && includeRE.size()>0) {
				if (!verificaCondizione(jsonobj, includeRE)) {
					logger.info(ogg.getAttributeByNL("NL_DESC") + " eliminato dalla lista dei risultati per condizioni di include." );
					iterator.remove();
					continue;
				}
			}
			
			if ((excludeRE != null) && excludeRE.size()>0) {
				if (verificaCondizione(jsonobj, excludeRE)) {
					logger.info(ogg.getAttributeByNL("NL_DESC") + " eliminato dalla lista dei risultati per condizioni di include." );
					iterator.remove();
					continue;
				}
			}
			
			if (filterGeom != null && !filterGeom.interseca(ogg)) {
    	        // Remove the current element from the iterator and the list.
    	        iterator.remove();
    	        continue;
    	    }
    	}
    	
    }
    
    /** 
     * Metodo da sovrascrivere se su vuole effettuare un filtraggio ulteriore
     * @param oggList
     */
    public final void postFilter(final List<OggettoTerritorio> oggList){
    	
    }
   
	/**
	 * Recupera urlServizio.
	 *
	 * @return urlServizio
	 */
	public final String getUrlServizio() {
		return urlServizio;
	}

	/**
	 * Imposta il valore di urlServizio.
	 * @param urlServizio valore di urlServizio da impostare
	 */
	public final void setUrlServizio(final String urlServizio) {
		this.urlServizio = urlServizio;
	}

	/**
	 * Recupera Codici nazione nei quali effettuare la ricerca.
	 *
	 * @return countryCodes
	 */
	public final String getCountryCodes() {
		return countryCodes;
	}

	/**
	 * Imposta i Codici nazione nei quali effettuare la ricerca..
	 * @param countryCodes valore di countryCodes da impostare
	 */
	public final void setCountryCodes(final String countryCodes) {
		this.countryCodes = countryCodes;
	}

	/**
	 * Recupera il Box geografico sul quale filtrare il risultato espresso in WGS84 (EPSG:4326) .
	 *
	 * @return viewbox
	 */
	public final String getViewbox() {
		return viewbox;
	}

	/**
	 * Imposta il valore del Box geografico sul quale filtrare il risultato espresso in WGS84 (EPSG:4326) .
	 * @param viewbox valore di viewbox da impostare
	 */
	public final void setViewbox(final String viewbox) {
		this.viewbox = viewbox;
	}

	/**
	 * Recupera  il valore delle Regular expression per controllare quali risposte includere. 
	 * La chiave della mappa  il nome dell'attributo json da testare, mentre il valore  la regular expression..
	 *
	 * @return includeRE
	 */
	public final Map<String, List<String>> getIncludeRE() {
		return includeRE;
	}

	/**
	 * Imposta il valore delle Regular expression per controllare quali risposte includere. 
	 * La chiave della mappa  il nome dell'attributo json da testare, mentre il valore  la regular expression.
	 * @param includeRE valore di includeRE da impostare
	 */
	public final void setIncludeRE(final Map<String, List<String>> includeRE) {
		this.includeRE = includeRE;
	}

	/**
	 * Recupera  il valore delle Regular expression per controllare quali risposte escludere. 
	 * La chiave della mappa  il nome dell'attributo json da testare, mentre il valore  la regular expression..
	 *
	 * @return excludeRE
	 */
	public final Map<String, List<String>> getExcludeRE() {
		return excludeRE;
	}

	/**
	 * Imposta il valore delle Regular expression per controllare quali risposte escludere. 
	 * La chiave della mappa  il nome dell'attributo json da testare, mentre il valore  la regular expression..
	 * @param excludeRE valore di excludeRE da impostare
	 */
	public final void setExcludeRE(final Map<String, List<String>> excludeRE) {
		this.excludeRE = excludeRE;
	}

	/**
	 * Recupera la geometria di filtro: nella risposta sar incluso solo ci che interseca questa geometria..
	 *
	 * @return filterGeom
	 */
	public final IGeomIntersecabile getFilterGeom() {
		return filterGeom;
	}

	/**
	 * Imposta il valore della geometria di filtro: nella risposta sar incluso solo ci che interseca questa geometria.
	 * @param filterGeom valore di filterGeom da impostare
	 */
	public final void setFilterGeom(final IGeomIntersecabile filterGeom) {
		this.filterGeom = filterGeom;
	}

	
}
