/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
/*
 * Copyright  Comune di Prato - Sistema Informativo
 * Project:    Package SIT per elaborazione/interrogazione/gestione dati cartografici 
 * File:       LayerTerritorioStorico.java
 * Author:     Alessandro Radaelli
 * Version:    1.0.0
 * CreationDate:  
 * ModifyDate:     
 *             
 */
package it.prato.comune.sit;

import it.prato.comune.utilita.core.beans.BasicException;
import it.prato.comune.utilita.core.type.DateType;
import it.prato.comune.utilita.core.type.EventType;
import it.prato.comune.utilita.core.type.IdType;
import it.prato.comune.utilita.core.type.ProgvalType;
import it.prato.comune.utilita.logging.interfaces.LogInterface;

import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;

import org.opengis.feature.simple.SimpleFeature;
import org.opengis.filter.spatial.BinarySpatialOperator;

/**
 * Clase  LayerTerritorioStorico.
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public abstract class LayerTerritorioStorico extends LayerTerritorio {

    /**
     * Costruttore della classe LayerTerritorioStorico.
     * 
     * @param configFilePath
     * @param logger
     * @param configBean
     * @param nome
     * @param codTPN
     * @param layer
     * @param nomiCampi
     */
    public LayerTerritorioStorico(String configFilePath, LogInterface logger,
            ConfigBean configBean, String nome, int codTPN, int layer,
            HashMap<String, String> nomiCampi) {
        super(configFilePath, logger, configBean, nome, codTPN, layer,
                nomiCampi);           
    }
        
    
    /**
     * Costruttore di dafault della classe LayerTerritorioStorico.
     */
    public LayerTerritorioStorico() {
    
    }    
    
    /*
     * @see it.prato.comune.sit.LayerTerritorio#initNomiCampiFull(java.util.Properties, java.lang.String, java.lang.String)
     */
    protected void initNomiCampiFull(Properties pr, String ente, String nomeLayer) {
        super.initNomiCampiFull(pr, ente, nomeLayer);
        addnomicampi(Layers.NL_ENTESTORICO, "ENTE");
        addnomicampi(Layers.NL_PROGIVAL, "PROGIVAL");
        addnomicampi(Layers.NL_PROGFIVAL, "PROGFIVAL");
        addnomicampi(Layers.NL_DTIVAL, "DTIVAL");
        addnomicampi(Layers.NL_DTFIVAL, "DTFIVAL");
        addnomicampi(Layers.NL_FLERR, "FLERR");  
    }
    
    /*
     * @see it.prato.comune.sit.LayerTerritorio#appendFeature(it.prato.comune.sit.OggettoTerritorio)
     */
    public void appendFeature(OggettoTerritorio oggettoTer) 
    throws IOException, SITIllegalAttributeException, SITException { 
    	// ///////////////////////////////////////////////////////
        // TODO controllo che sia OggettoTerritorioStorico
        // TODO controllo che siano presenti user e ipaddress
    	// ///////////////////////////////////////////////////////
        appendFeature(configBean.getLayersManager().getUser(),
                        configBean.getLayersManager().getIpAddress(),
                        (OggettoTerritorioStorico) oggettoTer, new DateType());
    }
    
    /* 
     * @see it.prato.comune.sit.LayerTerritorio#modifyFeature(it.prato.comune.sit.OggettoTerritorio)
     */
    public void modifyFeature(OggettoTerritorio oggettoTer) throws IOException, SITException, SQLException {
    	// ///////////////////////////////////////////////////////
        // TODO controllo che sia OggettoTerritorioStorico
        // TODO controllo che siano presenti user e ipaddress
    	// ///////////////////////////////////////////////////////
        modifyFeature(configBean.getLayersManager().getUser(),
                        configBean.getLayersManager().getIpAddress(),
                        (OggettoTerritorioStorico) oggettoTer, new DateType());
    }

    /*
     * @see it.prato.comune.sit.LayerTerritorio#removeFeature(it.prato.comune.sit.OggettoTerritorio)
     */
    public void removeFeature(OggettoTerritorio oggettoTer) throws IOException, SITException, SQLException {
    	// ////////////////////////////////////////////////////////
        // TODO controllo che sia OggettoTerritorioStorico
        // TODO controllo che siano presenti user e ipaddress
    	// ////////////////////////////////////////////////////////
        removeFeature(configBean.getLayersManager().getUser(),
                        configBean.getLayersManager().getIpAddress(),
                        (OggettoTerritorioStorico) oggettoTer);
    }

    /*
     * @see it.prato.comune.sit.LayerTerritorio#appendFeature(it.prato.comune.sit.OggettoTerritorio, it.prato.comune.sit.SITTransaction)
     */
    public void appendFeature(OggettoTerritorio oggettoTer, SITTransaction transaction) throws IOException, SITException {
    	// /////////////////////////////////////////////////////////
        // TODO controllo che sia OggettoTerritorioStorico
        // TODO controllo che siano presenti user e ipaddress
    	// /////////////////////////////////////////////////////////
        appendFeature(configBean.getLayersManager().getUser(),
                        configBean.getLayersManager().getIpAddress(),
                        (OggettoTerritorioStorico) oggettoTer, new DateType(), transaction);
    }

    /*
     * @see it.prato.comune.sit.LayerTerritorio#modifyFeature(it.prato.comune.sit.OggettoTerritorio, it.prato.comune.sit.SITTransaction)
     */
    public void modifyFeature(OggettoTerritorio oggettoTer, SITTransaction transaction) throws IOException, SITException, SQLException {
    	// ////////////////////////////////////////////////////////
        // TODO controllo che sia OggettoTerritorioStorico
        // TODO controllo che siano presenti user e ipaddress
    	// ////////////////////////////////////////////////////////
        modifyFeature(configBean.getLayersManager().getUser(),
                        configBean.getLayersManager().getIpAddress(),
                        (OggettoTerritorioStorico) oggettoTer, new DateType(), transaction);
    }

    /* 
     * @see it.prato.comune.sit.LayerTerritorio#removeFeature(it.prato.comune.sit.OggettoTerritorio, it.prato.comune.sit.SITTransaction)
     */
    public void removeFeature(OggettoTerritorio oggettoTer, SITTransaction transaction) throws IOException, SITException, SQLException {
    	// //////////////////////////////////////////////////////
        // TODO controllo che sia OggettoTerritorioStorico
        // TODO controllo che siano presenti user e ipaddress
    	// //////////////////////////////////////////////////////
        removeFeature(configBean.getLayersManager().getUser(),
                        configBean.getLayersManager().getIpAddress(),
                        (OggettoTerritorioStorico) oggettoTer, transaction);
    }

    /**
     *  
     * @param user
     * @param IPAddress
     * @param oggettoTer
     * @param dataInserimento
     * @throws IOException
     * @throws SITException
     * 
     * @see LayerTerritorioStorico#appendFeature(String, String, OggettoTerritorioStorico, DateType, SITTransaction)}
     */
    public void appendFeature(String user, String IPAddress, 
            OggettoTerritorioStorico oggettoTer, DateType dataInserimento) throws IOException, SITException {
        appendFeature(user, IPAddress, oggettoTer, dataInserimento, null);        
    }
    
    /**
     * Inserisce una nuova feature.
     * 
     * @param user
     * @param IPAddress
     * @param oggettoTer
     * @param dataInserimento
     * @param transaction
     * @throws IOException
     * @throws SITException
     */
    public void appendFeature(String user, String IPAddress, 
            OggettoTerritorioStorico oggettoTer, DateType dataInserimento,
            SITTransaction transaction) throws IOException, SITException {
        
        ProgvalType progival = null;
        SITTransaction transaction1 = null;
        
        try{
        
            if (transaction==null) {
                transaction1 = new SITDefaultTransaction();
            } else {
                transaction1 = transaction;
            }
        
            Connection conn =  this.getJDBCConnection(transaction1);            
            IdType id = null;
           
            try {
                progival = creaEvento(conn, user, IPAddress, configBean.getStoricoCodiceEvento(),
                            configBean.getStoricoCodiceVariazione(), EventType.INSERIMENTO );
                id = getNextId(conn);
            } catch (BasicException e1) {
                logger.error("BasicIOException " +  e1.getMessage());
                throw new SITBasicException("BasicIOException " +  e1.getMessage());
            }
           
            logger.debug("Classe: " + this.getClass().getName() + " Metodo appendFeature = " + id.toString());
            
            /* ALE
            try{
              
                logger.debug("INIZIO INSERIMENTO STATO LAYER STORICO " + getNome());
                
                StatoCantiereDAO scDAO = new StatoCantiereDAO(null,logger);
                scDAO.setIdCantieri(id);
                scDAO.setNewIdsStatoCantiere(conn);            
                scDAO.setIdTabStati(oggettoTer.getStatoCod());
                                                                           
                scDAO.setProgival(progival);
                scDAO.setDtival(dataInserimento);            
                scDAO.setFlerr('N');
                
                int rc = scDAO.inserisci(null,conn,logger);
                
                if(rc == 0){
                    logger.warn("Stato non inserito : " + scDAO);
                }
                
                oggettoTer.getFlagVariazione().remove(getNomiCampi(Layers.NL_CANTIERISTRAD_IDTABSTATI));
                                               
                logger.debug("FINE INSERIMENTO STATO CANTIERE STRADALE");
                
            }catch(IOBasicException iobe){
                logger.error("Errore durante l'inserimento dello stato del cantiere");
                throw new SITException("Errore durante l'aggiornamento dello stato del cantiere",iobe);
            }catch(SQLException sqle){
                logger.error("Errore durante l'inserimento dello stato del cantiere");
                throw new SITException("Errore durante l'aggiornamento dello stato del cantiere",sqle);
            }*/
                
            oggettoTer.setEnte(configBean.getEnte());
            oggettoTer.setFLERR("N");
            oggettoTer.setProgIVal(progival);
            oggettoTer.setProgFIVal(new ProgvalType(0));
            oggettoTer.setDTIVal(dataInserimento);
            oggettoTer.setDTFIVal(DateType.FINE_DEI_TEMPI);
          
            oggettoTer.setIDTPN(id);
            oggettoTer.setIDStorico();
            
            try {
                
                super.appendFeature(oggettoTer, transaction1);
                
            } catch (IOException e) {
                if (transaction==null) {
                    transaction1.rollback();
                }    
                
                logger.warn("Classe " + this.getClass().getName() + "Metodo: AppendFeature - IOException durante la modifica del record storico - Messaggio " + e.getMessage());
                throw (e);
                
            } catch (SITIllegalAttributeException e) {
                if (transaction == null) {
                    transaction1.rollback();
                }    
                
                logger.warn("Classe " + this.getClass().getName() + "Metodo: ModifyFeature - SITIllegalAttributeException durante la modifica del record storico - Messaggio " + e.getMessage());
                throw (e);
                
            } catch (SITException e) {
                logger.warn("SITException "+ e.getMessage());
                throw e;
            }
            
            if (transaction == null) {
                transaction1.commit();
            }
            
        }finally{            
            if (transaction == null && transaction1 != null) {
                transaction1.close();
            }
        }        
    }
    
    /**
     * @param user
     * @param IPAddress
     * @param oggettoTer
     * @param dataModifica
     * @return OggettoTerritorioStorico
     * @throws IOException
     * @throws SITException
     * @throws SQLException
     * 
     * @see LayerTerritorioStorico#modifyFeature(String, String, OggettoTerritorioStorico, DateType, SITTransaction)
     */
    public OggettoTerritorioStorico modifyFeature(String user, String IPAddress, 
            OggettoTerritorioStorico oggettoTer, DateType dataModifica) throws IOException, SITException, SQLException {        
        return modifyFeature(user, IPAddress, oggettoTer, dataModifica, null);                
    }
    
    /**
     * Modifica una feature esistente.
     * 
     * @param user
     * @param IPAddress
     * @param oggettoTer
     * @param dataModifica
     * @param transaction
     * @return OggettoTerritorioStorico
     * @throws IOException
     * @throws SITException
     * @throws SQLException
     */
    @SuppressWarnings("unchecked")
    public OggettoTerritorioStorico modifyFeature(String user, String IPAddress, 
            OggettoTerritorioStorico oggettoTer, DateType dataModifica, SITTransaction transaction) throws IOException, SITException, SQLException {
        
        logger.debug("MODIFICA LAYER STORICO" + getNome());
        
        if(!oggettoTer.isValido()){
            logger.error(getNome() + "- Errore: l'oggetto che si sta modificando non  quello attualmente valido.(DTFIVAL != 31-12-9999)");
            throw new SITStoricoException("Classe " + this.getClass().getName() + "Metodo: ModifyFeature - SITStoricoException durante la modifica del record storico - L'oggetto che si sta modificando non  quello attualmente valido.");
        }
        
        if(dataModifica.before(oggettoTer.getDTIVal())){
            logger.error(getNome() + " - Errore: la data di modifica  antecedente alla data di inizio validit (DTIVAL) del record.");
            throw new SITStoricoException("Classe " + this.getClass().getName() + "Metodo: ModifyFeature - SITStoricoException durante la modifica del record storico -  La data di modifica  antecedente alla data di inizio validit (DTIVAL) del record.");
        }
        
        ProgvalType progival = null;
        SITTransaction transaction1 = null;
        
        try{
        
            if (transaction == null) {
                transaction1 = new SITDefaultTransaction();
            } else {
                transaction1 = transaction;
            }        
                      
            Connection conn =  this.getJDBCConnection(transaction1); 
           
            try {
                progival = creaEvento(conn, user, IPAddress, configBean.getStoricoCodiceEvento(),
                            configBean.getStoricoCodiceVariazione(), EventType.MODIFICA );
            } catch (BasicException e1) {
                logger.error("BasicIOException " +  e1.getMessage());
                throw new SITBasicException("BasicIOException " +  e1.getMessage());
            }
            
            /**********************   DEBUG   *************************/
            HashMap nch = oggettoTer.getFlagVariazione();
            logger.debug("ELEMENTI VARIATI = " + nch.size());
            Iterator iter = nch.values().iterator();
                    
            while (iter.hasNext()) {                        
                logger.debug("ELEMENTI DA MODIFICARE = " + iter.next());
            }
             
            logger.debug("ELEMENTI ANCORA DA MODIFICARE = " + oggettoTer.getFlagVariazione().size());
                    
            if(oggettoTer.getFlagVariazione() != null && oggettoTer.getFlagVariazione().size() > 0){                    
                    
            	// ///////////////////////
                // Modifica record
            	// ///////////////////////
            	
                oggettoTer.resetFlagVariazione();
                
                if (dataModifica.equals(oggettoTer.getDTIVal())) {
                    oggettoTer.setFLERR("S");
                } else {
                    oggettoTer.setFLERR("N");
                }
                
                oggettoTer.setProgFIVal(progival);
                oggettoTer.setDTFIVal(dataModifica.addDay(-1));
                
                try {
                    super.modifyFeature(oggettoTer, transaction1);
                } catch (IOException e) {
                    if (transaction == null && transaction1 != null) {
                        transaction1.rollback();
                    }    
                    
                    logger.warn("Classe " + this.getClass().getName() + "Metodo: ModifyFeature - IOException durante la modifica del record storico - Messaggio " + e.getMessage());
                    throw (e);
                    
                } catch (SITIllegalAttributeException e) {
                    if (transaction == null && transaction1 != null) {
                        transaction1.rollback();
                    }
                    
                    logger.warn("Classe " + this.getClass().getName() + "Metodo: ModifyFeature - SITIllegalAttributeException durante la modifica del record storico - Messaggio " + e.getMessage());
                    throw (e);
                    
                } catch (SITFIDMapperException e) {
                    if (transaction == null && transaction1 != null) {
                        transaction1.rollback();
                    }
                    
                    logger.warn("Classe " + this.getClass().getName() + "Metodo: ModifyFeature - SITFIDMapperException durante la modifica del record storico - Messaggio " + e.getMessage());
                    throw (e);
                    
                } catch (SITException e) {                
                    logger.warn("SITException "+ e.getMessage());
                    throw e;
                } catch (SQLException e) {
                    if (transaction == null && transaction1 != null) {
                        transaction1.rollback();
                    }
                    
                    logger.warn("Classe " + this.getClass().getName() + "Metodo: ModifyFeature - SQLException durante la modifica del record storico - Messaggio " + e.getMessage());
                    throw (e);
				}
                
                // ///////////////////////////////////
                // Inserimento nuova situazione
                // ///////////////////////////////////
                
                oggettoTer.setFLERR("N");
                oggettoTer.setProgIVal(progival);
                oggettoTer.setProgFIVal(new ProgvalType(0));
                oggettoTer.setDTIVal(dataModifica);
                oggettoTer.setDTFIVal(new DateType(9999, 12,31));
                
                // ///////////////////////////////////////
                // Chiave primaria gestita trigger        
                // ///////////////////////////////////////
                
                try {
                    oggettoTer.setVariazioneAiNotNull();
                    super.appendFeature(oggettoTer, transaction1);
                } catch (IOException e) {
                    if (transaction==null) {
                        transaction1.rollback();
                    }
                    
                    logger.warn("Classe " + this.getClass().getName() + "Metodo: ModifyFeature - IOException durante l'inserimento nuovo record storico - Messaggio " + e.getMessage());
                    throw (e);
                    
                } catch (SITIllegalAttributeException e) {
                    if (transaction==null) {
                        transaction1.rollback();
                    }
                    
                    logger.warn("Classe " + this.getClass().getName() + "Metodo: ModifyFeature - SITIllegalAttributeException durante l'inserimento nuovo record storico - Messaggio" + e.getMessage());
                    throw (e);
                }
                
                if (transaction==null) {
                    transaction1.commit();
                }
            }
            
            // ///////////////
            //ALE ???
            // ///////////////
            String chiave = oggettoTer.getIDTPN();            
            return cercaIDTPNAdOggi(chiave,transaction1);
            
        }finally{
            if (transaction == null && transaction1 != null) {
                transaction1.close();
            }
        }        
    }

    /**
     * @param user
     * @param IPAddress
     * @param oggettoTer
     * @throws IOException
     * @throws SITException
     * @throws SQLException
     * 
     * @see LayerTerritorioStorico#removeFeature(String, String, OggettoTerritorioStorico, SITTransaction)
     */
    public void removeFeature(String user, String IPAddress, 
            OggettoTerritorioStorico oggettoTer) throws IOException, SITException, SQLException {       
        removeFeature(user, IPAddress, oggettoTer, null);        
    }

    /**
     * Rimuove una feature esistente. Metodo da usare in caso di cancellazione LOGICA.
     * 
     * @param user
     * @param IPAddress
     * @param oggettoTer
     * @param transaction
     * @throws IOException
     * @throws SITException
     * @throws SQLException
     */
    public void removeFeature(String user, String IPAddress, 
            OggettoTerritorioStorico oggettoTer, SITTransaction transaction) throws IOException, SITException, SQLException {

        if(!oggettoTer.isValido()){
            logger.error(getNome() + " - Errore: L'oggetto  che si sta modificando non  quello attualmente valido.(DTFIVAL != 31-12-9999)");
            throw new SITStoricoException("Classe " + this.getClass().getName() + "Metodo: removeFeature - SITStoricoException durante la modifica del record storico - L'oggetto che si sta modificando non  quello attualmente valido.");
        }
        
        DateType oggi = new DateType();
        ProgvalType progival = null;
        SITTransaction transaction1 = null;
                
        try{
        
            if (transaction==null) {
                transaction1 = new SITDefaultTransaction();
            } else {
                transaction1 = transaction;
            }
            
            Connection conn =  this.getJDBCConnection(transaction1); 
                    
            try {
                progival = creaEvento(conn, user, IPAddress, configBean.getStoricoCodiceEvento(),
                            configBean.getStoricoCodiceVariazione(), EventType.CESSAZIONE);
            } catch (BasicException e1) {
                logger.error("BasicIOException " +  e1.getMessage());
                throw new SITBasicException("BasicIOException " +  e1.getMessage());
            }        
            
            /* ALE
            try{
                
                logger.debug("INIZIO CANCELLAZIONE STATO (" + oggettoTer.getIDSStato() + ") CANTIERE STRADALE (" + oggettoTer.getIDCantieri()+")");
                
                StatoCantiereDAO scDAO = new StatoCantiereDAO(null,logger);
                scDAO.setIdsStatoCantiere(oggettoTer.getIDSStato());                      
                //scDAO.setIdCantieri(oggettoTer.getIDCantieri());
                boolean trovato = scDAO.seekLastImage(null,conn);
                
                if(!trovato){
                    throw new SITException("Stato cantiere non trovato " + scDAO);
                }
                
                if(scDAO.getProgfival().getProg() != 0){
                    int rc = scDAO.cessa(null,'C',progival,oggi,conn,logger);
                }else{
                    logger.warn("L'immagine risulta gi cessata : " + scDAO);
                }
                
                int rc = StatoCantiereDAO.cancellaImmagini(scDAO.getIdCantieri(),conn,logger);
                            
                logger.info("Cancellate logicamente " + rc + " immagini dello stato del cantiere " + scDAO);
                
                logger.debug("FINE CANCELLAZIONE STATO CANTIERE STRADALE");
                
            }catch(IOBasicException iobe){
                logger.error("Errore durante l'aggiornamento dello stato del cantiere");
                throw new SITException("Errore durante l'aggiornamento dello stato del cantiere",iobe);
            }catch(SQLException sqle){
                logger.error("Errore durante l'aggiornamento dello stato del cantiere");
                throw new SITException("Errore durante l'aggiornamento dello stato del cantiere",sqle);
            }            */
            
            // //////////////////////////
            // Modifica record
            // //////////////////////////
            
//            oggettoTer.setFLERR("C");     
            oggettoTer.setProgFIVal(progival);
            
            // ///////////////////////////////////////
            //TODO - VERIFICARE SE IL SETFLERR 
            //       E' GIUSTO .... 
            // ///////////////////////////////////////
            
//            oggettoTer.setDTFIVal(ieri);
            
            oggettoTer.setDTFIVal(oggi);            
            List<OggettoTerritorioStorico> immagini = this.cercaImmaginiOggetto(oggettoTer.getIDTPN(),transaction1);     
            
            try {
                super.modifyFeature(oggettoTer, transaction1);
                
                // ////////////////////////////////////////////////////////////////////
                // Viene fatta la "CANCELLAZIONE" logica di tutte le immagini 
                // e non solo la "CESSAZIONE" logica dell'ultima.
                // ////////////////////////////////////////////////////////////////////
                
                for (OggettoTerritorioStorico og: immagini) {
                    super.modifyFeature(og,transaction1);   
                }
                
            } catch (IOException e) {
                if (transaction==null) {
                    transaction1.rollback();
                }
                
                logger.warn("Classe " + this.getClass().getName() + "Metodo: RemoveFeature - IOException durante l'inserimento nuovo record storico - Messaggio " + e.getMessage());
                throw (e);
                
            } catch (SITIllegalAttributeException e) {
                if (transaction==null) {
                    transaction1.rollback();
                }
                
                logger.warn("Classe " + this.getClass().getName() + "Metodo: RemoveFeature - SITIllegalAttributeException durante l'inserimento nuovo record storico - Messaggio " + e.getMessage());
                throw (e);
                
            } catch (SITFIDMapperException e) {
                if (transaction==null) {
                    transaction1.rollback();
                }
                
                logger.warn("Classe " + this.getClass().getName() + "Metodo: RemoveFeature - SITFIDMapperException durante l'inserimento nuovo record storico - Messaggio " + e.getMessage());
                throw (e);
                
            } catch (SITException e) {
                if (transaction==null) {
                    transaction1.rollback();
                }
                
                logger.warn("SITException "+ e.getMessage());
                throw e;
                
            } catch (SQLException e) {
                if (transaction==null) {
                    transaction1.rollback();
                }
                
                logger.warn("Classe " + this.getClass().getName() + "Metodo: RemoveFeature - SQLException durante l'inserimento nuovo record storico - Messaggio " + e.getMessage());
                throw (e);
			}
            
            if (transaction == null) {
                transaction1.commit();
            }
            
        }finally{
            if (transaction == null && transaction1 != null) {
                transaction1.close();
            }
        }
    }

    /* 
     * Per questo tipo di layer la ricerca per chiave  rimappata sulla ricerca per chiave ad oggi.
     * 
     * @see it.prato.comune.sit.LayerTerritorio#cercaIDTPN(java.lang.String)
     */
    @Override
    public OggettoTerritorioStorico cercaIDTPN(String idTPN) {
        return cercaIDTPNAdOggi (idTPN);
    }

    /**
     * 
     * @param idOggetto
     * @return
     * 
     * @see LayerTerritorioStorico#cercaIDTPNAdOggi(String, SITTransaction)
     */
    public OggettoTerritorioStorico cercaIDTPNAdOggi (String idOggetto) {
        return cercaIDTPNAdOggi(idOggetto, null);           
    }
    
    /**
     * Ricerca IDTPN ad oggi presente.
     * 
     * @param idOggetto
     * @param transaction
     * @return OggettoTerritorioStorico
     */
    public OggettoTerritorioStorico cercaIDTPNAdOggi(String idOggetto, SITTransaction transaction) {
        
        ArrayList<Object> valore = new ArrayList<Object>();
        ArrayList<String> campi  = new ArrayList<String>();
        
        campi.add(Layers.NL_IDTPN);        
        valore.add(idOggetto);
        addCondizioniAdOggi();
        
        return (OggettoTerritorioStorico) super.cercaPoligono(campi, valore, transaction);        
    }
    
    /**
     * @param idOggetto
     * @return List
     * 
     * @see LayerTerritorioStorico#cercaImmaginiOggetto(String, SITTransaction)
     */
    public List<OggettoTerritorioStorico> cercaImmaginiOggetto (String idOggetto) {
        return cercaImmaginiOggetto (idOggetto, null);
    }
    
    /**
     * Ricerca immagini oggetto.
     * 
     * @param idImmagine
     * @param transaction
     * @return List
     */
    @SuppressWarnings("unchecked")
    public List<OggettoTerritorioStorico> cercaImmaginiOggetto (String idImmagine, SITTransaction transaction) {
           
        ArrayList<String> valore = new ArrayList<String>(); 
        ArrayList<String> campi  = new ArrayList<String>();
        
        campi.add(Layers.NL_IDTPN);
        valore.add(idImmagine); 
        
        ArrayList<OggettoTerritorioStorico> immagini = (ArrayList<OggettoTerritorioStorico>) super.cercaPoligoni(campi, valore, transaction);

        return immagini;
    }
    
   /**
    * Ricerca i poligoni ad oggi.
    * 
    * @param nomiCampi
 	* @param valori
 	* @return OggettoTerritorioStorico
 	*/
   @SuppressWarnings("unchecked")  
   public OggettoTerritorioStorico cercaPoligonoAdOggi (ArrayList nomiCampi, ArrayList valori) {        
        addCondizioniAdOggi();        
        return (OggettoTerritorioStorico) super.cercaPoligono(nomiCampi, valori);        
   }

   protected void addCondizioniAdOggi()   {

       /*nomiCampi.add(Layers.NL_PROGFIVAL);
       nomiCampi.add(Layers.NL_FLERR);
                      
       valori.add(new Integer(0));
       valori.add("N");*/
       
       try {
           if(filtroTotale==null){filtroTotale=this.getFiltroVuoto();}
           
//TOB           
//           filtroTotale.AndFiltro(Layers.NL_PROGFIVAL, new Integer(0), FilterType.COMPARE_EQUALS );           
		   filtroTotale.AndFiltro(SITFilterFactory.equals(Layers.NL_PROGFIVAL, new Integer(0)));
		   
//TOB           
//           filtroTotale.AndFiltro(Layers.NL_FLERR, "N", FilterType.COMPARE_EQUALS );
		   filtroTotale.AndFiltro(SITFilterFactory.equals(Layers.NL_FLERR, "N"));
		   
       } catch (SITException e) {
           logger.warn("Classe " + this.getClass().getName() + "Metodo: addCondizioniAdOggi - SITException - Messaggio " + e.getMessage(), e);
           // //////////////////////////////////////////////////////////////////////
           // TODOH Da rilanciare eccezione ma dovrebbero essere modificati tutti 
           // i programmi. Da fare quando  aggiorniamo Geotools 
           // ///////////////////////////////////////////////////////////////////////
           
//           throw (e);
       }       
   }

   /**
    * @param IDStorico
 	* @return OggettoTerritorioStorico
 	* 
 	* @see LayerTerritorioStorico#cercaPoligonoByIDStorico(String, SITTransaction)
 	*/
   public OggettoTerritorioStorico cercaPoligonoByIDStorico (String IDStorico) {
       return cercaPoligonoByIDStorico (IDStorico, null);
   }
   
   /**
    * Ricerca un policono per ID storico.
    * 
 	* @param IDStorico
 	* @param tr
 	* @return OggettoTerritorioStorico
 	*/
   public OggettoTerritorioStorico cercaPoligonoByIDStorico (String IDStorico, SITTransaction tr) {
       return (OggettoTerritorioStorico) super.cercaIDTPN(IDStorico, tr, Layers.NL_IDSTORICO );                
   }

   /**
    * Ricerca un policono per ID storico.
    * 
    * @param IDStorico
 	* @return OggettoTerritorioStorico
 	*/
   public OggettoTerritorioStorico cercaPoligonoByIDStorico (Long IDStorico) {
       return cercaPoligonoByIDStorico (IDStorico, null);
   }
    
   /**
    * Ricerca un policono per ID storico.
    * 
    * @param IDStorico
 	* @param tr
 	* @return OggettoTerritorioStorico
 	*/
   public OggettoTerritorioStorico cercaPoligonoByIDStorico (Long IDStorico, SITTransaction tr){       
       return (OggettoTerritorioStorico) cercaPoligonoByIDStorico ( IDStorico.toString(), tr);                
   }

   /**
    * Ricerca un policono per ID storico.
    *  
    * @param IDStorico
 	* @return OggettoTerritorioStorico
 	*/
   public OggettoTerritorioStorico cercaPoligonoByIDStorico (IdType IDStorico) {
       return cercaPoligonoByIDStorico (IDStorico, null);
   }
   
   /**
    * Ricerca un policono per ID storico. 
    * 
    * @param IDStorico
 	* @param tr
 	* @return OggettoTerritorioStorico
 	*/
   public OggettoTerritorioStorico cercaPoligonoByIDStorico (IdType IDStorico, SITTransaction tr) {
        return (OggettoTerritorioStorico) cercaPoligonoByIDStorico ( IDStorico.toString(),  tr);        
    }
   
   /**
    * Modifica una immagine storica (viene ritrovata tramite ids chiave primaria).
    * 
    * @param oggettoTer
    * @param tr
    * @throws IOException
    * @throws SITException
    * @throws SQLException 
    */
   public void modifyImmagineStorica(OggettoTerritorioStorico oggettoTer, SITTransaction tr) throws IOException, SITException, SQLException {
       super.modifyFeature(oggettoTer, tr);    
   }
   
   /**
    * Modifica una immagine storica (viene ritrovata tramite ids chiave primaria).
    * 
    * @param oggettoTer
    * @param tr
    * @throws IOException
    * @throws SITException
    * @throws SQLException 
    */
   public void modifyCoperturaImmagineStorica(OggettoTerritorioStorico oggettoTer, 
		   OggettoTerritorio polModifica, SITTransaction tr) throws IOException, SITException, SQLException {
       super.modifyCopertura(oggettoTer,polModifica, tr);      
   }
   
   /**
    * Modifica una immagine storica (viene ritrovata tramite ids chiave primaria).
    * 
    * @param oggettoTer
    * @param tr
    * @throws IOException
    * @throws SITException
    * @throws SQLException 
    */
   public void modifyAndClipCoperturaImmagineStorica(OggettoTerritorioStorico oggettoTer, OggettoTerritorio polModifica, 
		   LayerTerritorio clippingLayer, SITTransaction tr) throws IOException, SITException, SQLException {
       super.modifyAndClipCopertura(oggettoTer,polModifica,clippingLayer, tr);      
   }

   @Override
   protected abstract OggettoTerritorio creaOggetto(SimpleFeature curFeat);
    
//TOB    
//    @SuppressWarnings("unchecked") 
//    @Override
//    protected ArrayList FiltroGeografico(Geometry geom, short TipoFiltro)
//            throws IOException, SITException {
//        
//        addCondizioniAdOggi();
//        return super.FiltroGeografico(geom, TipoFiltro);
//    }
   
    /*
     * @see it.prato.comune.sit.LayerTerritorio#FiltroGeografico(org.opengis.filter.spatial.BinarySpatialOperator)
     */
    @SuppressWarnings("unchecked") 
    @Override
    protected ArrayList FiltroGeografico(BinarySpatialOperator filter) throws IOException, SITException {        
        addCondizioniAdOggi();
        return super.FiltroGeografico(filter);
    }    
    
    /*
     * @see it.prato.comune.sit.LayerTerritorio#getContentTypeLayer()
     */
    @Override
    abstract public int getContentTypeLayer() ;
    
    /* 
     * @see it.prato.comune.sit.LayerTerritorio#initNomiCampi(java.util.Properties, java.lang.String, java.lang.String)
     */
    @Override
    protected abstract void initNomiCampi(Properties pr, String ente, String nomeLayer) ;   
   
}
