/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/


package it.prato.comune.sit;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * Clase  MetadatoRicerche.
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public class MetadatoRicerche {

    public static final int TIPOCAMPOSCONOSCIUTO = -1;
    public static final int TIPOCAMPONUMERO      =  0;
    public static final int TIPOCAMPOSTRINGA     =  1;
    public static final int TIPOCAMPODATA        =  2;
    
    private Integer id;
    private String      descrizione = null;
    private Method      metodo      = null;
    //private String[]    nomiCampi   = {};
    private List<String>    nomiCampi   = new ArrayList<String>();
    private List<Integer>   tipiCampi   = new ArrayList<Integer>();
    private List<Boolean>   suggestDisponibile = null;
    private List<Integer>   suggestMinLength = null;
    private List<Boolean>   required 	= null;
    private List<ISuggestProvider>	suggestProvider		= null;

    /** Indica se nella ricerca  disponibile la possibilit di filtrare per geometria. */
    private boolean geomFilterAvailable = false;
    
    
    /**
     * Costruttore di default della classe MetadatoRicerche.
     * 
     */
    public MetadatoRicerche() {
        
    }
    
    public MetadatoRicerche(Integer id, String descrizione, Method metodo, Boolean[] suggestDisponibile, Integer[] suggestMinLength, Boolean[] required_G,
            String[] nomiCampi ) {
    	
    	this(id, descrizione, metodo, null, suggestDisponibile, suggestMinLength, required_G, nomiCampi);
    }
    
    /**
     * Costruttore
     * 
     * @param id
     * @param descrizione
     * @param nomeMetodo
     * @param classeMetodo
     * @param campiricerca
     * @throws SecurityException
     * @throws NoSuchMethodException
     */
    public MetadatoRicerche(Integer id, String descrizione,  String nomeMetodo, Class<?> classeMetodo, List<RicercaCampo> campiricerca) 
    		throws SecurityException, NoSuchMethodException {
    	
    	int nCampi = campiricerca.size();
		Class<?>[] tipoCampi = new Class<?>[nCampi];
		this.metodo = (nomeMetodo!=null) ? classeMetodo.getMethod(nomeMetodo, tipoCampi) : null;
		
	    suggestDisponibile 	= new ArrayList<Boolean>();
	    suggestMinLength 	= new ArrayList<Integer>();
	    required 			= new ArrayList<Boolean>();
	    suggestProvider		= new ArrayList<ISuggestProvider>();
		
		this.id = id;
        this.descrizione = descrizione;
		for (int i = 0 ; i < campiricerca.size(); i++) {
			tipoCampi[i] 	  		= campiricerca.get(i).getTipo();
			//nomiCampi[i] 			= campiricerca.get(i).getEtichetta();
			nomiCampi.add( campiricerca.get(i).getEtichetta());
			required.add(campiricerca.get(i).getRequired());
			suggestDisponibile.add(campiricerca.get(i).getSuggestDisponibile());
			suggestMinLength.add(campiricerca.get(i).getSuggestMinLength());
			suggestProvider.add(campiricerca.get(i).getSuggestProvider());
		};
		
		calcolaTipiCampi(tipoCampi);
		
    }
    
    
    /**
     * Costruttore della classe MetadatoRicerche.
     * 
     * @param id
     * @param descrizione
     * @param metodo
     * @param suggestDisponibile
     * @param suggestMinLength
     * @param nomiCampi
     */
    public MetadatoRicerche(Integer id, String descrizione,  Method metodo, Class<?>[] tipoCampi, Boolean[] suggestDisponibile, Integer[] suggestMinLength, Boolean[] required_G,
            String[] nomiCampi ) {
        super();
        this.id = id;
        this.descrizione = descrizione;
        this.metodo = metodo;
        this.nomiCampi = (nomiCampi==null) ? null : Arrays.asList(nomiCampi);
        this.suggestDisponibile = (suggestDisponibile==null) ? null : Arrays.asList(suggestDisponibile);
        this.suggestMinLength = (suggestMinLength==null) ? null : Arrays.asList(suggestMinLength);
        this.required= (required_G==null) ? null :  Arrays.asList(required_G);
        
        calcolaTipiCampi(tipoCampi);
        
    }

    private void calcolaTipiCampi( Class<?>[] tipoCampi) {
    	if (tipoCampi==null) {
            
	        Class<?>[] clparam = metodo.getParameterTypes();
	        for (int i = 0; i < clparam.length; i++) {
	            if (Number.class.isAssignableFrom(clparam[i])) {
	                tipiCampi.add(TIPOCAMPONUMERO);
	            } else if (String.class.isAssignableFrom(clparam[i])) {
	                tipiCampi.add( TIPOCAMPOSTRINGA);
	            } else if ((Calendar.class.isAssignableFrom(clparam[i])) ||
	                        (Date.class.isAssignableFrom(clparam[i]))) {
	                tipiCampi.add(TIPOCAMPODATA);
	            } else {
	                tipiCampi.add( TIPOCAMPOSCONOSCIUTO);
	            }
	        }
        } else {
	        for (int i = 0; i < tipoCampi.length; i++) {
	            if (Number.class.isAssignableFrom(tipoCampi[i])) {
	                tipiCampi.add(TIPOCAMPONUMERO);
	            } else if (String.class.isAssignableFrom(tipoCampi[i])) {
	                tipiCampi.add(TIPOCAMPOSTRINGA);
	            } else if ((Calendar.class.isAssignableFrom(tipoCampi[i])) ||
	                        (Date.class.isAssignableFrom(tipoCampi[i]))) {
	                tipiCampi.add(TIPOCAMPODATA);
	            } else {
	                tipiCampi.add(TIPOCAMPOSCONOSCIUTO);
	            }
	        }
        }
    }
    
    /**
     * @return String
     */
    public String getDescrizione() {
        return descrizione;
    }

    /**
     * @param descrizione
     */
    public void setDescrizione(String descrizione) {
        this.descrizione = descrizione;
    }

    /**
     * @return Method
     */
    public Method getMetodo() {
        return metodo;
    }

    /**
     * @param metodo
     */
    public void setMetodo(Method metodo) {
        this.metodo = metodo;
    }

    /**
     * @return String[]
     */
    public String[] getNomiCampi() {
        return ( nomiCampi==null) ? null : (String[]) nomiCampi.toArray(new String[nomiCampi.size()]);
    }

    /**
     * @return Integer
     */
    public Integer getId() {
        return id;
    }

    /**
     * @return Integer[]
     */
    public Integer[] getTipiCampi() {
        return ( tipiCampi==null) ? null : (Integer[]) tipiCampi.toArray(new Integer[tipiCampi.size()]); 
    }

    /** 
     * @return Boolean[]
     */
    public Boolean[] getSuggestDisponibile() {
        return ( suggestDisponibile==null) ? null : (Boolean[]) suggestDisponibile.toArray(new Boolean[suggestDisponibile.size()]);    
    }

    /**
     * @return Integer[]
     */
    public Integer[] getSuggestMinLength() {
        return ( suggestMinLength==null) ? null : (Integer[]) suggestMinLength.toArray(new Integer[suggestMinLength.size()]);  
    }

    /**
     * @param suggestMinLength
     */
    public void setSuggestMinLength(Integer[] suggestMinLength) {
        this.suggestMinLength = (suggestMinLength==null) ? null : Arrays.asList(suggestMinLength);
    }

    /**
     * @return Boolean[]
     */
    public Boolean[] getRequired() {
        return ( required==null) ? null : (Boolean[]) required.toArray(new Boolean[required.size()]);   
    }

    /**
     * @param required
     */
    public void setRequired(Boolean[] required) {
        this.required = (required==null) ? null : Arrays.asList( required);
    }

	/**
	 * @return the suggestProvider
	 */
	public ISuggestProvider[] getSuggestProvider() {
		return  ( suggestProvider==null) ? null :  (ISuggestProvider[]) suggestProvider.toArray(new ISuggestProvider[suggestProvider.size()]) ;
	}

	/** */
	
	/**
	 * Restituisce se nella ricerca  disponibile la possibilit di filtrare per geometria.
	 * @return true se  disponibile
	 */
	public final boolean isGeomFilterAvailable() {
		return geomFilterAvailable;
	}

	/**
	 * Imposta se nella ricerca  disponibile la possibilit di filtrare per geometria.
	 * 
	 * @param geomFilterAvailable valore da impostare
	 */
	public final void setGeomFilterAvailable(final boolean geomFilterAvailable) {
		this.geomFilterAvailable = geomFilterAvailable;
	}

}
