/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
/*
 * Copyright  Comune di Prato - Sistema Informativo
 * Project:    Package SIT per elaborazione/interrogazione/gestione dati cartografici 
 * File:       OggettoTerritorio.java
 * Author:     Alessandro Radaelli
 * Version:    1.0.0
 * CreationDate:   Giugno 2005
 * ModifyDate:     
 *
 * Function:   Classe astratta che rappresenta un oggetto territoriale 
 *          
 */

package it.prato.comune.sit;

import it.prato.comune.sit.IFeatureAttributeConfig.Permission;
import it.prato.comune.utilita.core.type.IdType;
import it.prato.comune.utilita.logging.interfaces.LogInterface;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.geotools.geometry.jts.JTS;
import org.geotools.referencing.CRS;
import org.opengis.feature.IllegalAttributeException;
import org.opengis.feature.simple.SimpleFeature;
import org.opengis.feature.type.GeometryType;
import org.opengis.geometry.MismatchedDimensionException;
import org.opengis.referencing.FactoryException;
import org.opengis.referencing.NoSuchAuthorityCodeException;
import org.opengis.referencing.crs.CoordinateReferenceSystem;
import org.opengis.referencing.operation.MathTransform;
import org.opengis.referencing.operation.TransformException;

import bsh.EvalError;
import bsh.Interpreter;

import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.Polygon;
import com.vividsolutions.jts.io.ParseException;
import com.vividsolutions.jts.io.WKTReader;
import com.vividsolutions.jts.io.WKTWriter;

/**
 * Classe OggettoTerritorio
 */
public abstract class OggettoTerritorio implements Serializable,
		IGetFeatureInfoObject, IGeomIntersecabile {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	protected LogInterface logger;
	private LayerTerritorio layer;

	private SimpleFeature Feat;
	private HashMap<String, String> flagVariazione = new HashMap<String, String>();

	/**
	 * Costruttore di default.
	 */
	public OggettoTerritorio() {
		layer = null;
		Feat = null;
	}

	/**
	 * Costruttore della classe OggettoTerritorio.
	 * 
	 * @param logger
	 * @param layer
	 * @param Feat
	 */
	protected OggettoTerritorio(LogInterface logger, LayerTerritorio layer,
			SimpleFeature Feat) {
		this.layer = layer;
		this.Feat = Feat;
		this.logger = logger;
	}

	/**
	 * Compara l'oggetto territorio corrente con quello passato come argomento
	 * 
	 * @param obj2
	 * @param senzaGeometria
	 * @return true se gli oggetti sono uguali (considerando la geometria e gli
	 *         attributi
	 */
	public boolean ugualeA(OggettoTerritorio obj2, boolean senzaGeometria) {
		return ugualeA(obj2, senzaGeometria, false);
	}

	/**
	 * Compara l'oggetto territorio corrente con quello passato come argomento
	 * 
	 * @param obj2
	 * @param senzaGeometria
	 * @boolean senzaIDTPN Indica se fare il confronto senza considerare IDTPN.
	 * @return true se gli oggetti sono uguali (considerando la geometria e gli
	 *         attributi
	 */
	public boolean ugualeA(OggettoTerritorio obj2, boolean senzaGeometria,
			boolean senzaIDTPN) {
		boolean retVal = true;

		for (String nlCampo : layer.getNomiCampi().keySet()) {

			if (!senzaIDTPN || !nlCampo.equals(Layers.NL_IDTPN)) {
				Object val1 = getAttributeByNL(nlCampo);
				Object val2 = obj2.getAttributeByNL(nlCampo);

				if ((val1 == null) || (val2 == null)) {
					retVal = (val1 == val2);
					if (!retVal)
						return false;
				} else {
					if (Comparable.class.isAssignableFrom(val2.getClass())) {
						retVal = (((Comparable) val2).compareTo(val1) == 0);
						if (!retVal)
							return false;
					} else {
						if (!val1.equals(val2))
							return false;
					}
				}
			}
		}

		if ((!senzaGeometria)
				&& (getGeometryAttributeWKT().equals(obj2
						.getGeometryAttributeWKT())))
			retVal = false;

		return retVal;
	}

	/**
	 * Metodo per settare il valore di un attributo a partire dal suo nome
	 * logico (vedere anche valori contenuti nella classe Costanti @link
	 * it.prato.comune.Costanti}
	 * 
	 * @param NLAttributo
	 * @param valore
	 * @throws SITIllegalAttributeException
	 */
	public void setAttributeByNL(String NLAttributo, Object valore)
			throws SITIllegalAttributeException {

		// TODO in questo modo consente scrittura anche a chi non ha setter.

		String nomeAttributo = layer.getNomiCampiScrittura().get(NLAttributo);
		setAttribute(nomeAttributo, valore);
	}

	/**
	 * Metodo per recuperare il valore di un attributo a partire dal suo nome
	 * logico (vedere anche valori contenuti nella classe Costanti
	 * {@link it.prato.comune.sit.Costanti}
	 * 
	 * @param NLAttributo
	 * @return Object
	 */
	public Object getAttributeByNL(String NLAttributo) {
		String nomeAttributo = layer.getNomiCampi().get(NLAttributo);
		return getAttribute(nomeAttributo);
	}

	/**
	 * Restituisce il valore dell'attributo il cui nome corrisponde con quello
	 * passato come parametro.
	 * 
	 * @param nomeAttributo
	 * @return Object
	 */
	protected Object getAttribute(String nomeAttributo) {

		if (nomeAttributo.equals(Layers.NL_FID)) {
			return getIDTPN();
		}
		Object obVal = null;
		obVal = Feat.getAttribute(nomeAttributo);

		if (obVal != null) {
			// logger.debug("Classe: " + this.getClass().getName() +
			// " - Metodo: getAttribute - " + "Attributo richiesto: "+
			// NomeAttributo + "- Valore " + obVal.toString());
		} else {
			// logger.debug("Classe: " + this.getClass().getName() +
			// " - Metodo: getAttribute - " + "Attributo richiesto: "+
			// NomeAttributo + "- Valore null" );
		}
		// TOB
		// return Feat.getAttribute(NomeAttributo);

		return obVal;
	}

	/**
	 * Chiama la setAttribute soltanto se il nuovo valore  diverso dal vecchio.
	 * Vecchio e nuovo devono essere oggetti che implementano comparable per
	 * poter essere confrontati.
	 * 
	 * @param nomeAttributo
	 * @param newVal
	 * @param oldVal
	 * @throws SITIllegalAttributeException
	 */
	protected void setAttributeOnlyIfChanged(String nomeAttributo,
			Comparable newVal, Comparable oldVal)
			throws SITIllegalAttributeException {

		logger.debug("Chiamo la setAttribute nel caso il nuovo valore  diverso dal nuovo...");
		logger.debug("Attributo: " + nomeAttributo + " - Vecchio valore: "
				+ oldVal + " - Nuovo valore: " + newVal);

		if ((oldVal == null && newVal != null)
				|| (oldVal != null && (newVal == null
						|| !newVal.getClass().isAssignableFrom(
								oldVal.getClass()) || oldVal.compareTo(newVal) != 0))) {
			logger.debug("Attributo " + nomeAttributo + " impostato!");
			setAttribute(nomeAttributo, newVal);
		}
	}

	/**
	 * Imposta il valore dell'attributo il cui nome corrisponde con quello
	 * passato come argomento.
	 * 
	 * @param nomeAttributo
	 * @param valore
	 * @throws SITIllegalAttributeException
	 */
	protected void setAttribute(String nomeAttributo, Object valore)
			throws SITIllegalAttributeException {
		try {
			Feat.setAttribute(nomeAttributo, valore);
			flagVariazione.put(nomeAttributo, nomeAttributo);
		} catch (IllegalAttributeException e) {
			String errMesg = "Errore durante setAttribute. Attributo: "
					+ nomeAttributo + " - Valore: " + valore;
			logger.error(errMesg, e);
			throw new SITIllegalAttributeException(errMesg, e);
		}
	}

	/**
	 * Cancellazione del flagVariazione.
	 */
	protected void resetFlagVariazione() {
		flagVariazione.clear();
	}

	/**
	 * Imposta il flagVariazione per gli attributi non nulli.
	 */
	protected void setVariazioneAiNotNull() {

		Iterator<String> iter = layer.getNomiCampiScrittura().values()
				.iterator();
		flagVariazione.clear();

		while (iter.hasNext()) {
			String nomecampo = (String) iter.next();

			if (getAttribute(nomecampo) != null) {
				flagVariazione.put(nomecampo, nomecampo);
			}
		}

		if (getGeometryAttribute() != null) {
			flagVariazione.put("##GEOM##", "##GEOM##");
		}
	}

	/**
	 * Restituisce il valore dell'attributo geometrico.
	 * 
	 * @return Geometry
	 */
	protected Geometry getGeometryAttribute() {
		// TOB
		// return Feat.getDefaultGeometry();

		return (Geometry) Feat.getDefaultGeometry();
	}

	/**
	 * Ritorna il nome decodificato dell'oggetto.<br/>
	 * 
	 * Per default  il valore di IDTPN, ma ogni specifica tipologia di oggetto
	 * pu sovrascrivere il metodo per fornire una descrizione pi significativa
	 * (per esempio)
	 * 
	 * @return Stringa per la descrizione dell'oggetto
	 */
	public String getNomeOggetto() {
		return evalEspressione(getLayer().getEspressioneNomeOggetto(), getIDTPN());
	}

	/**
	 * Ritorna una descrizione dell'oggetto, comprensibile per l'utente ed in
	 * gradi di fargli discriminare tra pi oggetti.<br/>
	 * 
	 * Per default :
	 * <ul>
	 *<li>il valore di getNomeOggetto() se {@link LayerTerritorio.getEspressioneDescrizione()}  null 
	 *o se la sua valutazione tramite beanshell va in eccezione </li>
	 *<li>la valutazione di {@link LayerTerritorio#getEspressioneDescrizione()} tramite beanshell negli altri casi</li>
	 *<br/>
	 * La shell di BeanShell per utilizzato per il calcolo ha a disposizione un oggetto attributes 
	 * che contiene il risultato del metodo {@link OggettoTerritorio#getFeatureInfoAttributes()} 
	 *
	 * Ogni specifica tipologia di oggetto pu sovrascrivere il metodo per fornire una descrizione pi
	 * significativa (per esempio "Sez. 201")
	 * 
	 * @return Stringa per la descrizione dell'oggetto
	 */
	public String getDescrizione() {
		
		return evalEspressione(getLayer().getEspressioneDescrizione(), getNomeOggetto());
	}

	private String evalEspressione(String espr, String defaut) {
		String retVal = null;
		if (espr!=null) {
			Interpreter i = new Interpreter();  // Construct an interpreter
			
			try {
				i.set("attributes", getFeatureInfoAttributes());
				
				return (String) i.eval(espr);
			} catch (EvalError e) {
				logger.warn("Errore di valutazione dell'espressione descrizione nel metodo getDescrizione di OggettoTerritorio, proseguo con descrizione standard. Espressione: " + getLayer().getEspressioneDescrizione(), e );
				retVal = defaut;
			}             
		} else {
			retVal = defaut;	
		}
		
		return retVal;
	}
	
	/**
	 * Ritorna una lista contente le descrizioni da usare in caso di suggest <br/>
	 * 
	 * Per default  il valore di IDTPN, ma ogni specifica tipologia di oggetto
	 * pu sovrascrivere il metodo per fornire una descrizione pi significativa
	 * 
	 * @return Lista contenente le descrizioni per il suggest sui vari campi.
	 *         Per esempio in civici il primo campo  la via, il secondo il
	 *         civico
	 */
	public List<String> getDescrizioneSuggest() {
		List<String> retVal = new ArrayList<String>();

		if (getLayer().getEspressioneDescrizioneSuggest()!=null) {
			for (String s: getLayer().getEspressioneDescrizioneSuggest()) {
				retVal.add(evalEspressione(s, getIDTPN()));
			}
		}
		
		return retVal;
	}

	/**
	 * Ritorna una lista contente nome campo e valore dei campi addizionali che
	 * saranno riportati nelle JSGeometry <br/>
	 * 
	 * Per default  una mappa vuota, ma ogni specifica tipologia di oggetto pu
	 * sovrascrivere il metodo per fornire una descrizione pi significativa
	 * 
	 * @return Ritorna una lista contente nome campo e valore dei campi
	 *         addizionali che saranno riportati nelle JSGeometry <br/>
	 *         .
	 */
	public Map<String, String> getJSGeomAdditionalFields() {
		HashMap<String, String> retVal = new HashMap<String, String>();
		return retVal;
	}

	/**
	 * Ritorna una lista contenente nome campo e class relativa a campi
	 * addizionali riportate nelle JSGeometry Per default  una mappa vuota, ma
	 * ogni specifica tipologia di oggetto pu sovrascrivere il metodo per
	 * fornire una descrizione pi significativa
	 * 
	 * @return Ritorna una lista contenente nome campo e class relativa a campi
	 *         addizionali riportate nelle JSGeometry <br/>
	 *         .
	 */
	public Map<String, Class> getJSGeomAdditionalFieldsType() {
		HashMap<String, Class> retVal = new HashMap<String, Class>();
		return retVal;
	}

	/**
	 * @param modello
	 * @throws SITGeometryCastNotSupported
	 * 
	 * @see {@link it.prato.comunt.sit.OggettoTerritorio#castToSameGeom(Geometry geomModello, Geometry geomToCast)}
	 */
	public void castToSameGeom(Geometry modello)
			throws SITGeometryCastNotSupported {

		try {
			OggettoTerritorio.castToSameGeom(modello,
					this.getGeometryAttribute());
		} catch (SITGeometryCastNotSupported e) {
			throw e;
		}
	}

	/**
	 * Gestisce il cast tra oggetti geometrici.
	 * 
	 * @param classModello
	 * @param geomToCast
	 * @return Geometry
	 * @throws SITGeometryCastNotSupported
	 */
	public static Geometry castToSameGeom(Class classModello,
			Geometry geomToCast) throws SITGeometryCastNotSupported {

		String modelloTipo = classModello.getName();
		String thisGeomTipo = geomToCast.getClass().getName();

		GeometryFactory geomFact = geomToCast.getFactory();
		Geometry casted = null;

		if (!(classModello.isAssignableFrom(geomToCast.getClass()))) {

			if (modelloTipo.equals("com.vividsolutions.jts.geom.MultiPolygon")) {
				if (thisGeomTipo.equals("com.vividsolutions.jts.geom.Polygon")) {
					Polygon[] pol = { (Polygon) geomToCast };
					casted = geomFact.createMultiPolygon(pol);
				}
			} else {
				if (modelloTipo
						.equals("com.vividsolutions.jts.geom.MultiLineString")) {
					if (thisGeomTipo.equals("com.vividsolutions.jts.geom.LineString")) {
						LineString[] line = { (LineString) geomToCast };
						casted = geomFact.createMultiLineString(line);
					}
				} else {
					if (modelloTipo
							.equals("com.vividsolutions.jts.geom.MultiPoint")) {
						if (thisGeomTipo
								.equals("com.vividsolutions.jts.geom.Point")) {
							Point[] point = { (Point) geomToCast };
							casted = geomFact.createMultiPoint(point);
						}
					}
				}
			}
		} else {
			casted = geomToCast;
		}

		if (casted == null) {
			throw new SITGeometryCastNotSupported("Conversione di "
					+ thisGeomTipo + " in " + modelloTipo + " non supportata");
		}

		return casted;
	}

	/**
	 * @param geomModello
	 * @param geomToCast
	 * @return Geometry
	 * @throws SITGeometryCastNotSupported
	 * 
	 * @see {@link it.prato.comunt.sit.OggettoTerritorio#castToSameGeom(Class classModello, Geometry geomToCast)}
	 */
	public static Geometry castToSameGeom(Geometry geomModello,
			Geometry geomToCast) throws SITGeometryCastNotSupported {
		return castToSameGeom(geomModello.getClass(), geomToCast);
	}

	/**
	 * Restituisce il valore del campo geometrico in formato WKT.
	 * 
	 * @return String
	 */
	public String getGeometryAttributeWKT() {
		WKTWriter wktW = new WKTWriter();

		// TOB
		// return wktW.writeFormatted(Feat.getDefaultGeometry());

		Geometry geom = (Geometry) Feat.getDefaultGeometry();
		if(geom == null) return null;
		return wktW.writeFormatted(geom);
	}
	
	/**
     * Restituisce il valore del campo geometrico in formato WKT nel sistema di riferimento desiderato.
     * 
     * @param desiredCrs CRS in cui si desidera la geometria
     * 
     * @return String
	 * @throws SITException 
     */
    public String getGeometryAttributeWKT(String desiredCrs) throws SITException {
        
        if(desiredCrs == null) return this.getGeometryAttributeWKT();
                        
        String fromCrs = this.getSRID();
        
        if(fromCrs == null || fromCrs.equals(desiredCrs)){
            return this.getGeometryAttributeWKT();
        }
        
        WKTWriter wktW = new WKTWriter();

        Geometry geom = (Geometry) Feat.getDefaultGeometry();        
        if(geom == null) return null;
        
        Geometry newGeom = OggettoTerritorio.trasforma(geom, fromCrs, desiredCrs, logger);
                        
        return wktW.writeFormatted(newGeom);
    }	
	

	/**
	 * Imposta il valore dell'attributo geometrico usando la stringa WKT passata
	 * come argomento.
	 * 
	 * @param WKTString
	 * @throws SITParseException
	 * @throws SITIllegalAttributeException
	 */
	public void setGeometryAttributeWKT(String WKTString)
			throws SITParseException, SITIllegalAttributeException {
		try {
			WKTReader wktReader = new WKTReader();
			Geometry geometry = (Geometry) wktReader.read(WKTString);
			setGeometryAttributeGeom(geometry);
		} catch (ParseException e) {
			String errMsg = "ParseException durante setting dell'attributo WKTString";
			logger.error(errMsg, e);
			throw new SITParseException(errMsg, e);
		} catch (SITIllegalAttributeException e) {
			String errMsg = "SITIllegalAttributeException durante setting dell'attributo WKTString";
			logger.error(errMsg, e);
			throw e;
		}
	}

	/**
	 * Imposta il valore dell'attributo geometrico utilizzando la geometria
	 * passata come argomento.
	 * 
	 * @param geom
	 * @throws SITIllegalAttributeException
	 */
	protected void setGeometryAttributeGeom(Geometry geom)
			throws SITIllegalAttributeException {
		try {
			// TOB
			// logger.debug("setGeometryAttributeGeom. Il layer deve contenere"
			// +
			// this.Feat.getFeatureType().getDefaultGeometry().getType().getName());
			// // Nel caso che venga passato un poligono dove serve un
			// multipoligono o casi simili si procede automaticamente alla
			// conversione
			// Class<?> defaultGeometryClass =
			// this.Feat.getFeatureType().getDefaultGeometry().getType();

			GeometryType geomType = this.Feat.getFeatureType()
					.getGeometryDescriptor().getType();
			logger.debug("Il layer deve contenere "
					+ geomType.getName().toString());
			Class<?> defaultGeometryClass = geomType.getBinding();

			if (defaultGeometryClass != geom.getClass()) {
				logger.debug("Inizio conversione da "
						+ geom.getClass().getName() + " a "
						+ defaultGeometryClass.getName());
				Geometry geom1 = castToSameGeom(defaultGeometryClass, geom);
				Feat.setDefaultGeometry(geom1);
			} else {
				Feat.setDefaultGeometry(geom);
			}

			flagVariazione.put("##GEOM##", "##GEOM##");

		} catch (IllegalAttributeException e) {
			String errMsg = "IllegalAttributeException durante setting dell'attributo Geometry";
			logger.error(errMsg, e);
			throw new SITIllegalAttributeException(errMsg, e);
		} catch (SITGeometryCastNotSupported e) {
			String errMsg = "SITGeometryCastNotSupported durante setting dell'attributo Geometry";
			logger.error(errMsg, e);
			throw new SITIllegalAttributeException(errMsg, e);
		}
	}

	/**
	 * Restituisce il feature id dell'attributo il cui nome  passato come
	 * argomento.
	 * 
	 * @param NLCampo
	 * @return
	 */
	public String getID(String NLCampo) {
		return getID(NLCampo, false);

	}

	/**
	 * Restituisce il feature id di scrittura dell'attributo il cui nome 
	 * passato come argomento.
	 * 
	 * @param NLCampo
	 * @return
	 */
	public String getIDScrittura(String NLCampo) {
		return getID(NLCampo, true);

	}

	/**
	 * Restituisce il feature id dell'attributo il cui nome  passato come
	 * argomento.
	 * 
	 * @param NLCampo
	 * @param bScrittura
	 * @return String
	 */
	// TODOH Aggiungere lancio eccezioni? Per adesso ritorna null
	// TODOH Per adesso gestisce solo chiavi singole, in caso di multiple
	// comunque ritorna null
	public String getID(String NLCampo, boolean bScrittura) {

		String szbuff = (bScrittura) ? layer.getNomiCampiScrittura(NLCampo)
				: layer.getNomiCampi(NLCampo);

		if ((szbuff != null) && (!szbuff.toUpperCase().equals(Layers.NL_FID))) {
			Object attVal = getAttribute(szbuff);

			if (attVal == null)
				return null;
			else
				return attVal.toString();

		} else {
			// TOB
			// try {
			//
			// Object[] chiavi =
			// getLayer().getFIDMapper().getPKAttributes(getFeat().getID());
			//
			// if (chiavi.length>0) {
			// return (String) chiavi[0].toString();
			// }else {
			// return null;
			// }

			// } catch (IOException e){
			// logger.warn("Eccezione IOException in getIDTPN", e);
			// return null;
			// } catch (SITException e){
			// logger.warn("Eccezione SITException in getIDTPN", e);
			// return null;
			// }

			// ???
			// String typnam = layer.configBean.getTypeNameScrittura();
			String featureId = getFeat().getIdentifier().getID();
			featureId = featureId.substring(featureId.indexOf(".") + 1);
			// String featureId = getFeat().getIdentifier().getID();
			return featureId;
		}
	}

	/**
	 * Restituisce l'NL_IDTPN.
	 * 
	 * @return String
	 * 
	 * @see {@link it.prato.comunt.sit.OggettoTerritorio#getID(String NLCampo)}
	 */
	// TODOH Aggiungere lancio eccezioni? Per adesso ritorna null
	// TODOH Per adesso gestisce solo chiavi singole, in caso di multiple
	// comunque ritorna null
	public String getIDTPN() {
		String retVal = null;
		
		if (getLayer().getEspressioneIDTPN()!=null && !getLayer().getEspressioneIDTPN().equals("")) {
			retVal = evalEspressione(getLayer().getEspressioneIDTPN(), getID(Layers.NL_IDTPN));	
		} else {
			retVal = getID(Layers.NL_IDTPN);
		}

		return retVal;
	}

	/**
	 * recupera il valore del campo che contiene il codice ente (definito nel
	 * config.txt tramite la chiave identificata da Layers.NL_CODENTE oppure
	 * null se non definito nel Config.txt
	 * 
	 * @return valore del campo o null se non definito il nome della campo in
	 *         Config.txt
	 */
	public Object getCodEnte() {
		if (layer.getNomiCampi().containsKey(Layers.NL_CODENTE)) {
			return getAttributeByNL(Layers.NL_CODENTE);
		} else {
			return null;
		}
	}

	/**
	 * Restituisce l'NL_IDTPN scrittura (utilizzando il secondo campo definito
	 * se presente altrimenti il primo.
	 * 
	 * @return String
	 * 
	 * @see {@link it.prato.comunt.sit.OggettoTerritorio#getID(String NLCampo)}
	 */
	// TODOH Aggiungere lancio eccezioni? Per adesso ritorna null
	// TODOH Per adesso gestisce solo chiavi singole, in caso di multiple
	// comunque ritorna null
	public String getIDTPNScrittura() {
		return getIDScrittura(Layers.NL_IDTPN);
	}

	/**
	 * @param IDTPN
	 * @throws SITIllegalAttributeException
	 * 
	 * @see {@link it.prato.comunt.sit.OggettoTerritorio#setID(String IDTPN, String NLCampo)}
	 */
	public void setIDTPN(String IDTPN) throws SITIllegalAttributeException {
		setID(IDTPN, Layers.NL_IDTPN);
	}

	/**
	 * Imposta il valore dell'NL_IDTPN.
	 * 
	 * @param IDTPN
	 * @param NLCampo
	 * @throws SITIllegalAttributeException
	 */
	protected void setID(String IDTPN, String NLCampo)
			throws SITIllegalAttributeException {
		String szbuff = layer.getNomiCampiScrittura(NLCampo);

		// TODOH nel caso di ##FID## gestito solo che faccia da solo (non viene
		// fatto alcun set) forse va gi bene cos
		// funziona nel caso di generazione automatica della chiave
		// (autoincrementi o simili)

		if ((szbuff != null) && (!szbuff.toUpperCase().equals(Layers.NL_FID))) {
			setAttribute(layer.getNomiCampiScrittura(NLCampo), IDTPN);
		}
	}

	/**
	 * Imposta il valore dell'NL_IDTPN.
	 * 
	 * @param IDTPN
	 * @param NLCampo
	 * @throws SITIllegalAttributeException
	 */
	protected void setID(IdType IDTPN, String NLCampo)
			throws SITIllegalAttributeException {
		String szbuff = layer.getNomiCampiScrittura(NLCampo);

		// TODOH nel caso di ##FID## gestito solo che faccia da solo (non viene
		// fatto alcun set) forse va gi bene cos
		// funziona nel caso di generazione automatica della chiave
		// (autoincrementi o simili)

		if ((szbuff != null) && (!szbuff.toUpperCase().equals(Layers.NL_FID))) {
			setAttribute(layer.getNomiCampiScrittura(NLCampo),
					new Long(IDTPN.getLongValue()));
		}
	}

	/**
	 * Restituisce l'internal envelope relativo al valore del campo geometrico.
	 * 
	 * @return RettangoloContenitore
	 */
	public RettangoloContenitore getRettContenitore() {

		// TOB
		// Envelope env = Feat.getDefaultGeometry().getEnvelopeInternal();

		Envelope env = ((Geometry) Feat.getDefaultGeometry())
				.getEnvelopeInternal();

		return new RettangoloContenitore(env.getMinX(), env.getMinY(),
				env.getMaxX(), env.getMaxY());
	}

	/**
	 * Calcola e restituisce il rettangolo contenitore di una lista di oggetti
	 * di tipo OggettoTerritorio.
	 * 
	 * @param oggettiTerritorio
	 * @return RettangoloContenitore
	 * @throws SITIllegalAttributeException
	 *             lanciata dall'operazione geometrica di unione
	 * @throws SITGeometryCastNotSupported
	 *             lanciata dall'operazione geometrica di unione
	 */
	public static RettangoloContenitore getRettContenitore(
			Collection<OggettoTerritorio> oggettiTerritorio)
			throws SITIllegalAttributeException, SITGeometryCastNotSupported {

		OggettoTerritorio unioneObj = getUnione(oggettiTerritorio);

		if (unioneObj == null) {
			return null;
		}

		// TOB
		// Envelope env =
		// unioneObj.getFeat().getDefaultGeometry().getEnvelopeInternal();

		Envelope env = ((Geometry) unioneObj.getFeat().getDefaultGeometry())
				.getEnvelopeInternal();

		return new RettangoloContenitore(env.getMinX(), env.getMinY(),
				env.getMaxX(), env.getMaxY());
	}

	/**
	 * Restituisce l'unione di una collezione di oggetti di tipo
	 * OggettoTerritorio sottoforma di un OggettoTerritorio, la cui unica
	 * caratteristica sfruttabile  la geometria.
	 * 
	 * @param oggettiTerritorio
	 * @return OggettoTerritorio
	 * @throws SITIllegalAttributeException
	 * @throws SITGeometryCastNotSupported
	 */
	public static OggettoTerritorio getUnione(Collection oggettiTerritorio)
			throws SITIllegalAttributeException, SITGeometryCastNotSupported {

		if (oggettiTerritorio == null || oggettiTerritorio.isEmpty()) {
			return null;
		}

		OggettoTerritorio objTerBase = null;
		Iterator<OggettoTerritorio> iter = oggettiTerritorio.iterator();

		for (; iter.hasNext();) {
			OggettoTerritorio objTer = (OggettoTerritorio) iter.next();

			if (objTerBase == null) {
				objTerBase = objTer;
			} else {
				objTerBase.unione(objTer);
			}
		}

		return objTerBase;
	}

	/**
	 * Restituisce il punto centrale della geometia.
	 * 
	 * @return Punto
	 */
	public Punto getCentroide() {
		// TOB
		// Geometry dg = Feat.getDefaultGeometry();
		Geometry dg = (Geometry) Feat.getDefaultGeometry();

		if (dg == null)
			return null;
		else
			return Punto.PointToPunto(dg.getCentroid());
	}

	/**
	 * Calcola la distanza tra la geometria dell'oggetto territorio corrente con
	 * quella dell'oggetto passato come argomento.
	 * 
	 * @param ogTer
	 * @return double
	 */
	public double distanza(OggettoTerritorio ogTer) {
		// TOB
		// return
		// Feat.getDefaultGeometry().distance(ogTer.getFeat().getDefaultGeometry());
		return ((Geometry) Feat.getDefaultGeometry()).distance((Geometry) ogTer
				.getFeat().getDefaultGeometry());
	}

	/**
	 * Calcola la distanza tra la geometria dell'oggetto territorio corrente e
	 * il punto passato come argomento.
	 * 
	 * @param p1
	 * @return double
	 */
	public double distanza(Punto p1) {
		// TOB
		// return Feat.getDefaultGeometry().distance(Punto.PuntoToPoint(p1));
		return ((Geometry) Feat.getDefaultGeometry()).distance(Punto
				.PuntoToPoint(p1));
	}

	/**
	 * Estrae la defaultGeometry e la trasforma nello stesso sistema di riferimento di questo oggetto.
	 * In caso di problemi rilancia una RuntimeException per non rompere la compatibilit con l'API esistente.
	 * @param ogTer oggetto di cui estrarre e trasformare la geometria
	 * 
	 * @return Geometria trasformata
	 */
	private Geometry trasformaStessoSRID(OggettoTerritorio ogTer) {
		Geometry buff;
		try {
			buff = trasforma(ogTer.getGeometryAttribute(), ogTer.getSRID(), getSRID(), logger);
		} catch (SITException e) {
			throw new SITRuntimeException(e);
		}
		
		return buff;
	}
	
	/**
	 * Riduce o aumenta la geometria dell'oggetto territorio in base al buffer 
	 * @param buffer
	 * @throws SITIllegalAttributeException
	 */
	public void buffer(double buffer) throws SITIllegalAttributeException{
	    Geometry geom = (Geometry) Feat.getDefaultGeometry();
        if(geom == null) return;
        Geometry bufferedGeom = geom.buffer(buffer);
        setGeometryAttributeGeom(bufferedGeom);
	}
	
	/**
	 * Controlla se la geometria dell'oggetto territorio corrente interseca o
	 * meno quella dell'oggetto passato come argomento.
	 * 
	 * @param ogTer
	 * @return True se interseca
	 */
	public boolean interseca(OggettoTerritorio ogTer) {
		// TOB
		// return
		// Feat.getDefaultGeometry().intersects(ogTer.getFeat().getDefaultGeometry());
		
		Geometry buff = trasformaStessoSRID(ogTer);
		
		return getGeometryAttribute().intersects(buff);
	}
	
	public boolean interseca(String wkt, String SRID) {
		// TOB
		// return
		// Feat.getDefaultGeometry().intersects(ogTer.getFeat().getDefaultGeometry());
		
		String wkt1;
		try {
			wkt1 = trasformaWKT(wkt, SRID, getSRID(), logger);
		} catch (SITException e) {
			throw new SITRuntimeException(e);
		}
		Geometry buff = JSGeometry.WKTToGeometry(wkt1, logger);
		
		return getGeometryAttribute().intersects(buff);
	}

	/**
	 * Controlla se la geometria dell'oggetto territorio corrente copre o meno
	 * quella dell'oggetto passato come argomento.
	 * 
	 * @param ogTer
	 * @return boolean
	 */
	public boolean copre(OggettoTerritorio ogTer) {
		Geometry buffer = getGeometryAttribute().buffer(0.01);
		Geometry buffered = getGeometryAttribute().union(buffer);

		// TOB
		// return (buffered).covers(ogTer.getFeat().getDefaultGeometry());
		Geometry buff = trasformaStessoSRID(ogTer);
		return (buffered).covers(buff);
	}

	/**
	 * Controlla se la geometria dell'oggetto territorio corrente  coperta o
	 * meno da quella dell'oggetto passato come argomento.
	 * 
	 * @param ogTer
	 * @return boolean
	 */
	public boolean copertoDa(OggettoTerritorio ogTer) {
		Geometry buff = trasformaStessoSRID(ogTer);
		Geometry buffer = buff.buffer(0.01);
		Geometry buffered = buff.union(buffer);

		// TOB
		// return (Feat.getDefaultGeometry()).coveredBy(buffered);
		return getGeometryAttribute().coveredBy(buffered);

	}

	/**
	 * Controlla se la geometria dell'oggetto territorio corrente contiene o
	 * meno da quella dell'oggetto passato come argomento.
	 * 
	 * @param ogTer
	 * @return boolean
	 */
	public boolean contiene(OggettoTerritorio ogTer) {
		// TOB
		// return
		// Feat.getDefaultGeometry().contains(ogTer.getFeat().getDefaultGeometry());
		Geometry buff = trasformaStessoSRID(ogTer);
		
		return (getGeometryAttribute()).contains(buff);
	}

	/**
	 * Controlla se la geometria dell'oggetto territorio corrente dista meno di
	 * un dato range passato come argomento </br> da quella dell'oggetto passato
	 * come argomento.
	 * 
	 * @param ogTer
	 * @param range
	 * @return boolean
	 */
	public boolean distamenodi(OggettoTerritorio ogTer, double range) {
		// TOB
		// return
		// Feat.getDefaultGeometry().isWithinDistance(ogTer.getFeat().getDefaultGeometry(),
		// range);
		Geometry buff = trasformaStessoSRID(ogTer);
		return getGeometryAttribute().isWithinDistance(buff, range);
	}

	/**
	 * Calcola l'intersezione tra la geometria dell'oggetto territorio corrente
	 * e quella dell'oggetto passato come argomento, settando successivamente il
	 * valore dell'attributo geometrico dell'oggetto territorio corrente.
	 * 
	 * @param ogTer
	 * @throws SITIllegalAttributeException
	 */
	public void intersezione(OggettoTerritorio ogTer)
			throws SITIllegalAttributeException {
		Geometry geom = this.getGeometryAttribute();

		// TOB
		// geom = geom.intersection(ogTer.getFeat().getDefaultGeometry());
		Geometry buff = trasformaStessoSRID(ogTer);
		geom = geom.intersection(buff);
		setGeometryAttributeGeom(geom);
	}

	/**
	 * Calcola l'unione tra la geometria dell'oggetto territorio corrente e
	 * quella dell'oggetto passato come argomento, settando successivamente il
	 * valore dell'attributo geometrico dell'oggetto territorio corrente.
	 * 
	 * @param ogTer
	 * @throws SITIllegalAttributeException
	 * @throws SITGeometryCastNotSupported
	 */
	public void unione(OggettoTerritorio ogTer)
			throws SITIllegalAttributeException, SITGeometryCastNotSupported {
		Geometry geom = this.getGeometryAttribute();

		// TOB
		// Geometry geomnew = geom.union(ogTer.getFeat().getDefaultGeometry());
		Geometry buff = trasformaStessoSRID(ogTer);
		Geometry geomnew = geom.union(buff);

		// dovrebbe essere superfluo perch adesso il cast viene fatto dentro
		// setGeometryAttributeGeom
		/*
		 * try { geomnew = OggettoTerritorio.castToSameGeom(geom, geomnew); }
		 * catch (SITGeometryCastNotSupported e) { logger.warn(
		 * "Durante unione di geometrie si e' verificata l'eccezione SITGeometryCastNotSupported - "
		 * + e.getMessage() + " Proseguo lo stesso..."); //throw e; }
		 */

		setGeometryAttributeGeom(geomnew);
	}

	/**
	 * Calcola l'unione tra la geometria dell'oggetto territorio corrente e
	 * quella passata come stringa WKT, settando successivamente il
	 * valore dell'attributo geometrico dell'oggetto territorio corrente. La geometria WKT deve 
	 * essere espressa nello stesso sistema di riferimento dell'oggetto corrente.
	 * 
	 * @param ogTer
	 * @throws SITIllegalAttributeException
	 * @throws SITGeometryCastNotSupported
	 */
	public void unioneWKT(String geom)
			throws SITIllegalAttributeException, SITGeometryCastNotSupported {
		Geometry geomOrig = this.getGeometryAttribute();
		Geometry buff = JSGeometry.WKTToGeometry(geom, logger);
		
		// TOB
		// Geometry geomnew = geom.union(ogTer.getFeat().getDefaultGeometry());
		//Geometry buff = trasformaStessoSRID(ogTer);
		Geometry geomnew = geomOrig.union(buff);

		setGeometryAttributeGeom(geomnew);
	}

	
	/**
	 * Calcola la differenza tra la geometria dell'oggetto territorio corrente e
	 * quella passata come argomento, settando successivamente il valore
	 * dell'attributo geometrico dell'oggetto territorio corrente.
	 * 
	 * @param geom1
	 * @throws SITIllegalAttributeException
	 * @throws SITGeometryCastNotSupported
	 */
	protected void differenzaGeom(Geometry geom1)
			throws SITIllegalAttributeException, SITGeometryCastNotSupported {

		Geometry geom = this.getGeometryAttribute();
		Geometry geomnew = geom.difference(geom1);

		// ///////////////////////////////////////////////////
		// dovrebbe essere superfluo perch adesso il cast
		// viene fatto dentro setGeometryAttributeGeom.
		// //////////////////////////////////////////////////
		/*
		 * try { geomnew = OggettoTerritorio.castToSameGeom(geom, geomnew); }
		 * catch (SITGeometryCastNotSupported e) { throw e; }
		 */

		setGeometryAttributeGeom(geomnew);
	}

	/**
	 * Calcola la differenza tra la geometria dell'oggetto territorio corrente e
	 * quella dell'oggetto passato come argomento.
	 * 
	 * @param ogTer
	 * @throws SITIllegalAttributeException
	 * @throws SITGeometryCastNotSupported
	 */
	public void differenza(OggettoTerritorio ogTer)
			throws SITIllegalAttributeException, SITGeometryCastNotSupported {
		// TOB
		// differenzaGeom(ogTer.getFeat().getDefaultGeometry());
		Geometry buff = trasformaStessoSRID(ogTer);
		differenzaGeom(buff);
	}

	/**
	 * Calcola la differenza tra la geometria dell'oggetto territorio corrente e
	 * quella passata come argomento in formato WKT.
	 * 
	 * @param WKT
	 * @throws SITIllegalAttributeException
	 * @throws SITGeometryCastNotSupported
	 */
	public void differenza(String WKT) throws SITIllegalAttributeException,
			SITGeometryCastNotSupported {
		differenzaGeom(JSGeometry.WKTToGeometry(WKT, logger));
	}

	/**
	 * @return String
	 * @throws SITException
	 * 
	 * @see {@link it.prato.comunt.sit.JSGeometry#oggettoTerritorioToJSGeometry(OggettoTerritorio ogg, LogInterface logger)}
	 */
	public String getJSGeometry() throws SITException {
		return JSGeometry.oggettoTerritorioToJSGeometry(this, logger)
				.toString();
	}

	/**
	 * Trasforma la geometria passata sotto format di WKT in un differente sistema di
	 * riferimento.
	 * 
	 * @param wkt Geometria da connvertire
	 * @param fromCRS sistema di riferimento di partenza, espresso nella forma EPSG:XXXX
	 * @param toCRS sistema di riferimento di partenza, espresso nella forma EPSG:XXXX
	 * @param logger 
	 * @return String Stringa WKT che rappresenta la geometria convertita
	 * @throws SITException
	 * 
	 * @see {@link OggettoTerritorio#trasforma(Geometry sourceGeometry, CoordinateReferenceSystem fromCRS, CoordinateReferenceSystem toCRS, LogInterface logger)}
	 */
	public static String trasformaWKT(String wkt, String fromCRS, String toCRS, LogInterface logger) throws SITException {
		
		if (fromCRS.equals(toCRS)) {
			return wkt;
		} else {
			// Conversione coordinate del WKT
			WKTReader wktr = new WKTReader();
			
			try {
				Geometry sourceGeometry = wktr.read(wkt);
				Geometry trasformata = trasforma(sourceGeometry, fromCRS, toCRS, logger);
				
				WKTWriter wktw = new WKTWriter();
	
				return wktw.write(trasformata);
			} catch (ParseException e) {
				String errMsg = "Errore durante trasformazione WKT: ParseException";
				logger.error(errMsg, e);
				throw new SITException(errMsg, e);
			}
		}
			
	}
	
	/**
	 * Trasforma la geometria passata come argomento in un differente sistema di
	 * riferimento.
	 * 
	 * @param sourceGeometry
	 * @param fromCRS
	 * @param toCRS
	 * @param logger
	 * @return Geometry
	 * @throws SITException
	 * 
	 * @see {@link OggettoTerritorio#trasforma(Geometry sourceGeometry, CoordinateReferenceSystem fromCRS, CoordinateReferenceSystem toCRS, LogInterface logger)}
	 */
	protected static Geometry trasforma(Geometry sourceGeometry,
			String fromCRS, CoordinateReferenceSystem toCRS, LogInterface logger)
			throws SITException {

		CoordinateReferenceSystem sourceCRS = null;

		// ///////////////////////////////////////////////////////////////////////////////
		// Se definiti i sistemi di riferimento lo decodifica
		// e converte, altrimenti ritorna la geometria iniziale senza
		// trasformazioni
		// ///////////////////////////////////////////////////////////////////////////////

		if ((fromCRS != null) && (!fromCRS.equals("")) && (toCRS != null)) {
			try {
				sourceCRS = CRS.decode(fromCRS,true);
			} catch (NoSuchAuthorityCodeException e) {
				String errMsg = "Errore durante trasformazione CRS: authority code non valido";
				logger.error(errMsg, e);
				throw new SITException(errMsg, e);
			} catch (FactoryException e) {
				String errMsg = "Errore durante trasformazione CRS: FactoryException";
				logger.error(errMsg, e);
				throw new SITException(errMsg, e);
			}

			return trasforma(sourceGeometry, sourceCRS, toCRS, logger);

		} else {
			return sourceGeometry;
		}
	}

	/**
	 * Trasforma la geometria passata come argomento in un differente sistema di
	 * riferimento.
	 * 
	 * @param sourceGeometry
	 * @param fromCRS
	 * @param toCRS
	 * @param logger
	 * @return Geometry
	 * @throws SITException
	 * 
	 * @see {@link OggettoTerritorio#trasforma(Geometry sourceGeometry, CoordinateReferenceSystem fromCRS, CoordinateReferenceSystem toCRS, LogInterface logger)}
	 */
	protected static Geometry trasforma(Geometry sourceGeometry,
			CoordinateReferenceSystem fromCRS, String toCRS, LogInterface logger)
			throws SITException {

		CoordinateReferenceSystem targetCRS = null;

		try {
			targetCRS = CRS.decode(toCRS,true);
		} catch (NoSuchAuthorityCodeException e) {
			String errMsg = "Errore durante trasformazione CRS: authority code non valido";
			logger.error(errMsg, e);
			throw new SITException(errMsg, e);
		} catch (FactoryException e) {
			String errMsg = "Errore durante trasformazione CRS: FactoryException";
			logger.error(errMsg, e);
			throw new SITException(errMsg, e);
		}

		return trasforma(sourceGeometry, fromCRS, targetCRS, logger);
	}

	/**
	 * Trasforma la geometria passata come argomento in un differente sistema di
	 * riferimento.
	 * 
	 * @param sourceGeometry
	 * @param fromCRS
	 * @param toCRS
	 * @param logger
	 * @return Geometry
	 * @throws SITException
	 * 
	 * @see {@link OggettoTerritorio#trasforma(Geometry sourceGeometry, String fromCRS, CoordinateReferenceSystem toCRS, LogInterface logger)}
	 */
	protected static Geometry trasforma(Geometry sourceGeometry,
			String fromCRS, String toCRS, LogInterface logger)
			throws SITException {

		CoordinateReferenceSystem targetCRS = null;

		try {
			targetCRS = CRS.decode(toCRS,true);
		} catch (NoSuchAuthorityCodeException e) {
			String errMsg = "Errore durante trasformazione CRS: authority code non valido";
			logger.error(errMsg, e);
			throw new SITException(errMsg, e);
		} catch (FactoryException e) {
			String errMsg = "Errore durante trasformazione CRS: FactoryException";
			logger.error(errMsg, e);
			throw new SITException(errMsg, e);
		}

		return trasforma(sourceGeometry, fromCRS, targetCRS, logger);
	}

	/**
	 * Trasforma la geometria passata come argomento in un differente sistema di
	 * riferimento.
	 * 
	 * @param sourceGeometry
	 * @param fromCRS
	 * @param toCRS
	 * @param logger
	 * @return Geometry
	 * @throws SITException
	 */
	protected static Geometry trasforma(Geometry sourceGeometry,
			CoordinateReferenceSystem fromCRS, CoordinateReferenceSystem toCRS,
			LogInterface logger) throws SITException {

		Geometry targetGeometry = null;

		try {
			// TOB
			// MathTransform transform = CRS.transform(fromCRS, toCRS);
			// targetGeometry = JTS.transform( sourceGeometry, transform);
		    
			MathTransform transform = CRS.findMathTransform(fromCRS, toCRS, true);
//			logger.info("Eseguo trasformazione da "+ fromCRS + " a " + toCRS);
			targetGeometry = JTS.transform(sourceGeometry, transform);

		} catch (MismatchedDimensionException e) {
			String errMsg = "Errore durante trasformazione CRS: MismatchedDimensionException";
			logger.error(errMsg, e);
			throw new SITException(errMsg, e);
		} catch (NoSuchAuthorityCodeException e) {
			String errMsg = "Errore durante trasformazione CRS: authority code non valido";
			logger.error(errMsg, e);
			throw new SITException(errMsg, e);
		} catch (FactoryException e) {
			String errMsg = "Errore durante trasformazione CRS: FactoryException";
			logger.error(errMsg, e);
			throw new SITException(errMsg, e);
		} catch (TransformException e) {
			String errMsg = "Errore durante trasformazione CRS: TransformException";
			logger.error(errMsg, e);
			throw new SITException(errMsg, e);
		}

		return targetGeometry;
	}
	
	

	
	/**
	 * Trasforma un punto in un diverso sistema di riferimento
	 * 
	 * @param in Punto da trasformare
	 * @param toSRID Sistema di riferimento di destinazione
	 * @param logger 
	 * @return
	 * @throws SITException
	 */
	public static Punto trasformaPunto(Punto in, String toSRID, LogInterface logger) throws SITException {
		
		Point g = (Point) trasforma(Punto.PuntoToPoint(in), in.getSRID(), toSRID, logger);
		
		return new Punto(g.getX(), g.getY(),toSRID);
		
	}
	
	/**
	 * @return SimpleFeature
	 * 
	 */
	protected SimpleFeature getFeat() {
		return Feat;
	}

	/**
	 * @param feat
	 *            il campo feat da impostare.
	 * 
	 */
	protected void setFeat(SimpleFeature feat) {
		Feat = feat;
	}

	/**
	 * @return HashMap il flagVariazione.
	 * 
	 */
	protected HashMap<String, String> getFlagVariazione() {
		return flagVariazione;
	}

	/**
	 * Restituisce l'elemento Layer
	 * 
	 * @return LayerTerritorio
	 */
	public LayerTerritorio getLayer() {
		return layer;
	}

	/**
	 * Imposta il campo LayerTerritorio
	 * 
	 * @param layer
	 */
	public void setLayer(LayerTerritorio layer) {
		this.layer = layer;
	}

	/**
	 * 
	 * @see IGetFeatureInfoObject#getFeatureInfoAttributes()
	 */
	public Map<String, Object> getFeatureInfoAttributes() {
		Map<String, Object> retVal = new HashMap<String, Object>();

		for (String nlCampo : layer.getNomiCampi().keySet()) {

			retVal.put(nlCampo, getAttributeByNL(nlCampo));
		}


		return retVal;
	}
	
	/**
	 * Ritorna tutti gli attributi non essendo supportate le permission su questo tipo di fonte.
	 * @see IGetFeatureInfoObject#getFeatureInfoAttributes(Permission) 
	 */
	public Map<String, Object> getFeatureInfoAttributes(Permission perm) {
		return new HashMap<String, Object>(getFeatureInfoAttributes());
	}

	/**
	 * @see IGetFeatureInfoObject#getFeatureInfoLayer()
	 */
	public IGetFeatureInfoLayer getFeatureInfoLayer() {
		// TODO Auto-generated method stub
		return (IGetFeatureInfoLayer) getLayer();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see it.prato.comune.sit.IGetFeatureInfoObject#getSRID()
	 */
	public String getSRID() throws SITException {
		return getLayer().getSRID();
	}

	/* (non-Javadoc)
	 * @see it.prato.comune.sit.IGetFeatureInfoObject#getBoundingBoxWKT()
	 */
	public String getBoundingBoxWKT() {

		Geometry geom = (Geometry) Feat.getDefaultGeometry();
		return new WKTWriter().write(geom.getEnvelope());

	}

	/* (non-Javadoc)
	 * @see it.prato.comune.sit.IGetFeatureInfoObject#getGetFeatureInfoLink()
	 */
	public String getGetFeatureInfoLink() {
		return null;
	}
	
	
	public double getArea(){
		if(this.getLayer().isLayerOfPolygons()){
			return ((Geometry)getFeat().getDefaultGeometry()).getArea();
		} else {
			return 0;
		}		
	}

	/* (non-Javadoc)
	 * @see it.prato.comune.sit.IGetFeatureInfoObject#getSourceHTTP()
	 */
	@Override
	public String getSourceHTTP() {
		return null;
	}

	/* (non-Javadoc)
	 * @see it.prato.comune.sit.IGetFeatureInfoObject#getSourceLayer()
	 */
	@Override
	public String getSourceLayer() {
		return null;
	}

	/* (non-Javadoc)
	 * @see it.prato.comune.sit.IGetFeatureInfoObject#getSourceBBOX()
	 */
	@Override
	public String getSourceBBOX() {
		return null;
	}

	/* (non-Javadoc)
	 * @see it.prato.comune.sit.IGetFeatureInfoObject#getSourceStyle()
	 */
	@Override
	public String getSourceStyle() {
		return null;
	}

	@Override
	public Integer getSourceHeight() {
		return null;
	}
	
	@Override
	public Integer getSourceWidth() {
		return null;
	}

	@Override
	public Integer getSourceX() {
		return null;
	}
	
	@Override
	public Integer getSourceY() {
		return null;
	}
}
