/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
/*
 * Copyright  Comune di Prato - Sistema Informativo
 * Project:    Package SIT per elaborazione/interrogazione/gestione dati cartografici 
 * File:       PoligonoCircoscrizione.java
 * Author:     Alessandro Radaelli
 * Version:    1.0.0
 * CreationDate:   Giugno 2005
 * ModifyDate:     
 *
 * Function:   Poligono relativo ad una zona circoscrizione 
 *             
 */
package it.prato.comune.sit;

import java.util.HashMap;
import java.util.Map;

import org.opengis.feature.simple.SimpleFeature;

import bsh.EvalError;
import bsh.Interpreter;
import it.prato.comune.sit.IFeatureAttributeConfig.Permission;
import it.prato.comune.utilita.logging.interfaces.LogInterface;

/**
 * Clase  PoligonoCircoscrizione.
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public class OggettoTerritorioGenerico extends OggettoTerritorio{

	public OggettoTerritorioGenerico(LogInterface logger, LayerTerritorio layer,
			SimpleFeature Feat) {
		super(logger, layer, Feat);
		// TODO Auto-generated constructor stub
	}

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Costruttore di default della classe PoligonoCircoscrizione.
	 * 
	 */
    public OggettoTerritorioGenerico() {
        super();
    }

    /*
     * @see it.prato.comune.sit.OggettoTerritorio#getNomeOggetto()
     */
    @Override
    public String getNomeOggetto() {
    	return evalEspressione(getLayer().getEspressioneNomeOggetto());
    }
    
    /*
     * @see it.prato.comune.sit.OggettoTerritorio#getDescrizione()
     */
    @Override
    public String getDescrizione() {
        return evalEspressione(getLayer().getEspressioneDescrizione());
    }

	public String evalEspressione(String espr) {
		
		String retVal = null;
		
		if (espr!=null) {
			Interpreter i = new Interpreter();  // Construct an interpreter
			
			try {
				i.set("attributes", getFeatureInfoAttributes());
				
				return (String) i.eval(espr);
			} catch (EvalError e) {
				//TODO logger.warn("Errore di valutazione dell'espressione descrizione nel metodo getDescrizione di OggettoTerritorio, proseguo con descrizione standard. Espressione: " + getLayer().getEspressioneDescrizione(), e );
				retVal = e.getMessage();
			}             
		} else {
			retVal = "";	
		}
		
		return retVal;
	}
	
	/**
	 * Ritorna tutti gli attributi con la permission indicata.
	 * @see IGetFeatureInfoObject#getFeatureInfoAttributes(Permission) 
	 */
	public Map<String, Object> getFeatureInfoAttributes(Permission perm) {
		Map<String, Object> retVal = new HashMap<String, Object>();
		
		if (LayerTerritorioGenerico.class.isAssignableFrom(getLayer().getClass())) {
			LayerTerritorioGenerico l = (LayerTerritorioGenerico) getLayer();
			Map<String, Object> fa = getFeatureInfoAttributes();
			for (String nl: fa.keySet()) {
				IFeatureAttributeConfig fac = l.getFeatureAttributeConfig(nl);
				if (fac.hasPermission(perm)) {
					retVal.put(nl, fa.get(nl));
				}
			}
			return retVal;
		} else {
			return super.getFeatureInfoAttributes(perm);
		}
		
	}

}
