/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
/*
 * Copyright  Comune di Prato - Sistema Informativo
 * Project:    Package SIT per elaborazione/interrogazione/gestione dati cartografici 
 * File:       OggettoTerritorioStorico.java
 * Author:     Alessandro Radaelli
 * Version:    1.0.0
 * CreationDate:  
 * ModifyDate:     
 *             
 */
package it.prato.comune.sit;

import it.prato.comune.utilita.core.type.DateType;
import it.prato.comune.utilita.core.type.IdType;
import it.prato.comune.utilita.core.type.ProgvalType;
import it.prato.comune.utilita.logging.interfaces.LogInterface;

import java.math.BigDecimal;
import java.util.Date;

import org.opengis.feature.simple.SimpleFeature;

/**
 * Clase  OggettoTerritorioStorico.
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public class OggettoTerritorioStorico extends OggettoTerritorio {

	/**
	 * Costruttore di default della classe OggettoTerritorioStorico.
	 * 
	 */
    public OggettoTerritorioStorico() {
    }

    /**
     * Costruttore della classe OggettoTerritorioStorico.
     * 
     * @param logger
     * @param layer
     * @param Feat
     */
    public OggettoTerritorioStorico(LogInterface logger, LayerTerritorio layer,
    		SimpleFeature Feat) {
        super(logger, layer, Feat);
    }
    
    /**
     * Restituisce l'ente. 
     * 
     * @return String
     */
    public String getEnte() {        
        return (String) super.getAttribute(getLayer().getNomiCampiScrittura(Layers.NL_ENTESTORICO));            
    }

    /**
     * Restituisce l'ente.
     *   
     * @param valore
     * @throws SITIllegalAttributeException
     */
    protected void setEnte(String valore) throws SITIllegalAttributeException {
        super.setAttributeOnlyIfChanged(getLayer().getNomiCampiScrittura(Layers.NL_ENTESTORICO), valore, getEnte());
    }
    
    /**
     * @return String
     */
    public String getFLERR() {        
        return (String) super.getAttribute(getLayer().getNomiCampiScrittura(Layers.NL_FLERR));            
    }

    /**
     * @param valore
     * @throws SITIllegalAttributeException
     */
    protected void setFLERR(String valore)throws SITIllegalAttributeException {
        super.setAttribute(getLayer().getNomiCampiScrittura(Layers.NL_FLERR), valore);
    }
    
    /**
     * @param valore
     * @throws SITIllegalAttributeException
     */
    protected void setProgIVal(ProgvalType valore) throws SITIllegalAttributeException {
        super.setAttribute(getLayer().getNomiCampiScrittura(Layers.NL_PROGIVAL), valore.toString());
    }
    
    /**
     * @return ProgvalType
     */
    public ProgvalType getProgIVal() {
        BigDecimal szBuff = (BigDecimal) super.getAttribute(getLayer().getNomiCampi(Layers.NL_PROGIVAL));
        
        return (szBuff == null) ? null : new ProgvalType(szBuff.longValue()); 
    }

    /**
     * @param valore
     * @throws SITIllegalAttributeException
     */
    protected void setProgFIVal(ProgvalType valore) throws SITIllegalAttributeException {
        super.setAttribute(getLayer().getNomiCampiScrittura(Layers.NL_PROGFIVAL), valore.toString());
    }
    
    /**
     * @return ProgvalType
     */
    public ProgvalType getProgFIVal() {        
        BigDecimal szBuff = (BigDecimal) super.getAttribute(getLayer().getNomiCampi(Layers.NL_PROGFIVAL));        
        return new ProgvalType(szBuff.longValue());
    }
    
    /**
     * @return DateType
     */
    public DateType getDTIVal() {        
        Date dd = (Date) super.getAttribute(getLayer().getNomiCampi(Layers.NL_DTIVAL));
        
        DateType gr = new DateType(dd);
        return gr;       
    }

    /**
     * @param valore
     * @throws SITIllegalAttributeException
     */
    protected void setDTIVal(DateType valore)throws SITIllegalAttributeException {
        super.setAttribute(getLayer().getNomiCampiScrittura(Layers.NL_DTIVAL), valore.toDate());
    }
    
    /**
     * @return DateType
     */
    public DateType getDTFIVal() {        
        Date dd = (Date) super.getAttribute(getLayer().getNomiCampi(Layers.NL_DTFIVAL));
        
        DateType gr = new DateType(dd);
        return gr;               
    }

    /**
     * @param valore
     * @throws SITIllegalAttributeException
     */
    protected void setDTFIVal(DateType valore)throws SITIllegalAttributeException {
        super.setAttribute(getLayer().getNomiCampiScrittura(Layers.NL_DTFIVAL), valore.toDate());
    }
    
    /**
     * @return boolean
     */
    public boolean isValido(){
        return this.getProgFIVal().getProg()==0;
    }
    
    /** 
     * @return IdType
     */
    public IdType getIDStorico() {
        return new IdType(getID(Layers.NL_IDSTORICO));
    }
    
    /**
     * Setta IDStorico. Questo campo pu essere la chiave primaria della tabelle (nel qual caso su config.txt 
     * deve essere presente ##FID## altrimenti, se la chiave non e' presente su config.txt, non viene fatto nulla (la chiave 
     * la combinazione di IDTPN+PROGIVAL+FLERR.
     * 
     * @throws SITIllegalAttributeException
     */
    public void setIDStorico() throws SITIllegalAttributeException{ 
        setID((String) null, Layers.NL_IDSTORICO);        
    }
 
    /**
     * @param IDTPN
     * @throws SITIllegalAttributeException
     */
    public void setIDTPN(IdType IDTPN) throws SITIllegalAttributeException{
        setID( IDTPN, Layers.NL_IDTPN);        
    }

}
