/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
package it.prato.comune.sit;

import it.prato.comune.utilita.logging.interfaces.LogInterface;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

public abstract class RicercaBase implements IRicerca, IRicercaComponibile {

	private String nomeRicerca = "";
	private List<RicercaCampo> campiRicerca=null;
	private List<IRicercaComponibile> ricerche = new ArrayList<IRicercaComponibile>();
	private LayerTerritorio layerTer;
	private Boolean geomFilterAvailable = true;
	
	private LogInterface logger; 
	
	/**
	 * @see {IRicerca{@link #addRicerca(int, LayerTerritorio)}
	 */
	public void addRicerca(int id, LayerTerritorio layerTer) throws SITException{
		
		setLayerTer(layerTer);
		List<RicercaCampo> ricTot =  getCampiRicercaTot();
		layerTer.addRicerca(id, nomeRicerca, null, ricTot, geomFilterAvailable);
		
	}
	
	/**
	 * Recupera un filtro corrispondente all'intersezione con la geometria passata come argomento.
	 * 
	 * @param geomWKT Geometria di filtro
	 * @return se geomWKT diverso da null e da stringa vuota ritorna un filtro corrispondente all'intersezione 
	 * 			con la geometria passata come argomento, altrimenti ritorna null.
	 * @throws SITException eccezione rilanciata in caso di errore 
	 * @throws IOException eccezione rilanciata in caso di errore IO
	 */
	public final Filtro getGeomFilter(String geomWKT) throws SITException, IOException {
		
		if (geomWKT == null || geomWKT.equals("")) {
			return null;
		}
		
		try {
			Filtro fv = getLayerTer().getFiltroVuoto();
			/*
			OggettoTerritorio ogg = getLayerTer().creaNuovoOggettoTerritorio();
			ogg.setGeometryAttributeWKT(geomWKT);
			fv.AndFiltroGeog(ogg, Filtro.GEOMETRY_INTERSECTS);
			*/
			fv.AndFiltroGeog(geomWKT, Filtro.GEOMETRY_INTERSECTS);
			return fv;
			
		} catch (SITException e) {
			String msg = "Eccezione durante composizione filtro geometrico con parametro " + geomWKT;			
			getLogger().error(msg,e);
			throw e;
		} /*catch (IOException e) {
			String msg = "Eccezione durante composizione filtro geometrico con parametro " + geomWKT;			
			getLogger().error(msg,e);
			throw e;
		}*/
		
	}
	
	@SuppressWarnings({"unchecked"})
	public ArrayList<OggettoTerritorio> cerca(Object...args) throws SITException {
		ArrayList<OggettoTerritorio> retVal = null;
		String geomWKT = null;
		
		List<Object> lArgs = new LinkedList<Object>(Arrays.asList(args));
		Filtro fGeom = null;
		
		try {
			if (geomFilterAvailable) {
				geomWKT = (String) lArgs.remove(0);
				if (geomWKT != null) {
					fGeom = getGeomFilter(geomWKT);		
				}
			}
			
			Filtro f = getFiltro(lArgs.toArray());
			getLayerTer().setFiltro(f);
			if (fGeom!=null) {
				f.AndFiltro(fGeom);
			}
			retVal = getLayerTer().cercaFiltro();
			getLayerTer().setFiltro((getLayerTer().getFiltroVuoto()));
		
		} catch (SITException e) {
			getLogger().error("Eccezione durante ricerca di oggetti cartografici",e);
			throw e;
		} catch (IOException e) {
			getLogger().error("Eccezione durante ricerca di oggetti cartografici",e);
			throw new SITException("Eccezione durante ricerca di oggetti cartografici",e);
		}
	
		return retVal; 
	}

	protected LayerTerritorio getLayerTer() {
		return layerTer; 
	}
	
	/**
	 * @return the nomeRicerca
	 */
	public String getNomeRicerca() {
		return nomeRicerca;
	}
	
	/**
	 * @param nomeRicerca the nomeRicerca to set
	 */
	public void setNomeRicerca(String nomeRicerca) {
		this.nomeRicerca = nomeRicerca;
	}

	/**
	 * @return the campiRicerca
	 */
	public List<RicercaCampo> getCampiRicerca() {
		return campiRicerca;
	}
	/**
	 * @param campiRicerca the campiRicerca to set
	 */
	public void setCampiRicerca(List<RicercaCampo> campiRicerca) {
		this.campiRicerca = campiRicerca;
	}
	
	/*
	 * 
	 * (non-Javadoc)
	 * @see it.prato.comune.sit.IRicerca#ricercaSuggest2(int, int, boolean, java.lang.Object[])
	 */
	public JSGeometryArrayList<? extends JSGeometry> ricercaSuggest2(
			int nCampo, boolean withGeometry, Object... valori)
			throws SITException {

		JSGeometryArrayList<JSGeometry> retVal = new JSGeometryArrayList<JSGeometry>(logger);
		ArrayList<OggettoTerritorio> ret = cerca(valori);
		retVal = JSGeometryArrayList.toJSGeometryArrayList(ret, null, logger,withGeometry);
		
		return retVal;
		
	}
	/**
	 * @return the logger
	 */
	public LogInterface getLogger() { 
		return logger;
	}
	
	/**
	 * @param logger the logger to set
	 */
	public void setLogger(LogInterface logger) {
		this.logger = logger;
	}
	
	/**
	 * @return the ricerche
	 */
	public List<IRicercaComponibile> getRicerche() {
		return ricerche;
	}

	/**
	 * @param ricerche the ricerche to set
	 */ 
	public void setRicerche(List<IRicercaComponibile> ricerche) {
		this.ricerche = ricerche;
	}

	/* (non-Javadoc)
	 * @see it.prato.comune.sit.IRicercaComponibile#getCampiRicercaTot()
	 */
	public List<RicercaCampo> getCampiRicercaTot() {
		
		List<RicercaCampo> retVal = new ArrayList<RicercaCampo>();
		if (getCampiRicerca()!=null) {
			retVal.addAll(getCampiRicerca());
		}
		if (getRicerche()!=null) {
			for (IRicercaComponibile ric: getRicerche()) {
				retVal.addAll(ric.getCampiRicercaTot());
			}
		}
		
		return retVal;
	}

	/**
	 * @param layerTer the layerTer to set
	 */
	public void setLayerTer(LayerTerritorio layerTer) {
		this.layerTer = layerTer;
		for (IRicerca ric: ricerche) {
			ric.setLayerTer(layerTer);
		}
		
	}
	
	/**
	 * Permette di impostare se la ricerca consente di impostare il filtro geometrico.
	 * 
	 * @param geomFilterAvailable true se la ricerca consente di impostare il filtro geometrico.
	 */
	public final void setGeomFilterAvailable(final Boolean pGeomFilterAvailable) {
		this.geomFilterAvailable = pGeomFilterAvailable;
	}
	
}
