/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
package it.prato.comune.sit;

/**
 * 
 * Classe che definisce un campo sul quale effettuare una ricerca. <br/>
 * Esempio di definizione Spring: <br/>
 * <pre>
 * 	<bean class="it.prato.comune.sit.RicercaCampo">
		<property name="NLCampo" value="NL_TOPONIMO" />
		<property name="etichetta" value="Province" />
		<property name="tipo" value="java.lang.String" />
		<property name="suggestDisponibile" value="true" />
		<property name="suggestMinLength" value="3" /> 
		<property name="suggestProvider">
			<bean class="it.prato.comune.sit.SuggestProviderLocalDatastore" >
				<constructor-arg ref="logger1" />
				<property name="datastore" ref="pgWFSgtTinyRT" />
				<property name="featureName" value="sita:listaprovincertpoly" />
				<!-- <property name="cql" value="????? " /> -->
				<property name="displayFieldName" value="nprov" />
				<property name="valueFieldName"   value="prov_istat" />								
			</bean> 
		</property>
		<property name="operatore" value="LIKE" /> 
	</bean>
 *  
 * </pre>
 * 
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public class RicercaCampo {
	private String   	NLCampo				= null;
	private String   	etichetta 			= null;
	private Class<?>   	tipo				= String.class;
	private Boolean 	suggestDisponibile 	= null;
    private Integer 	suggestMinLength 	= null;
    private Boolean		required			= true;
    private String 		operatore			= "=";
    private String 		trasforma			= null; 	 
    private ISuggestProvider	suggestProvider		= null;
	
	/**
	 * @return the operatore
	 */
	public String getOperatore() {
		return operatore;
	}

	/**
	 * @param operatore the operatore to set
	 */
	public void setOperatore(String operatore) {
		this.operatore = operatore.trim();
	}

    
	/**
	 * @return the nLCampo
	 */
	public String getNLCampo() {
		return NLCampo;
	}
	/**
	 * @param nLCampo the nLCampo to set
	 */
	public void setNLCampo(String nLCampo) {
		NLCampo = nLCampo;
	}
	/**
	 * @return the etichetta
	 */
	public String getEtichetta() {
		return etichetta;
	}
	/**
	 * @param etichetta the etichetta to set
	 */
	public void setEtichetta(String etichetta) {
		this.etichetta = etichetta;
	}
	
	/**
	 * @return the tipo
	 */
	public Class<?> getTipo() {
		return tipo;
	}
	/**
	 * @param tipo the tipo to set
	 */
	public void setTipo(Class<?> tipo) {
		this.tipo = tipo;
	}
	/**
	 * @return the suggestDisponibile
	 */
	public Boolean getSuggestDisponibile() {
		return suggestDisponibile;
	}
	/**
	 * @param suggestDisponibile the suggestDisponibile to set
	 */
	public void setSuggestDisponibile(Boolean suggestDisponibile) {
		this.suggestDisponibile = suggestDisponibile;
	}
	/**
	 * @return the suggestMinLength
	 */
	public Integer getSuggestMinLength() {
		return suggestMinLength;
	}
	/**
	 * @param suggestMinLength the suggestMinLength to set
	 */
	public void setSuggestMinLength(Integer suggestMinLength) {
		this.suggestMinLength = suggestMinLength;
	}
	/**
	 * @return the required
	 */
	public Boolean getRequired() {
		return required;
	}
	/**
	 * @param required the required to set
	 */
	public void setRequired(Boolean required) {
		this.required = required;
	}

	/**
	 * @return the dominio
	 */
	public ISuggestProvider getSuggestProvider() {
		return suggestProvider;
	}

	/**
	 * @param dominio the dominio to set
	 */
	public void setSuggestProvider(ISuggestProvider dominioSuggest) {
		this.suggestProvider = dominioSuggest;
	}

	/**
	 * @return il valore del parametro trasforma ({@link RicercaCampo#setTrasforma(String)}
	 */
	public String getTrasforma() {
		return trasforma;
	}

	/**
	 * @param trasforma indica se prima di fare la ricerca il parametro deve essere trasformato. <br/>
	 * I valori possibili sono:
	 * 		vuoto, nullo o un valore non compreso in quelli elencati di seguito -> lascia invariato
	 *  	u -> trasforma in maiuscolo <br/>
	 *  	U -> trasforma in maiuscolo <br/>
	 *  	l -> trasforma in minuscolo <br/>
	 *  	L -> trasforma in minuscolo <br/>
	 */
	public void setTrasforma(String trasforma) {
		this.trasforma = trasforma;
	}

	public Object trasforma(Object val) {
		
		if (getTipo().equals(String.class) && getTrasforma()!=null) {
			if (getTrasforma().equalsIgnoreCase("u")) {
				return val.toString().toUpperCase(); 
			} else if (getTrasforma().equalsIgnoreCase("l")) {
				return val.toString().toLowerCase(); 
			} else return val;
		} else return val;
		
	}
	
}
