/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
package it.prato.comune.sit;

/**
 * 
 * Classe che implementa una ricerca basata su comparazione
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public class RicercaCompara extends RicercaBase {

	private String operatore = "AND";
	
	public Filtro getFiltro(Object...args) {
		
		Filtro filtro = null;		
		try {
	        filtro = getLayerTer().getFiltroVuoto();
	        // FiltroVuoto  filter.Include, se gli altri filtri si compongono in OR occorre che diventi filter.EXCLUDE altrimenti viene tutto
	        if (operatore.equals("OR")) filtro.NotFiltro();
			
			for (int i = 0 ; i<getCampiRicerca().size(); i++) {
				RicercaCampo rc= getCampiRicerca().get(i);
				if (rc.getRequired() || (args[i]!=null && !args[i].equals(""))) {
					if (rc.getOperatore().trim().toUpperCase().equals("LIKE")) {
						String nomeCampo = filtro.getNomeCampo(rc.getNLCampo());
						//String val = rc.trasforma(args[i]).toString().replaceAll("'", "''");
						String val = rc.trasforma(args[i]).toString();
						val = SITFilterFactory.escapeOriginalValue(val);
						if (operatore.equals("AND")) {
							filtro.AndFiltro(SITFilterFactory.like(nomeCampo, 
									"%" + val + "%" ,false));
						} else {
							filtro.OrFiltro(SITFilterFactory.like(nomeCampo, 
									"%" + val + "%" ,false));
						}
					} else {
						if (rc.getOperatore().trim().toUpperCase().equals("LIKETOKEN")) {
							Filtro filtro1 = getLayerTer().getFiltroVuoto();
							String nomeCampo = filtro.getNomeCampo(rc.getNLCampo());
							String val = rc.trasforma(args[i]).toString();
							val = SITFilterFactory.escapeOriginalValue(val);
							
							String[] tokens = val.split(" ");
							for (String s: tokens){
								String val1 = rc.trasforma(s).toString();
								val1 = SITFilterFactory.escapeOriginalValue(val1);
								filtro1.AndFiltro(SITFilterFactory.like(nomeCampo, "%" + val1 + "%", false));
							}
							if (operatore.equals("AND")) {
								filtro.AndFiltro(filtro1);
							} else {
								filtro.OrFiltro(filtro1);
							}
						} else { 
							if (operatore.equals("AND")) {
								filtro.AndFiltro(rc.getNLCampo(), rc.trasforma(args[i]),rc.getOperatore());
							} else {
								filtro.OrFiltro(rc.getNLCampo(), rc.trasforma(args[i]),rc.getOperatore());
							}
						}
					}
				}
			}		
		} catch (SITException e){
			
		}
        return filtro;		
	}

	/**
	 * @return the operatore
	 */
	public String getOperatore() {
		return operatore;
	}

	/**
	 * Definisce l'operatore di composizione della ricerca. Sono possibili i valori AND (default) e OR
	 * 
	 * @param operatore the operatore to set
	 */
	public void setOperatore(String operatore) {
		this.operatore = operatore.trim().toUpperCase();
	}
	
}
