/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
package it.prato.comune.sit;

import java.util.Arrays;
import java.util.List;

/**
 * Implementa una ricerca "componibile" in OR tra i campi definiti e le altre ricerche annidate  tramite il metodo setRicerche  
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public class RicercaOr extends RicercaBase {
	
	
	public Filtro getFiltro(Object...args) throws SITException {
		
		List<Object> argList = Arrays.asList(args);
		Filtro filtro = null;
		int ncampi = 0;
		int ncampicurr = 0;
		try {
	        filtro = getLayerTer().getFiltroVuoto();
	        // FiltroVuoto  filter.Include, se gli altri filtri si compongono in OR occorre che diventi filter.EXCLUDE altrimenti viene tutto
	        filtro.NotFiltro();
			
	        if (getCampiRicerca()!=null) {
	        	ncampicurr = getCampiRicerca().size();
	        	Object[] argsQuestaRicerca = argList.subList(ncampi, ncampicurr).toArray();
	        	for (int i = 0 ; i<getCampiRicerca().size(); i++) {
					RicercaCampo rc= getCampiRicerca().get(i);
					if (rc.getRequired() || (argsQuestaRicerca[i]!=null && !argsQuestaRicerca[i].equals(""))) {
						if (rc.getOperatore().trim().toUpperCase().equals("LIKE")) {
							String nomeCampo = filtro.getNomeCampo(rc.getNLCampo());
							//String val = rc.trasforma(argsQuestaRicerca[i]).toString().replaceAll("'", "''");
							String val = rc.trasforma(argsQuestaRicerca[i]).toString();
			                val = SITFilterFactory.escapeOriginalValue(val);
			                filtro.OrFiltro(SITFilterFactory.like(nomeCampo, 
			                        "%" + val + "%" , false));
						} else {
							if (rc.getOperatore().trim().toUpperCase().equals("LIKETOKEN")) {
								Filtro filtro1 = getLayerTer().getFiltroVuoto();
								String nomeCampo = filtro.getNomeCampo(rc.getNLCampo());
								String val = rc.trasforma(argsQuestaRicerca[i]).toString();
								val = SITFilterFactory.escapeOriginalValue(val);
								
								String[] tokens = val.split(" ");
								for (String s: tokens){
									String val1 = rc.trasforma(s).toString();
									val1 = SITFilterFactory.escapeOriginalValue(val1);
									filtro1.AndFiltro(SITFilterFactory.like(nomeCampo, "%" + val1 + "%", false));
								}
								filtro.OrFiltro(filtro1);
							} else {
								filtro.OrFiltro(rc.getNLCampo(), rc.trasforma(argsQuestaRicerca[i]),rc.getOperatore());
							}
						}
					}
				}
				ncampi+=ncampicurr;
	        }
	        
	        for (IRicercaComponibile ric: getRicerche()) {
	        	ncampicurr = ric.getCampiRicerca().size();
	        	Object[] argsQuestaRicerca = argList.subList(ncampi, ncampi + ncampicurr).toArray();
	        	filtro.OrFiltro(ric.getFiltro(argsQuestaRicerca));
	        	ncampi+=ncampicurr;
	        }
		} catch (SITException e){
			getLogger().error("Eccezione durante ricerca con and di oggetti cartografici",e);
			throw e;
		}
        return filtro;
	}

}
