/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
/*
 * Copyright  Comune di Prato - Sistema Informativo
 * Project:    Package SIT per elaborazione/interrogazione/gestione dati cartografici 
 * File:       SITDataStorePool.java
 * Author:     Alessandro Radaelli
 * Version:    1.0.0
 * CreationDate:   Giugno 2005
 * ModifyDate:     
 * 
 */

package it.prato.comune.sit;

import it.prato.comune.utilita.logging.interfaces.LogInterface;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.geotools.data.DataStore;
import org.geotools.data.DataStoreFactorySpi;
import org.geotools.data.DataUtilities;

/**
 * Classe  SITDataStorePool.
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public class SITDataStorePool {

    protected HashMap<String, DataStore> pool = new HashMap<String, DataStore>();
    protected HashMap<String, String> poolDsParam = new HashMap<String, String>();
    protected HashMap<String, String> poolDsFactory = new HashMap<String, String>();
    protected HashMap<String, String> poolDsHints = new HashMap<String, String>();
    protected HashMap<String, String> poolDsCaseSensitive = new HashMap<String, String>();
    
    protected LogInterface logger;
    protected String configFilePath=null;
    
    /**
     * Costruttore della classe SITDataStorePool.
     * 
     * @param configFilePath
     * @param logger
     * @throws IOException
     */
    public SITDataStorePool(String configFilePath, LogInterface logger) throws IOException {
        this.logger = logger;
        this.configFilePath=configFilePath;       
    }

    /**
     * Consente l'impostazione dei parametri necessari.
     * 
     * @param id
     * @param dsParam
     * @param dsFactory
     * @param dsHints
     * @param dsCaseSensitive
     */
    protected void setParam (String id, String dsParam, String dsFactory, String dsHints, String dsCaseSensitive) {        
        poolDsParam.put(id, dsParam);
        poolDsFactory.put(id, dsFactory);
        poolDsHints.put(id, dsHints);
        poolDsCaseSensitive.put(id, dsCaseSensitive);    
    }
    
    /**
     * Restituisce il data store configurato che  stato richiesto in base all'operazione da svolgere.
     * 
     * @param configBean
     * @return DataStore
     * @throws IOException
     * @throws SITException
     */
    protected DataStore getDataStore(ConfigBean configBean) throws IOException, SITException {
        DataStore ds = null;
        
        try {
            if (configBean.getDsPoolID()!=null) {
                if (pool.containsKey(configBean.getDsPoolID())) {
                    ds = (DataStore) pool.get(configBean.getDsPoolID());
                } else {
                    ds = openDataStore(configBean.getDsPoolID());
                    pool.put(configBean.getDsPoolID(), ds);
                }
            } else {
                ds = openDataStore(configBean.getDataStoreParams(), configBean.getDataStoreFactory(), configBean.isDataStoreConHints());
            }
        } catch (IOException e) {
            logger.error("IOException durante recupero DataStore: " + e.getMessage(), e);
            throw e;
        } catch (SITException e) {
            logger.warn("SITException durante recupero DataStore: " + e.getMessage(), e);
            throw e;
        }

        return ds;
    }

    /**
     * Imposta il data store sulla base dei parametri letti dalla configurazione.
     * 
     * @param szDataStoreParams
     * @param szDataStoreFactory
     * @param isConHints
     * @return DataStore
     * @throws IOException
     * @throws SITException
     */
    protected DataStore openDataStore(String szDataStoreParams, 
            String szDataStoreFactory, boolean isConHints) throws IOException, SITException{
    	
        DataStore ds = null;
//TOB        
//        Map<String, String>  params = new HashMap<String, String>();
        Map<String, Serializable>  params = new HashMap<String, Serializable>();
        
        // ////////////////////////////////////////////////////////
        // parsing della stringa per estrarre i parametri
        // ////////////////////////////////////////////////////////
        
        logger.debug("Inizio lettura parametri DataStore...");
        
        try {
	        String[] parametri = szDataStoreParams.split("!!");
	        Pattern pattern = Pattern.compile("[^\\\\];");
	        
	        for (int i = 0; i < parametri.length; i++) {
	            	            	            
	            Matcher m = pattern.matcher(parametri[i]);
	            String name  = null;
	            String value = null;	            	            
	            
	            if(m.find()){
	                name = parametri[i].substring(0,m.start()+1).replace("\\;", ";");
	                if(m.end() < parametri[i].length()){
	                    value = parametri[i].substring(m.end()).replace("\\;", ";");;
	                }	                
	            }	            	           

                params.put(name, value);
	            
//	            String[] nomeEPar = parametri[i].split(";");
//	            params.put(nomeEPar[0], nomeEPar[1]);	            
	            
	        }
//        } catch (ArrayIndexOutOfBoundsException e) {
//        	logger.error("Eccezione durante lettura parametri DataStore. Controlla la sintassi del DSPOOLDATASTORE!", e);
        } catch (Exception e) {
        	logger.error("Eccezione durante lettura parametri DataStore. Controlla la sintassi del DSPOOLDATASTORE!", e);
        }
        
        try {
            
            Class a = null;
            
            try {
                a = Class.forName(szDataStoreFactory);
            } catch (ClassNotFoundException e) {
            	String errMsg = "ClassNotFoundException durante recupero della Factory del DataStore " + e.getMessage();
                logger.error(errMsg, e);
                new SITBasicException(errMsg, e);
            }

            DataStoreFactorySpi ldsf = null;
             
            try {
                if (isConHints) {
                    Class[] tipiarg = {java.util.Map.class};
                    Object[] argom = {new HashMap()}; 
                    
                    ldsf = (DataStoreFactorySpi) (a.getConstructor(tipiarg).newInstance(argom));
                } else {
                    ldsf = (DataStoreFactorySpi) (a.newInstance());  
                }
                
            } catch (IllegalArgumentException e) {
            	String warnMsg = "IllegalArgumentException durante costruzione DataStoreFactorySpi " + e.getMessage();
                logger.warn(warnMsg, e);
                throw new SITException(warnMsg,e);
            } catch (SecurityException e) {
            	String warnMsg = "SecurityException durante costruzione DataStoreFactorySpi " + e.getMessage();
            	logger.warn(warnMsg, e);
            	throw new SITException(warnMsg,e);
            } catch (InstantiationException e) {
            	String warnMsg = "InstantiationException durante costruzione DataStoreFactorySpi " + e.getMessage();
            	logger.warn(warnMsg, e);
            	throw new SITException(warnMsg,e);
            } catch (IllegalAccessException e) {
            	String warnMsg = "IllegalAccessException durante costruzione DataStoreFactorySpi " + e.getMessage();
            	logger.warn(warnMsg, e);
            	throw new SITException(warnMsg,e);
            } catch (InvocationTargetException e) {
            	String warnMsg = "InvocationTargetException durante costruzione DataStoreFactorySpi " + e.getMessage();
            	logger.warn(warnMsg, e);
            	throw new SITException(warnMsg,e);
            } catch (NoSuchMethodException e) {
            	String warnMsg = "NoSuchMethodException durante costruzione DataStoreFactorySpi " + e.getMessage();
            	logger.warn(warnMsg, e);
            	throw new SITException(warnMsg,e);
            }
            
            if (ldsf == null) {
                logger.warn("Non sono riuscito a costruire il DataStoreFactorySpi ldsf null");
            }
            
            logger.debug("Inizio la creazione del DataStore...");  
            
            if (params.containsKey("url")) {
            	
            	File shpFile = DataUtilities.urlToFile(new URL(params.get("url").toString()));
            	
            	if(!shpFile.exists())
            		ds = (DataStore) ldsf.createNewDataStore(params);
            	else
            		ds = (DataStore) ldsf.createDataStore(params);
            	
            } else {
            	ds = (DataStore) ldsf.createDataStore(params);
            }
            
//            dataStore = DataStoreFinder.getDataStore(params);
            
            if (ds != null) {
               /*
                logger.debug("LayerTerritorio"
                        + " getDataStore(" + szDataStoreParams
                        + " - eseguito con successo ");
               */
                logger.debug("Ho istanziato correttamente il DataStore " + ds.getClass().getName());
                
            } else {
                /*
                logger.debug("LayerTerritorio"
                        + " getDataStore(" + szDataStoreParams
                        + " ritorna valore null");
                */
            	logger.error("Non sono riuscito a instanziare il DataStore: null");
            }
            
        } catch (IOException e) {
            logger.error("Eccezione durante creazione DataStore " + szDataStoreParams, e);
            throw e;
        }
        
        return ds;
    }
 
    /**
     * 
     * @param dsPoolID
     * @return DataStore
     * @throws IOException
     * @throws SITException
     * 
     * @see {@link SITDataStorePool#openDataStore(String szDataStoreParams, String szDataStoreFactory, boolean isConHints)}
     */
    private DataStore openDataStore(String dsPoolID) throws IOException, SITException {             
        boolean isConHints;
        
        if ( ((String) (poolDsHints.get(dsPoolID))).compareToIgnoreCase("N")==0) {
            isConHints=false;
        } else {
            isConHints=true;
        }
        
        return openDataStore((String) poolDsParam.get(dsPoolID), (String) poolDsFactory.get(dsPoolID), isConHints );      
    }

    /**
     * @param dsPoolID
     * @return String
     */
    public String getPoolDsCaseSensitive(String dsPoolID) {
        return poolDsCaseSensitive.get(dsPoolID);
    }
    
    public void disposeDataStores(){
		if(pool!=null&& pool.size()>0){
			
    		final Collection<DataStore> collection = this.pool.values();
    		final Iterator<DataStore> iter = collection.iterator();
    		
    		while(iter.hasNext()){
    			final DataStore ds = (DataStore)iter.next();    			
    			if(ds != null){
    				logger.debug("Eseguo il disposing del DataStore...");
    				ds.dispose();
    			}        				
    		}
		}	
    }
}
