/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
/*
 * Copyright  Comune di Prato - Sistema Informativo
 * Project:    Package SIT per elaborazione/interrogazione/gestione dati cartografici 
 * File:       SITException.java
 * Author:     Alessandro Radaelli
 * Version:    1.0.0
 * CreationDate:   Giugno 2005
 * ModifyDate:     
 *       
 */

package it.prato.comune.sit;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.sql.SQLException;

/**
 * Classe SITException.
 * <p>Title: SITException</p>
 * <p>Description: Eccezione Operazioni Certificato Destinazione Urbanistica</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Comune di Prato</p>
 * </br>
 *
 * @author Radaelli Alessandro
 * @version 1.0
 */
public class SITException extends Exception implements Serializable {
	
	private static final long serialVersionUID = -1718179316747194856L;

	public static final char FATAL_ERROR     = 'E';
	public static final char PROCESS_ERROR   = 'P';

	public static final int iERR_GENERICERROR = 1;
	public static final String szERR_GENERICERROR = "Errore del package SIT";
	public static final int iERR_FIDMAPPER = 2;
	public static final int iERR_FUNZIONE = 3;
	public static final int iERR_GEOMETRYCAST = 4;
	public static final int iERR_ILLEGALATTRIBUTE = 5;
	public static final int iERR_ILLEGALFILTER = 6;
	public static final int iERR_PARSEWKT = 7;
	public static final int iERR_SCHEMA = 8;
	public static final int iERR_SQLENCORDER = 9;
	public static final int iERR_STORICO = 9;

	/**
	 * Codice Errore
	 */
	private int codice;

	/**
	 * Tipo Errore:
	 * 'V' = Errore visualizzabile (significativo per l'operatore)
	 * 'N' = Errore non visualizzabile (non significativo per l'operatore)
	 */
	private char tipoErr;

	/**
	 * Eccezione originante. Contiene lo stack per debug avanzato.
	 */
	private Exception eccezioneOriginante;

	/**
	 * Costruttore della classe SITException. Istanzia un'eccezione non qualificata
	 */
	public SITException() {
		codice=-1;
		tipoErr = FATAL_ERROR;
		eccezioneOriginante = null;
	}

	/**
	 * Istanzia una specifica eccezione con codice, messaggio e riferimento all'eccezione originaria.
	 * 
	 * @param codice Codice dell'errore
	 * @param tipoErr Tipo errore
	 * @param messaggio Messaggio d'errore
	 * @param e Eccezione originante
	 */
	public SITException(int codice, char tipoErr, String messaggio, Exception e) {
		super(messaggio);
		if (codice != 0)
			this.codice=codice;
		else
			// ///////////////////////////////////////////
			// il return code non puo' essere zero !!!
			// ///////////////////////////////////////////			
			this.codice = -1;
		
		this.tipoErr = tipoErr;
		eccezioneOriginante = e;
	}

	/**
	 * Istanzia una specifica eccezione con codice, messaggio e riferimento all'eccezione originaria.
	 * 
	 * @param codice Codice dell'errore
	 * @param messaggio Messaggio d'errore
	 * @param e Eccezione originante
	 */
	public SITException(int codice, String messaggio, Exception e) {
		this(codice, FATAL_ERROR, messaggio, e);
	}

	/**
	 * Istanzia una specifica eccezione con codice e messaggio.
	 * 
	 * @param codice Codice dell'Errore
	 * @param messaggio Messaggio d'Errore
	 */
	public SITException(int codice, String messaggio) {
		this(codice, FATAL_ERROR, messaggio, (Exception) null);
	}

	/**
	 * Istanzia una specifica eccezione con codice, messaggio e tipoErr.
	 * 
	 * @param codice Codice dell'Errore
	 * @param tipoErr Tipo dell'Errore
	 * @param messaggio Messaggio d'Errore
	 */
	public SITException(int codice, char tipoErr, String messaggio) {
		this(codice, tipoErr, messaggio, (Exception) null);
	}

	/**
	 * Istanzia un eccezione con un certo Messaggio e Codice -1.
	 * 
	 * @param messaggio
	 * @param e Eccezione originante
	 */
	public SITException(String messaggio, Exception e) {
		this(-1, FATAL_ERROR, messaggio, e);
	}

	/**
	 * Istanzia un eccezione con un certo Messaggio e Codice -1.
	 * 
	 * @param tipoErr Tipo dell'Errore
	 * @param messaggio
	 * @param e Eccezione originante
	 */
	public SITException(char tipoErr, String messaggio, Exception e) {
		this(-1, tipoErr, messaggio, e);
	}

	/**
	 * Istanzia un eccezione con un messaggio.
	 * 
	 * @param messaggio
	 */
	public SITException(String messaggio) {
		this(-1, FATAL_ERROR, messaggio, (Exception) null);
	}

	/**
	 * Istanzia un eccezione con un tipoErr e messaggio.
	 * 
	 * @param tipoErr
	 * @param messaggio
	 */
	public SITException(char tipoErr, String messaggio) {
		this(-1, tipoErr, messaggio, (Exception) null);	}


	/**
	 * @param ois
	 * @throws ClassNotFoundException
	 * @throws IOException
	 */
	private void readObject(ObjectInputStream ois) throws ClassNotFoundException, IOException {
		ois.defaultReadObject();
	}
	
	/**
	 * @param oos
	 * @throws IOException
	 */
	private void writeObject(ObjectOutputStream oos) throws IOException {
		oos.defaultWriteObject();
	}
	
	/**
	 * @return int
	 */
	public int getCodice() {
		return codice;
	}
	
	/**
	 * @param codice
	 */
	public void setCodice(int codice) {
		this.codice = codice;
	}
	
	/**
	 * @return char
	 */
	public char getTipoErr() {
		return tipoErr;
	}
	
	/**
	 * @param tipoErr
	 */
	public void setTipoErr(char tipoErr) {
		this.tipoErr = tipoErr;
	}
	
	/**
	 * @return Exception
	 */
	public Exception getEccezioneOriginante() {
		return eccezioneOriginante;
	}
	
	/** 
	 * @return boolean
	 */
	public boolean isProcessErrorType() {
		if (tipoErr == PROCESS_ERROR)
			return true;
		
		return false;
	}

	/** 
	 * @return SQLException
	 */
	private SQLException searchSQLException(){
		Throwable ex = eccezioneOriginante;

		while (ex != null){	  
			if (ex instanceof SQLException){
				break;
			}
			if(ex.getCause() != null && !ex.equals(ex.getCause())){
				ex = ex.getCause();
			}else{
				ex = null;
				break;
			}
		}	

		return (SQLException)ex;
	}

	/**
	 * @return boolean
	 */
	public boolean isSQLEccezioneOriginante() {
		return (eccezioneOriginante != null && eccezioneOriginante instanceof SQLException);      
	}

	/**
	 * @return boolean
	 */
	public boolean isSQLCause() {
		return searchSQLException() != null;	      
	}

	/**
	 * @return boolean
	 */
	public boolean isEccezioneOriginanteSQLCause() {
		if (eccezioneOriginante != null && (eccezioneOriginante instanceof SQLException || eccezioneOriginante.getCause() instanceof SQLException))
			return true;
		return false;
	}

	/**
	 * @return int
	 */
	public int getSQLErrorCode() {
		if (isSQLCause())
			return searchSQLException().getErrorCode();
		return 0;
	}
}
