/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
/*
 * Copyright  Comune di Prato - Sistema Informativo
 * Project:    Package SIT per elaborazione/interrogazione/gestione dati cartografici 
 * File:       SITGeometryFilter.java
 * Author:     Tobia Di Pisa
 * Version:    1.0.0
 * CreationDate:  
 * ModifyDate:     
 *             
 */
package it.prato.comune.sit;

import java.util.HashSet;
import java.util.Set;

import org.geotools.factory.CommonFactoryFinder;
import org.geotools.factory.GeoTools;
import org.opengis.filter.And;
import org.opengis.filter.Filter;
import org.opengis.filter.FilterFactory2;
import org.opengis.filter.Id;
import org.opengis.filter.Not;
import org.opengis.filter.Or;
import org.opengis.filter.PropertyIsEqualTo;
import org.opengis.filter.PropertyIsGreaterThan;
import org.opengis.filter.PropertyIsGreaterThanOrEqualTo;
import org.opengis.filter.PropertyIsLessThan;
import org.opengis.filter.PropertyIsLessThanOrEqualTo;
import org.opengis.filter.PropertyIsLike;
import org.opengis.filter.PropertyIsNotEqualTo;
import org.opengis.filter.PropertyIsNull;
import org.opengis.filter.expression.Expression;
import org.opengis.filter.identity.FeatureId;
import org.opengis.filter.spatial.BBOX;
import org.opengis.filter.spatial.Beyond;
import org.opengis.filter.spatial.Contains;
import org.opengis.filter.spatial.Crosses;
import org.opengis.filter.spatial.Disjoint;
import org.opengis.filter.spatial.Equals;
import org.opengis.filter.spatial.Intersects;
import org.opengis.filter.spatial.Overlaps;
import org.opengis.filter.spatial.Touches;
import org.opengis.filter.spatial.Within;
import org.opengis.geometry.BoundingBox;

import com.vividsolutions.jts.geom.Geometry;

/**
 * Classe SITFilterFactory. Questa classe fa uso della FilterFactory2 delle GeoAPI per consentire </br>
 * la creazione dei filtri Opengis (sia spaziali che non).
 * 
 * @author Tobia Di Pisa (tobia.dipisa@geo-solutions.it)
 *
 */
public final class SITFilterFactory {

	public static FilterFactory2 filterFactory = CommonFactoryFinder.getFilterFactory2(GeoTools.getDefaultHints());
	public static String 	escapeChar 		= "!";
	public static String 	wildcardChar 	= "%";
	public static String 	singleChar 		= "?";
	
	/**
	 * Controlla se la geometria specificata col primo openando contiene il secondo operando geometrico.
	 * 
	 * @param propertyName
	 * @param geometry
	 * @return Contains
	 */
	public static Contains contains(String propertyName, Geometry geometry){
        Expression expr1 = filterFactory.property(propertyName);
        Expression expr2 = filterFactory.literal(geometry);
		return filterFactory.contains(expr1, expr2);
	}
	
	/**
	 * Produce un filtro AND tra due filtri.
	 * 
	 * @param first
	 * @param second
	 * @return And
	 */
	public static And and(Filter first, Filter second){
		return filterFactory.and(first, second);
	}
	
	/**
	 * Produce un filtro OR tra due filtri.
	 * 
	 * @param first
	 * @param second
	 * @return Or
	 */
	public static Or or(Filter first, Filter second){
		return filterFactory.or(first, second);
	}
	
	/**
	 * Inverte il valore logico del filtro.
	 * 
	 * @param filter
	 * @return Not
	 */
	public static Not not(Filter filter){
		return filterFactory.not(filter);
	}
	
	/**
	 * Controlla se la geometria specificata col primo openando interseca il secondo operando geometrico.
	 * 
	 * @param propertyName
	 * @param geometry
	 * @return Intersects
	 */
	public static Intersects intersects(String propertyName, Geometry geometry){
        Expression expr1 = filterFactory.property(propertyName);
        Expression expr2 = filterFactory.literal(geometry);
		return filterFactory.intersects(expr1, expr2);
	}
	
	/**
	 * Controlla se la geometria specificata col primo openando si sovrappone il secondo operando geometrico.
	 * 
	 * @param propertyName
	 * @param geometry
	 * @return Overlaps
	 */
	public static Overlaps overlaps(String propertyName, Geometry geometry){
        Expression expr1 = filterFactory.property(propertyName);
        Expression expr2 = filterFactory.literal(geometry);
		return filterFactory.overlaps(expr1, expr2);
	}
	
	/**
	 * Controlla se la geometria specificata col primo openando tocca il secondo operando geometrico.
	 * 
	 * @param propertyName
	 * @param geometry
	 * @return Touches
	 */
	public static Touches touches(String propertyName, Geometry geometry){
        Expression expr1 = filterFactory.property(propertyName);
        Expression expr2 = filterFactory.literal(geometry);
		return filterFactory.touches(expr1, expr2);
	}
	
	/**
	 * Controlla se la geometria specificata col primo openando  al di la di un certo raggio dal secondo operando geometrico.
	 * 
	 * @param propertyName
	 * @param geometry
	 * @param distance
	 * @param units
	 * @return Beyond
	 */
	public static Beyond beyond(String propertyName, Geometry geometry, double distance, String units){
        Expression expr1 = filterFactory.property(propertyName);
        Expression expr2 = filterFactory.literal(geometry);
		return filterFactory.beyond(expr1, expr2, distance, units);
	}
	
	/**
	 * Produce un filtro sull'ID specificato come argomento.
	 * 
	 * @param id
	 * @return Id
	 */
	public static Id id(String id){		
		FeatureId featureId = filterFactory.featureId(id);
		
		Set<FeatureId> featureIdSet = new HashSet<FeatureId>() ; 
		featureIdSet.add(featureId);
		
		return filterFactory.id(featureIdSet);
	}
	
	/**
	 * Controlla se il valore del campo 'nome'  uguale a 'valore'.
	 * 
	 * @param nome
	 * @param valore
	 * @return PropertyIsEqualTo
	 */
	public static PropertyIsEqualTo equals(String nome, Object valore){		
        Expression expr1 = filterFactory.property(nome);
        Expression expr2 = filterFactory.literal(valore);

		return filterFactory.equals(expr1, expr2);
	}

	/**
	 * Esegue escape dei caratteri nella stringa originale usando i caratteri di escape, wildcard e single di default
	 * 
	 * @param originalValue
	 * @param wildcard
	 * @param singleChar
	 * @param escape
	 * @return
	 */
	public static String escapeOriginalValue(String originalValue) {	
		return escapeOriginalValue(originalValue, wildcardChar, singleChar, escapeChar);
	}
	
	/**
	 * Esegue escape dei caratteri nella stringa originale
	 * 
	 * @param originalValue
	 * @param wildcard
	 * @param singleChar
	 * @param escape
	 * @return
	 */
	public static String escapeOriginalValue(String originalValue, String wildcard, String single, String escape) {
		
		String retVal = originalValue.replace(escape, escape+escape);
		retVal 		  = retVal.replace(wildcard, escape+wildcard);
		retVal 		  = retVal.replace(single, escape+single);
		
		return retVal;
	}
	
	/**
	 * Compara una stringa utilizzando un pattern ('valore') di confronto e una specifica wildcard.
	 * La ricerca  case insensitive se il datastore lo permette.
	 * 
	 * @param nome
	 * @param valore
	 * @param wildcard
	 * @param singleChar
	 * @param escape
	 * @return PropertyIsLike
	 */
	public static PropertyIsLike like(String nome, String valore, String wildcard, String singleChar, String escape){				
		return like(nome, valore, wildcard, singleChar, escape, false);
	}
	
	/**
     * Compara una stringa utilizzando un pattern ('valore') di confronto, utilizzando wildcard e escape di default ed un booleano che identifica
     * se la ricerca debba essere case sensitive o no. Anche se  impostata una ricerca case insensitive non  detto che 
     * lo specifico data store la implementi.
     * 
     * @param nome
     * @param valore
     * @param wildcard
     * @param singleChar
     * @param escape
     * @param matchCase
     * @return PropertyIsLike
     */
    public static PropertyIsLike like(String nome, String valore, boolean matchCase){       
        Expression expr1 = filterFactory.property(nome);
        return filterFactory.like(expr1, valore, wildcardChar, singleChar, escapeChar);
    }
	
	/**
     * Compara una stringa utilizzando un pattern ('valore') di confronto, una specifica wildcard ed un booleano che identifica
     * se la ricerca debba essere case sensitive o no. Anche se  impostata una ricerca case insensitive non  detto che 
     * lo specifico data store la implementi.
     * 
     * @param nome
     * @param valore
     * @param wildcard
     * @param singleChar
     * @param escape
     * @param matchCase
     * @return PropertyIsLike
     */
    public static PropertyIsLike like(String nome, String valore, String wildcard, String singleChar, String escape, boolean matchCase){       
        Expression expr1 = filterFactory.property(nome);
        return filterFactory.like(expr1, valore, wildcard, singleChar, escape);
    }
	
	/**
	 * Controlla se il valore del campo 'nome'  minore di 'valore'.
	 * 
	 * @param nome
	 * @param valore
	 * @return PropertyIsLessThan
	 */
	public static PropertyIsLessThan less(String nome, Object valore){
        Expression expr1 = filterFactory.property(nome);
        Expression expr2 = filterFactory.literal(valore);

		return filterFactory.less(expr1, expr2);
	}
	
	/**
	 * Controlla se il valore del campo 'nome'  maggiore di 'valore'.
	 * 
	 * @param nome
	 * @param valore
	 * @return PropertyIsGreaterThan
	 */
	public static PropertyIsGreaterThan greater(String nome, Object valore){
        Expression expr1 = filterFactory.property(nome);
        Expression expr2 = filterFactory.literal(valore);

		return filterFactory.greater(expr1, expr2);
	}
	
	/**
	 * Controlla se il valore del campo 'nome'  minore o uguale a 'valore'.
	 * 
	 * @param nome
	 * @param valore
	 * @return PropertyIsLessThanOrEqualTo
	 */
	public static PropertyIsLessThanOrEqualTo lessOrEqual(String nome, Object valore){
        Expression expr1 = filterFactory.property(nome);
        Expression expr2 = filterFactory.literal(valore);

		return filterFactory.lessOrEqual(expr1, expr2);
	}
	
	/**
	 * Controlla se il valore del campo 'nome'  maggiore o uguale a 'valore'.
	 * 
	 * @param nome
	 * @param valore
	 * @return PropertyIsGreaterThanOrEqualTo
	 */
	public static PropertyIsGreaterThanOrEqualTo greaterOrEqual(String nome, Object valore){
        Expression expr1 = filterFactory.property(nome);
        Expression expr2 = filterFactory.literal(valore);

		return filterFactory.greaterOrEqual(expr1, expr2);
	}
	
	/**
	 * Controlla se il valore del campo 'nome' non  uguale a 'valore'.
	 * 
	 * @param nome
	 * @param valore
	 * @return PropertyIsNotEqualTo
	 */ 
	public static PropertyIsNotEqualTo notEqual(String nome, Object valore){
        Expression expr1 = filterFactory.property(nome);
        Expression expr2 = filterFactory.literal(valore);
        
		return filterFactory.notEqual(expr1, expr2);
	}
	
	/**
	 * Controlla se il valore del campo 'nome'  nullo.
	 * 
	 * @param nome
	 * @return PropertyIsNull
	 */
	public static PropertyIsNull isNull(String nome){
        Expression expr = filterFactory.property(nome);        
		return filterFactory.isNull(expr);
	}
	
	/**
	 * Controlla se il valore del campo 'nome'  non nullo.
	 * 
	 * @param nome
	 * @return PropertyIsNotEqualTo
	 */
	public static PropertyIsNotEqualTo isNotNull(String nome){
        Expression expr = filterFactory.property(nome);        
		return filterFactory.notEqual(expr, null);
	}
	
	/**
	 * Produce un filtro di tipo BBOX sulla base dei parametri passati coem argomento.
	 * 
	 * @param geometry
	 * @param boundingBox
	 * @return BBOX
	 */
	public static BBOX bbox(Geometry geometry, BoundingBox boundingBox){
        Expression expr = filterFactory.literal(geometry);
		return filterFactory.bbox(expr, boundingBox);
	}
	
	/**
	 * Produce un filtro di tipo Crosses sulla base dei parametri passati coem argomento.
	 * 
	 * @param propertyName
	 * @param geometry
	 * @return Crosses
	 */
	public static Crosses crosses(String propertyName, Geometry geometry){
        Expression expr1 = filterFactory.property(propertyName);
        Expression expr2 = filterFactory.literal(geometry);
		return filterFactory.crosses(expr1, expr2);
	}
	
	/**
	 * Produce un filtro di tipo Disjoint sulla base dei parametri passati coem argomento.
	 * 
	 * @param propertyName
	 * @param geometry
	 * @return Disjoint
	 */
	public static Disjoint disjoint(String propertyName, Geometry geometry){
        Expression expr1 = filterFactory.property(propertyName);
        Expression expr2 = filterFactory.literal(geometry);
		return filterFactory.disjoint(expr1, expr2);
	}
	
	/**
	 * Produce un filtro di tipo Equals sulla base dei parametri passati coem argomento.
	 * 
	 * @param propertyName
	 * @param geometry
	 * @return Equals
	 */
	public static Equals geometryEquals(String propertyName, Geometry geometry){
        Expression expr1 = filterFactory.property(propertyName);
        Expression expr2 = filterFactory.literal(geometry);
		return filterFactory.equal(expr1, expr2);
	}
	
	/**
	 * Produce un filtro di tipo Within sulla base dei parametri passati coem argomento.
	 * 
	 * @param propertyName
	 * @param geometry
	 * @return Within
	 */
	public static Within within(String propertyName, Geometry geometry){
        Expression expr1 = filterFactory.property(propertyName);
        Expression expr2 = filterFactory.literal(geometry);
		return filterFactory.within(expr1, expr2);
	}
}
