/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
/*
 * Copyright  Comune di Prato - Sistema Informativo
 * Project:    Package SIT per elaborazione/interrogazione/gestione dati cartografici 
 * File:       Territorio.java
 * Author:     Alessandro Radaelli
 * Version:    1.0.0
 * CreationDate:   Giugno 2005
 * ModifyDate:     
 *
 * Function:   Classe di gestione SIT di un territorio. 
 * Al suo interno sono contenuti i layer
 * 
 *             
 */

package it.prato.comune.sit;


import it.prato.comune.sit.runtime.SITCoreVersion;
import it.prato.comune.sit.runtime.util.APIVersion;
import it.prato.comune.sit.runtime.util.PluginVersion;
import it.prato.comune.sit.runtime.util.SITVersionException;
import it.prato.comune.sit.runtime.util.Version;
import it.prato.comune.utilita.logging.interfaces.LogInterface;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.support.DefaultListableBeanFactory;
import org.springframework.context.support.FileSystemXmlApplicationContext;
import org.springframework.context.support.GenericApplicationContext;

/**
 * Classe SITLayersManager. Implementa l'interfaccia {@link ILayersManager} per il SIT.
 * 
 * @author Ing. bf44
 *
 */
public class SITLayersManager implements ILayersManager{
    
    private static final String CLASSE = "SITLayersManager";

    
    HashMap<Integer, LayerTerritorio> TipologieDivisioniTerritorio  = new HashMap<Integer, LayerTerritorio>();
    private HashMap<Integer, IGetFeatureInfoLayer> getFeatureInfoLayers  = new HashMap<Integer, IGetFeatureInfoLayer>();
    
    
    //HashMap<String, Integer> AssocNomeLayerCodLayer = new HashMap<String, Integer>();
    String  Nome = new String();
    SITDataStorePool dsPool = null;
    
    private String configFilePath=null;
    private LogInterface logger=null;
    private String ente=null;
    
    private String user=null;
    private String ipAddress=null;
    
    private String jdbc_sit_driver;
    private String jdbc_sit_url;
    private String jdbc_sit_user;
    private String jdbc_sit_password;
    
    protected SITLayersManager(){};
    
    protected void init(String configFilePath, LogInterface logger, String Nome, String ente) throws SITException {
        this.configFilePath = configFilePath;
        this.ente = ente;

        // //////////////////////////////////////////////////////
        // Istanzio classe di gestione del log della procedura
        // //////////////////////////////////////////////////////
        
        this.logger = logger ;
        
        try {
            dsPool = new SITDataStorePool(configFilePath, logger);
        } catch (IOException e1) {
            String msg = "Problemi con la creazione del SITDataStorePool";
            logger.error(msg,e1);
            throw new SITException(msg,e1);
        }
        
        // ///////////////////////////////////
        // Apertura file di configurazione
        // ///////////////////////////////////
        
        File configFile = new File(configFilePath);
        FileInputStream input = null;
    
        try  {
            input = new FileInputStream(configFile);
        } catch (FileNotFoundException e) {
            String msg = "File " + configFilePath + " non trovato";
            logger.error(msg,e);            
            throw new SITException(msg,e);
            
        }

        Properties pr = new Properties();
    
        try {
            pr.load(input);
        } catch (IOException e) {
            String msg = "Impossibile leggere il file " + configFilePath;
            logger.error(msg,e);            
            throw new SITException(msg,e);            
        }
        
        // ////////////////////////////
        // Inizializzazione DsPool
        // ////////////////////////////
        
        String szDsPoolBaseString = ente + "DSPOOL";
        
        int iConta = 1;
        String szDsParName = szDsPoolBaseString + "DATASTOREPAR" + iConta;
        
        while (pr.getProperty(szDsParName) != null) {
            String dsParam = pr.getProperty(szDsParName);
            String dsFactory = pr.getProperty(szDsPoolBaseString + "DATASTOREFACTORY" + iConta);
            String dsHints = pr.getProperty(szDsPoolBaseString + "DATASTORECONHINTS" + iConta);
            String dsCaseSensitive = pr.getProperty(szDsPoolBaseString + "DATASTORECASESENSITIVE" + iConta);
            
            dsPool.setParam(String.valueOf(iConta), dsParam, dsFactory, dsHints, dsCaseSensitive);
            iConta++;
            szDsParName = szDsPoolBaseString + "DATASTOREPAR" + iConta;
        }
        
        // ///////////////////////////////////////////////////
        // INIZIALIZZO I PARAMETRI DI CONNESSIONE CLASSICI
        // ///////////////////////////////////////////////////
        
        String alias        = pr.getProperty("DBALIAS");
        jdbc_sit_driver     = pr.getProperty("JDBC."+alias+".driver");
        jdbc_sit_url        = pr.getProperty("JDBC."+alias+".url");
        jdbc_sit_user       = pr.getProperty("JDBC."+alias+".user");
        jdbc_sit_password   = pr.getProperty("JDBC."+alias+".password");
        
        try {
            
            // Legge elenco plugin e li inizializza
            iConta = 1;
            String szPluginParNameBase = ente + "PLUGIN";
            String szPluginParName = szPluginParNameBase + iConta;
            APIVersion apiVersion = SITCoreVersion.getInstance().getAPIVersion();
            
            while (pr.getProperty(szPluginParName) != null) {
                String pluginName = pr.getProperty(szPluginParName).trim();
                logger.debug("Caricamento layer plugin " + pluginName);
                // Istanzia la classe 
                Class c = Class.forName(pluginName);
                IInitLayerPlugin initPlugin = null;
                try {
                    initPlugin = (IInitLayerPlugin)c.newInstance();
                } catch (InstantiationException e) {
                	logger.error("Errore durante l'istanziazione plugin " + pluginName, e);
                }
                // Controllo che la versione sia coerente.
                Method getPluginVersion = c.getDeclaredMethod("getPluginVersion");
                PluginVersion pluginVersion = (PluginVersion)getPluginVersion.invoke(initPlugin);
                APIVersion apiComplianceVersion = pluginVersion.getAPIComplianceVersion();
                Version version = pluginVersion.getVersion();
                System.out.println("apiVersion " + apiVersion.getMajor() );
                System.out.println("apiComplianceVersion " + apiComplianceVersion.getMajor() );
                
                if(apiVersion.getMajor() != apiComplianceVersion.getMajor()){
                    throw new SITVersionException("Il plugin " + pluginName + " versione " + version.format() + " risulta NON essere compatibile con la versione " + apiVersion.format() + " delle API");
                }
                
                if(apiVersion.getMinor()<apiComplianceVersion.getMinor()){
                    throw new SITVersionException("Il plugin " + pluginName + " versione " + version.format() + " potrebbe utilizzare funzionalit non presenti nella versione " + apiVersion.format() + " delle API");
                }
                
                initPlugin.init(pr, this);
                
                //Method m = c.getDeclaredMethod("init",Properties.class, ILayersManager.class);
                //m.invoke(initPlugin, pr, this);
                
                iConta++;
                szPluginParName = szPluginParNameBase + iConta;
            }    
           
        } catch (ClassNotFoundException e) {
            logger.error("ClassNotFoundException durante l'inizializzazione dell'oggetto Territorio - Caricamento plugin", e);
            throw new SITException("",e);
        } catch (SecurityException e) {
            logger.error("SecurityException durante l'inizializzazione dell'oggetto Territorio - Caricamento plugin", e);
            throw new SITException("",e);
        } catch (NoSuchMethodException e) {
            logger.error("NoSuchMethodException durante l'inizializzazione dell'oggetto Territorio - Caricamento plugin", e);
            throw new SITException("",e);
        } catch (IllegalArgumentException e) {
            logger.error("IllegalArgumentException durante l'inizializzazione dell'oggetto Territorio - Caricamento plugin", e);
            throw new SITException("",e);
        } catch (IllegalAccessException e) {
            logger.error("IllegalAccessException durante l'inizializzazione dell'oggetto Territorio - Caricamento plugin", e);
            throw new SITException("",e);
        } catch (InvocationTargetException e) {
            logger.error("InvocationTargetException durante l'inizializzazione dell'oggetto Territorio - Caricamento plugin", e);
            logger.error("L'eccezione originale  " + e.getTargetException().getMessage(), e.getTargetException());
            if (!SITException.class.isAssignableFrom(e.getTargetException().getClass())) {
                throw (SITException)e.getTargetException();                
            } else {
                throw new SITException("",(Exception)e.getTargetException());
            }
        }
       
        /////////////////////////////////////////////////////////////////////////
        // Caricamento IGetFeatureInfoLayers
        /////////////////////////////////////////////////////////////////////////
        String getFeatureInfoConfig = pr.getProperty(ente + "GETFEATUREINFOLAYERSCONFIG");
        if (getFeatureInfoConfig!=null && !getFeatureInfoConfig.equals("")) {
	        //create parent BeanFactory       
			DefaultListableBeanFactory parentBeanFactory = new DefaultListableBeanFactory(); 
		    //register your pre-fabricated object in it
		    parentBeanFactory.registerSingleton("logger1", logger);
		    //wrap BeanFactory inside ApplicationContext
		    GenericApplicationContext parentContext = 
		            new GenericApplicationContext(parentBeanFactory);
		    parentContext.refresh();
		    
		    if (!getFeatureInfoConfig.startsWith("/") && !getFeatureInfoConfig.startsWith("file:")) {
		    	String absolutePath = new File(configFilePath).getAbsolutePath();
		    	String absPath = absolutePath.substring(0,absolutePath.lastIndexOf(File.separator));;
		    	getFeatureInfoConfig = "file:///" + absPath + File.separator + getFeatureInfoConfig;
		    }
		    
		    try {
		    	logger.debug("Apro il file " + getFeatureInfoConfig + " per leggere le definizioni di GetFeatureInfoLayers");
		    	FileSystemXmlApplicationContext applicationContext = new FileSystemXmlApplicationContext(new String[] {getFeatureInfoConfig}, parentContext);

		    	@SuppressWarnings("unchecked")
		    	List<IGetFeatureInfoLayer> gfiList =  (List<IGetFeatureInfoLayer>) applicationContext.getBean("getFeatureInfoLayers");
					   	
		    	for (IGetFeatureInfoLayer lay: gfiList) {
		    		logger.debug("Definisco GetFeatureInfoLayer " + lay.getNome() + " con codTPN=" + lay.getCodTPN());
				   	addGetFeatureInfoLayer(lay);   
		    	}
		    } catch (BeansException e) {
		    	logger.warn("Errore spring nel file definizioni di GetFeatureInfoLayers "+getFeatureInfoConfig,e );
		    }
		    //ApplicationContext applicationContext = new ClassPathXmlApplicationContext(new String[] {"/application-context.xml"}, parentContext);
		    		       

        }
        
        /////////////////////////////////////////////////////////////////////////
        // Caricamento Layers
        /////////////////////////////////////////////////////////////////////////
        String layersConfig = pr.getProperty(ente + "LAYERSCONFIG");
        if (layersConfig!=null && !layersConfig.equals("")) {
	        //create parent BeanFactory       
			DefaultListableBeanFactory parentBeanFactory = new DefaultListableBeanFactory(); 
		    //register your pre-fabricated object in it
		    parentBeanFactory.registerSingleton("logger1", logger);
		    //wrap BeanFactory inside ApplicationContext
		    GenericApplicationContext parentContext = 
		            new GenericApplicationContext(parentBeanFactory);
		    parentContext.refresh();
		    
		    if (!layersConfig.startsWith("/") && !layersConfig.startsWith("file:")) {
		    	String absolutePath = new File(configFilePath).getAbsolutePath();
		    	String absPath = absolutePath.substring(0,absolutePath.lastIndexOf(File.separator));;
		    	layersConfig = "file:///" + absPath + File.separator + layersConfig;
		    }
		    
		    try {
		    	logger.debug("Apro il file " + layersConfig + " per leggere le definizioni di Layers");
		    	FileSystemXmlApplicationContext applicationContext = new FileSystemXmlApplicationContext(new String[] {layersConfig}, parentContext);

		    	@SuppressWarnings("unchecked")
		    	List<LayerTerritorioGenerico> gfiList =  (List<LayerTerritorioGenerico>) applicationContext.getBean("layers");
					   	
		    	for (LayerTerritorioGenerico lay: gfiList) {
		    		logger.debug("Definisco Layer " + lay.getNome() + " con codTPN=" + lay.getCodTPN());
		    		lay.setDSPool(dsPool);
		    		lay.setLayersManager(this);
		    		addLayer(lay.getCodTPN(), lay);
		    		//initLayers(pr, lay.getDefaultCodTPN(), lay);
		    	}
		    } catch (BeansException e) {
		    	logger.warn("Errore spring nel file definizioni di Layers "+layersConfig,e );
		    }
		    //ApplicationContext applicationContext = new ClassPathXmlApplicationContext(new String[] {"/application-context.xml"}, parentContext);
		    		       

        }
        
        
        // ///////////////////////////////////////////////////////////////
        //Chiudo il file di configurazione (in realta lo stream di input).
        // /////////////////////////////////////////////////////////////////
        
        try {
            input.close();
        } catch(IOException io) { 
            
        }
    };
    
    /**
     * Costruttore della classe Territorio.
     * 
     * @param configFilePath
     * @param logger
     * @param Nome
     * @throws FileNotFoundException
     * @throws IOException
     */
    public SITLayersManager(String configFilePath, LogInterface logger, String Nome) throws SITException {
    	this(configFilePath, logger, Nome, "001");
    }

    /**
     * Costruttore della classe Territorio.
     * 
     * @param configFilePath
     * @param logger
     * @param Nome
     * @param ente
     * @throws FileNotFoundException
     * @throws IOException
     */
    public SITLayersManager(String configFilePath, LogInterface logger, String nome, String ente) throws SITException {
    	init(configFilePath, logger, nome, ente);
    }
    
    /**
     * @param dsPool
     * @param dsPoolID
     * @param inString
     * @return String
     */
    public static String dsCaseSensitiveTransform(SITDataStorePool dsPool, String dsPoolID,  String inString) {

    	String dsCaseSensitive = dsPool.getPoolDsCaseSensitive(dsPoolID);

    	if (dsCaseSensitive==null) return inString;
    	else if (dsCaseSensitive.equals("U")) return inString.toUpperCase();
    	else if (dsCaseSensitive.equals("L")) return inString.toLowerCase();
    	else return inString;
    }
    
    private String TrimIfNotNull(String str) {
        return (str==null) ? str : str.trim();
    }
    
    

    /**
     * Inizializza un layer
     * 
     * @param pr
     * @param IDLayer
     * @param layer
     * @throws MalformedURLException
     * @throws SITException
     */
    public void initLayers(Properties pr, int codTPN, LayerTerritorio layer) throws SITException {
        
        initLayers( pr, codTPN, layer.getConfigPrefix(), layer);
        
    }
    /**
     * Se il nome di file passato  relativo lo trasforma in assoluto interpretando il path relativo a partire dal file Config.txt
     * @param fileName
     * @return
     */
    private String ifRelativeSetAbsolute(String fileName) {
 	    
       if ((fileName==null) || fileName.equals("")){
           return fileName;
       }        
        
       try{
           new URL(fileName);
           return fileName;
       }catch(MalformedURLException murle){
           File f = new File(fileName);           
     	   if (f.isAbsolute()) {
     		   return f.toURI().toString();
     	   } else {
     	       File cf = new File(this.configFilePath);
     		   File af = new File(cf.getParent(),fileName);
     		   return af.toURI().toString();
     	   }           
 	   }               	   
    }

    // TODO Da eliminare in favore della precedente quando tutti i layer saranno adeguati a nuova struttura
    /**
     * Inizializzazione di tutti i layers SIT.
     * 
     * @param pr
     * @param IDLayer
     * @param nome
     * @param layer
     * @throws MalformedURLException
     * @throws SITException
     */
    private void initLayers(Properties pr, int codTPN, String nome, LayerTerritorio layer) throws SITException {
        
        
    	String nomeLayer = pr.getProperty(ente + nome + "NOMELAYER");

    	//AssocNomeLayerCodLayer.put(nomeLayer,new Integer(codTPN));
    	
    	// Controllo se lo stesso codice TPN  utilizzato da un altro Layer
        if(TipologieDivisioniTerritorio.containsKey(codTPN)){
            LayerTerritorio layerTerritorio = (LayerTerritorio) TipologieDivisioniTerritorio.get(codTPN);
            String msg = "Errore nell'inizializzazione del layer " + nome + " : il codTPN " + codTPN + "  gi utilizzato ";
            if(layerTerritorio == null){
                msg += "da un layer non definito nel file di configurazione"; 
            }else{
                msg += "dal layer " + layerTerritorio.getConfigPrefix() + " (" + layerTerritorio.getNome() + ")";
                
            }
            logger.error(msg);
            throw new SITException(msg);            
        }
        
    	if (nomeLayer != null) {    	        	    
            
    		ConfigBean configBean = new ConfigBean();

    		configBean.setLayersManager(this);
    		configBean.setDsPool(dsPool);
    		configBean.setEnte(ente);
    		
    		String szBasePath = TrimIfNotNull(pr.getProperty(ente + "SHPBASEPATH"));
    		szBasePath = ifRelativeSetAbsolute(szBasePath);
    		pr.setProperty(ente + "SHPBASEPATH", szBasePath);
    		
    		String szDsParams = TrimIfNotNull(pr.getProperty(ente + nome + "DATASTOREPAR"));
    		
    		// ////////////////////////////////////
    		// sostituzione eventuale macro path
    		// ////////////////////////////////////
    		
    		if (szDsParams!=null && szBasePath!=null) {
    			szDsParams=szDsParams.replaceAll("%BASEPATH%",szBasePath);
    		}

    		configBean.setDataStoreParams(szDsParams);
    		
    		String szSRID = TrimIfNotNull(pr.getProperty(ente + nome + "SRID"));
    		configBean.setSRID(szSRID);

    		String szDsPoolID = TrimIfNotNull(pr.getProperty(ente + nome + "DSPOOLID"));
    		configBean.setDsPoolID(szDsPoolID);

    		//int codTPN = IDLayer;

    		//AssocCodTPNCodLayer.put(codTPN, new Integer(IDLayer));

    		String typeName = TrimIfNotNull(pr.getProperty(ente + nome + "TYPENAME"));

    		// ////////////////////////////////////////////////////////
    		// Modifico in funzione del parametro DSPOOLCASESENSITIVE
    		// ////////////////////////////////////////////////////////
    		
    		typeName = dsCaseSensitiveTransform(dsPool, szDsPoolID, typeName);

    		configBean.setTypeName(typeName);

    		String  queryDirettaJdbcUrl = TrimIfNotNull(pr.getProperty(ente + nome + "QUERYDIRETTAJDBCURL"));
    		configBean.setQueryDirettaJdbcUrl(queryDirettaJdbcUrl);

    		String  queryDirettaJdbcDriver = TrimIfNotNull(pr.getProperty(ente + nome + "QUERYDIRETTAJDBCDRIVER"));
    		configBean.setQueryDirettaJdbcDriver(queryDirettaJdbcDriver);

    		String  queryDirettaUser = TrimIfNotNull(pr.getProperty(ente + nome + "QUERYDIRETTAUSER"));
    		configBean.setQueryDirettaUser(queryDirettaUser);

    		String  queryDirettaPasswd = TrimIfNotNull(pr.getProperty(ente + nome + "QUERYDIRETTAPASSWD"));
    		configBean.setQueryDirettaPasswd(queryDirettaPasswd);

    		String typeNameScrittura = TrimIfNotNull(pr.getProperty(ente + nome + "TYPENAMESCRITTURA"));
    		configBean.setTypeNameScrittura(typeNameScrittura);

    		String dataStoreFactory = TrimIfNotNull(pr.getProperty(ente + nome + "DATASTOREFACTORY"));

    		configBean.setDataStoreFactory(dataStoreFactory);

    		String conHints = TrimIfNotNull(pr.getProperty(ente + nome + "DATASTORECONHINTS"));

    		if ((conHints!=null) && (conHints.toUpperCase().equals("N"))) {
    			configBean.setDataStoreConHints(false);
    		} else {
    			configBean.setDataStoreConHints(true);
    		}

    		// ///////////////////////////////////////////////
    		// Lettura parametri legati alla storicizzazione
    		// ///////////////////////////////////////////////
    		
    		String StoricoCodiceEvento = TrimIfNotNull(pr.getProperty(ente + nome + "STORICOCODICEEVENTO"));

    		// ////////////////////
    		// Tabella storica
    		// ////////////////////
    		
    		if (StoricoCodiceEvento!=null) { 
    			configBean.setStoricoCodiceEvento(StoricoCodiceEvento);

    			String StoricoCodiceVariazione = TrimIfNotNull(pr.getProperty(ente + nome + "STORICOCODICEVARIAZIONE"));
    			configBean.setStoricoCodiceVariazione(StoricoCodiceVariazione);

    			String StoricoCodiceNumeratore = TrimIfNotNull(pr.getProperty(ente + nome + "STORICOCODICENUMERATORE"));
    			configBean.setStoricoCodiceNumeratore(StoricoCodiceNumeratore);
    		}
    		    		
    		layer.initLayer(configFilePath, logger, configBean, nomeLayer, codTPN);

    		String szQl = pr.getProperty(ente + nome + "QUERYLIMIT");
    		try {
    			Integer ql = Integer.valueOf(szQl);
    			layer.setQueryLimit(ql);
    		} catch (NumberFormatException e) {
    			logger.warn("Valore del parametro del config.txt  "+ ente + nome + "QUERYLIMIT " + "non numerico. Continuo senza imporre limiti alle query ");
    		}

    		// ////////////////////////////////
    		// Lettura Mappa dei nomi campi
    		// ////////////////////////////////
    		
    		layer.initNomiCampiFull(pr, ente, nome);
    		layer.initRicerche();

    	} 
    	
    	addLayer(codTPN, layer);
    }
    
    /**
     * Restituisce la tipologia di divisione del territorio.
     * 
     * @deprecated Adesso IDLayer coincide con codTPN, utilizzare {@link #getLayerByCodTPN(int)} 
     * @param IDLayer
     * @return LayerTerritorio
     */
    public LayerTerritorio getTipologiaDivisioneTerritorio(int IDLayer) {
    	//return (LayerTerritorio) TipologieDivisioniTerritorio.get(new Integer(IDLayer));
        return getLayerByCodTPN(IDLayer);
    }

    /**
     * Recupera il codTPN del layer passando uno drgli id interni individuati dalle costanti statiche di questa classe.
     * 
     * @deprecated Adesso IDLayer coincide con codTPN, utilizzare {@link #getLayerByCodTPN(int)}
     * @param internalId
     * @return int
     */
    public int getCodTPN(int internalId) {                     
    	return getLayerById(internalId).getCodTPN();      
    }

    /**
     * Recupera l'oggetto layer passando uno deggli id interni individuati dalle costanti statiche di questa classe.
     * 
     * @param internalId id fittizio assegnato da questa classe
     * @return LayerTerritorio
     */
    public LayerTerritorio getLayerById(int internalId) {
        return getLayerByCodTPN(internalId);
    	//return getTipologiaDivisioneTerritorio(internalId);      
    }        

    /**
     * Recupera l'oggetto layer passando il codTPN definito nel file di config.
     * 
     * @param codTPN
     * @return LayerTerritorio
     */
    public LayerTerritorio getLayerByCodTPN(int codTPN) {
    	return (LayerTerritorio) TipologieDivisioniTerritorio.get(Integer.valueOf(codTPN));
    }
    
    /**
     * Resituisce la collezione dei layers definiti nel file di configurazione
     * @return Collection
     */
    public Collection<LayerTerritorio> getAllLayers(){
        
        Collection<LayerTerritorio> layersOnConfig = new ArrayList<LayerTerritorio>();
        
        for (Iterator<LayerTerritorio> iterator = TipologieDivisioniTerritorio.values().iterator(); iterator.hasNext();) {
            LayerTerritorio layer = iterator.next();
            if(layer != null){
                layersOnConfig.add(layer);
            }
        }                
        
        return layersOnConfig;
    }
    
    /**
    *  Recupera il getFeatureInfoLayer passando il codTPN definito.
    * 
    * @param codTPN
    * @return IGetFeatureInfoLayer
    */
	public IGetFeatureInfoLayer getGetFeatureInfoLayerByCodTPN(int codTPN) {
		
		IGetFeatureInfoLayer retVal = null;
		
		// Cerca tra i layerTerritorio
		retVal = (IGetFeatureInfoLayer) getLayerByCodTPN(codTPN);
		if (retVal==null) {
			// se non presente tra i layerTerritorio cerca tra il GetFeatureInfoLayers
			retVal = getFeatureInfoLayers.get(Integer.valueOf(codTPN));
		}
		
	 	return retVal;
	}
    
	/**
	 * Aggiunge un GetFeatureInfoLayer
	 * 
	 * @param codTPN
	 * @param layer
	 */
	public void addGetFeatureInfoLayer(IGetFeatureInfoLayer layer) {
		getFeatureInfoLayers.put(layer.getCodTPN(), layer);
	}
	
	
	public void addLayer(int codTPN, LayerTerritorio layer) {
		if (layer.getNome()!=null) {
			TipologieDivisioniTerritorio.put(new Integer(codTPN), layer);
			
		} else {
		    //logger.info("Il layer " + nome + " con codTPN " + codTPN + " non  definito nel file di configurazione!");
			TipologieDivisioniTerritorio.put(new Integer(codTPN), null);    
			
		}
	}
	
	
    
    /**
     * Recupera l'oggetto layer passando il nome del layer.
     * 
     * @param name
     * @return LayerTerritorio
     */
    /* **********************************************************************************************
     * Metodo che non pu pi funzionare, perch allo stesso nome possono corrispondere layer diversi
     * **********************************************************************************************
    public LayerTerritorio getLayerByName(String name) {
    	Integer codTPN = (Integer) AssocNomeLayerCodLayer.get(name);

    	if (codTPN!=null) {
    		return getLayerByCodTPN(codTPN) ;
    	} else {
    		return null;
    	}
    }
    */
    
    /**
     * Modifica LA GEOMETRIA dell'oggetto identificato da idTPN all'interno del layer codTPN con quella contenuta in szJSGeometry
     * 
     * N.B. Il valore degli attributi rimane invariato
     * 
     * @param codTPN
     * @param idTPN
     * @param szJSGeometry
     * @return OggettoTerritorio
     * @throws SITException
     */
    private OggettoTerritorio editOperation (int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica, ILayersManager.EditOperationType op, Integer clippingCodTPN ) throws SITException {
    	OggettoTerritorio ogDaAgg = null;

    	LayerTerritorio layerTerr = getLayerByCodTPN(codTPN);
    	ogDaAgg = JSGeometry.jsGeometryToOggettoTerritorio(this, szJSGeometry, logger);

    	if (layerTerr == null) {
    		throw new SITException("Nessun layer definito per il codice indicato");
    	} 

    	if (bSoloImmagineStorica) { 
    		if (!LayerTerritorioStorico.class.isAssignableFrom(layerTerr.getClass())) {
    			throw new SITException("Territorio.modify-Operazione storica richiesta su layer non storico: codTPN=" + codTPN);
    		} else {
    			
    			LayerTerritorioStorico layerStorico = (LayerTerritorioStorico) layerTerr;
    			OggettoTerritorioStorico ogIniziale = layerStorico.cercaPoligonoByIDStorico(idTPN, tr);
    			if (ogIniziale==null) throw new SITException("L'oggetto non esiste");
    			
    			try {
    				switch (op) {
    				case EdOpModify:
    					ogIniziale.setGeometryAttributeWKT(ogDaAgg.getGeometryAttributeWKT());           
    					layerStorico.modifyImmagineStorica(ogIniziale, tr);
    					break;
    				case EdOpModifyUnione:
    					ogIniziale.unione(ogDaAgg);
    					layerStorico.modifyImmagineStorica(ogIniziale, tr);
    					break;
    				case EdOpModifyDifferenza:
    					ogIniziale.differenza(ogDaAgg);
    					layerStorico.modifyImmagineStorica(ogIniziale, tr);
    					break;
    				case EdOpModifyCopertura:
    					layerStorico.modifyCoperturaImmagineStorica(ogIniziale,ogDaAgg, tr);
    					break;
    				case EdOpModifyAndClipCopertura:
    					
    					if (clippingCodTPN == null) {
    						throw new SITException("Codice layer di clipping null");
    					}
    					
    					LayerTerritorio clippingLayer = getLayerByCodTPN(clippingCodTPN);
    					
    					if (clippingLayer == null) {
    						throw new SITException("Nessun layer di clipping definito per il codice indicato : " + clippingCodTPN);
    					}
    					
    					layerStorico.modifyAndClipCoperturaImmagineStorica(ogIniziale, ogDaAgg, clippingLayer, tr);
    					break;
    				}
    			} catch (IOException e) {
    				logger.warn("Eccezione IOException verificata durante il metodo modify",e);
    				throw new SITException("Eccezione IOException verificata durante il metodo modify", e);
    			} catch (SQLException e) {
    				logger.warn("Eccezione SQLException verificata durante il metodo modify",e);
    				throw new SITException("Eccezione SQLException verificata durante il metodo modify", e);
				}    
    			
    			return ogIniziale;
    		}
    		
    	} else {
    		
    		OggettoTerritorio ogIniziale = layerTerr.cercaIDTPN(idTPN, tr);
    		
    		if (ogIniziale==null) throw new SITException("L'oggetto non esiste");
    		
    		try {
    			switch (op) {
    			case EdOpModify:
    				ogIniziale.setGeometryAttributeWKT(ogDaAgg.getGeometryAttributeWKT());   
    				layerTerr.modifyFeature(ogIniziale, tr);
    				break;
    			case EdOpModifyUnione:
    				ogIniziale.unione(ogDaAgg);
    				layerTerr.modifyFeature(ogIniziale, tr);
    				break;
    			case EdOpModifyDifferenza:
    				ogIniziale.differenza(ogDaAgg);
    				layerTerr.modifyFeature(ogIniziale, tr);
    				break;
    			case EdOpModifyCopertura:
    				layerTerr.modifyCopertura(ogIniziale,ogDaAgg, tr);
    				break;
    			case EdOpModifyAndClipCopertura:
    				
    				if (clippingCodTPN == null) {
    					throw new SITException("Codice layer di clipping null");
    				}
    				
    				LayerTerritorio clippingLayer = getLayerByCodTPN(clippingCodTPN);
    				
    				if (clippingLayer == null) {
    					throw new SITException("Nessun layer di clipping definito per il codice indicato : " + clippingCodTPN);
    				}
    				
    				layerTerr.modifyAndClipCopertura(ogIniziale, ogDaAgg, clippingLayer, tr);
    				break;
    			}

    		} catch (IOException e) {
    			logger.warn("Eccezione IOException verificata durante il metodo modify",e);
    			throw new SITException("Eccezione IOException verificata durante il metodo modify", e);
    		} catch (SQLException e) {
    			logger.warn("Eccezione IOException verificata durante il metodo modify",e);
    			throw new SITException("Eccezione SQLException verificata durante il metodo modify", e);
			}  
    		
    		return ogIniziale;
    	}
    }

    /**
     * 
     * @param codTPN
     * @param idTPN
     * @param szJSGeometry
     * @return OggettoTerritorio
     * @throws SITException
     * 
     * @see {@link it.prato.comunt.sit.Territorio#modify(int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica)}
     */   
    public OggettoTerritorio modify (int codTPN, String idTPN, String szJSGeometry) throws SITException {
    	return modify(codTPN, idTPN, szJSGeometry, null, false);
    }

    /**
     * Modifica LA GEOMETRIA dell'oggetto identificato da idTPN all'interno del layer codTPN con quella contenuta in szJSGeometry
     * 
     * N.B. Il valore degli attributi rimane invariato
     * 
     * @param codTPN
     * @param idTPN
     * @param szJSGeometry
     * @return OggettoTerritorio
     * @throws SITException
     */
    public OggettoTerritorio modify(int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica) throws SITException {
    	return editOperation (codTPN, idTPN, szJSGeometry, tr, bSoloImmagineStorica, ILayersManager.EditOperationType.EdOpModify, null);
    }

    /**
     * @param codTPN
     * @param idTPN
     * @param szJSGeometry
     * @return OggettoTerritorio
     * @throws SITException
     * 
     * @see {@link it.prato.comunt.sit.Territorio#modifyUnione(int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica)}
     */
    public OggettoTerritorio modifyUnione(int codTPN, String idTPN, String szJSGeometry) throws SITException {
    	return modifyUnione(codTPN, idTPN, szJSGeometry, null, false) ;      
    }
   
    /**
     * @param codTPN
     * @param idTPN
     * @param szJSGeometry
     * @param tr
     * @param bSoloImmagineStorica
     * @return OggettoTerritorio
     * @throws SITException
     * 
     * @see {@link it.prato.comunt.sit.Territorio#editOperation(int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica, EditOperationType op, Integer clippingCodTPN)}
     */
    public OggettoTerritorio modifyUnione (int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica ) throws SITException {
	   return editOperation (codTPN, idTPN, szJSGeometry, tr, bSoloImmagineStorica,ILayersManager.EditOperationType.EdOpModifyUnione, null );	   
    }

    /**
     * @param codTPN
     * @param idTPN
     * @param szJSGeometry
     * @return OggettoTerritorio
     * @throws SITException
     * 
     * @see {@link it.prato.comunt.sit.Territorio#modifyDifferenza(int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica)}
     */
    public OggettoTerritorio modifyDifferenza (int codTPN, String idTPN, String szJSGeometry) throws SITException {
    	return  modifyDifferenza ( codTPN,  idTPN,  szJSGeometry,  null, false) ;
    }
   
    /**
     * @param codTPN
     * @param idTPN
     * @param szJSGeometry
     * @param tr
     * @param bSoloImmagineStorica
     * @return OggettoTerritorio
     * @throws SITException
     * 
     * @see {@link it.prato.comunt.sit.Territorio#editOperation(int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica, EditOperationType op, Integer clippingCodTPN)}
     */
    public OggettoTerritorio modifyDifferenza (int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica ) throws SITException {
       return editOperation ( codTPN,  idTPN,  szJSGeometry,  tr,  bSoloImmagineStorica, ILayersManager.EditOperationType.EdOpModifyDifferenza , null );       
   }

    /**
     * @param codTPN
     * @param idTPN
     * @param szJSGeometry
     * @return OggettoTerritorio
     * @throws SITException
     * 
     * @see {@link it.prato.comunt.sit.Territorio#appendFeature(int codTPN, String idTPN, String szJSGeometry, SITTransaction tr)}
     */
    public OggettoTerritorio appendFeature (int codTPN, String idTPN, String szJSGeometry) throws SITException {
    	return  appendFeature ( codTPN,  idTPN,  szJSGeometry,  null) ;
   }
    
    /**
     * Consente di inserire una nuova feature.
     * 
     * @param codTPN
     * @param idTPN
     * @param szJSGeometry
     * @param tr
     * @return OggettoTerritorio
     * @throws SITException
     */
    public OggettoTerritorio appendFeature (int codTPN, String idTPN, String szJSGeometry, SITTransaction tr) throws SITException {

    	// ///////////////////////////////////////////////////////////////
    	// Attenzione se idTPN null crea ed aggiunge senza settare valore 
    	// attributi (solo chiave primaria se gestita da FIDMApper.
    	// ///////////////////////////////////////////////////////////////
    	
    	LayerTerritorio layer = getLayerByCodTPN(codTPN);

    	if (layer == null) {
    		throw new SITException("Nessun layer definito per il codice indicato");
    	}

    	OggettoTerritorio ogDaAgg = null;

    	try {
    		ogDaAgg = JSGeometry.jsGeometryToOggettoTerritorio(this, szJSGeometry, logger);

    		if (idTPN != null) {
    			ogDaAgg.setIDTPN(idTPN);
    		}

    		layer.appendFeature(ogDaAgg, tr);

    	} catch (IOException e) {
    		logger.error("Errore appendFeature - ", e);
    		throw new SITException("Eccezione verificata durante il metodo appendFeature", e);
    	} catch (SITException e) {
    		throw e;
    	}

    	return ogDaAgg;
    }

    /**
     * 
     * @param codTPN
     * @param keyValue
     * @throws SITException
     * 
     * @see {@link it.prato.comunt.sit.Territorio#removeFeature(int codTPN, String keyValue, SITTransaction tr)}
     */
    public void removeFeature (int codTPN, String keyValue) throws SITException {
    	removeFeature ( codTPN,  keyValue,  null) ;    
    }

    /**
     * Consente la rimozione di una feature.
     * 
     * @param codTPN
     * @param keyValue
     * @param tr
     * @throws SITException
     */
    public void removeFeature (int codTPN, String keyValue, SITTransaction tr) throws SITException {

	   LayerTerritorio layer = getLayerByCodTPN(codTPN);

	   if (layer == null) {
		   throw new SITException("Nessun layer definito per il codice indicato");
	   }

	   OggettoTerritorio oggToDelete = layer.cercaIDTPN(keyValue);
	   if (oggToDelete==null) throw new SITException("L'oggetto non esiste");
	   
	   try {
		   layer.removeFeature(oggToDelete, tr);
	   } catch (IOException e) {
		   throw new SITException("Eccezione appendFeature verificata durante il metodo removeFeature",  e);
	   } catch (SITException e) {
		   throw e;
	   } catch (SQLException e) {
		   throw new SITException("Eccezione SQLException verificata durante il metodo removeFeature",  e);
	   }
   }

    /**
     * @param codTPN
     * @param idTPN
     * @param szJSGeometry
     * @param clippingCodTPN
     * @return OggettoTerritorio
     * @throws SITException
     * 
     * @see {@link it.prato.comunt.sit.Territorio#modifyAndClipCopertura(int codTPN, String idTPN, String szJSGeometry, int clippingCodTPN, SITTransaction tr, boolean bSoloImmagineStorica)}
     */
    public OggettoTerritorio modifyAndClipCopertura (int codTPN, String idTPN, String szJSGeometry, int clippingCodTPN) throws SITException {
    	return  modifyAndClipCopertura ( codTPN,  idTPN,  szJSGeometry,  clippingCodTPN,  null, false);
   }
   
   /**
    * Il metodo permette di modificare la copertura di un poligono e di vincolare tale modifica ad un layer
    * che ne limita i confini.
    * 
    * @param codTPN codice tpn del layer da modificare
    * @param clippingCodTPN codice tpn del layer di vincolo
    * @param idTPN id del poligono interessato dalla modifica (o idstorico se bSoloImmagineStorica=true)
    * @param szJSGeometry stringa rrappresentante la geometry della modifica richiesta
    * @param tr Transazione - se null viene usata transazione di default con autocommit
    * @param bSoloImmagineStorica - indica se, in caso di layer storici, deve essere modificata l'immagine identificata da IDTPN (inteso come valore di IDStorico) oppure se deve essere fatto una modifica storica completa (con generazione di nuova immagine) 
    * @return OggettoTerritorio oggetto iniziale con la geometria modificata
    * @throws SITException
    * 
    * @see {@link it.prato.comunt.sit.Territorio#editOperation(int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica, EditOperationType op, Integer clippingCodTPN)}
    */
   public OggettoTerritorio modifyAndClipCopertura (int codTPN, String idTPN, String szJSGeometry, int clippingCodTPN, SITTransaction tr, boolean bSoloImmagineStorica) throws SITException {
       return editOperation ( codTPN,  idTPN,  szJSGeometry,  tr,  bSoloImmagineStorica, ILayersManager.EditOperationType.EdOpModifyAndClipCopertura , clippingCodTPN );              
   }

   /**
    * @param codTPN
    * @param idTPN
    * @param szJSGeometry
    * @return OggettoTerritorio
    * @throws SITException
    * 
    * @see {@link it.prato.comunt.sit.Territorio#modifyCopertura(int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica)}
    */
   public OggettoTerritorio modifyCopertura (int codTPN, String idTPN, String szJSGeometry) throws SITException {
	   return  modifyCopertura ( codTPN,  idTPN,  szJSGeometry, null, false);
   }
   
   /**
    * @param codTPN
    * @param idTPN
    * @param szJSGeometry
    * @param tr
    * @param bSoloImmagineStorica
    * @return OggettoTerritorio
    * @throws SITException
    * 
    * @see {@link it.prato.comunt.sit.Territorio#editOperation(int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica, EditOperationType op, Integer clippingCodTPN)}
    */
   public OggettoTerritorio modifyCopertura (int codTPN, String idTPN, String szJSGeometry, SITTransaction tr, boolean bSoloImmagineStorica) throws SITException {
	   return editOperation ( codTPN,  idTPN,  szJSGeometry,  tr,  bSoloImmagineStorica, ILayersManager.EditOperationType.EdOpModifyCopertura , null );       
   }
   
   /**
    * Restituisce una connessione standard alla base di dati.
    * 
    * @return Connection
    * @throws SITException
    */
   public Connection getSitStandardConnection() throws SITException{
       
       Connection conn;
       
       try {
           
           Class.forName(jdbc_sit_driver);
           conn = DriverManager.getConnection(jdbc_sit_url,jdbc_sit_user,jdbc_sit_password);
           conn.setTransactionIsolation(Connection.TRANSACTION_READ_COMMITTED);
           conn.setAutoCommit(false);
           
        } catch (SQLException e) {
           String msg = "Errore durante l'Acquisizione della Connessione standard a SIT" + jdbc_sit_url;
           logger.error(msg,e);
           throw new SITException (msg,e);
        } catch (Exception e) {
           String msg = "Errore durante l'Acquisizione della Connessione standard a SIT" + jdbc_sit_url;
           logger.error(msg,e);
           throw new SITException (msg,e);
        }
     
        return conn;
   }

   /**
    * @param codTPN
    * @param idTPN
    * @param szClippingCodTPN
    * @param geoCoord
    * @param geoOp
    * @throws SITException
    * 
    * @see {@link it.prato.comunt.sit.Territorio#EditManager(Integer codTPN, String idTPN, String szClippingCodTPN, String geoCoord, String geoOp, SITTransaction tr, boolean bStoricoSoloImmagine)}
    */
   public void EditManager (Integer codTPN, String idTPN, String szClippingCodTPN, String geoCoord, String geoOp )    throws SITException  {
	   EditManager(codTPN, idTPN, szClippingCodTPN, geoCoord, geoOp, null, false);
   }
   
   /**
    * Esegue append, modify, remove e quanto necessari per la gestione della geometria a partire dai parametri nel formato gestito in Tolomeo.
    * 
    * @param codTPN Codice del layer interessato all'operazione
    * @param idTPN  ID dell'oggetto interessato dall'operazione (non utilizzato se si tratta di una operazione di inserimento). Se si tratta di un layer storico contiene IDStorico se BStoricoSoloImmagine=true, l'ID in caso contrario
    * @param szClippingCodTPN codice del layer su cui fare il clipping. null se clipping non necessario
    * @param geoCoord geometria nel formato utilizzato in tolomeo (ed utilizzato da jsGeometryToOggettoTerritorio)
    * @param geoOp (operazione richiesta. E' uno dei valori Costanti.digitizeOperation*)
    * @param tr
    * @param bStoricoSoloImmagine flag che indica se eventuali operazioni storiche devono generare lo storico o sono riferite ad una specifica immagine identificata da idTPN
    * @throws SITException
    */
   public void EditManager (Integer codTPN, String idTPN, String szClippingCodTPN, String geoCoord, 
		   String geoOp, SITTransaction tr, boolean bStoricoSoloImmagine) throws SITException {
	  
	   int clippingCodTPN = 0;        
	   SITTransaction tr1 = (tr == null) ? new SITDefaultTransaction() : tr;
	   boolean error = false;

	   try {
		   if((szClippingCodTPN != null) && !szClippingCodTPN.equals("")) {
			   clippingCodTPN = Integer.parseInt(szClippingCodTPN);
		   }
		   
		   logger.debug("clippingCodTPN = " + clippingCodTPN);
		   
		   if (geoOp.equals(ILayersManager.digitizeOperationSubtract)) {
			   // Sub tr1act
			   modifyDifferenza(codTPN,idTPN, geoCoord,  tr1, bStoricoSoloImmagine);
		   } else if (geoOp.equals(ILayersManager.digitizeOperationAdd)) {
			   // Add
			   modifyUnione(codTPN,idTPN, geoCoord,  tr1, bStoricoSoloImmagine);
		   } else if (geoOp.equals(ILayersManager.operationFeatureVertexEditing)) {
			   // Vertex editing
			   modify(codTPN,idTPN, geoCoord,  tr1, bStoricoSoloImmagine);
		   } else if (geoOp.equals(ILayersManager.operationFeatureDragDrop)) {
			   // DragDrop
			   modify(codTPN,idTPN, geoCoord,  tr1, bStoricoSoloImmagine);
		   } else if (geoOp.equals(ILayersManager.operationGeometryModify)) {
			   // Generica modifica
			   modify(codTPN,idTPN, geoCoord,  tr1,bStoricoSoloImmagine);
		   } else if (geoOp.equals(ILayersManager.digitizeOperationAddSub)) {
			   // AddSub
			   if(clippingCodTPN != 0){
				   modifyAndClipCopertura(codTPN, idTPN, geoCoord, clippingCodTPN,  tr1, bStoricoSoloImmagine);
			   }else{
				   modifyCopertura(codTPN,idTPN, geoCoord,  tr1, bStoricoSoloImmagine);
			   }                    
		   } else if (geoOp.equals(ILayersManager.digitizeOperationInsert)) {
			   // Inserimento nuovo
			   appendFeature(codTPN, idTPN, geoCoord,  tr1);
		   } else if (geoOp.equals(ILayersManager.operationFeatureDelete)) {
			   // Cancellazione Feature
			   removeFeature(codTPN, idTPN,  tr1);
		   }           
	   } catch (NumberFormatException e) {
		   logger.warn("NumberFormatException in AjaxQueryServlet", e);
		   error = true;
		   throw new SITException("", e);
	   } catch (SITException e) {
		   logger.warn("SITException in AjaxQueryServlet", e);
		   error = true;
		   throw e;
	   } finally {
		   if(tr == null && tr1 != null){
			   try{
				   if(error) {    			   
					   tr1.rollback();    			   
				   } else {
					   tr1.commit();
				   }
			   }catch(Exception e){
				   logger.error("Problemi durante rollbaco o commit della transazione",e);    			   
			   }
		   }
	   }
   }

   /**
    * @return String
    */
   public String getUser() {
	   return user;
   }

   /**
    * @param user
    */
   public void setUser(String user) {
	   this.user = user;
   }

   /**
    * @return String
    */
   public String getIpAddress() {
	   return ipAddress;
   }

   /**
    * @param ipAddress
    */
   public void setIpAddress(String ipAddress) {
	   this.ipAddress = ipAddress;
   }

   /**
    * Disposing dei data stores
    */
   public void dispose() throws SITException{              
       if(dsPool != null) {
           try {
               dsPool.disposeDataStores();
           } catch (Exception e) {
               logger.error("Errore durante il dispose dei database", e);
               throw new SITException("Errore in dispose dei data store",e);
           }
       }   		     
    }

	public LogInterface getLogger() {
		return this.logger;
	}

    @Override
    protected void finalize() throws Throwable {
        dispose();
        // TODO Auto-generated method stub
        super.finalize();
    }
	
	
}