/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing, 
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Tolomeo; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
]*
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate
 * su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License come 
 * pubblicato  dalla Free Software Foundation, sia la versione 3 della 
 * licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di
 * COMMERCIABILIT o IDONEIT PER UN PARTICOLARE SCOPO. Vedere la
 * GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public
 * insieme a Tolomeo, in caso contrario, si scriva alla Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *   
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
/*
 * Copyright  2007 Comune di Prato - All Right Reserved
 * Project:      Tolomeo - WebGis con funzioni di editing
 * File:         SITPunto.java
 * Function:     Semplice punto
 * Author:       Alessandro Radaelli
 * Version:      1.0.0
 * CreationDate: 23/05/2008
 * ModifyDate:   
 */

package it.prato.comune.sit;

import it.prato.comune.utilita.logging.interfaces.LogInterface;

import com.vividsolutions.jts.geom.Point;

/**
 * Classe SITPunto.
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public class SITPunto {
    private Double x = null;
    private Double y = null;
    private String SRID = null;
    private LogInterface logger = null;
    
    /**
     * Costruttore della classe SITPunto.
     * 
     * @param logger
     * @param x
     * @param y
     * @param SRID
     */
    public SITPunto(LogInterface logger, Double x, Double y, String SRID) {
        super();
        this.x = x;
        this.y = y;
        this.SRID = SRID;
        this.logger = logger;
    }

    /**
     * @return Double
     */
    public Double getX() {
        return x;
    }

    /**
     * @param x
     */
    public void setX(Double x) {
        this.x = x;
    }

    /**
     * @return Double
     */
    public Double getY() {
        return y;
    }

    /**
     * @param y
     */
    public void setY(Double y) {
        this.y = y;
    }

    /**
     * @return String
     */
    public String getSRID() {
        return SRID;
    }

    /**
     * @param srid
     */
    public void setSRID(String srid) {
        SRID = srid;
    }
    
    /**
     * Converte le coordinate del punto corrente nel sistema di riferimento specificato come parametro.
     * 
     * @param targetSRID
     * @throws SITException
     */
    public void trasforma (String targetSRID) throws SITException {
        try {
            Point pt = LayerPuntiTerritorio.creaPoint(this.getX(), this.getY(), this.getSRID(), targetSRID);
            this.setX(pt.getCoordinate().x);
            this.setY(pt.getCoordinate().y);
            this.setSRID(targetSRID);
        } catch (SITException e) {
            logger.warn("SITException in trasforma", e);
            throw e;
        }        
    }
}
