/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/

package it.prato.comune.sit;

import it.prato.comune.utilita.logging.interfaces.LogInterface;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

/**
 *  Implementazione di un suggestProvider locale e inizializzato con i valori derivanti da una select su datasource sql
 * Esempio di inizializzazione tramite Spring:
 * <pre>
 * {@code
 
<bean id="suggestPGDataSource" class="org.apache.commons.dbcp.BasicDataSource">
    <property name="driverClassName" value="org.postgresql.Driver"/>
    <property name="url" value="jdbc:postgresql://devpg2.comune.prato.it:6432/comune_di_prato"/>
    <property name="username" value="piano_strutturale"/>
    <property name="password" value="zzzzzzzz"/>
</bean>
 

<bean class="it.prato.comune.sit.RicercaCompara" > 
	<property name="nomeRicerca" value="Autosuggest gerarchico da DB" />
    	<property name="campiRicerca">
			<list>
				<bean class="it.prato.comune.sit.RicercaCampo">
					<property name="NLCampo" value="NL_TOPONIMO" />
					<property name="etichetta" value="Province" />
					<property name="tipo" value="java.lang.String" />
					<property name="suggestDisponibile" value="true" />
					<property name="suggestMinLength" value="3" /> 
					<property name="suggestProvider">
						<bean class="it.prato.comune.sit.SuggestProviderLocalDatasource" >
							<constructor-arg ref="logger1" />	
							<property name="datasource" ref="suggestPGDataSource" />
							<property name="sql" value="SELECT gid, substring(codprov from 3 for 3) as codprov,nome,sigla FROM province " />
							<property name="displayFieldName" value="nome" />
							<property name="valueFieldName"   value="codprov" />								
						</bean> 
					</property>
					<property name="operatore" value="LIKE" /> 
				</bean>
				<bean class="it.prato.comune.sit.RicercaCampo">
					<property name="NLCampo" value="NL_TOPONIMO" />
					<property name="etichetta" value="Comuni" />
					<property name="tipo" value="java.lang.String" />
					<property name="suggestDisponibile" value="true" />
					<property name="suggestMinLength" value="3" /> 
					<property name="suggestProvider">
						<bean class="it.prato.comune.sit.SuggestProviderLocalDatasource" >
							<constructor-arg ref="logger1" />
							<property name="datasource" ref="suggestPGDataSource" />
							<property name="sql" value="SELECT gid, codcom, codprov, nome FROM comuni "/>
							<property name="displayFieldName" value="nome" />
							<property name="valueFieldName"   value="codcom" />								
							<property name="filterBind" value="0#codprov" />
						</bean> 
					</property>
					<property name="operatore" value="LIKE" /> 
				</bean>
			</list>
    	</property>
	        </bean>
}
 * </pre>
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public class SuggestProviderLocalDatasource extends SuggestProviderBase implements ISuggestProvider {


	private DataSource			datasource	= null;
	private String				sql 		= null;
	private LogInterface		logger		= null;
	
	/**
	 * Costruttore
	 * 
	 * @param logger
	 */
	public SuggestProviderLocalDatasource(LogInterface logger) {
		super();
		this.logger=logger;
	}
	
	/**
	 * @see {ISuggestProvider{@link #getValori()}
	 */
	public  List<Map<String, Object>> getValori() throws SITException {
		List<Map<String, Object>> valori = new ArrayList<Map<String, Object>>();
		Connection connection = null;
		
		if (datasource==null || sql==null) {
			String errMsg = "Datasource o sql non inizializzato in classe "+ this.getClass().getCanonicalName();
			logger.error(errMsg);
			 throw new SITException(errMsg);
		}
		
		try {
			connection = datasource.getConnection();
			Statement select = connection.createStatement();
			ResultSet result = select.executeQuery(sql);
			ResultSetMetaData rsmd = result.getMetaData();
			int columnCount = rsmd.getColumnCount();
			
			while (result.next()) {
				Map<String, Object> rec = new HashMap<String, Object>();
				// The column count starts from 1
				for (int i = 1; i < columnCount + 1; i++ ) {
				  String colname = rsmd.getColumnName(i);
				  Object colVal  = result.getObject(i);
				  rec.put(colname,colVal);
				}
				valori.add(rec);
			}

		} catch (Exception e) {
			String errMsg = "Eccezione in getValori della classe "+ this.getClass().getCanonicalName();
			logger.error(errMsg,e);
			 throw new SITException(errMsg, e);
		} finally {
			if (connection!=null) {
				try {
					connection.close();
				} catch (Exception e) {
					String errMsg = "Eccezione in getValori della classe "+ this.getClass().getCanonicalName();
					logger.error(errMsg,e);
					 throw new SITException(errMsg, e);
				}
			}
		}
		return valori;
	}

	/**
	 * @see {ISuggestProvider{@link #isRemote()}
	 */
	public boolean isRemote() {
		return false;
	}
	
	/**
	 * Permette di impostare lo statement sql da eseguire per ottenere i valori di suggest
	 * @param sql the sql to set
	 */
	public void setSql(String sql) {
		this.sql = sql;
	}

	/**
	 * @param datasource the datasource to set
	 */
	public void setDatasource(DataSource datasource) {
		this.datasource = datasource;
	}

	/**
	 * @param logger the logger to set
	 */
	public void setLogger(LogInterface logger) {
		this.logger = logger;
	}
	

}
