/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/

package it.prato.comune.sit;

import it.prato.comune.utilita.logging.interfaces.LogInterface;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.geotools.data.DataStore;
import org.geotools.data.Query;
import org.geotools.data.simple.SimpleFeatureCollection;
import org.geotools.data.simple.SimpleFeatureIterator;
import org.geotools.data.simple.SimpleFeatureSource;
import org.opengis.feature.simple.SimpleFeature;
import org.opengis.feature.type.AttributeDescriptor;
import org.opengis.filter.Filter;

/**
 * 
 *  Implementazione di un suggestProvider locale e inizializzato con i valori derivanti da una interrogazione di un datastore geotools 
 * Esempio di inizializzazione tramite Spring:
 * <pre>
 * {@code

<!-- factory-method="newInstance" -->
<!-- the factory bean, which contains a method called createInstance() -->
<bean id="gtDSFactoryBean" 
		class="org.geotools.data.postgis.PostgisNGDataStoreFactory"
		 >
</bean>

<!-- the bean to be created via the factory bean -->
<bean id="pggtpianostrutturale"
      factory-bean="gtDSFactoryBean"
      factory-method="createDataStore">
	  <constructor-arg >
			<!-- dbtype;postgis!!user;piano_strutturale!!passwd;xeipho4U!!host;devpg2.comune.prato.it!!port;6432!!database;comune_di_prato!!schema;piano_strutturale -->
			<util:map map-class="java.util.HashMap" >
				<entry key="dbtype" 	value="postgis" 			/>
				<entry key="user" 		value="piano_strutturale" 	/>
				<entry key="passwd" 	value="zzzzzzzz" 			/>
				<entry key="host" 		value="devpg2.comune.prato.it" />
				<entry key="port" 		value="6432" 				/>
				<entry key="database" 	value="comune_di_prato" 	/>
				<entry key="schema" 	value="piano_strutturale" 	/>
			</util:map>
	  </constructor-arg>
</bean>

<bean class="it.prato.comune.sit.RicercaCompara" > 
	<property name="nomeRicerca" value="Autosuggest gerarchico da GT" />
	<property name="campiRicerca">
		<list>
			<bean class="it.prato.comune.sit.RicercaCampo">
				<property name="NLCampo" value="NL_TOPONIMO" />
				<property name="etichetta" value="Province" />
				<property name="tipo" value="java.lang.String" />
				<property name="suggestDisponibile" value="true" />
				<property name="suggestMinLength" value="3" /> 
				<property name="suggestProvider">
					<bean class="it.prato.comune.sit.SuggestProviderLocalDatastore" >
						<constructor-arg ref="logger1" />
						<property name="datastore" ref="pggtpianostrutturale" />
						<property name="featureName" value="provinceale" />
						<!-- <property name="cql" value="????? " /> -->
						<property name="displayFieldName" value="nome" />
						<property name="valueFieldName"   value="codprov" />								
					</bean> 
				</property>
				<property name="operatore" value="LIKE" /> 
			</bean>
			<bean class="it.prato.comune.sit.RicercaCampo">
				<property name="NLCampo" value="NL_TOPONIMO" />
				<property name="etichetta" value="Comuni" />
				<property name="tipo" value="java.lang.String" />
				<property name="suggestDisponibile" value="true" />
				<property name="suggestMinLength" value="3" /> 
				<property name="suggestProvider">
					<bean class="it.prato.comune.sit.SuggestProviderLocalDatastore" >
						<constructor-arg ref="logger1" />
						<property name="datastore" ref="pggtpianostrutturale" />
						<property name="featureName" value="comuni" />
						<!-- <property name="cql" value="????? " /> -->
						<property name="displayFieldName" value="nome" />
						<property name="valueFieldName"   value="codcom" />								
						<property name="filterBind" value="0#codprov" />
					</bean> 
				</property>
				<property name="operatore" value="LIKE" /> 
			</bean>
		</list>
	</property>
</bean>

}
 * </pre>
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public class SuggestProviderLocalDatastore extends SuggestProviderBase implements ISuggestProvider {

	private DataStore			datastore	= null;
	//private String				cql 		= null;
	private LogInterface		logger		= null;
	private String				featureName	= null;
	
	/**
	 * Costruttore
	 * 
	 * @param logger
	 */
	public SuggestProviderLocalDatastore(LogInterface logger) {
		super();
		this.logger=logger;
	}
	
	/**
	 * @see {ISuggestProvider{@link #getValori()}
	 */
	public  List<Map<String, Object>> getValori() throws SITException {
		List<Map<String, Object>> valori = new ArrayList<Map<String, Object>>();
		
		if (datastore==null || featureName==null || featureName.equals("")) {
			String errMsg = "Datastore o featureName non inizializzato in classe "+ this.getClass().getCanonicalName();
			logger.error(errMsg);
			 throw new SITException(errMsg);
		}
		
		try {
			SimpleFeatureSource source = datastore.getFeatureSource(featureName);
			SimpleFeatureCollection features;
		//	if (cql!=null && !cql.equals("")) {
			//	Filter filter = CQL.toFilter(cql);
			//	features = source.getFeatures(filter);
			//} else {
				features = source.getFeatures();
			//}
			
		//	int count = features.size();
			List<String> fields = getFilterFields();
			if (!fields.contains(getDisplayFieldName())) fields.add(getDisplayFieldName());
			if (!fields.contains(getValueFieldName())) 	 fields.add(getValueFieldName());
				
			Query query = new Query(featureName, Filter.INCLUDE, (String[]) fields.toArray(new String[fields.size()]));
			//query.setCoordinateSystem(system);
			//query.setMaxFeatures(maxFeatures);
			
			features = source.getFeatures(query);
			
			SimpleFeatureIterator iterator = features.features();
			try {
				while (iterator.hasNext()) {
					SimpleFeature feature = iterator.next();
					List<AttributeDescriptor> attDescriptors = feature.getFeatureType().getAttributeDescriptors();
                
	                Map<String, Object> rec = new HashMap<String, Object>();
					// The column count starts from 1
					for (int i = 0; i < attDescriptors.size(); i++ ) {
						// Escludo campo geom
						if ( feature.getFeatureType().getGeometryDescriptor()==null || 
								(attDescriptors.get(i).getName() !=feature.getFeatureType().getGeometryDescriptor().getName())) {
							String colname = attDescriptors.get(i).getLocalName().toString();
							Object colVal  = feature.getAttribute(i);
							rec.put(colname,colVal);	
						}
					}
					valori.add(rec);
	            //    count++;
	            }
	        } finally {
	            iterator.close(); // IMPORTANT
	        }
		} catch (IOException e) {
			String errMsg = "Eccezione in getValori della classe "+ this.getClass().getCanonicalName();
			logger.error(errMsg,e);
			 throw new SITException(errMsg, e);
		//} catch (CQLException e) {
		//	String errMsg = "Eccezione in getValori della classe "+ this.getClass().getCanonicalName();
		//	logger.error(errMsg,e);
		//	 throw new SITException(errMsg, e);
		}
		return valori;
	}

	/**
	 * @see {ISuggestProvider{@link #isRemote()}
	 */
	public boolean isRemote() {
		return false;
	}

	/**
	 * @param sql the sql to set
	 */
	//public void setCql(String cql) {
	//	this.cql = cql;
	//}

	/**
	 * Permette di impostare il datastore da utilizzare
	 * @param datastore the datastore to set
	 */
	public void setDatastore(DataStore datastore) {
		this.datastore = datastore;
	}

	/**
	 * @param logger the logger to set
	 */
	public void setLogger(LogInterface logger) {
		this.logger = logger;
	}

	/**
	 * Permette di impostare la featuretype geotools che sar utilizzata per fare le richieste
	 * @param featuretype the featuretype to set
	 */
	public void setFeatureName(String featureName) {
		this.featureName = featureName;
	}
	

}
