/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/

package it.prato.comune.sit;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Implementazione di un suggestProvider locale e statico (tutti i valori sono definiti settandoli tramite il metodo  {@link SuggestProviderLocalStatic#setValori(List)}
 * Esempio di inizializzazione tramite Spring:
 * <pre>
 * {@code
<bean class="it.prato.comune.sit.RicercaCampo">
	<property name="NLCampo" value="NL_TOPONIMO" />
	<property name="etichetta" value="Regioni" />
	<property name="tipo" value="java.lang.String" />
	<property name="suggestDisponibile" value="true" />
	<property name="suggestMinLength" value="3" /> 
	<property name="suggestProvider">
		<bean class="it.prato.comune.sit.SuggestProviderLocalStatic" >
			<property name="Valori">
			   	<list>
					<util:map map-class="java.util.HashMap">
					   <entry key="COD" value="TO" />
					   <entry key="DESC" value="TOSCANA" />
					  </util:map>
					  <util:map map-class="java.util.HashMap">
					   <entry key="COD" value="LA" />
					   <entry key="DESC" value="LAZIO" />
					  </util:map>
					  <util:map map-class="java.util.HashMap">
					   <entry key="COD" value="LO" />
					   <entry key="DESC" value="LOMBARDIA" />
				  	</util:map>
		  		</list>
			 </property>
		</bean> 
	</property>
	<property name="operatore" value="LIKE" /> 
</bean>
}
 * </pre>
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public class SuggestProviderLocalStatic extends SuggestProviderBase implements ISuggestProvider {

	private List<Map<String, Object>> 	valori = new ArrayList<Map<String, Object>>();
	
	
	/**
	 * @see {ISuggestProvider{@link #getValori()}
	 */
	public  List<Map<String, Object>> getValori() {
		return valori;
	}

	/**
	 * Imposta i valori 
	 * @param valori
	 */
	public void setValori( List<Map<String, Object>> valori) {
		this.valori = new ArrayList<Map<String, Object>>( valori);
	}

	/**
	 * @see {ISuggestProvider{@link #isRemote()}
	 */
	public boolean isRemote() {
		return false;
	}

}
