/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
package it.prato.comune.sit;


import it.prato.comune.utilita.logging.interfaces.LogInterface;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

/**
 * 
 * jZip.java --
 * author - Tom Valesky
 * purpose - example of how to use java.util.zip classes; compresses one
 *           file. 
 * Modificato ed integrato da A. Radaelli
 *
 */
public class jZip {
    
    
    public static void compress_file(String[] filename, File outfile, 
    		LogInterface logger) throws IOException{  
        
        try {
            byte[] buf = new byte[4096];            //read buffer            
            int retval;                             //return value

            // ///////////////////////////
            // set up output stream
            // ///////////////////////////
            
            FileOutputStream os = new FileOutputStream(outfile);
            
            // /////////////////////////////////////////
            // wrap output stream in ZipOutputStream
            // /////////////////////////////////////////
            
            ZipOutputStream zos = new ZipOutputStream(os);
    
            for(int i = 0; i < filename.length; i++) {
                File f = new File(filename[i]);
                
                FileInputStream is = new FileInputStream(f);
                
                // ///////////////////////////////////////////////////////
                // set entry field in zipfile; do this once for each
                // file you're compressing
                // ///////////////////////////////////////////////////////
                zos.putNextEntry(new ZipEntry(f.getName()));
                
                // /////////////////////////////////////////////////////////
                // read the input file in and write it to the zipfile
                // /////////////////////////////////////////////////////////
                do{
                	// ////////////////////////////////////
                    // read file in from input stream
                	// ////////////////////////////////////
                    retval = is.read(buf, 0, 4096);
                    
                    // /////////////////////////////
                    // -1 indicates end of file
                    // /////////////////////////////
                    if (retval != -1){
                    	// ///////////////////////////////
                        // write buffer to output stream
                    	// ///////////////////////////////
                    	
                    	zos.write(buf, 0, retval);
                    }
                    
                }while (retval != -1);
            
                is.close();              
            }
  
            // ////////////////////////
            // close this ZipEntry
            // ////////////////////////
            zos.closeEntry();
            
            // /////////////////////////////
            // close input stream
            // /////////////////////////////
            
            // //////////////////////////////
            // close output stream
            // //////////////////////////////
            
            zos.close();
        } catch (FileNotFoundException e) {
            logger.warn("File non trovato" + e.getMessage());
            throw e;
        } catch (IOException e) {
            logger.warn("IOException " + e.getMessage() );
            throw e;
        }
    }
    
    public static List<File> uncompress_file(File inFile, File outDir, 
    		LogInterface logger) throws IllegalArgumentException, IOException {       
    	
        String errMsg = null;
        
        if(inFile == null){
            errMsg = "Il file di input  nullo";
            logger.error(errMsg);
            throw new IllegalArgumentException(errMsg);
        }
        
        if(!inFile.exists()){
            errMsg = "Il file di input(" + inFile.getAbsolutePath() + ") non esiste";
            logger.error(errMsg);
            throw new IllegalArgumentException(errMsg);
        }
        
        if (!inFile.canRead()) {
            errMsg = "Il file di input(" + inFile.getAbsolutePath() + ")  protetto da lettura";
            logger.error(errMsg);
            throw new IllegalArgumentException(errMsg);
        }
        
        if(outDir == null){
            errMsg = "La directory di output  nulla";
            logger.error(errMsg);
            throw new IllegalArgumentException(errMsg);            
        }
        
        if(outDir.exists() && !outDir.isDirectory()){
            errMsg = "Il file di output (" + outDir.getAbsolutePath() + ") non  una directory";
            logger.error(errMsg);
            throw new IllegalArgumentException(errMsg);
            
        }else if(!outDir.exists()){                                    
            boolean createdOutDir = outDir.mkdir();
            
            if(!createdOutDir){
                errMsg = "Impossibile creare la directory di output";
                logger.error(errMsg);
                throw new IOException(errMsg);
            }
            
            logger.info("Creata directory " + outDir.getAbsolutePath());                        
        }
        
        ZipInputStream zipIn = null;
        ZipEntry entry = null;
        FileOutputStream outStream = null;
        int size;                             //return value
        
        ArrayList<File> files = new ArrayList<File>();
        
        try {                        
            byte[] buf = new byte[4096];   //read buffer
                    
            // //////////////////////////
            // imposto input stream
            // //////////////////////////
            
            FileInputStream is = new FileInputStream(inFile);
            
            // /////////////////////////////////////////
            // wrappo output stream in ZipOutputStream
            // /////////////////////////////////////////
            zipIn = new ZipInputStream(is);
                       
            while ((entry = zipIn.getNextEntry()) != null) {
                           
                File out = new File(outDir,entry.getName());                
                files.add(out);
                
                if (entry.isDirectory()) {
                    if (!out.mkdirs()) {
                        logger.error("Impossibile creare la directory: " + out.getName());
                        throw new IOException("Impossibile creare la directory: " + out.getAbsolutePath());
                    }
                } else {
                    try{
                        outStream = new FileOutputStream(new File(outDir,entry.getName()));
                                        
                        while ((size = zipIn.read(buf, 0, buf.length)) != -1) {                        
                            outStream.write(buf, 0, size);                                            
                        }               
                        
                    }finally{
                        if(outStream != null)
                            outStream.close();
                    }                    
                }                     
            }
            
            return files;

        } catch (FileNotFoundException e) {
            logger.error("File non trovato" + e.getMessage());
            throw e;
        } catch (IOException e) {
            logger.error("IOException " + e.getMessage() );
            throw e;
        }finally{
            if(zipIn != null){
                zipIn.closeEntry();
                zipIn.close();
            }
        }
    }
    
    /**
     * Ritorna la dimensione totale dei file decompressi contenuti nello zip
     * @param inFile Zip file
     * @param logger Logger
     * @return
     * @throws IllegalArgumentException
     * @throws IOException
     */
    public static long getUncompressedSize(File inFile, LogInterface logger) 
    		throws IllegalArgumentException, IOException {       
    	
        String errMsg = null;
        
        if(inFile == null){
            errMsg = "Il file di input  nullo";
            logger.error(errMsg);
            throw new IllegalArgumentException(errMsg);
        }
        
        if(!inFile.exists()){
            errMsg = "Il file di input(" + inFile.getAbsolutePath() + ") non esiste";
            logger.error(errMsg);
            throw new IllegalArgumentException(errMsg);
        }
        
        if (!inFile.canRead()) {
            errMsg = "Il file di input(" + inFile.getAbsolutePath() + ")  protetto da lettura";
            logger.error(errMsg);
            throw new IllegalArgumentException(errMsg);
        }
        
        ZipInputStream zipIn = null;
        ZipEntry entry = null;
        long size = 0L;
        
        
        try {                        
            FileInputStream is = new FileInputStream(inFile);
            zipIn = new ZipInputStream(is);
                       
            while ((entry = zipIn.getNextEntry()) != null) {
            	if (!entry.isDirectory()) size += entry.getSize();
            }
            return size;

        } catch (FileNotFoundException e) {
            logger.error("File non trovato" + e.getMessage());
            throw e;
        } catch (IOException e) {
            logger.error("IOException " + e.getMessage() );
            throw e;
        }finally{
            if(zipIn != null){
                zipIn.closeEntry();
                zipIn.close();
            }
        }
    }
    
    /**
     * Ritorna l'elenco dei nomi dei file contenuti nello zip.
     * @param inFile Zip file
     * @param logger Logger
     * @return elenco dei nomi
     * @throws IllegalArgumentException
     * @throws IOException
     */
    public static List<String> getFileNameList(File inFile, LogInterface logger) 
    		throws IllegalArgumentException, IOException {       
    	
        String errMsg = null;
        List<String> retVal= new ArrayList<String>();
        
        if(inFile == null){
            errMsg = "Il file di input  nullo";
            logger.error(errMsg);
            throw new IllegalArgumentException(errMsg);
        }
        
        if(!inFile.exists()){
            errMsg = "Il file di input(" + inFile.getAbsolutePath() + ") non esiste";
            logger.error(errMsg);
            throw new IllegalArgumentException(errMsg);
        }
        
        if (!inFile.canRead()) {
            errMsg = "Il file di input(" + inFile.getAbsolutePath() + ")  protetto da lettura";
            logger.error(errMsg);
            throw new IllegalArgumentException(errMsg);
        }
        
        ZipInputStream zipIn = null;
        ZipEntry entry = null;
        
        try {                        
            FileInputStream is = new FileInputStream(inFile);
            zipIn = new ZipInputStream(is);
                       
            while ((entry = zipIn.getNextEntry()) != null) {
            	if (!entry.isDirectory()) retVal.add(entry.getName());
            }
            return retVal;

        } catch (FileNotFoundException e) {
            logger.error("File non trovato" + e.getMessage());
            throw e;
        } catch (IOException e) {
            logger.error("IOException " + e.getMessage() );
            throw e;
        }finally{
            if(zipIn != null){
                zipIn.closeEntry();
                zipIn.close();
            }
        }
    }
    
}
