/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/


package it.prato.comune.sit.ows;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 *  Bean destinato a contenere le informazioni relative ad un layer e recuperate dalla chiamata GetCapabilities
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public class WMSLayerCapabilitiesBean {

	private String name=null;
	private String title=null;
	private String layerAbstract=null;
	private List<WMSStyleCapabilities> styles = new ArrayList<WMSStyleCapabilities>();
	private Double MinScaleDenominator=null;
	private Double MaxScaleDenominator=null;
	private Map<String, WMSLayerCapabilitiesBBox> bboxMap = new HashMap<String, WMSLayerCapabilitiesBBox>();
	private WMSLayerCapabilitiesBBox bboxlatlon = null;
	private WMSLayerCapabilitiesAttributionBean attribution = null;
	private List<WMSCapabilitiesMetadataURLBean> metadataUrlList = new ArrayList<WMSCapabilitiesMetadataURLBean>();
	
	/** Indica se il layer  interrogabile o no. */
	private Boolean queryable = false;
	/** Formati disponibili per la chiamata GetFeatureInfo. */
	private final List<String> getFeatureInfoFormats = new ArrayList<String>();
	
	/**
	 * @param styles the styles to set
	 */
	public void setStyles(List<WMSStyleCapabilities> styles) {
		this.styles = styles;
	}

	/**
	 * @return the styles
	 */
	public List<WMSStyleCapabilities> getStyles() {
		return styles;
	}

	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return the title
	 */
	public String getTitle() {
		return title;
	}

	/**
	 * @param title the title to set
	 */
	public void setTitle(String title) {
		this.title = title;
	}

	/**
	 * @return the layerAbstract
	 */
	public String getLayerAbstract() {
		return layerAbstract;
	}

	/**
	 * @param layerAbstract the layerAbstract to set
	 */
	public void setLayerAbstract(String layerAbstract) {
		this.layerAbstract = layerAbstract;
	}

	/**
	 * @return the minScaleDenominator
	 */
	public Double getMinScaleDenominator() {
		return MinScaleDenominator;
	}

	/**
	 * @param minScaleDenominator the minScaleDenominator to set
	 */
	public void setMinScaleDenominator(Double minScaleDenominator) {
		MinScaleDenominator =  (minScaleDenominator!=null &&  minScaleDenominator.equals(Double.NaN) && !Double.isInfinite(minScaleDenominator)) ? null :  minScaleDenominator ;
	}

	/**
	 * @param minScaleDenominator the minScaleDenominator to set
	 */
	public void setMinScaleDenominatorFromString(String minScaleDenominatorString) {
		setMinScaleDenominator(Double.valueOf(minScaleDenominatorString));
	}
	
	public void setMinScaleDenominatorFromScaleHint(Double scaleHintMin) {
		setMinScaleDenominator(convertScaleHintToScaleDenominator(scaleHintMin));
	}
	
	/**
	 * @return the maxScaleDenominator
	 */
	public Double getMaxScaleDenominator() {
		return MaxScaleDenominator;
	}

	/**
	 * @param maxScaleDenominator the maxScaleDenominator to set
	 */
	public void setMaxScaleDenominator(Double maxScaleDenominator) {
		MaxScaleDenominator = (maxScaleDenominator!=null && !maxScaleDenominator.equals(Double.NaN) && !Double.isInfinite(maxScaleDenominator)) ? maxScaleDenominator : null;
	}
	
	/**
	 * @param maxScaleDenominator the maxScaleDenominator to set
	 */
	public void setMaxScaleDenominatorFromString(String maxScaleDenominatorString) {
		setMaxScaleDenominator(Double.valueOf(maxScaleDenominatorString));
	}
	
	public void setMaxScaleDenominatorFromScaleHint(Double scaleHintMax) {
		setMaxScaleDenominator(convertScaleHintToScaleDenominator(scaleHintMax));
	}
	
	/**
	 * Calcolo scaleDenominator a partire da ScaleHint
	 * Formule trovate su http://wiki.deegree.org/deegreeWiki/HowToUseScaleHintAndScaleDenominator
	 * @param scaleHint
	 * @return
	 */
	private Double convertScaleHintToScaleDenominator(Double scaleHint) {
		
		// Quanto trovato su  http://wiki.deegree.org/deegreeWiki/HowToUseScaleHintAndScaleDenominator indicherebbe 0.00028
		return  scaleHint / 0.00028 / Math.sqrt(2)  ;//Math.sqrt(((Math.pow((scaleHint/0.00028), 2))/2));
		
	}

	/**
	 * Contiene i bounding box nei diversi sitemi di riferimento. La chiave della mappa  il codice epsg convertito toUpperCase
	 * @return the bboxMap
	 */
	public Map<String, WMSLayerCapabilitiesBBox> getBboxMap() {
		return bboxMap;
	}

	/**
	 * @return the bboxlatlon
	 */
	public WMSLayerCapabilitiesBBox getBboxlatlon() {
		return bboxlatlon;
	}

	/**
	 * @param bboxlatlon the bboxlatlon to set
	 */
	public void setBboxlatlon(WMSLayerCapabilitiesBBox bboxlatlon) {
		this.bboxlatlon = bboxlatlon;
	}

	/**
	 * Indica se il layer  interrogabile o no.
	 * @return true se  interrogabile
	 */
	public final Boolean isQueryable() {
		return queryable;
	}

	/**
	 * Imposta se il layer  interrogabile o no.
	 * @param queryable valore da impostare
	 */
	public void setQueryable(final Boolean queryable) {
		this.queryable = queryable;
	}
	
	/**
	 * Restituisce i formati disponibili per la chiamata GetFeatureInfo.
	 * 
	 * @return the getFeatureInfoFormats
	 */
	public final List<String> getGetFeatureInfoFormats() {
		return getFeatureInfoFormats;
	}

	/**
	 * Restituisce l'attribuzione.
	 * @return attribuzione
	 */
	public final WMSLayerCapabilitiesAttributionBean getAttribution() {
		return attribution;
	}

	/**
	 * Imposta l'attribuzione.
	 * 
	 * @param attribution Attribuzione da impostare
	 */
	public final void setAttribution(WMSLayerCapabilitiesAttributionBean attribution) {
		this.attribution = attribution;
	}

	/**
	 * Restituisce metadataUrlList
	 * @return metadataUrlList
	 */
	public final List<WMSCapabilitiesMetadataURLBean> getMetadataUrlList() {
		return metadataUrlList;
	}

}
