/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/

package it.prato.comune.sit.ows;

import it.prato.comune.sit.SITException;
import it.prato.comune.sit.runtime.util.SITVersion;
import it.prato.comune.utilita.logging.interfaces.LogInterface;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;

import org.geotools.data.ows.CRSEnvelope;
import org.geotools.data.ows.HTTPClient;
import org.geotools.data.ows.Layer;
import org.geotools.data.ows.Service;
import org.geotools.data.ows.SimpleHttpClient;
import org.geotools.data.ows.StyleImpl;
import org.geotools.data.ows.WMSCapabilities;
import org.geotools.data.wms.WMSUtils;
import org.geotools.data.wms.WebMapServer;
import org.geotools.data.wms.xml.Attribution;
import org.geotools.data.wms.xml.LogoURL;
import org.geotools.data.wms.xml.MetadataURL;
import org.geotools.ows.ServiceException;

/**
 * 
 * Classe che consente di recuperare informazioni da servizi WMS (come elenco layer disponibili, stili per ogni layer etc.)
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public class WMSService {

	private URL	url= null;
	private LogInterface logger=null; 
	private WMSCapabilitiesBean capabilities=null;
	private int timeout = 0; 
	
	/**
	 * Costruttore 
	 * 
	 * @param url Url del servizio WMS 
	 * @param logger
	 * @throws SITException
	 */
	public WMSService(String url, LogInterface logger) throws SITException {
		this(url, 0, logger);
	}
	
	
	/**
     * Costruttore 
     * 
     * @param url Url del servizio WMS
     * @param logger timeout chiamate al server 
     * @param logger
     * @throws SITException
     */
    public WMSService(String url, int timeout, LogInterface logger) throws SITException {
        try {
            this.url=new URL(url);
        } catch (MalformedURLException e) {
            String msg= "WMSService: Url (" + url + ") malformata. Verificare url e credenziali geoserver (in preset e tolomeo.properties) e configurazione di geoserver" ;
            logger.warn(msg, e );
            throw new SITException(msg, e);
        }
        this.logger=logger;
        this.timeout = timeout;
    }
    
	
	/*
	Su richiesta di RT era stato inserito il calcolo dell'intervallo di visibilit anche in funzione dell'intervallo di antenati e successori
	08/02/2013 rimosso calcolo che tiene conto di antenati e successori su richiesta di RT
		
	private Double[] calcolaDenomTuttiSuccessori(Layer lay, boolean bUseScaleDenom) {
		
		// lui
		Double max = (bUseScaleDenom) ? lay.getScaleDenominatorMax() : lay.getScaleHintMax();
		Double min = (bUseScaleDenom) ? lay.getScaleDenominatorMin() : lay.getScaleHintMin();
		
		Double[] minmax = { (min!=null && !min.equals(Double.NaN)) ? min : Double.MAX_VALUE , (max!=null && !max.equals(Double.NaN)) ? max : Double.MIN_VALUE };
		
		// tutti i successori
		Layer[] layChildren = lay.getChildren();
		if (layChildren!=null) {
			for (Layer lay1: layChildren) {
				
				Double[] minmaxbuff = calcolaDenomTuttiSuccessori(lay1, bUseScaleDenom);
				if (!minmaxbuff[0].equals(Double.NaN)) minmax[0] = Math.min(minmax[0],minmaxbuff[0] );
				if (!minmaxbuff[1].equals(Double.NaN)) minmax[1] = Math.max(minmax[1],minmaxbuff[1] );
			}
			
		}
		
		return minmax;
	}
	*/
	/*
	 * 	Su richiesta di RT era stato inserito il calcolo dell'intervallo di visibilit anche in funzione dell'intervallo di antenati e successori
		08/02/2013 rimosso calcolo che tiene conto di antenati e successori su richiesta di RT
	
	private Double[] calcolaDenomTuttiAntenati(Layer lay, boolean bUseScaleDenom) {
		
		// lui
		Double max = (bUseScaleDenom) ? lay.getScaleDenominatorMax() : lay.getScaleHintMax();
		Double min = (bUseScaleDenom) ? lay.getScaleDenominatorMin() : lay.getScaleHintMin();
				
		Double[] minmax = { (min!=null && !min.equals(Double.NaN)) ? min : Double.MAX_VALUE , (max!=null && !max.equals(Double.NaN)) ? max : Double.MIN_VALUE  };
		
		// tutti i successori
		Layer layParent = lay.getParent();
		// Esamino l'antenato a patto che non sia l'ultimo
		if (layParent!=null && layParent.getParent()!=null) {
			
			Double[] minmaxbuff = calcolaDenomTuttiAntenati(layParent, bUseScaleDenom);
			if (!minmaxbuff[0].equals(Double.NaN)) {
				minmax[0] = Math.min(minmax[0],minmaxbuff[0] );
			} 
			
			if (!minmaxbuff[1].equals(Double.NaN)) {
				minmax[1] = Math.max(minmax[1],minmaxbuff[1] );
			}
		}
		
		return minmax;
	}
	*/
	
	private Double[] calcolaDenom(Layer lay, boolean bUseScaleDenom) {
		
		// Antenati e successori, il layer stesso viene in realt considerato sia tra i successori che tra gli antenati
		/*
		Su richiesta di RT era stato inserito il calcolo dell'intervallo di visibilit anche in funzione dell'intervallo di antenati e successori
		08/02/2013 rimosso calcolo che tiene conto di antenati e successori su richiesta di RT
		
		 
		Double[] minmax = calcolaDenomTuttiAntenati(lay, bUseScaleDenom);
		Double[] minmax1 = calcolaDenomTuttiSuccessori(lay, bUseScaleDenom);

		Double retVal[] = new Double[2];
		retVal[0] = Math.min(minmax[0], minmax1[0]);
		retVal[1] = Math.max(minmax[1], minmax1[1]);
		retVal[0] = (retVal[0].equals(Double.MAX_VALUE)) ? Double.NaN : retVal[0];
		retVal[1] = (retVal[1].equals(Double.MIN_VALUE)) ? Double.NaN : retVal[1];
		 */
		
		// lui
		Double max = (bUseScaleDenom) ? lay.getScaleDenominatorMax() : lay.getScaleHintMax();
		Double min = (bUseScaleDenom) ? lay.getScaleDenominatorMin() : lay.getScaleHintMin();
		
		Double[] minmax = { (min!=null && !min.equals(Double.NaN)) ? min : Double.NaN , (max!=null && !max.equals(Double.NaN)) ? max : Double.NaN  };
		
		
		return minmax;
		
	}
		
	
	/**
	 * Recupera le informazioni relative alla getCapabilities. 
	 * 
	 * TODO: per il momento sono recuperate solo le info strettamente necessarie per la gestione della legenda in Tolomeo, ma le classi "wrappate" recuperano molto di pi
	 * 
     * @return
     * @throws SITException
	 */
	public WMSCapabilitiesBean getCapabilities() throws SITException {
		
		if (capabilities!=null) return capabilities;
		
		capabilities = new WMSCapabilitiesBean();
			
			try {			
			    
				WebMapServer wms = null;
				
				if (this.timeout == 0) {
				    wms = new WebMapServer(url);
				} else {
				    HTTPClient httpClient = new SimpleHttpClient();
				    httpClient.setConnectTimeout(timeout);
				    httpClient.setReadTimeout(timeout);				    
				    wms = new WebMapServer(url,httpClient);
				}
				
				WMSCapabilities cap = wms.getCapabilities();
				
				Service ser = cap.getService();
				if (ser!=null) {
					capabilities.setMaxHeight(ser.getMaxHeight());
					capabilities.setMaxWidth(ser.getMaxWidth());
					capabilities.setLayerLimit(ser.getLayerLimit());
				}
				
				Layer[] layerList = WMSUtils.getNamedLayers(cap);
				
				for (Layer lay: layerList) {
					
					
					// Istanzio il layer bean
					WMSLayerCapabilitiesBean layercb = new WMSLayerCapabilitiesBean();
					// Inizializzo i valori del layer a partire dall'oggetto layer di geotools 
					layercb.setName(lay.getName());
					layercb.setLayerAbstract(lay.get_abstract());
					layercb.setTitle(lay.getTitle());
					SITVersion sv = SITVersion.parse(cap.getVersion());
					boolean bUseScaleDenom = true;
					if ( sv.getMajor()>1 || 
						 (sv.getMajor()==1 && sv.getMinor()>=3)) {
						bUseScaleDenom = true;
						Double[] den = calcolaDenom(lay, bUseScaleDenom);	
						layercb.setMaxScaleDenominator(den[1]);
						layercb.setMinScaleDenominator(den[0]);
					} else {
						bUseScaleDenom = false;
						Double[] den = calcolaDenom(lay, bUseScaleDenom);
						layercb.setMaxScaleDenominatorFromScaleHint(den[1]);
						layercb.setMinScaleDenominatorFromScaleHint(den[0]);
					}
								
					// Imposto se il layer  interrogabile
					layercb.setQueryable(lay.isQueryable());
					
					// Aggiunta formati supportati dalla chiamata getFeatureInfo
					layercb.getGetFeatureInfoFormats().clear();
					if (cap.getRequest().getGetFeatureInfo() != null 
							&& cap.getRequest().getGetFeatureInfo().getFormats() != null) {
						layercb.getGetFeatureInfoFormats().addAll(cap.getRequest().getGetFeatureInfo().getFormats());
					}
					
					// Scorro gli stili e li aggiungo
					for (StyleImpl st: lay.getStyles()) {
                		// Istanzio lo style bean
						WMSStyleCapabilities stylecb = new WMSStyleCapabilities();
                		//Inizializzo i valori dello stile a partire dall'oggetto di geotools
						stylecb.setName(st.getName());
						String styleTitle = (st.getTitle()!=null) ? st.getTitle().toString() : "";
						stylecb.setTitle(styleTitle);
						String styleAbstract = (st.getAbstract()!=null) ? st.getAbstract().toString() : "";
						stylecb.setStyleAbstract(styleAbstract);
						
						//Aggiungo lo stile nel layer bean
						layercb.getStyles().add(stylecb);
                	}

					// Bounding box
					List<CRSEnvelope> bboxmap = lay.getLayerBoundingBoxes();
					for (CRSEnvelope crsenv: bboxmap) {
							WMSLayerCapabilitiesBBox lcbbbox = new WMSLayerCapabilitiesBBox();
							lcbbbox.setEpsg(crsenv.getEPSGCode());
							lcbbbox.setMaxX(crsenv.getMaxX());
							lcbbbox.setMaxY(crsenv.getMaxY());
							lcbbbox.setMinX(crsenv.getMinX());
							lcbbbox.setMinY(crsenv.getMinY());
							layercb.getBboxMap().put(crsenv.getEPSGCode().toUpperCase(), lcbbbox); 
					}
					// LatLon BBOX 
					CRSEnvelope latlonenv = lay.getLatLonBoundingBox();
					WMSLayerCapabilitiesBBox latlonbbox = new WMSLayerCapabilitiesBBox();
					latlonbbox.setEpsg("EPSG:4326");
					latlonbbox.setMaxX(latlonenv.getMaxX());
					latlonbbox.setMaxY(latlonenv.getMaxY());
					latlonbbox.setMinX(latlonenv.getMinX());
					latlonbbox.setMinY(latlonenv.getMinY());
					layercb.setBboxlatlon(latlonbbox);
					
					// Attribution
					Attribution attribution = lay.getAttribution();
					WMSLayerCapabilitiesAttributionBean attribBean = null;
					if (attribution!=null) {
						attribBean = new WMSLayerCapabilitiesAttributionBean();
						attribBean.setTitle(attribution.getTitle());
						attribBean.setOnlineResource(attribution.getOnlineResource());
						LogoURL lu = attribution.getLogoURL(); 
						if (lu!=null) {
							WMSLayerCapabilitiesLogoURLBean logoUrl = new WMSLayerCapabilitiesLogoURLBean();
							logoUrl.setFormat(lu.getFormat());
							logoUrl.setHeight(lu.getHeight());
							logoUrl.setWidth(lu.getWidth());
							logoUrl.setOnlineResource(lu.getOnlineResource());
							attribBean.setLogoURL(logoUrl);
						}
					}
					layercb.setAttribution(attribBean);
				
					// MetadataURL
					List<MetadataURL> mdList = lay.getMetadataURL();
					if (mdList!=null && mdList.size()>0) {
						List<WMSCapabilitiesMetadataURLBean> mdCapList = layercb.getMetadataUrlList();
						for(MetadataURL mu: mdList) {
							WMSCapabilitiesMetadataURLBean muBean = new WMSCapabilitiesMetadataURLBean();
							muBean.setFormat(mu.getFormat());
							muBean.setType(mu.getType());
							muBean.setUrl(mu.getUrl().toExternalForm());
							mdCapList.add(muBean);
						}
					}
					
					// Aggiungo il layer al capabilities bean
					capabilities.getLayers().add(layercb);
					
				}
				
			} catch (ServiceException e) {
				String msg= "Errore durante la connessione con il WMS (" + url +"). Verificare la url e la disponibilit del servizio" ;
				logger.warn(msg, e);
				throw new SITException(msg, e);
			} catch (IOException e) {
				String msg= "Errore durante la connessione con il WMS(" + url +"). Verificare la url e la disponibilit del servizio" ;
				logger.warn(msg, e);
				throw new SITException(msg, e);
			}
			
			return capabilities;
		
	}
	
}
