/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
package it.prato.comune.sit.runtime;

import it.prato.comune.sit.SITException;
import it.prato.comune.sit.runtime.util.SITAPIVersion;
import it.prato.comune.sit.runtime.util.SITVersion;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

/**
 * Classe che recupera la versione del SIT core e dell'API dal file di propriet in cui 
 * questi dati sono contenuti.
 * 
 * @author bf44
 *
 */
public class SITCoreVersion {

    private static final String PROP_FILE_NAME = "application.properties";
    public static final String PROP_NAME_VERSION = "application.version";
    private static SITCoreVersion coreVersion = new SITCoreVersion();
    private static boolean loaded = false;
    private SITVersion version;

    private SITCoreVersion() {}

    /**
     * Resituisce l'unica istanza della classe SITCoreVersion
     * @return
     * @throws SITException
     */
    public static synchronized SITCoreVersion getInstance() throws SITException {        

        if(!loaded){                
            InputStream inStream = null;
            try {             
                
                inStream = SITCoreVersion.class.getResourceAsStream(PROP_FILE_NAME);        
                Properties versionProp = new Properties();                        
                versionProp.load(inStream);
                coreVersion.version = SITVersion.parse(versionProp.getProperty(PROP_NAME_VERSION));
                loaded = true;
                
            } catch (IOException e) {
                throw new SITException("Erorre nel recupero delle propriet di versione",e);
            } finally {
                if (inStream != null){
                    try {
                        inStream.close();
                    } catch (IOException e) {}
                }
            }
        }
        return coreVersion;
    }


    /**
     * Restituisce la versione del SIT core.
     * @return SITVersion.
     */
    public SITVersion getCoreVersion(){    	
        return this.version; 
    }

    /**
     * Restituisce la versione delle API a cui corrisponde il SIT core.
     * @return SITAPIVersion
     */
    public SITAPIVersion getAPIVersion(){
        return new SITAPIVersion(version.getMajor(),version.getMinor());
    }

}
