/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
package it.prato.comune.sit.runtime.util;

import it.prato.comune.sit.SITParseException;

import java.text.MessageFormat;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Classe che implementa l'interfaccia APIVersion
 * @author bf44
 *
 */
public class SITAPIVersion implements APIVersion {

    private int major;
    private int minor;    
            
    /**
     * Csotruttore che accetta i due interi rappresentanti la major e la minor version 
     * @param major int. Major version.
     * @param minor int. Minor version.
     */
    public SITAPIVersion(int major, int minor){
        this.major = major;
        this.minor = minor;       
    }

    /**
     * @see APIVersion#getMajor()
     */
    public int getMajor() {
        return this.major;
    }

    /**
     * @see APIVersion#getMinor()
     */
    public int getMinor() {
        return this.minor;
    }
    
    /**
     * Metodo statico che restituisce un oggetto SITAPIVersion parsando una stringa che rappresenti la versione.
     * 
     * @param version
     * @return SITVersion
     * @throws SITParseException Lanciata qualora la stringa che rappresenta la versione delle API non sia correttamente formattata.
     */
    public static SITAPIVersion parse(String apiVersion) throws SITParseException {
        
        String regExp = "^(\\d+)\\.(\\d+)$";
        Pattern pattern = Pattern.compile(regExp);    
        Matcher m = pattern.matcher(apiVersion);
        if(!m.matches()){
            throw new SITParseException("La versione fornita (" + apiVersion + ") non risulta aver un formato corretto : major.minor");
        }
        
        String majorS = m.group(1);
        String minorS = m.group(2);
        
        int major = Integer.parseInt(majorS);
        int minor = Integer.parseInt(minorS);

        return new SITAPIVersion(major,minor);
        
    }

    /**
     * @see APIVersion#format()
     */
    public String format() {
        MessageFormat mf = new MessageFormat("{0,number,#}.{1,number,#}");                    
        Object[] arguments = {new Integer(getMajor()),new Integer(getMinor())};        
        return mf.format(arguments);
    }           
        
}
