/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
package it.prato.comune.sit.runtime.util;

import it.prato.comune.sit.SITException;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

/**
 * Classe astratta che implementa l'interfaccia PluginVersion.
 * Questa classe permette la gestione della versione per mezzo di un file id propriet.
 * Pu essere utilizzata dai plugins per la gestione della loro versione.
 * @author bf44
 */
public abstract class SITPluginVersion implements PluginVersion{
    
    private Properties runtimeProp;
    private SITVersion version;
    private SITAPIVersion apiComplianceVersion;
    
    protected SITPluginVersion(){}
    
    /**
     * Restituisce l'InputStream del file di properties che contiene le informazioni sulla versione.
     * @return InputStream del file di properties
     * @throws SITException. Da lanciare in caso di problemi nella lettura del file
     */
    public abstract InputStream getVersionProperties() throws SITException;
    
    /**
     * Restituisce il nome della propriet che nel file di properties rappresenta la versione del plugin.<br />
     * Da sovrascrivere nel caso in cui il nome sia diverso da quello di default.
     * @see PluginVersion#PROP_NAME_PLUGIN_VERSION  
     * @return String. Nome della propriet che rappresenta la versione del plugin.
     * @throws SITException
     */
    public String getPluginVersionPropertyName() throws SITException {
        return PluginVersion.PROP_NAME_PLUGIN_VERSION;
    }
    
    /**
     * Restituisce il nome della propriet che nel file di properties rappresenta la versione delle API 
     * con cui il plugin  compatibile. <br />
     * Da sovrascrivere nel caso in cui il nome sia diverso da quello di default.
     * @see PluginVersion#PROP_NAME_APICOMPLIANCE_VERSION
     * @return String. Nome della propriet che rappresenta la versione delle API con cui il plugin  compatibile. 
     * @throws SITException
     */
    public String getAPIComplianceVersionPropertyName() throws SITException {
        return PluginVersion.PROP_NAME_APICOMPLIANCE_VERSION;
    } 
    
    /**
     * Inizializza le variabili di classe leggendo il file di properties.
     * 
     * @throws IOException. Lanciata se ci sono problemi di lettura del file.
     * @throws SITException
     */
    protected void init() throws IOException,SITException {
        if(runtimeProp != null) return;
        
        InputStream inStream = getVersionProperties();
        this.runtimeProp = new Properties();
        try{
            this.runtimeProp.load(inStream);
        }finally{
            if (inStream != null){
                inStream.close();
            }
        }
            
        this.version = SITVersion.parse(this.runtimeProp.getProperty(getPluginVersionPropertyName()));
        this.apiComplianceVersion = SITAPIVersion.parse(this.runtimeProp.getProperty(getAPIComplianceVersionPropertyName()));
    }
    
    /**
     * @see PluginVersion#getVersion()
     */
    public Version getVersion(){
        return this.version; 
    }
    
    /**
     * @see PluginVersion#getAPIComplianceVersion()
     */
    public APIVersion getAPIComplianceVersion(){
        return this.apiComplianceVersion;
    }
    
}
