/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
/*
 * Copyright  Comune di Prato - Sistema Informativo
 * Project:    Package SIT per elaborazione/interrogazione/gestione dati cartografici 
 * File:       TestSITBaseLayer.java
 * Author:     Alessandro Radaelli
 * Version:    1.0.0
 * CreationDate:  
 * ModifyDate:     
 *             
 */
package it.prato.comune.sit.test;

import it.prato.comune.sit.SITDefaultTransaction;
import it.prato.comune.sit.SITException;
import it.prato.comune.sit.SITLayersManager;
import it.prato.comune.sit.SITTransaction;
import it.prato.comune.utilita.logging.PratoLogger;
import it.prato.comune.utilita.logging.interfaces.LogInterface;

import java.util.Properties;

import junit.framework.TestCase;

import org.apache.log4j.PropertyConfigurator;
import org.junit.AfterClass;
import org.junit.BeforeClass;

/**
 * Classe TestSITBaseLayer.
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public abstract class TestSITBase extends TestCase{

	protected LogInterface logger = null;
	protected SITLayersManager layersManager = null;
	protected SITTransaction transaction = null;
	
	/**
	 * Path assoluto del file di configurazione per il LayersManager
	 * @return String
	 */
	protected abstract String getConfigFileName();
	
	/**
     * Path assoluto del file di log dell'applicazione di test.
     * Se non sovrascritto il valore  c:/logSITProve.txt 
     * @return String
     */
    protected String getLogFileName(){
        return "logSITProve.txt";
    };
	
	/**
	 * Inizializzazione del contesto.
	 * 
	 * @throws Exception
	 */
	@BeforeClass
	public void setUp() throws Exception {

		// ///////////////////////////////////
		// ... Acquisizione Logger
		// ///////////////////////////////////
		Properties prop = new Properties();
        prop.put("log4j.rootLogger","DEBUG,stdout");
        prop.put("log4j.appender.stdout","org.apache.log4j.ConsoleAppender");
        //prop.put("log4j.appender.stdout.Threshold","FATAL");
        prop.put("log4j.appender.stdout.layout","org.apache.log4j.PatternLayout");
        prop.put("log4j.appender.stdout.layout.ConversionPattern","%d{HH:mm:ss}-%C{1}[%M] %-5p %x -%m%n");
		prop.put("log4j.logger.pippo","DEBUG,fileout");
		prop.put("log4j.appender.fileout","org.apache.log4j.DailyRollingFileAppender");
		prop.put("log4j.appender.fileout.File",getLogFileName());
		prop.put("log4j.appender.fileout.layout","org.apache.log4j.PatternLayout");
		prop.put("log4j.appender.fileout.layout.ConversionPattern","%d{HH:mm:ss} %X{user} %X{address} %-8p %C{1}[%M] %x %m%n");
		prop.put("log4j.appender.fileout.DatePattern",".yyyyMMdd");
		PropertyConfigurator.configure(prop);
		logger = new PratoLogger("pippo","---.---.---.---","NOUSER");
		//System.out.println("*** Logger pronto!");

		//logger.debug("Inizio log");            
		logger.setIpAddress("0.0.0.0");
		logger.setUser("prova");
		this.layersManager = new SITLayersManager(getConfigFileName(), logger,  "Comune di Prato");		
		layersManager.setUser("TEST");
		layersManager.setIpAddress("0.0.0.0");
		transaction = new SITDefaultTransaction();
		
	}
	
	@AfterClass
	public void tearDown(){
		try {
			this.layersManager.dispose();
		} catch (SITException e) {
			e.printStackTrace();
		}
	}

}
