/*******************************************************************************
 * Tolomeo is a developing framework for visualization, editing,  
 * geoprocessing and decisional support application based on cartography.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * This file is part of Tolomeo.
 * 
 * Tolomeo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 3 of the License, 
 * or (at your option) any later version.
 * 
 * Tolomeo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with Tolomeo; 
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  021101301  USA
 * 
 * Developers Information:
 * 
 * Tolomeo is developed by Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it 
 * 
 * 
 * Versione in Italiano LGPL
 * 
 * Tolomeo  un framework per lo sviluppo di applicazioni per
 * visualizzazione, editing, geoprocessing e supporto alla decisione basate su cartografia.
 * 
 * Tolomeo Copyright 2011 Comune di Prato;
 * 
 * Questo file fa parte di Tolomeo.
 * 
 * Tolomeo  un software libero;  possibile redistribuirlo e / o 
 * modificarlo sotto i termini della GNU Lesser General Public License, 
 * come pubblicato dalla Free Software Foundation, sia la versione 3 della licenza o (a propria scelta) una versione successiva.
 *  
 * Tolomeo  distribuito nella speranza che possa essere utile,
 * ma SENZA ALCUNA GARANZIA, senza neppure la garanzia implicita di COMMERCIABILIT o
 * IDONEIT PER UN PARTICOLARE SCOPO. Vedere la GNU Lesser General Public License per ulteriori dettagli.
 * 
 * Si dovrebbe avere ricevuto una copia della GNU Lesser General Public insieme a Tolomeo, in caso contrario, 
 * si scriva alla Free Software  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 021101301 USA
 *  
 * 
 * Informazioni Sviluppatori:
 * 
 * Tolomeo  sviluppato dal Comune di Prato
 * 
 * Alessandro Radaelli
 * Federico Nieri
 * Mattia Gennari
 * 
 * sit@comune.prato.it
 ******************************************************************************/
/*
 * Copyright  Comune di Prato - Sistema Informativo
 * Project:    Package SIT per elaborazione/interrogazione/gestione dati cartografici 
 * File:       TestSITBaseLayer.java
 * Author:     Alessandro Radaelli
 * Version:    1.0.0
 * CreationDate:  
 * ModifyDate:     
 *             
 */
package it.prato.comune.sit.test;

import it.prato.comune.sit.Filtro;
import it.prato.comune.sit.LayerTerritorio;
import it.prato.comune.sit.MetadatoRicerche;
import it.prato.comune.sit.OggettoTerritorio;
import it.prato.comune.sit.Punto;
import it.prato.comune.sit.SITException;
import it.prato.comune.sit.SITPaginatedResult;
import it.prato.comune.sit.SortItem;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 * Classe TestSITBaseLayer.
 * 
 * @author Ing. Alessandro Radaelli
 *
 */
public abstract class TestSITBaseLayer extends TestSITBase{
	
	@BeforeClass
    public void setUp() throws Exception {
	    super.setUp();
	}
	
	@AfterClass
    public void tearDown(){
        super.tearDown();
    }
	
	
	abstract protected LayerTerritorio getTestLayer();
	
	protected  HashMap<String, Object> getDefaultFieldsContent() {
	    return new HashMap<String, Object> ();
	}
	
	protected String getTestIDTPNNotPresent() {
	    return "9918998";
	}
	
	protected String getTestIDTPNPresent() {
        return null;
    }
	
	protected Filtro getFiltroSubset() throws SITException {
		
		Filtro flt = getTestLayer().getFiltroVuoto();
		return flt;
		
	}
	
	protected List<String> getSortFieldsNL() {
		
		//List<String> retVal = new ArrayList<String>();
		//retVal.add(e);
		return null;
	}
	
	protected int getTestPageLength() {
		return 100;
	}
	

	/**
	 * Controlla che gli oggetti contenuti in actual siano uguali a quelli identificati dalle chiavi contenute in idTPNList
	 * Non  necessario che gli oggetti siano nello stesso ordine, ma e' necessario che siano tutti e soli quelli identificati dalle chiavi.
	 * 
	 * @param actual
	 * @param idTPNList
	 */
	protected void utiTestListaUgualeA(List<OggettoTerritorio> actual, String... idTPNList ) {

		assertNotNull(actual);
		assertNotNull(idTPNList);
		assertEquals(actual.size(), idTPNList.length);

		boolean bFlagtotale = true;

		for (OggettoTerritorio og:actual) {
			boolean bFlag = false;

			for (String idTPN:idTPNList) {
				if (utiTestUgualeNoAssert(og,idTPN)) {
					bFlag=true ;
					break;
				}
			}

			if (!bFlag) {
				bFlagtotale = false;
				break;
			}
		}

		assertTrue(bFlagtotale);        
	}    

	/**
	 * Test di uguaglianza tra due oggetti.
	 * 
	 * @param actual
	 * @param idTPN
	 * 
	 * @see {@link TestSITBaseLayer#utiTestUgualeA(OggettoTerritorio actual, String idTPN)}
	 */
	protected void utiTestUgualeA(List<OggettoTerritorio> actual, String idTPN ) {        
		assertNotNull(actual);
		assertEquals(1, actual.size());

		utiTestUgualeA (actual.get(0), idTPN);        
	}

	/**
	 * Test di uguaglianza tra due oggetti.
	 * 
	 * @param actual
	 * @param idTPN
	 * @return
	 */
	protected boolean utiTestUgualeNoAssert(OggettoTerritorio actual, String idTPN ) {
		OggettoTerritorio oggDaIDTPN = actual.getLayer().cercaIDTPN(idTPN);

		assertNotNull(oggDaIDTPN);
		assertNotNull(actual);
		return oggDaIDTPN.ugualeA(actual, true);
	}

	/**
	 * Test di uguaglienza tra due oggetti.
	 * 
	 * @param actual
	 * @param idTPN
	 * 
	 * @see {@link TestSITBaseLayer#utiTestUgualeNoAssert(OggettoTerritorio actual, String idTPN)}
	 */
	protected void utiTestUgualeA(OggettoTerritorio actual, String idTPN ) {
		assertTrue(utiTestUgualeNoAssert( actual,  idTPN ));
	}

	/**
	 * Test di rimozione oggetto.
	 * 
	 * @param lay
	 * @param oggTerr
	 * @throws SITException
	 * @throws IOException
	 * @throws SQLException
	 */
	protected void utiTestLayerRemoveOggetto(LayerTerritorio lay, OggettoTerritorio oggTerr) throws SITException, IOException, SQLException {

		// //////////////////////
		//Cancellazione
		// //////////////////////
		lay.removeFeature(oggTerr);

		// ////////////////////////////////////////////////////
		//Verifica che sia stato effettivamente cancellato
		// ////////////////////////////////////////////////////
		OggettoTerritorio oggTerrTrovato = lay.cercaIDTPN(oggTerr.getIDTPNScrittura());

		assertNull(oggTerrTrovato);
	}

	/**
	 * Test di modifica oggetto.
	 * 
	 * @param lay
	 * @param oggTerr
	 * @return
	 * @throws SITException
	 * @throws IOException
	 * @throws SQLException
	 */
	protected OggettoTerritorio utiTestLayerModifyOggetto(LayerTerritorio lay, OggettoTerritorio oggTerr) throws SITException, IOException, SQLException {

		// //////////////
		//Modifica 
		// //////////////
		lay.modifyFeature(oggTerr);

		// ///////////////////////////////////////
		// Verifica se la modifica  riuscita
		// ///////////////////////////////////////
		OggettoTerritorio oggTerrTrovato = lay.cercaIDTPN(oggTerr.getIDTPNScrittura());

		assertNotNull(oggTerrTrovato);

		if (oggTerr.getGeometryAttributeWKT()!=null) {
			assertNotNull(oggTerrTrovato.getGeometryAttributeWKT());
		}

		return oggTerrTrovato;
	}

	/**
	 * Test di inserimento oggetto.
	 * 
	 * @param lay
	 * @param oggTerr
	 * @return
	 * @throws SITException
	 * @throws IOException
	 */
	protected OggettoTerritorio utiTestLayerInsertOggetto(LayerTerritorio lay, OggettoTerritorio oggTerr) throws SITException, IOException {

		// /////////////////////
		//Inserimento
		// /////////////////////
		lay.appendFeature(oggTerr);

		// /////////////////////////////////////////////////////
		//Ricerca e verifica che sia uguale a quello inserito
		// /////////////////////////////////////////////////////
		
		String id =  oggTerr.getIDTPNScrittura();
		OggettoTerritorio oggTerrTrovato = lay.cercaIDTPN(id);

		assertNotNull(oggTerrTrovato);

		if (oggTerr.getGeometryAttributeWKT()!=null) {
			assertNotNull(oggTerrTrovato.getGeometryAttributeWKT());
		}

		return oggTerrTrovato;
	}

	/**
	 * Test creazione nuovo oggetto.
	 * 
	 * @param lay
	 * @return
	 * @throws SITException
	 * @throws IOException
	 */
	protected OggettoTerritorio utiTestLayerGetOggetto(LayerTerritorio lay) throws SITException, IOException {
		assertNotNull(lay);
		OggettoTerritorio oggTerr = lay.creaNuovoOggettoTerritorio();
		assertNotNull(oggTerr);
		return oggTerr;
	}

	/**
	 * Test delle operazioni di CRUD.
	 * 
	 * @param lay
	 * @param oggTerr
	 * @param campiDaMod
	 * @throws SITException
	 * @throws IOException
	 */
	protected void utiTestCRUDLayer(LayerTerritorio lay, OggettoTerritorio oggTerr, Map<String, Object> campiDaMod) throws SITException, IOException{

		try {
			OggettoTerritorio oggTrovato = utiTestLayerInsertOggetto(lay, oggTerr);
			assertNotNull(oggTerr);
			boolean ug = oggTerr.ugualeA(oggTrovato, true, true);
			assertTrue(ug);

			// //////////////////
			//Modifica
			// //////////////////
			if (campiDaMod.size()>0) {
    			for (String nlCampo: campiDaMod.keySet()) { 
    				oggTerr.setAttributeByNL(nlCampo, campiDaMod.get(nlCampo));
    			}
    
    			oggTrovato = utiTestLayerModifyOggetto(lay, oggTerr);
    			assertTrue(oggTerr.ugualeA(oggTrovato, true));
			}
		} catch (Exception e) {
			logger.warn("Errore durante utiTestCRUDLayer -" + e.getMessage(), e);
			throw new SITException(e.getMessage(),e);
		}finally {
			if (lay!=null && oggTerr!=null) {
				// //////////////////////
				// Cancellazione
				// //////////////////////
				try {
					utiTestLayerRemoveOggetto(lay, oggTerr);
				} catch (SQLException exc) {
					logger.warn("SQLException -" + exc.getMessage(), exc);
					throw new SITException(exc.getMessage(), exc);
				}
			}
		} 
	}
	
	private void checkPrerequisiti() {        
		assertNotNull(getTestLayer());        
	}
    
    @Test
    public void testNomiCampi() {
    
        checkPrerequisiti();
        // NomiCampi
        // Per adesso ci sono dei casi nei quali  null??       
        HashMap<String, String> nc = getTestLayer().getNomiCampi();
        assertNotNull(nc);
        for(String nl: nc.keySet()) {
            String nlv = nc.get(nl);
            assertNotNull(nlv);
            assertTrue(!nlv.equals(""));
        }
    
    }
    
    @Test
    public void testLetturaCampiDaNomiCampi() throws SITException {
    
        checkPrerequisiti();
        if (getTestIDTPNPresent()!=null) {
        	LayerTerritorio layer = getTestLayer();
            OggettoTerritorio trova = layer.cercaIDTPN(getTestIDTPNPresent());
            assertNotNull(trova);
            HashMap<String, String> nc1 = getTestLayer().getNomiCampi();
            assertNotNull(nc1);
            for(String nl: nc1.keySet()) {
                Object valorecampo = trova.getAttributeByNL(nl);
                // Non so quale debba essere il valore, quindi mi limito a leggerli tutti per vedere che non vada in eccezione;
            }   
            
        }    
    }
    
    @Test
    public void testCodTPN() {
        
        checkPrerequisiti();
        // getCodTPN
        int codTPN = getTestLayer().getCodTPN();
        assertTrue(codTPN!=0);        
    }

    @Test
    public void testConfigPrefix() {
        
        checkPrerequisiti();
        // getConfigPrefix
        String configPrefix = getTestLayer().getConfigPrefix();
        assertNotNull(configPrefix);
        assertTrue(!configPrefix.equals(""));
        
    }
    
    @Test
    public void testCercaIDTPN() throws SITException {
        
        checkPrerequisiti();
        // cercaIDTPN
        // Se non trova risponde null
        LayerTerritorio layer = getTestLayer();
        OggettoTerritorio nontrova = layer.cercaIDTPN(getTestIDTPNNotPresent());
        assertNull(nontrova);
        if (getTestIDTPNPresent()!=null) {
            OggettoTerritorio trova = layer.cercaIDTPN(getTestIDTPNPresent());
            assertNotNull(trova);
            
        }
        
    }
    
    @Test
    public void testCreaNuovoOggettoTerritorio() throws IOException, SITException {
        
        checkPrerequisiti();
        LayerTerritorio layer = getTestLayer();
        
        OggettoTerritorio nuovo = layer.creaNuovoOggettoTerritorio();
        assertNotNull(nuovo);
        // Lettura di tutti campi definiti essendo un oggetto nuovo sono tutti null ma devono essere letti correttamente 
        OggettoTerritorio nuovo1 = layer.creaNuovoOggettoTerritorio();
        assertNotNull(nuovo1);
        HashMap<String, String> nc1 = layer.getNomiCampi();
        assertNotNull(nc1);
        HashMap<String, Object> defaultFieldContents = getDefaultFieldsContent();
        
        for(String nl: nc1.keySet()) {
            Object valorecampo = nuovo1.getAttributeByNL(nl);
            if (defaultFieldContents.containsKey(nl)) {
                assertEquals(defaultFieldContents.get(nl), valorecampo);
            } else {
                assertNull("Il valore del campo " + nl + " non  nullo come atteso, ma " + ((valorecampo!=null) ? valorecampo.toString() + " di tipo " + valorecampo.getClass().getName() : "") ,valorecampo);
            }
        }
        
        
    }
    
    @Test
    public void testGeometryAttribute() throws IOException, SITException {
        
        checkPrerequisiti();
        // Scrittura e Lettura campo geometrico
        // Attenzione, se la geometria  null va in eccezione
        String testWKT = "MULTIPOLYGON(((1660436.82350625 4867176.52215365,1660436.82350625 4867176.52215365,1660436.82350625 4867176.52215365,1663742.90230223 4865287.33427023,1660268.14601666 4865388.54076399,1660436.82350625 4867176.52215365)))";
        
        LayerTerritorio layer = getTestLayer();
        
        OggettoTerritorio nuovo2 = layer.creaNuovoOggettoTerritorio();
        assertNotNull(nuovo2);
        nuovo2.setGeometryAttributeWKT(testWKT);
        String wkt = nuovo2.getGeometryAttributeWKT();
        assertNotNull(wkt);
        assertNotNull(nuovo2.getCentroide());
        assertNotNull(nuovo2.getRettContenitore());
        assertTrue(nuovo2.distanza(new Punto(0,0))!=0);
        // ... altri
        
        
    }
    
    @Test
    public void testGetLayerByCodTPN() throws IOException, SITException {
        
        checkPrerequisiti();
        
        LayerTerritorio layer = getTestLayer();
        
        int codTPN = layer.getCodTPN();
        LayerTerritorio layer1 = layersManager.getLayerByCodTPN(codTPN);
        assertNotNull(layer1);
        assertTrue(layer==layer1);
        
    }
    
    @Test
    public void testChiInterseca() throws IOException, SITException {
        
        checkPrerequisiti();
        // Metodo chiInterseca
        LayerTerritorio layer = getTestLayer();
        ArrayList<OggettoTerritorio> interseca =  layer.chiInterseca(0, 0);
        assertNotNull(interseca);
        assertEquals(0, interseca.size());
        if (getTestIDTPNPresent()!=null) {
            OggettoTerritorio trovato = layer.cercaIDTPN(getTestIDTPNPresent());
            assertNotNull(trovato);
            interseca =  layer.chiInterseca(trovato);
            assertNotNull(interseca);
            // interseca almeno se stesso
            assertTrue(interseca.size()>0);
        }
        
        
    }

    @Test
    public void testContiene() throws IOException, SITException {
        
        checkPrerequisiti();
        // Metodo chiInterseca
        LayerTerritorio layer = getTestLayer();
        ArrayList<OggettoTerritorio> contiene =  layer.chiContiene(0, 0);
        assertNotNull(contiene);
        assertEquals(0, contiene.size());
        /*if (getTestIDTPNPresent()!=null) {
            OggettoTerritorio trovato = getTestLayer().cercaIDTPN(getTestIDTPNPresent());
            assertNotNull(trovato);
            contiene =  getTestLayer().chiContiene(trovato);
            assertNotNull(contiene);
            // interseca almeno se stesso
            assertTrue(contiene.size()>0);
        }*/
        
        
    }
    
    @Test
    public void testGetFiltroVuoto() throws IOException, SITException {
        
        checkPrerequisiti();
        LayerTerritorio layer = getTestLayer();
        Filtro filtro = layer.getFiltroVuoto();
        assertNotNull(filtro);
                
        
    }
    
    @Test
    public void testGetRicerche() throws IOException, SITException {
        
        checkPrerequisiti();
        LayerTerritorio layer = getTestLayer();
        HashMap<Integer, MetadatoRicerche> ricerche = layer.getRicerche();
        assertNotNull(ricerche);
        for(MetadatoRicerche ric: ricerche.values()) {
            String desc = ric.getDescrizione();
            assertNotNull(desc);
            assertTrue(!desc.equals(""));
            // ... etc.
            
        }                
    }
   
   
    @Test
    public void testFiltroPaginatoSingolaPagina() throws IOException, SITException {
        
        checkPrerequisiti();
        LayerTerritorio layer = getTestLayer();
        
        // Il test viene fatto utilizzando i dati del subset se definito
        Filtro flt = getFiltroSubset();
        
        assertNotNull(flt);
        layer.setFiltro(flt);
        
        int nRecord = layer.getCountFiltro();
        
        if (nRecord > 0) {
			SITPaginatedResult risPag=layer.cercaFiltro(null, 1, 0, null );
	        assertNotNull(risPag);
	        List<? extends OggettoTerritorio> ris = risPag.getResult();
	        assertNotNull(ris);
	        assertNotNull(risPag.getMaxFeatures());
	        assertEquals((Integer) 1,risPag.getMaxFeatures());
	        assertNotNull(risPag.getStartIndex());
	        assertEquals((Integer) 0,risPag.getStartIndex());
	        assertNull(risPag.getSortItems());
	        assertTrue(ris.size()==1);
	        
	        // Verifica metodo senza paginazione
	        flt = getFiltroSubset();
	        assertNotNull(flt);
	        layer.setFiltro(flt);
	        ris = layer.cercaFiltro();
	        assertNotNull(ris);
	        assertEquals(nRecord, ris.size());
        }
    }
    
    @Test
    public void testFiltroPaginatoLeggiDuePagine() throws IOException, SITException {
        
        checkPrerequisiti();
        LayerTerritorio layer = getTestLayer();
        
        // Il test viene fatto utilizzando i dati del subset se definito
        Filtro flt = getFiltroSubset();
        
        assertNotNull(flt);
        layer.setFiltro(flt);
        
        int nRecord = layer.getCountFiltro();
        
        if (nRecord>1) {
        	// Prelevo pagina 0 di un elemento
        	SITPaginatedResult risPag =layer.cercaFiltro(null, 1, 0, null );
        	assertNotNull(risPag);
	        List<? extends OggettoTerritorio> ris = risPag.getResult();
	        assertNotNull(ris);
	        assertTrue(ris.size()==1);
	        OggettoTerritorio ris1 = ris.get(0);
	        
	        // Prelevo pagina 1 di un elemento 
	        risPag =layer.cercaFiltro(null, 1, 1, null );
        	assertNotNull(risPag);
	        ris = risPag.getResult();
	        assertNotNull(ris);
	        assertTrue(ris.size()==1);
	        OggettoTerritorio ris2 = ris.get(0);
	        
	        // Verifico che siano diversi
	        assertTrue(!ris1.getIDTPN().equals(ris2.getIDTPN()));
        }
    }
    
    @Test
    public void testFiltroPaginatoTuttePagine() throws IOException, SITException {
        
        checkPrerequisiti();
        LayerTerritorio layer = getTestLayer();
        
        // Il test viene fatto utilizzando i dati del subset se definito
        Filtro flt = getFiltroSubset();
        
        assertNotNull(flt);
        layer.setFiltro(flt);
        
        int nRecord = layer.getCountFiltro();
        
       
        if (nRecord>1) {
        	
	        // verifico che chiedendo tutte le pagine vengono recuperati tutti i record
	        double pageLen = getTestPageLength();
	        int recLetti = 0;
	        for (int page=0; page < Math.ceil(nRecord/pageLen) ; page++) {
	        	Integer start = (int) (pageLen*page);
	        	SITPaginatedResult risPag = layer.cercaFiltro(transaction, (int)pageLen, start, null);
	        	assertNotNull(risPag);
	        	assertEquals((Integer) (int)pageLen, risPag.getMaxFeatures());
	        	assertEquals(start,  risPag.getStartIndex());
	        	List<? extends OggettoTerritorio> ris = risPag.getResult();
	        	//ris = layer.cercaFiltro(transaction, null, null, null);

	        	assertNotNull(ris);
	        	recLetti += ris.size();
	        }
	        assertEquals(nRecord, recLetti);
        }
    }
    
    @Test
    public void testFiltroPaginatoOrdinamentoCresc() throws IOException, SITException {
        
        checkPrerequisiti();
        LayerTerritorio layer = getTestLayer();
        
        // Il test viene fatto utilizzando i dati del subset se definito
        Filtro flt = getFiltroSubset();
        
        assertNotNull(flt);
        layer.setFiltro(flt);
        
        int nRecord = layer.getCountFiltro();
        
        
        if (nRecord>1) {
        	// Test su ordinamenti
	        if (getSortFieldsNL()!=null) {
	        	// Richiesta solo ordinamento senza paginazione, tutti crescenti
	        	List<SortItem> sort = new ArrayList<SortItem>();
		        for (String sf: getSortFieldsNL()) {
		        	SortItem si = new SortItem();
		        	si.setNomeLogico(sf);
		        	si.setOrdine(SortItem.Dir.CRESCENTE);
		        	sort.add(si);
		        }
		        SITPaginatedResult risPagCresc =layer.cercaFiltro(null, null, null, sort.toArray(new SortItem[sort.size()]));
	        	assertNotNull(risPagCresc);
				List<? extends OggettoTerritorio> risCresc = risPagCresc.getResult();
		        assertNotNull(risCresc);
		        assertEquals(nRecord, risCresc.size());
	        }
        }
    }
    
    @Test
    public void testFiltroPaginatoOrdinamentoDecresc() throws IOException, SITException {
        
        checkPrerequisiti();
        LayerTerritorio layer = getTestLayer();
        
        // Il test viene fatto utilizzando i dati del subset se definito
        Filtro flt = getFiltroSubset();
        
        assertNotNull(flt);
        layer.setFiltro(flt);
        
        int nRecord = layer.getCountFiltro();
        
        if (nRecord>1) {
        	// Test su ordinamenti
	        if (getSortFieldsNL()!=null) {
	        	
		        // Richiesta solo ordinamento senza paginazione, tutti decrescenti
		        List<SortItem> sort = new ArrayList<SortItem>();
		        for (String sf: getSortFieldsNL()) {
		        	SortItem si = new SortItem();
		        	si.setNomeLogico(sf);
		        	si.setOrdine(SortItem.Dir.DECRESCENTE);
		        	sort.add(si);
		        }
		        SITPaginatedResult risDecrescPag = layer.cercaFiltro(null, null, null, sort.toArray(new SortItem[sort.size()]));
	        	assertNotNull(risDecrescPag);
				List<? extends OggettoTerritorio> risDecresc = risDecrescPag.getResult();
		        assertNotNull(risDecresc);
		        assertEquals(nRecord, risDecresc.size());
		        
	        }
        }
    }
    
    @Test
    public void testFiltroPaginatoOrdinamento() throws IOException, SITException {
        
        checkPrerequisiti();
        LayerTerritorio layer = getTestLayer();
        
        // Il test viene fatto utilizzando i dati del subset se definito
        Filtro flt = getFiltroSubset();
        
        assertNotNull(flt);
        layer.setFiltro(flt);
        
        int nRecord = layer.getCountFiltro();
        
        if (nRecord>1) {
        	// Test su ordinamenti
	        if (getSortFieldsNL()!=null) {
	        	
	        	// Richiesta solo ordinamento senza paginazione, tutti crescenti
	        	List<SortItem> sort = new ArrayList<SortItem>();
		        for (String sf: getSortFieldsNL()) {
		        	SortItem si = new SortItem();
		        	si.setNomeLogico(sf);
		        	si.setOrdine(SortItem.Dir.CRESCENTE);
		        	sort.add(si);
		        }
		        SITPaginatedResult risPagCresc =layer.cercaFiltro(null, null, null, sort.toArray(new SortItem[sort.size()]));
	        	assertNotNull(risPagCresc);
				List<? extends OggettoTerritorio> risCresc = risPagCresc.getResult();
		        assertNotNull(risCresc);
		        assertEquals(nRecord, risCresc.size());
	        	
		        // Richiesta solo ordinamento senza paginazione, tutti decrescenti
		        sort.clear();
		        for (String sf: getSortFieldsNL()) {
		        	SortItem si = new SortItem();
		        	si.setNomeLogico(sf);
		        	si.setOrdine(SortItem.Dir.DECRESCENTE);
		        	sort.add(si);
		        }
		        SITPaginatedResult risDecrescPag = layer.cercaFiltro(null, null, null, sort.toArray(new SortItem[sort.size()]));
	        	assertNotNull(risDecrescPag);
				List<? extends OggettoTerritorio> risDecresc = risDecrescPag.getResult();
		        assertNotNull(risDecresc);
		        assertEquals(nRecord, risDecresc.size());
		        
		        // Verifica dei due casi crescente e decrescente
		        // Stesso numero elementi
		        assertEquals(risCresc.size(), risDecresc.size());
		        // Stessi elementi ma in ordine invertito
		        for (int i= 0;  i< risCresc.size(); i++) {
		        	assertEquals(risCresc.get(i).getIDTPN(), risDecresc.get(nRecord-i-1).getIDTPN());
		        }   
		        
	        }
        }
    }
    
    
}
